@@ -4,7 +4,7 @@
 
 # Get version and revision number
 MAJ=0
-MIN=10
+MIN=11
 REV=0
 
 if svn --xml info >/dev/null 2>&1; then
@@ -25,7 +25,7 @@ else
 fi
 
 # create m4 subdirectory which fails for some older versions of autotools
-[[ -d m4 ]] || mkdir m4
+[ -d m4 ] || mkdir m4
 
 if which glibtoolize > /dev/null 2>&1; then
 	echo "calling glibtoolize"
@@ -58,3 +58,7 @@ autoconf
 
 echo "Done"
 echo "Please run configure."
+
+# touch file to update Version.h
+touch src/Build/Version.h.in
+
