@@ -30,7 +30,7 @@
 # include <iostream>
 # include <sstream>
 # include <exception>
-# if defined(FC_OS_LINUX) || defined(FC_OS_MACOSX)
+# if defined(FC_OS_LINUX) || defined(FC_OS_MACOSX) || defined(FC_OS_BSD)
 # include <unistd.h>
 # include <pwd.h>
 # include <sys/types.h>
@@ -68,6 +68,7 @@
 #include <Base/BoundBoxPy.h>
 #include <Base/PlacementPy.h>
 #include <Base/RotationPy.h>
+#include <Base/Sequencer.h>
 #include <Base/Tools.h>
 #include <Base/UnitsApi.h>
 
@@ -222,6 +223,10 @@ Application::Application(ParameterManager * /*pcSysParamMngr*/,
           "The Unit API");
     Py_INCREF(pUnitsModule);
     PyModule_AddObject(pAppModule, "Units", pUnitsModule);
+
+    Base::ProgressIndicatorPy::init_type();
+    Base::Interpreter().addType(Base::ProgressIndicatorPy::type_object(),
+        pBaseModule,"ProgressIndicator");
 }
 
 Application::~Application()
@@ -271,6 +276,12 @@ Document* Application::newDocument(const char * Name, const char * UserName)
     _pActiveDoc->signalRenamedObject.connect(boost::bind(&App::Application::slotRenamedObject, this, _1));
     _pActiveDoc->signalActivatedObject.connect(boost::bind(&App::Application::slotActivatedObject, this, _1));
 
+    // make sure that the active document is set in case no GUI is up
+    {
+        Base::PyGILStateLocker lock;
+        Py::Object active(_pActiveDoc->getPyObject(), true);
+        Py::Module("FreeCAD").setAttr(std::string("ActiveDocument"),active);
+    }
 
     signalNewDocument(*_pActiveDoc);
 
@@ -408,6 +419,18 @@ Document* Application::getActiveDocument(void) const
 void Application::setActiveDocument(Document* pDoc)
 {
     _pActiveDoc = pDoc;
+
+    // make sure that the active document is set in case no GUI is up
+    if (pDoc) {
+        Base::PyGILStateLocker lock;
+        Py::Object active(pDoc->getPyObject(), true);
+        Py::Module("FreeCAD").setAttr(std::string("ActiveDocument"),active);
+    }
+    else {
+        Base::PyGILStateLocker lock;
+        Py::Module("FreeCAD").setAttr(std::string("ActiveDocument"),Py::None());
+    }
+
     if (pDoc)
         signalActiveDocument(*pDoc);
 }
@@ -904,7 +927,9 @@ void Application::init(int argc, char ** argv)
 
         initTypes();
 
+#if (BOOST_VERSION < 104600) || (BOOST_FILESYSTEM_VERSION == 2)
         boost::filesystem::path::default_name_check(boost::filesystem::no_check);
+#endif
 
         initConfig(argc,argv);
         initApplication();
@@ -939,15 +964,17 @@ void Application::initTypes(void)
     App ::PropertyAngle             ::init();
     App ::PropertyDistance          ::init();
     App ::PropertyLength            ::init();
+    App ::PropertySpeed             ::init();
+    App ::PropertyAcceleration      ::init();
     App ::PropertyInteger           ::init();
     App ::PropertyIntegerConstraint ::init();
     App ::PropertyPercent           ::init();
     App ::PropertyEnumeration       ::init();
     App ::PropertyIntegerList       ::init();
     App ::PropertyString            ::init();
+    App ::PropertyFont              ::init();
     App ::PropertyStringList        ::init();
     App ::PropertyLink              ::init();
-    App ::PropertyLinkWeak          ::init();
     App ::PropertyLinkSub           ::init();
     App ::PropertyLinkList          ::init();
     App ::PropertyLinkSubList       ::init();
@@ -956,6 +983,7 @@ void Application::initTypes(void)
     App ::PropertyVectorList        ::init();
     App ::PropertyPlacement         ::init();
     App ::PropertyPlacementLink     ::init();
+    App ::PropertyGeometry          ::init();
     App ::PropertyComplexGeoData    ::init();
     App ::PropertyColor             ::init();
     App ::PropertyColorList         ::init();
@@ -977,6 +1005,7 @@ void Application::initTypes(void)
     App ::InventorObject            ::init();
     App ::VRMLObject                ::init();
     App ::Annotation                ::init();
+    App ::AnnotationLabel           ::init();
     App ::MeasureDistance           ::init();
 }
 
@@ -1094,6 +1123,11 @@ void Application::initApplication(void)
     if (!(mConfig["Verbose"] == "Strict")) Console().Log("Create Application\n");
     Application::_pcSingleton = new Application(0,0,mConfig);
 
+    // set up Unit system default
+    ParameterGrp::handle hGrp = App::GetApplication().GetParameterGroupByPath
+       ("User parameter:BaseApp/Preferences/Units");
+    UnitsApi::setSchema((UnitSystem)hGrp->GetInt("UserSchema",0));
+
     // starting the init script
     Interpreter().runString(Base::ScriptFactory().ProduceScript("FreeCADInit"));
 }
@@ -1203,10 +1237,13 @@ void Application::LoadParameters(void)
 
     try {
         if (_pcSysParamMngr->LoadOrCreateDocument(mConfig["SystemParameter"].c_str()) && !(mConfig["Verbose"] == "Strict")) {
-            Console().Warning("   Parameter not existing, write initial one\n");
-            Console().Message("   This warning normally means that FreeCAD is running the first time\n"
-                              "   or the configuration was deleted or moved. Build up the standard\n"
-                              "   configuration.\n");
+            // Configuration file optional when using as Python module
+            if (!Py_IsInitialized()) {
+                Console().Warning("   Parameter not existing, write initial one\n");
+                Console().Message("   This warning normally means that FreeCAD is running the first time\n"
+                                  "   or the configuration was deleted or moved. Build up the standard\n"
+                                  "   configuration.\n");
+            }
         }
     }
     catch (const Base::Exception& e) {
@@ -1219,11 +1256,13 @@ void Application::LoadParameters(void)
 
     try {
         if (_pcUserParamMngr->LoadOrCreateDocument(mConfig["UserParameter"].c_str()) && !(mConfig["Verbose"] == "Strict")) {
-            Console().Warning("   User settings not existing, write initial one\n");
-            Console().Message("   This warning normally means that FreeCAD is running the first time\n"
-                              "   or your configuration was deleted or moved. The system defaults\n"
-                              "   will be reestablished for you.\n");
-
+            // Configuration file optional when using as Python module
+            if (!Py_IsInitialized()) {
+                Console().Warning("   User settings not existing, write initial one\n");
+                Console().Message("   This warning normally means that FreeCAD is running the first time\n"
+                                  "   or your configuration was deleted or moved. The system defaults\n"
+                                  "   will be reestablished for you.\n");
+            }
         }
     }
     catch (const Base::Exception& e) {
@@ -1246,6 +1285,15 @@ namespace boost { namespace program_options {
 #endif
 #endif
 
+// reported for SUSE in issue #0000208
+#if defined(__GNUC__)
+#if BOOST_VERSION == 104400
+namespace boost { namespace filesystem { 
+    bool no_check( const std::string & ) { return true; }
+} }
+#endif
+#endif
+
 pair<string, string> customSyntax(const string& s)
 {
     if (s.find("-display") == 0)
@@ -1507,7 +1555,7 @@ void Application::ExtractUserPath()
     mConfig["BinPath"] = mConfig["AppHomePath"] + "bin" + PATHSEP;
     mConfig["DocPath"] = mConfig["AppHomePath"] + "doc" + PATHSEP;
 
-#if defined(FC_OS_LINUX) || defined(FC_OS_CYGWIN)
+#if defined(FC_OS_LINUX) || defined(FC_OS_CYGWIN) || defined(FC_OS_BSD)
     // Default paths for the user depending stuff on the platform
     struct passwd *pwd = getpwuid(getuid());
     if (pwd == NULL)
@@ -1530,7 +1578,7 @@ void Application::ExtractUserPath()
     if (mConfig.find("AppDataSkipVendor") == mConfig.end()) {
         appData += mConfig["ExeVendor"];
         fi.setFile(appData.c_str());
-        if (!fi.exists()) {
+        if (!fi.exists() && !Py_IsInitialized()) {
             if (!fi.createDirectory()) {
                 std::string error = "Cannot create directory ";
                 error += appData;
@@ -1544,7 +1592,7 @@ void Application::ExtractUserPath()
     
     appData += mConfig["ExeName"];
     fi.setFile(appData.c_str());
-    if (!fi.exists()) {
+    if (!fi.exists() && !Py_IsInitialized()) {
         if (!fi.createDirectory()) {
             std::string error = "Cannot create directory ";
             error += appData;
@@ -1585,7 +1633,7 @@ void Application::ExtractUserPath()
     if (mConfig.find("AppDataSkipVendor") == mConfig.end()) {
         appData += mConfig["ExeVendor"];
         fi.setFile(appData.c_str());
-        if (!fi.exists()) {
+        if (!fi.exists() && !Py_IsInitialized()) {
             if (!fi.createDirectory()) {
                 std::string error = "Cannot create directory ";
                 error += appData;
@@ -1599,7 +1647,7 @@ void Application::ExtractUserPath()
     
     appData += mConfig["ExeName"];
     fi.setFile(appData.c_str());
-    if (!fi.exists()) {
+    if (!fi.exists() && !Py_IsInitialized()) {
         if (!fi.createDirectory()) {
             std::string error = "Cannot create directory ";
             error += appData;
@@ -1650,7 +1698,7 @@ void Application::ExtractUserPath()
             appData += PATHSEP;
             appData += mConfig["ExeVendor"];
             fi.setFile(appData.c_str());
-            if (!fi.exists()) {
+            if (!fi.exists() && !Py_IsInitialized()) {
                 if (!fi.createDirectory()) {
                     std::string error = "Cannot create directory ";
                     error += appData;
@@ -1664,7 +1712,7 @@ void Application::ExtractUserPath()
         appData += PATHSEP;
         appData += mConfig["ExeName"];
         fi.setFile(appData.c_str());
-        if (!fi.exists()) {
+        if (!fi.exists() && !Py_IsInitialized()) {
             if (!fi.createDirectory()) {
                 std::string error = "Cannot create directory ";
                 error += appData;
@@ -1684,7 +1732,7 @@ void Application::ExtractUserPath()
 #endif
 }
 
-#if defined (FC_OS_LINUX) || defined(FC_OS_CYGWIN)
+#if defined (FC_OS_LINUX) || defined(FC_OS_CYGWIN) || defined(FC_OS_BSD)
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/param.h>
