@@ -21,8 +21,8 @@
  ***************************************************************************/
 
 
-/*! \class App::Document
-This is bisides the Application class the most importand class in FreeCAD
+/*! \namespace App \class App::Document
+This is besides the Application class the most important class in FreeCAD
 It contains all the data of the opened, saved or newly created FreeCAD Document.
 The Document manage the Undo and Redo mechanism and the linking of documents.
 
@@ -31,11 +31,11 @@ App::Application. Only the Application can Open or destroy a document.
 
 \section Exception Exception handling
 As the document is the main data structure of FreeCAD we have to take a close
-look on how Exceptions affect the integratie of the App::Document.
+look on how Exceptions affect the integrity of the App::Document.
 
 \section UndoRedo Undo Redo an Transactions
-Undo Redo handling is one of the major machanism of an document in terms of
-user friendlines and speed (no on will whait for Undo to long).
+Undo Redo handling is one of the major mechanism of an document in terms of
+user friendliness and speed (no one will wait for Undo too long).
 
 \section Dependency Graph and dependency handling
 The FreeCAD document handles the dependencies of its DocumentObjects with
@@ -99,13 +99,12 @@ using namespace std;
 using namespace boost;
 using namespace zipios;
 
-
 #ifdef MemDebugOn
 # define new DEBUG_CLIENTBLOCK
 #endif
 
 #if FC_DEBUG
-//#  define FC_LOGFEATUREUPDATE
+#  define FC_LOGFEATUREUPDATE
 #endif 
 
 // typedef boost::property<boost::vertex_root_t, DocumentObject* > VertexProperty;
@@ -531,7 +530,7 @@ Document::~Document()
 // Exported functions
 //--------------------------------------------------------------------------
 
-void Document::Save (Writer &writer) const
+void Document::Save (Base::Writer &writer) const
 {
     writer.Stream() << "<?xml version='1.0' encoding='utf-8'?>" << endl
     << "<!--" << endl
@@ -605,7 +604,8 @@ void Document::Restore(Base::XMLReader &reader)
     Label.setValue(DocLabel.c_str());
 
     // create new transient directory
-    Base::FileInfo TransDirNew(Base::FileInfo::getTempPath() + Id.getValue());
+    std::string basePath = Base::FileInfo::getTempPath() + GetApplication().getExecutableName();
+    Base::FileInfo TransDirNew(basePath + "_Doc_"  + Id.getValue());
     if(!TransDirNew.exists())
         TransDirNew.createDirectory();
     TransientDir.setValue(TransDirNew.filePath());
@@ -1023,6 +1023,22 @@ int Document::countObjects(void) const
    return static_cast<int>(d->objectArray.size());
 }
 
+std::vector<App::DocumentObject*> Document::getInList(const DocumentObject* me) const
+{
+    // result list
+    std::vector<App::DocumentObject*> result;
+    // go through all objects
+    for (std::map<std::string,DocumentObject*>::const_iterator It = d->objectMap.begin(); It != d->objectMap.end();++It) {
+        // get the outList and search if me is in that list
+        std::vector<DocumentObject*> OutList = It->second->getOutList();
+        for (std::vector<DocumentObject*>::const_iterator It2=OutList.begin();It2!=OutList.end();++It2)
+            if (*It2 && *It2 == me)
+                // add the parent object
+                result.push_back(It->second);
+    }
+    return result;
+}
+
 void Document::recompute()
 {
     // delete recompute log
@@ -1107,7 +1123,7 @@ void Document::recompute()
     d->vertexMap.clear();
 }
 
-const char *Document::getErrorDescription(const App::DocumentObject*Obj) const
+const char * Document::getErrorDescription(const App::DocumentObject*Obj) const
 {
     for (std::vector<App::DocumentObjectExecReturn*>::const_iterator it=_RecomputeLog.begin();it!=_RecomputeLog.end();++it)
         if ((*it)->Which == Obj)
@@ -1182,7 +1198,7 @@ void Document::recomputeFeature(DocumentObject* Feat)
     _recomputeFeature(Feat);
 }
 
-DocumentObject *Document::addObject(const char* sType, const char* pObjectName)
+DocumentObject * Document::addObject(const char* sType, const char* pObjectName)
 {
     Base::BaseClass* base = static_cast<Base::BaseClass*>(Base::Type::createInstanceByName(sType,true));
 
@@ -1312,6 +1328,9 @@ void Document::_remObject(DocumentObject* pcObject)
 {
     std::map<std::string,DocumentObject*>::iterator pos = d->objectMap.find(pcObject->getNameInDocument());
 
+    if (d->activeObject == pcObject)
+        d->activeObject = 0;
+
     signalDeletedObject(*pcObject);
 
     // Transaction stuff
@@ -1398,7 +1417,7 @@ DocumentObject* Document::_copyObject(DocumentObject* obj, std::map<DocumentObje
                     copy_map[link] = link_copy;
                     static_cast<PropertyLink*>(it->second)->setValue(link_copy);
                 }
-                else if (link->getDocument() == this) {
+                else if (link && link->getDocument() == this) {
                     //static_cast<PropertyLink*>(it->second)->setValue(link);
                 }
             }
@@ -1491,12 +1510,12 @@ DocumentObject* Document::moveObject(DocumentObject* obj, bool recursive)
     return obj;
 }
 
-DocumentObject *Document::getActiveObject(void) const
+DocumentObject * Document::getActiveObject(void) const
 {
     return d->activeObject;
 }
 
-DocumentObject *Document::getObject(const char *Name) const
+DocumentObject * Document::getObject(const char *Name) const
 {
     std::map<std::string,DocumentObject*>::const_iterator pos;
 
@@ -1508,7 +1527,7 @@ DocumentObject *Document::getObject(const char *Name) const
         return 0;
 }
 
-const char *Document::getObjectName(DocumentObject *pFeat) const
+const char * Document::getObjectName(DocumentObject *pFeat) const
 {
     std::map<std::string,DocumentObject*>::const_iterator pos;
 
@@ -1558,18 +1577,15 @@ std::string Document::getStandardObjectName(const char *Name, int d) const
 
 std::vector<DocumentObject*> Document::getObjects() const
 {
-    std::vector<DocumentObject*> Objects;
-    for (std::map<std::string,DocumentObject*>::const_iterator it = d->objectMap.begin(); it != d->objectMap.end(); ++it)
-        Objects.push_back(it->second);
-    return Objects;
+    return d->objectArray;
 }
 
 std::vector<DocumentObject*> Document::getObjectsOfType(const Base::Type& typeId) const
 {
     std::vector<DocumentObject*> Objects;
-    for (std::map<std::string,DocumentObject*>::const_iterator it = d->objectMap.begin(); it != d->objectMap.end(); ++it) {
-        if (it->second->getTypeId().isDerivedFrom(typeId))
-            Objects.push_back(it->second);
+    for (std::vector<DocumentObject*>::const_iterator it = d->objectArray.begin(); it != d->objectArray.end(); ++it) {
+        if ((*it)->getTypeId().isDerivedFrom(typeId))
+            Objects.push_back(*it);
     }
     return Objects;
 }
