@@ -161,6 +161,8 @@ public:
     /// Returns a list of all Objects
     std::vector<DocumentObject*> getObjects() const;
     std::vector<DocumentObject*> getObjectsOfType(const Base::Type& typeId) const;
+    /// Returns an array with the correct types already.
+    template<typename T> inline std::vector<T*> getObjectsOfType() const;
     int countObjectsOfType(const Base::Type& typeId) const;
     /// get the number of objects in the document
     int countObjects(void) const;
@@ -228,7 +230,10 @@ public:
     //@{
     /// write GraphViz file
     void writeDependencyGraphViz(std::ostream &out);
+    /// checks if the graph is directed and has no cycles
     bool checkOnCycle(void);
+    /// get a list of all objects linking to the given object
+    std::vector<App::DocumentObject*> getInList(const DocumentObject* me) const;
     // set Changed
     //void setChanged(DocumentObject* change);
     //@}
@@ -276,6 +281,17 @@ private:
     struct DocumentP* d;
 };
 
+template<typename T>
+inline std::vector<T*> Document::getObjectsOfType() const
+{
+    std::vector<T*> type;
+    std::vector<App::DocumentObject*> obj = this->getObjectsOfType(T::getClassTypeId());
+    type.reserve(obj.size());
+    for (std::vector<App::DocumentObject*>::iterator it = obj.begin(); it != obj.end(); ++it)
+        type.push_back(static_cast<T*>(*it));
+    return type;
+}
+
 
 } //namespace App
 
