@@ -47,6 +47,14 @@ DynamicProperty::~DynamicProperty()
 {
 }
 
+void DynamicProperty::getPropertyList(std::vector<Property*> &List) const
+{
+    // get the properties of the base class first and insert the dynamic properties afterwards
+    this->pc->PropertyContainer::getPropertyList(List);
+    for (std::map<std::string,PropData>::const_iterator it = props.begin(); it != props.end(); ++it)
+        List.push_back(it->second.property);
+}
+
 void DynamicProperty::getPropertyMap(std::map<std::string,Property*> &Map) const
 {
     // get the properties of the base class first and insert the dynamic properties afterwards
@@ -243,6 +251,27 @@ std::string DynamicProperty::getUniquePropertyName(const char *Name) const
     }
 }
 
+std::string DynamicProperty::encodeAttribute(const std::string& str) const
+{
+    std::string tmp;
+    for (std::string::const_iterator it = str.begin(); it != str.end(); ++it) {
+        if (*it == '<')
+            tmp += "&lt;";
+        else if (*it == '"')
+            tmp += "&quot;";
+        else if (*it == '&')
+            tmp += "&amp;";
+        else if (*it == '>')
+            tmp += "&gt;";
+        else if (*it == '\n')
+            tmp += " ";
+        else
+            tmp += *it;
+    }
+
+    return tmp;
+}
+
 void DynamicProperty::Save (Base::Writer &writer) const 
 {
     std::map<std::string,Property*> Map;
@@ -263,7 +292,8 @@ void DynamicProperty::Save (Base::Writer &writer) const
         else {
             writer.Stream() << writer.ind() << "<Property name=\"" << it->first
                             << "\" type=\"" << it->second->getTypeId().getName()
-                            << "\" group=\"" << pt->second.group << "\" doc=\"" << pt->second.doc
+                            << "\" group=\"" << encodeAttribute(pt->second.group)
+                            << "\" doc=\"" << encodeAttribute(pt->second.doc)
                             << "\" attr=\"" << pt->second.attr << "\" ro=\"" << pt->second.readonly
                             << "\" hide=\"" << pt->second.hidden << "\">" << std::endl;
         }
