@@ -148,41 +148,6 @@ const char* PropertyContainer::getName(const Property* prop)const
     return getPropertyData().getName(this,prop);
 }
 
-std::vector<DocumentObject*> PropertyContainer::getOutList(void) const
-{
-    std::vector<Property*> List;
-    std::vector<DocumentObject*> ret;
-    getPropertyList(List);
-    for(std::vector<Property*>::const_iterator It = List.begin();It != List.end(); ++It)
-    {
-        if ((*It)->isDerivedFrom(PropertyLinkList::getClassTypeId()))
-        {
-            const std::vector<DocumentObject*> &OutList = static_cast<PropertyLinkList*>(*It)->getValues();
-            for(std::vector<DocumentObject*>::const_iterator It2 = OutList.begin();It2 != OutList.end(); ++It2)
-                ret.push_back(*It2 );
-        }
-        else if ((*It)->isDerivedFrom(PropertyLink::getClassTypeId()))
-        {
-            ret.push_back(static_cast<PropertyLink*>(*It)->getValue() );
-        }
-    }
-    return ret;
-}
-
-std::vector<DocumentObject*> PropertyContainer::getWeakOutList(void) const
-{
-    std::vector<Property*> List;
-    std::vector<DocumentObject*> ret;
-    getPropertyList(List);
-    for(std::vector<Property*>::const_iterator It = List.begin();It != List.end(); ++It)
-    {
-        if ((*It)->isDerivedFrom(PropertyLinkWeak::getClassTypeId()))
-        {
-            ret.push_back(static_cast<PropertyLinkWeak*>(*It)->getValue() );
-        }
-    }
-    return ret;
-}
 
 const PropertyData * PropertyContainer::getPropertyDataPtr(void){return &propertyData;} 
 const PropertyData & PropertyContainer::getPropertyData(void) const{return propertyData;} 
@@ -190,7 +155,7 @@ const PropertyData & PropertyContainer::getPropertyData(void) const{return prope
 PropertyData PropertyContainer::propertyData;
 
 /**
- * Binary function to query the flags for use with generic STL functions..
+ * Binary function to query the flags for use with generic STL functions.
  */
 template <class TCLASS>
 class PropertyAttribute : public std::binary_function<TCLASS, typename App::PropertyType, bool>
@@ -203,7 +168,7 @@ private:
     const PropertyContainer* cont;
 };
 
-void PropertyContainer::Save (Writer &writer) const 
+void PropertyContainer::Save (Base::Writer &writer) const 
 {
     std::map<std::string,Property*> Map;
     getPropertyMap(Map);
@@ -506,6 +471,7 @@ void PropertyData::getPropertyList(const PropertyContainer *container,std::vecto
 
 
 /** \defgroup PropFrame Property framework
+    \ingroup APP
 \section Introduction
 The property framework introduces the ability to access attributes (member variables) of a class by name without
 knowing the class type. It's like the reflection mechanism of Java or C#.
