@@ -105,9 +105,9 @@ public:
   virtual Property *getPropertyByName(const char* name) const;
   /// get the name of a property
   virtual const char* getName(const Property* prop) const;
-  /// get all properties of the class (including properies of the parent)
+  /// get all properties of the class (including properties of the parent)
   virtual void getPropertyMap(std::map<std::string,Property*> &Map) const;
-  /// get all properties of the class (including properies of the parent)
+  /// get all properties of the class (including properties of the parent)
   virtual void getPropertyList(std::vector<Property*> &List) const;
   /// set the Status bit of all properties at once
   void setPropertyStatus(unsigned char bit,bool value);
@@ -145,10 +145,6 @@ public:
       return 0;
   }
 
-  /// returns a list of objects this object is pointing to by Links
-  std::vector<DocumentObject*> getOutList(void) const;
-  std::vector<DocumentObject*> getWeakOutList(void) const;
-
   virtual void Save (Base::Writer &writer) const;
   virtual void Restore(Base::XMLReader &reader);
 
