@@ -24,6 +24,7 @@
 #include "PreCompiled.h"
 
 #ifndef _PreComp_
+# include <sstream>
 #endif
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
@@ -87,22 +88,45 @@ void PropertyFileIncluded::setValue(const char* sFile, const char* sName)
         if (_cValue == sFile)
             throw Base::Exception("Not possible to set the same file!");
 
+        std::string pathTrans = getDocTransientPath();
+        Base::FileInfo file(sFile);
+        std::string path = file.dirPath();
+        if (!file.exists()) {
+            std::stringstream str;
+            str << "File " << file.filePath() << " does not exist.";
+            throw Base::Exception(str.str());
+        }
+
         aboutToSetValue(); // undo redo by move the file away with temp name
 
         // remove old file (if not moved by undo)
         Base::FileInfo value(_cValue);
+        std::string pathAct = value.dirPath();
         if (value.exists())
             value.deleteFile();
 
-        Base::FileInfo file(sFile);
-        std::string pathTrans = getDocTransientPath();
-        std::string path = file.dirPath();
-        std::string pathAct = value.dirPath();
-
         // if a special name given, use this instead
         if (sName) {
-            _cValue = path + "/" + sName;
-            _BaseFileName = sName;
+            Base::FileInfo ExtraName(path + "/" + sName);
+            if (ExtraName.exists() ) {
+                // if a file with this name already exists search for a new one
+                int i=0;
+                
+                do {
+                    i++;
+                    std::stringstream str;
+                    str << path << "/" << sName << i;
+                    ExtraName.setFile(str.str());
+                }
+                while (ExtraName.exists());
+                _cValue = ExtraName.filePath();
+                _BaseFileName = ExtraName.fileName();
+
+            }
+            else {
+                _cValue = path + "/" + sName;
+                _BaseFileName = sName;
+            }
         }
         else if (value.fileName().empty()) {
             _cValue = pathTrans + "/" + file.fileName();
@@ -112,12 +136,36 @@ void PropertyFileIncluded::setValue(const char* sFile, const char* sName)
         // if the files is already in transient dir of the document, just use it
         if (path == pathTrans) {
             bool done = file.renameFile(_cValue.c_str());
-            assert(done);
+            //assert(done);
+            if (!done) {
+                std::stringstream str;
+                str << "Cannot rename file " << file.filePath() << " to " << _cValue;
+                throw Base::Exception(str.str());
+            }
         }
         // otherwise copy from origin location 
         else {
-            /*bool done = */file.copyTo(_cValue.c_str());
-            //assert(done);
+            // if file already exists in transient dir make a new unique name
+            Base::FileInfo fi(_cValue);
+            if (fi.exists()) {
+                Base::FileInfo fi2(Base::FileInfo::getTempFileName());
+                std::stringstream str;
+                str << fi.dirPath() << "/" << fi2.fileNamePure();
+                std::string ext = fi.extension(false);
+                if (!ext.empty())
+                    str << "." << ext;
+                Base::FileInfo fi3(str.str());
+                _cValue = fi3.filePath();
+                _BaseFileName = fi3.fileName();
+            }
+
+            bool done = file.copyTo(_cValue.c_str());
+            //assert(done); 
+            if (!done) {
+                std::stringstream str;
+                str << "Cannot copy file from " << file.filePath() << " to " << _cValue;
+                throw Base::Exception(str.str());
+            }
         }
 
         hasSetValue();
@@ -206,7 +254,7 @@ void PropertyFileIncluded::setPyObject(PyObject *value)
     setValue(string.c_str());
 }
 
-void PropertyFileIncluded::Save (Writer &writer) const
+void PropertyFileIncluded::Save (Base::Writer &writer) const
 {
     if (writer.isForceXML()) {
         writer.Stream() << writer.ind() << "<FileIncluded file=\"\">" << endl;
@@ -237,6 +285,7 @@ void PropertyFileIncluded::Restore(Base::XMLReader &reader)
     if (!file.empty()) {
         // initate a file read
         reader.addFile(file.c_str(),this);
+
         // is in the document transient path
         aboutToSetValue();
         _cValue = getDocTransientPath() + "/" + file;
