@@ -39,6 +39,10 @@ namespace Base {
 class Writer;
 }
 
+namespace Data {
+class ComplexGeoData;
+}
+
 namespace App
 {
 class Feature;
@@ -277,10 +281,30 @@ public:
     virtual void Paste(const Property &from);
 };
 
+/** The base class for all basic geometry properties.
+ * @author Werner Mayer
+ */
+class AppExport PropertyGeometry : public App::Property
+{
+    TYPESYSTEM_HEADER();
+
+public:
+    PropertyGeometry();
+    ~PropertyGeometry();
+
+    /** @name Modification */
+    //@{
+    /// Applies a transformation on the real geometric data type
+    virtual void transformGeometry(const Base::Matrix4D &rclMat) = 0;
+    /// Retrieve bounding box information
+    virtual Base::BoundBox3d getBoundingBox() const = 0;
+    //@}
+};
+
 /** The base class for all complex data properties.
  * @author Werner Mayer
  */
-class AppExport PropertyComplexGeoData : public App::Property
+class AppExport PropertyComplexGeoData : public App::PropertyGeometry
 {
     TYPESYSTEM_HEADER();
 
@@ -296,6 +320,7 @@ public:
 
     /** @name Getting basic geometric entities */
     //@{
+    virtual const Data::ComplexGeoData* getComplexData() const = 0;
     virtual Base::BoundBox3d getBoundingBox() const = 0;
     virtual void getFaces(std::vector<Base::Vector3d> &Points,
         std::vector<Data::ComplexGeoData::FacetTopo> &Topo,
