@@ -24,6 +24,8 @@
 #include "PreCompiled.h"
 
 #ifndef _PreComp_
+# include <boost/version.hpp>
+# include <boost/filesystem/path.hpp>
 #endif
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
@@ -100,7 +102,7 @@ void PropertyInteger::setPyObject(PyObject *value)
     }
 }
 
-void PropertyInteger::Save (Writer &writer) const
+void PropertyInteger::Save (Base::Writer &writer) const
 {
     writer.Stream() << writer.ind() << "<Integer value=\"" <<  _lValue <<"\"/>" << std::endl;
 }
@@ -166,9 +168,13 @@ void PropertyPath::setValue(const boost::filesystem::path &Path)
 void PropertyPath::setValue(const char * Path)
 {
     aboutToSetValue();
+#if (BOOST_VERSION < 104600) || (BOOST_FILESYSTEM_VERSION == 2)
     _cValue = boost::filesystem::path(Path,boost::filesystem::no_check );
     //_cValue = boost::filesystem::path(Path,boost::filesystem::native );
     //_cValue = boost::filesystem::path(Path,boost::filesystem::windows_name );
+#else
+    _cValue = boost::filesystem::path(Path);
+#endif
     hasSetValue();
 }
 
@@ -179,17 +185,14 @@ boost::filesystem::path PropertyPath::getValue(void) const
 
 PyObject *PropertyPath::getPyObject(void)
 {
-    const std::string s = _cValue.string() ;
-    std::string s2 = _cValue.native_file_string();
-    std::string s3 = _cValue.native_directory_string();
-
-    // decomposition functions:
-    std::string  r = _cValue.root_name();
-    std::string  d = _cValue.root_directory() ;
-    std::string  l = _cValue.leaf();
+#if (BOOST_VERSION < 104600) || (BOOST_FILESYSTEM_VERSION == 2)
+    std::string str = _cValue.native_file_string();
+#else
+    std::string str = _cValue.string();
+#endif
 
     // Returns a new reference, don't increment it!
-    PyObject *p = PyUnicode_DecodeUTF8(_cValue.native_file_string().c_str(),_cValue.native_file_string().size(),0);
+    PyObject *p = PyUnicode_DecodeUTF8(str.c_str(),str.size(),0);
     if (!p) throw Base::Exception("UTF8 conversion failure at PropertyPath::getPyObject()");
     return p;
 }
@@ -216,7 +219,7 @@ void PropertyPath::setPyObject(PyObject *value)
 }
 
 
-void PropertyPath::Save (Writer &writer) const
+void PropertyPath::Save (Base::Writer &writer) const
 {
     std::string val = encodeAttribute(_cValue.string());
     writer.Stream() << writer.ind() << "<Path value=\"" <<  val <<"\"/>" << std::endl;
@@ -261,7 +264,7 @@ TYPESYSTEM_SOURCE(App::PropertyEnumeration, App::PropertyInteger);
 
 
 PropertyEnumeration::PropertyEnumeration()
-  : _EnumArray(0)
+  : _CustomEnum(false), _EnumArray(0)
 {
 
 }
@@ -406,6 +409,50 @@ const char** PropertyEnumeration::getEnums(void) const
     return _EnumArray;
 }
 
+void PropertyEnumeration::Save(Base::Writer &writer) const
+{
+    writer.Stream() << writer.ind() << "<Integer value=\"" <<  _lValue <<"\"";
+    if (_CustomEnum)
+        writer.Stream() << " CustomEnum=\"true\"";
+    writer.Stream() << "/>" << std::endl;
+    if (_CustomEnum) {
+        std::vector<std::string> items = getEnumVector();
+        writer.Stream() << writer.ind() << "<CustomEnumList count=\"" <<  items.size() <<"\">" << endl;
+        writer.incInd();
+        for(std::vector<std::string>::iterator it = items.begin(); it != items.end(); ++it) {
+            std::string val = encodeAttribute(*it);
+            writer.Stream() << writer.ind() << "<Enum value=\"" <<  val <<"\"/>" << endl;
+        }
+        writer.decInd();
+        writer.Stream() << writer.ind() << "</CustomEnumList>" << endl;
+    }
+}
+
+void PropertyEnumeration::Restore(Base::XMLReader &reader)
+{
+    // read my Element
+    reader.readElement("Integer");
+    // get the value of my Attribute
+    long val = reader.getAttributeAsInteger("value");
+
+    if (reader.hasAttribute("CustomEnum")) {
+        reader.readElement("CustomEnumList");
+        int count = reader.getAttributeAsInteger("count");
+        std::vector<std::string> values(count);
+        for(int i = 0; i < count; i++) {
+            reader.readElement("Enum");
+            values[i] = reader.getAttribute("value");
+        }
+
+        reader.readEndElement("CustomEnumList");
+
+        _CustomEnum = true;
+        setEnumVector(values);
+    }
+
+    setValue(val);
+}
+
 PyObject *PropertyEnumeration::getPyObject(void)
 {
     if (!_EnumArray) {
@@ -451,6 +498,7 @@ void PropertyEnumeration::setPyObject(PyObject *value)
             values[i] = PyString_AsString(item);
         }
 
+        _CustomEnum = true;
         setEnumVector(values);
         setValue((long)0);
     }
@@ -623,7 +671,7 @@ void PropertyIntegerList::setPyObject(PyObject *value)
     }
 }
 
-void PropertyIntegerList::Save (Writer &writer) const
+void PropertyIntegerList::Save (Base::Writer &writer) const
 {
     writer.Stream() << writer.ind() << "<IntegerList count=\"" <<  getSize() <<"\">" << endl;
     writer.incInd();
@@ -731,7 +779,7 @@ void PropertyFloat::setPyObject(PyObject *value)
     }
 }
 
-void PropertyFloat::Save (Writer &writer) const
+void PropertyFloat::Save (Base::Writer &writer) const
 {
     writer.Stream() << writer.ind() << "<Float value=\"" <<  _dValue <<"\"/>" << std::endl;
 }
@@ -805,6 +853,19 @@ void PropertyFloatConstraint::setPyObject(PyObject *value)
         _dValue = temp;
         hasSetValue();
     } 
+    else if (PyInt_Check(value)) {
+        float temp = (float)PyInt_AsLong(value);
+        if (_ConstStruct) {
+            if (temp > _ConstStruct->UpperBound)
+                temp = _ConstStruct->UpperBound;
+            else if (temp < _ConstStruct->LowerBound)
+                temp = _ConstStruct->LowerBound;
+        }
+    
+        aboutToSetValue();
+        _dValue = temp;
+        hasSetValue();
+    } 
     else {
         std::string error = std::string("type must be float, not ");
         error += value->ob_type->tp_name;
@@ -819,6 +880,20 @@ void PropertyFloatConstraint::setPyObject(PyObject *value)
 
 TYPESYSTEM_SOURCE(App::PropertyDistance, App::PropertyFloat);
 
+//**************************************************************************
+//**************************************************************************
+// PropertySpeed
+//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+TYPESYSTEM_SOURCE(App::PropertySpeed, App::PropertyFloat);
+
+//**************************************************************************
+//**************************************************************************
+// PropertyAcceleration
+//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+TYPESYSTEM_SOURCE(App::PropertyAcceleration, App::PropertyFloat);
+
 //**************************************************************************
 //**************************************************************************
 // PropertyLength
@@ -851,7 +926,7 @@ void PropertyLength::setPyObject(PyObject *value)
 // PropertyAngle
 //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
-TYPESYSTEM_SOURCE(App::PropertyAngle, App::PropertyFloat);
+TYPESYSTEM_SOURCE(App::PropertyAngle, App::PropertyFloatConstraint);
 
 
 //**************************************************************************
@@ -940,7 +1015,7 @@ void PropertyFloatList::setPyObject(PyObject *value)
     }
 }
 
-void PropertyFloatList::Save (Writer &writer) const
+void PropertyFloatList::Save (Base::Writer &writer) const
 {
     if (writer.isForceXML()) {
         writer.Stream() << writer.ind() << "<FloatList count=\"" <<  getSize() <<"\">" << endl;
@@ -1074,7 +1149,7 @@ void PropertyString::setPyObject(PyObject *value)
     setValue(string);
 }
 
-void PropertyString::Save (Writer &writer) const
+void PropertyString::Save (Base::Writer &writer) const
 {
     std::string val = encodeAttribute(_cValue);
     writer.Stream() << writer.ind() << "<String value=\"" <<  val <<"\"/>" << std::endl;
@@ -1107,6 +1182,22 @@ unsigned int PropertyString::getMemSize (void) const
     return static_cast<unsigned int>(_cValue.size());
 }
 
+//**************************************************************************
+// PropertyFont
+//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+TYPESYSTEM_SOURCE(App::PropertyFont , App::PropertyString);
+
+PropertyFont::PropertyFont()
+{
+
+}
+
+PropertyFont::~PropertyFont()
+{
+
+}
+
 //**************************************************************************
 // PropertyStringList
 //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -1209,7 +1300,7 @@ unsigned int PropertyStringList::getMemSize (void) const
     return static_cast<unsigned int>(size);
 }
 
-void PropertyStringList::Save (Writer &writer) const
+void PropertyStringList::Save (Base::Writer &writer) const
 {
     writer.Stream() << writer.ind() << "<StringList count=\"" <<  getSize() <<"\">" << endl;
     writer.incInd();
@@ -1311,7 +1402,7 @@ void PropertyBool::setPyObject(PyObject *value)
     }
 }
 
-void PropertyBool::Save (Writer &writer) const
+void PropertyBool::Save (Base::Writer &writer) const
 {
     writer.Stream() << writer.ind() << "<Bool value=\"" ;
     if (_lValue)
@@ -1466,7 +1557,7 @@ void PropertyColor::setPyObject(PyObject *value)
     setValue( cCol );
 }
 
-void PropertyColor::Save (Writer &writer) const
+void PropertyColor::Save (Base::Writer &writer) const
 {
     writer.Stream() << writer.ind() << "<PropertyColor value=\"" 
     <<  _cCol.getPackedValue() <<"\"/>" << endl;
@@ -1597,7 +1688,7 @@ void PropertyColorList::setPyObject(PyObject *value)
     }
 }
 
-void PropertyColorList::Save (Writer &writer) const
+void PropertyColorList::Save (Base::Writer &writer) const
 {
     if (!writer.isForceXML()) {
         writer.Stream() << writer.ind() << "<ColorList file=\"" << writer.addFile(getName(), this) << "\"/>" << std::endl;
@@ -1607,11 +1698,13 @@ void PropertyColorList::Save (Writer &writer) const
 void PropertyColorList::Restore(Base::XMLReader &reader)
 {
     reader.readElement("ColorList");
-    std::string file (reader.getAttribute("file") );
+    if (reader.hasAttribute("file")) {
+        std::string file (reader.getAttribute("file"));
 
-    if (!file.empty()) {
-        // initate a file read
-        reader.addFile(file.c_str(),this);
+        if (!file.empty()) {
+            // initate a file read
+            reader.addFile(file.c_str(),this);
+        }
     }
 }
 
@@ -1731,14 +1824,13 @@ void PropertyMaterial::setTransparency(float val)
 
 PyObject *PropertyMaterial::getPyObject(void)
 {
-    return new MaterialPy(&_cMat);
+    return new MaterialPy(new Material(_cMat));
 }
 
 void PropertyMaterial::setPyObject(PyObject *value)
 {
     if (PyObject_TypeCheck(value, &(MaterialPy::Type))) {
-        MaterialPy  *pcObject = (MaterialPy*)value;
-        setValue( *pcObject->_pcMaterial );
+        setValue(*static_cast<MaterialPy*>(value)->getMaterialPtr());
     }
     else {
         std::string error = std::string("type must be 'Material', not ");
@@ -1747,7 +1839,7 @@ void PropertyMaterial::setPyObject(PyObject *value)
     }
 }
 
-void PropertyMaterial::Save (Writer &writer) const
+void PropertyMaterial::Save (Base::Writer &writer) const
 {
     writer.Stream() << writer.ind() << "<PropertyMaterial ambientColor=\"" 
         <<  _cMat.ambientColor.getPackedValue() 
