@@ -179,7 +179,11 @@ public:
     virtual PyObject *getPyObject(void);
     virtual void setPyObject(PyObject *);
 
+    virtual void Save (Base::Writer &writer) const;
+    virtual void Restore(Base::XMLReader &reader);
+
 private:
+    bool _CustomEnum;
     const char** _EnumArray;
 };
 
@@ -207,7 +211,7 @@ public:
         long LowerBound, UpperBound, StepSize;
     };
     /** setting the boundaries
-     * This sets the constriant struct. It can be dynamcly 
+     * This sets the constraint struct. It can be dynamcly 
      * allocated or set as an static in the class the property
      * blongs to:
      * \code
@@ -215,7 +219,7 @@ public:
      * \endcode
      */
     void setConstraints(const Constraints* sConstraint);
-    /// get the constriant struct
+    /// get the constraint struct
     const Constraints*  getConstraints(void) const;
     //@}
 
@@ -252,6 +256,7 @@ class AppExport PropertyIntegerList: public PropertyLists
 
 public:
     /**
+
      * A constructor.
      * A more elaborate description of the constructor.
      */
@@ -364,7 +369,7 @@ public:
         float LowerBound, UpperBound, StepSize;
     };
     /** setting the boundaries
-     * This sets the constriant struct. It can be dynamcly 
+     * This sets the constraint struct. It can be dynamcly 
      * allocated or set as an static in the class the property
      * blongs to:
      * \code
@@ -372,7 +377,7 @@ public:
      * \endcode
      */
     void setConstraints(const Constraints* sConstrain);
-    /// get the constriant struct
+    /// get the constraint struct
     const Constraints*  getConstraints(void) const;
     //@}
 
@@ -419,12 +424,38 @@ public:
  * This is a property for representing angles. It basicly a float
  * property. On the Gui it has a quantity like RAD.
  */
-class AppExport PropertyAngle: public PropertyFloat
+class AppExport PropertyAngle: public PropertyFloatConstraint
 {
     TYPESYSTEM_HEADER();
 public:
     PropertyAngle(void){}
     virtual ~PropertyAngle(){}
+    virtual const char* getEditorName(void) const { return "Gui::PropertyEditor::PropertyAngleItem"; }
+};
+
+/** Speed property
+ * This is a property for representing speed. It is basically a float
+ * property. On the Gui it has a quantity like m/s or km/h.
+ */
+class AppExport PropertySpeed: public PropertyFloat
+{
+    TYPESYSTEM_HEADER();
+public:
+    PropertySpeed(void){}
+    virtual ~PropertySpeed(){}
+    virtual const char* getEditorName(void) const { return "Gui::PropertyEditor::PropertyFloatItem"; }
+};
+
+/** Acceleration property
+ * This is a property for representing acceleration. It is basically a float
+ * property. On the Gui it has a quantity like m/s^2.
+ */
+class AppExport PropertyAcceleration: public PropertyFloat
+{
+    TYPESYSTEM_HEADER();
+public:
+    PropertyAcceleration(void){}
+    virtual ~PropertyAcceleration(){}
     virtual const char* getEditorName(void) const { return "Gui::PropertyEditor::PropertyFloatItem"; }
 };
 
@@ -523,6 +554,19 @@ private:
     std::string _cValue;
 };
 
+/** Property handling with font names.
+ */
+class AppExport PropertyFont : public PropertyString
+{
+    TYPESYSTEM_HEADER();
+
+public:
+    PropertyFont(void);
+    virtual ~PropertyFont();
+    virtual const char* getEditorName(void) const
+    { return "Gui::PropertyEditor::PropertyFontItem"; }
+};
+
 class AppExport PropertyStringList: public PropertyLists
 {
     TYPESYSTEM_HEADER();
