@@ -41,6 +41,9 @@ template <class _Precision>
 class BoundBox3
 {
 public:
+  typedef _Precision num_type;
+  typedef float_traits<num_type> traits_type;
+
   /**  Public attributes */
   //@{
   _Precision MinX;
@@ -336,7 +339,7 @@ inline bool BoundBox3<_Precision>::GetOctantFromVector (const Vector3<_Precision
 }
 
 template <class _Precision>
-inline BoundBox3<_Precision> BoundBox3<_Precision>::CalcOctant (OCTANT Octant) const
+inline BoundBox3<_Precision> BoundBox3<_Precision>::CalcOctant (typename BoundBox3< _Precision >::OCTANT Octant) const
 {
   BoundBox3<_Precision> cOct (*this);
 
@@ -507,12 +510,12 @@ inline Vector3<_Precision> BoundBox3<_Precision>::IntersectionPoint (const Vecto
   Vector3<_Precision>   cVctRes;
 
   // Vergleichs-BB um REEN_EPS vergroessern
-  cCmpBound.MaxX += FLOAT_EPS;
-  cCmpBound.MaxY += FLOAT_EPS;
-  cCmpBound.MaxZ += FLOAT_EPS;
-  cCmpBound.MinX -= FLOAT_EPS;
-  cCmpBound.MinY -= FLOAT_EPS;
-  cCmpBound.MinZ -= FLOAT_EPS;
+  cCmpBound.MaxX += traits_type::epsilon();
+  cCmpBound.MaxY += traits_type::epsilon();
+  cCmpBound.MaxZ += traits_type::epsilon();
+  cCmpBound.MinX -= traits_type::epsilon();
+  cCmpBound.MinY -= traits_type::epsilon();
+  cCmpBound.MinZ -= traits_type::epsilon();
 
   // Liegt Punkt innerhalb ?
   if (cCmpBound.IsInBox (rcVct))
