@@ -21,11 +21,14 @@
  *                                                                         *
  *   Juergen Riegel 2002                                                   *
  ***************************************************************************/
+
+
 #ifndef BASE_EXCEPTION_H
 #define BASE_EXCEPTION_H
 
 #include <exception>
 #include <string>
+#include <signal.h>
 #include "FileInfo.h"
 #include "BaseClass.h"
 
@@ -161,6 +164,21 @@ inline void Exception::setMessage(const std::string& sMessage)
   _sErrMsg = sMessage;
 }
 
+#if defined(__GNUC__) && defined (FC_OS_LINUX)
+class SignalException
+{
+public:
+    SignalException();
+    ~SignalException();
+
+private:
+    static void throw_signal(int signum);
+
+private:
+    struct sigaction new_action, old_action;
+    bool ok;
+};
+#endif
 
 } //namespace Base
 
