@@ -28,6 +28,9 @@
 #if defined (_POSIX_C_SOURCE)
 #   undef  _POSIX_C_SOURCE
 #endif // (re-)defined in pyconfig.h
+#if defined (_XOPEN_SOURCE)
+#   undef _XOPEN_SOURCE
+#endif // (re-)defined in pyconfig.h
 
 
 #include <Python.h>
