@@ -265,7 +265,15 @@ void Base::XMLReader::readFiles(zipios::ZipInputStream &zipstream) const
     // up. In this case the associated GUI document asks for its file which is not part of the ZIP
     // file, then.
     // In either case it's guaranteed that the order of the files is kept.
-    zipios::ConstEntryPointer entry = zipstream.getNextEntry();
+    zipios::ConstEntryPointer entry;
+    try {
+        entry = zipstream.getNextEntry();
+    }
+    catch (const std::exception&) {
+        // There is no further file at all. This can happen if the
+        // project file was created without GUI
+        return;
+    }
     std::vector<FileEntry>::const_iterator it = FileList.begin();
     Base::SequencerLauncher seq("Importing project files...", FileList.size());
     while (entry->isValid() && it != FileList.end()) {
@@ -337,12 +345,13 @@ bool Base::XMLReader::isRegistered(Base::Persistence *Object) const
 // ---------------------------------------------------------------------------
 //  Base::XMLReader: Implementation of the SAX DocumentHandler interface
 // ---------------------------------------------------------------------------
-void Base::XMLReader::startElement(const XMLCh* const /*uri*/, const XMLCh* const localname, const XMLCh* const /*qname*/, const Attributes& attrs)
+void Base::XMLReader::startElement(const XMLCh* const /*uri*/, const XMLCh* const localname, const XMLCh* const /*qname*/, const XERCES_CPP_NAMESPACE_QUALIFIER Attributes& attrs)
 {
     Level++; // new scope
     LocalName = StrX(localname).c_str();
 
-    // saving attributes
+    // saving attributes of the current scope, delete all previously stored ones
+    AttrMap.clear();
     for (unsigned int i = 0; i < attrs.getLength(); i++) {
         AttrMap[StrX(attrs.getQName(i)).c_str()] = StrXUTF8(attrs.getValue(i)).c_str();
     }
@@ -386,7 +395,7 @@ void Base::XMLReader::resetDocument()
 // ---------------------------------------------------------------------------
 //  Base::XMLReader: Overrides of the SAX ErrorHandler interface
 // ---------------------------------------------------------------------------
-void Base::XMLReader::error(const SAXParseException& e)
+void Base::XMLReader::error(const XERCES_CPP_NAMESPACE_QUALIFIER SAXParseException& e)
 {
     // print some details to error output and throw an
     // exception to abort the parsing
@@ -396,7 +405,7 @@ void Base::XMLReader::error(const SAXParseException& e)
     throw e;
 }
 
-void Base::XMLReader::fatalError(const SAXParseException& e)
+void Base::XMLReader::fatalError(const XERCES_CPP_NAMESPACE_QUALIFIER SAXParseException& e)
 {
     // print some details to error output and throw an
     // exception to abort the parsing
@@ -406,7 +415,7 @@ void Base::XMLReader::fatalError(const SAXParseException& e)
     throw e;
 }
 
-void Base::XMLReader::warning(const SAXParseException& e)
+void Base::XMLReader::warning(const XERCES_CPP_NAMESPACE_QUALIFIER SAXParseException& e)
 {
     // print some details to error output and throw an
     // exception to abort the parsing
