@@ -105,6 +105,8 @@ public:
     boost::signal<void (const char*)> signalAddWorkbench;
     /// signal on removed workbench
     boost::signal<void (const char*)> signalRemoveWorkbench;
+    /// signal on activating view
+    boost::signal<void (const Gui::MDIView*)> signalActivateView;
     //@}
 
     /** @name methods for Document handling */
@@ -188,6 +190,8 @@ public:
     PYFUNCDEF_S(sGetWorkbenchHandler);      // retrieves the workbench handler
     PYFUNCDEF_S(sListWorkbenchHandlers);    // retrieves a list of all workbench handlers
     PYFUNCDEF_S(sActiveWorkbenchHandler);   // retrieves the active workbench object
+    PYFUNCDEF_S(sAddResPath);               // adds a path where to find resources
+    PYFUNCDEF_S(sAddLangPath);              // adds a path to a qm file
     PYFUNCDEF_S(sAddIconPath);              // adds a path to an icon file
     PYFUNCDEF_S(sAddIcon);                  // adds an icon to the cache
 
@@ -195,6 +199,7 @@ public:
 
     PYFUNCDEF_S(sUpdateGui);
     PYFUNCDEF_S(sUpdateLocale);
+    PYFUNCDEF_S(sGetLocale);
     PYFUNCDEF_S(sCreateDialog);
     PYFUNCDEF_S(sAddPreferencePage);
 
