@@ -26,6 +26,9 @@
 # include <QDir>
 # include <QKeySequence>
 # include <QMessageBox>
+# include <Inventor/actions/SoGetBoundingBoxAction.h>
+# include <Inventor/nodes/SoOrthographicCamera.h>
+# include <Inventor/nodes/SoPerspectiveCamera.h>
 #endif
 
 #include "Command.h"
@@ -41,6 +44,8 @@
 #include "WhatsThis.h"
 #include "WaitCursor.h"
 #include "Control.h"
+#include "View3DInventor.h"
+#include "View3DInventorViewer.h"
 
 #include <Base/Console.h>
 #include <Base/Exception.h>
@@ -57,6 +62,7 @@ using namespace Gui::Dialog;
 using namespace Gui::DockWnd;
 
 /** \defgroup commands Command Framework
+    \ingroup GUI
  * \section Overview
  * In GUI applications many commands can be invoked via a menu item, a toolbar button or an accelerator key. The answer of Qt to master this
  * challenge is the class \a QAction. A QAction object can be added to a popup menu or a toolbar and keep the state of the menu item and
@@ -76,7 +82,7 @@ using namespace Gui::DockWnd;
  * \a Gui::CommandBase and \a Gui::Action that represent the link between Qt's QAction world and the FreeCAD's command  world. 
  *
  * The Action class holds a pointer to QAction and CommandBase and acts as a mediator and -- to save memory -- that gets created 
- * (@ref CommandBase::createAction()) not before it is added (@ref Command::addTo()) to a menu or toolbar.
+ * (@ref Gui::CommandBase::createAction()) not before it is added (@ref Gui::Command::addTo()) to a menu or toolbar.
  *
  * Now, the implementation of the slots of MainWindow can be done in the method \a activated() of subclasses of Command instead.
  *
@@ -106,7 +112,7 @@ using namespace Gui::DockWnd;
  *   }
  * };
  * \endcode
- * An instance of \a OpenCommand must be created and added to the \ref CommandManager to make the class known to FreeCAD.
+ * An instance of \a OpenCommand must be created and added to the \ref Gui::CommandManager to make the class known to FreeCAD.
  * To see how menus and toolbars can be built go to the @ref workbench.
  *
  * @see Gui::Command, Gui::CommandManager
@@ -237,9 +243,9 @@ Gui::Document* Command::getActiveGuiDocument(void) const
 
 App::Document* Command::getDocument(const char* Name) const
 {
-    if (Name)
+    if (Name) {
         return App::GetApplication().getDocument(Name);
-
+    }
     else {
         Gui::Document * pcDoc = getGuiApplication()->activeDocument();
         if (pcDoc)
@@ -258,7 +264,7 @@ App::DocumentObject* Command::getObject(const char* Name) const
         return 0;
 }
 
-void Command::invoke (int i)
+void Command::invoke(int i)
 {
     // Do not query _pcAction since it isn't created necessarily
 #ifdef FC_LOGUSERACTION
@@ -304,12 +310,14 @@ void Command::testActive(void)
         return;
     }
 
-    if ((!Gui::Control().isAllowedAlterDocument()  && eType & AlterDoc)    ||
-        (!Gui::Control().isAllowedAlterView()      && eType & Alter3DView) ||
-        (!Gui::Control().isAllowedAlterSelection() && eType & AlterSelection)) {
-         _pcAction->setEnabled(false);
-        return;
-    }
+    if (!(eType & ForEdit))  // special case for commands which are only in some edit modes active
+        
+        if ((!Gui::Control().isAllowedAlterDocument()  && eType & AlterDoc)    ||
+            (!Gui::Control().isAllowedAlterView()      && eType & Alter3DView) ||
+            (!Gui::Control().isAllowedAlterSelection() && eType & AlterSelection)) {
+             _pcAction->setEnabled(false);
+            return;
+        }
 
     bool bActive = isActive();
     _pcAction->setEnabled(bActive);
@@ -356,22 +364,22 @@ std::string Command::getUniqueObjectName(const char *BaseName) const
 void Command::openCommand(const char* sCmdName)
 {
     // Using OpenCommand with no active document !
-    assert(getGuiApplication()->activeDocument());
+    assert(Gui::Application::Instance->activeDocument());
 
     if (sCmdName)
-        getGuiApplication()->activeDocument()->openCommand(sCmdName);
+        Gui::Application::Instance->activeDocument()->openCommand(sCmdName);
     else
-        getGuiApplication()->activeDocument()->openCommand(sName);
+        Gui::Application::Instance->activeDocument()->openCommand("Command");
 }
 
 void Command::commitCommand(void)
 {
-    getGuiApplication()->activeDocument()->commitCommand();
+    Gui::Application::Instance->activeDocument()->commitCommand();
 }
 
 void Command::abortCommand(void)
 {
-    getGuiApplication()->activeDocument()->abortCommand();
+    Gui::Application::Instance->activeDocument()->abortCommand();
 }
 
 bool Command::_blockCmd = false;
@@ -393,12 +401,12 @@ void Command::doCommand(DoCmd_Type eType,const char* sCmd,...)
     va_end(namelessVars);
 
     if (eType == Gui)
-        getGuiApplication()->macroManager()->addLine(MacroManager::Gui,format);
+        Gui::Application::Instance->macroManager()->addLine(MacroManager::Gui,format);
     else
-        getGuiApplication()->macroManager()->addLine(MacroManager::Base,format);
+        Gui::Application::Instance->macroManager()->addLine(MacroManager::Base,format);
 
     try {
-        Interpreter().runString(format);
+        Base::Interpreter().runString(format);
     }
     catch (...) {
         // free memory to avoid a leak if an exception occurred
@@ -423,7 +431,7 @@ void Command::updateActive(void)
 {
     WaitCursor wc;
 
-    getGuiApplication()->activeDocument()->getDocument()->recompute();
+    Gui::Application::Instance->activeDocument()->getDocument()->recompute();
     //getGuiApplication()->UpdateActive();
 }
 
@@ -435,7 +443,7 @@ void Command::updateAll(std::list<Gui::Document*> cList)
             (*It)->onUpdate();
     }
     else {
-        getGuiApplication()->onUpdate();
+        Gui::Application::Instance->onUpdate();
     }
 }
 
@@ -491,8 +499,48 @@ int Command::keySequenceToAccel(int sk) const
 {
     QKeySequence::StandardKey type = (QKeySequence::StandardKey)sk;
     QKeySequence ks(type);
-    QVariant v = ks.operator QVariant();
-	return v.toInt();
+    QVariant v = QVariant::fromValue<QKeySequence>(ks);
+    return v.toInt();
+}
+
+void Command::adjustCameraPosition()
+{
+    Gui::Document* doc = Gui::Application::Instance->activeDocument();
+    if (doc) {
+        Gui::View3DInventor* view = static_cast<Gui::View3DInventor*>(doc->getActiveView());
+        Gui::View3DInventorViewer* viewer = view->getViewer();
+        SoCamera* camera = viewer->getCamera();
+        if (!camera || !camera->isOfType(SoOrthographicCamera::getClassTypeId()))
+            return;
+
+        // get scene bounding box
+        SoGetBoundingBoxAction action(viewer->getViewportRegion());
+        action.apply(viewer->getSceneGraph());
+        SbBox3f box = action.getBoundingBox();
+        if (box.isEmpty()) return;
+
+        // get cirumscribing sphere and check if camera is inside
+        SbVec3f cam_pos = camera->position.getValue();
+        SbVec3f box_cnt = box.getCenter();
+        SbSphere bs;
+        bs.circumscribe(box);
+        float radius = bs.getRadius();
+        float distance_to_midpoint = (box_cnt-cam_pos).length();
+        if (radius >= distance_to_midpoint) {
+            // Move the camera to the edge of the bounding sphere, while still
+            // pointing at the scene.
+            SbVec3f direction = cam_pos - box_cnt;
+            (void) direction.normalize(); // we know this is not a null vector
+            camera->position.setValue(box_cnt + direction * radius);
+
+            // New distance to mid point
+            distance_to_midpoint =
+                (camera->position.getValue() - box.getCenter()).length();
+            camera->nearDistance = distance_to_midpoint - radius;
+            camera->farDistance = distance_to_midpoint + radius;
+            camera->focalDistance = distance_to_midpoint;
+        }
+    }
 }
 
 Action * Command::createAction(void)
@@ -523,9 +571,9 @@ void Command::languageChange()
 
 MacroCommand::MacroCommand(const char* name)
 #if defined (_MSC_VER)
-        : Command( _strdup(name) )
+  : Command( _strdup(name) )
 #else
-        : Command( strdup(name) )
+  : Command( strdup(name) )
 #endif
 {
     sGroup = QT_TR_NOOP("Macros");
@@ -616,7 +664,7 @@ void MacroCommand::save()
 //===========================================================================
 
 PythonCommand::PythonCommand(const char* name,PyObject * pcPyCommand, const char* pActivationString)
-        :Command(name),_pcPyCommand(pcPyCommand)
+  : Command(name),_pcPyCommand(pcPyCommand)
 {
     if (pActivationString)
         Activation = pActivationString;
@@ -628,7 +676,7 @@ PythonCommand::PythonCommand(const char* name,PyObject * pcPyCommand, const char
     // call the method "GetResources()" of the command object
     _pcPyResourceDict = Interpreter().runMethodObject(_pcPyCommand, "GetResources");
     // check if the "GetResources()" method returns a Dict object
-    if (! PyDict_Check(_pcPyResourceDict) )
+    if (!PyDict_Check(_pcPyResourceDict))
         throw Base::Exception("PythonCommand::PythonCommand(): Method GetResources() of the Python command object returns the wrong type (has to be Py Dictonary)");
 }
 
@@ -638,9 +686,9 @@ const char* PythonCommand::getResource(const char* sName) const
 
     // get the "MenuText" resource string
     pcTemp = PyDict_GetItemString(_pcPyResourceDict,sName);
-    if (! pcTemp )
+    if (!pcTemp)
         return "";
-    if (! PyString_Check(pcTemp) )
+    if (!PyString_Check(pcTemp))
         throw Base::Exception("PythonCommand::getResource(): Method GetResources() of the Python command object returns a dictionary which holds not only strings");
 
     return PyString_AsString(pcTemp);
@@ -652,11 +700,11 @@ void PythonCommand::activated(int iMsg)
         try {
             Interpreter().runMethodVoid(_pcPyCommand, "Activated");
         }
-        catch ( const Base::PyException& e) {
+        catch (const Base::PyException& e) {
             Base::Console().Error("Running the Python command '%s' failed:\n%s\n%s",
                                   sName, e.getStackTrace().c_str(), e.what());
         }
-        catch ( const Base::Exception&) {
+        catch (const Base::Exception&) {
             Base::Console().Error("Running the Python command '%s' failed, try to resume",sName);
         }
     }
@@ -690,11 +738,13 @@ bool PythonCommand::isActive(void)
 
 void PythonCommand::languageChange()
 {
-    if ( _pcAction ) {
-        _pcAction->setText       ( QObject::tr( getMenuText()    ) );
-        _pcAction->setToolTip    ( QObject::tr( getToolTipText() ) );
-        _pcAction->setStatusTip  ( QObject::tr( getStatusTip()   ) );
-        _pcAction->setWhatsThis  ( QObject::tr( getWhatsThis()   ) );
+    if (_pcAction) {
+        _pcAction->setText         (qApp->translate(getName(), getMenuText   ()));
+        _pcAction->setToolTip      (qApp->translate(getName(), getToolTipText()));
+        _pcAction->setStatusTip    (qApp->translate(getName(), getStatusTip  ()));
+        _pcAction->setWhatsThis    (qApp->translate(getName(), getWhatsThis  ()));
+        if (_pcAction->statusTip().isEmpty())
+            _pcAction->setStatusTip(qApp->translate(getName(), getToolTipText()));
     }
 }
 
@@ -714,24 +764,25 @@ Action * PythonCommand::createAction(void)
     Action *pcAction;
 
     pcAction = new Action(this,getMainWindow());
-    pcAction->setText(QObject::trUtf8(getResource("MenuText")));
-    pcAction->setToolTip(QObject::trUtf8(getResource("ToolTip")));
-    pcAction->setStatusTip(QObject::trUtf8(getResource("StatusTip")));
-    pcAction->setWhatsThis(QString::fromUtf8(getResource("WhatsThis")));
+
+    pcAction->setText         (qApp->translate(getName(), getMenuText   ()));
+    pcAction->setToolTip      (qApp->translate(getName(), getToolTipText()));
+    pcAction->setStatusTip    (qApp->translate(getName(), getStatusTip  ()));
+    pcAction->setWhatsThis    (qApp->translate(getName(), getWhatsThis  ()));
+    if (pcAction->statusTip().isEmpty())
+        pcAction->setStatusTip(qApp->translate(getName(), getToolTipText()));
     if (strcmp(getResource("Pixmap"),"") != 0)
         pcAction->setIcon(Gui::BitmapFactory().pixmap(getResource("Pixmap")));
 
-    if (pcAction->statusTip().isEmpty())
-        pcAction->setStatusTip(QObject::trUtf8(getResource("ToolTip")));
-    if (pcAction->whatsThis().isEmpty())
-        pcAction->setWhatsThis(QObject::trUtf8(getResource("ToolTip")));
-
     return pcAction;
 }
 
 const char* PythonCommand::getWhatsThis() const
 {
-    return getResource("WhatsThis");
+    const char* whatsthis = getResource("WhatsThis");
+    if (!whatsthis || whatsthis[0] == '\0')
+        whatsthis = this->getName();
+    return whatsthis;
 }
 
 const char* PythonCommand::getMenuText() const
@@ -774,7 +825,7 @@ CommandManager::~CommandManager()
 
 void CommandManager::addCommand(Command* pCom)
 {
-    _sCommands[pCom->getName()] = pCom;//	pCom->Init();
+    _sCommands[pCom->getName()] = pCom;// pCom->Init();
 }
 
 void CommandManager::removeCommand(Command* pCom)
@@ -793,7 +844,7 @@ void CommandManager::clearCommands()
     _sCommands.clear();
 }
 
-bool CommandManager::addTo(const char* Name,QWidget *pcWidget)
+bool CommandManager::addTo(const char* Name, QWidget *pcWidget)
 {
     if (_sCommands.find(Name) == _sCommands.end()) {
         // Print in release mode only a log message instead of an error message to avoid to annoy the user
