@@ -111,9 +111,10 @@ public:
     //@}
 
 protected:
-    /** @name Attributes 
-     *  Set by the inherited constructor to set up the most important properties 
-     *  of the command. In the constructor are set default values. 
+    /** @name Attributes set by the inherited constructor.
+     *
+     *  They set up the most important properties  of the command.
+     *  In the constructor are set default values. 
      *  The real values should be set in the constructor of the inheriting class.
      */
     //@{
@@ -128,14 +129,15 @@ protected:
     Action *_pcAction; /**< The Action item. */
 };
 
-/** The Command class
- * This class is mostly used for commands implemented directly in C++ (@see PythonCommand).
+/** The Command class.
+ *
+ * This class is mostly used for commands implemented directly in C++ (see PythonCommand).
  * It contains also a lot of helper methods to make implementing commands for FreeCAD as easy as possible.
  *
  * @note This class is intended to handle the GUI interaction like:
  * - starting a dialog
  * - doing view and window stuff
- * - anything else, especially altering the document must be done on application level @see doCommand() for details.
+ * - anything else, especially altering the document must be done on application level. See doCommand() for details.
  *
  * @see CommandManager
  * @author Jrgen Riegel
@@ -159,6 +161,7 @@ protected:
     /// Applies the menu text, tool and status tip to the passed action object
     void applyCommandData(Action* );
     int keySequenceToAccel(int) const;
+    void adjustCameraPosition();
     //@}
 
 public:
@@ -179,13 +182,13 @@ public:
     //@{
     /// Get pointer to the Application Window
     static Application*  getGuiApplication(void);   
-    /// Get a referenc to the selection 
+    /// Get a reference to the selection 
     Gui::SelectionSingleton&  getSelection(void);
     /// Get pointer to the active gui document
     Gui::Document*  getActiveGuiDocument(void) const;
     /** Get pointer to the named or active App document
-     *  Returns a pointer to the named docuement or the active
-     *  document when no name is given. NULL is returnd
+     *  Returns a pointer to the named document or the active
+     *  document when no name is given. NULL is returned
      *  when the name does not exist or no document is active!
      */
     App::Document*  getDocument(const char* Name=0) const;
@@ -200,15 +203,15 @@ public:
     /** @name Helper methods for the Undo/Redo and Update handling */
     //@{
     /// Open a new Undo transaction on the active document
-    void openCommand(const char* sName=0);
+    static void openCommand(const char* sName=0);
     /// Commit the Undo transaction on the active document
-    void commitCommand(void);
+    static void commitCommand(void);
     /// Abort the Undo transaction on the active document
-    void abortCommand(void);
+    static void abortCommand(void);
     /// Updates the (active) document (propagate changes)
-    void updateActive(void);
+    static void updateActive(void);
     /// Updates the (all or listed) documents (propagate changes)
-    void updateAll(std::list<Gui::Document*> cList);
+    static void updateAll(std::list<Gui::Document*> cList);
     /// Translate command
     void languageChange();
     //@}
@@ -256,7 +259,7 @@ public:
     /** @name checking of internal state */
     //@{
     /// returns the name to which the command belongs
-    const char* getAppModuleName(void) const {return sAppModule;}	
+    const char* getAppModuleName(void) const {return sAppModule;}
     /// Get the command name
     const char* getName() const { return sName; }
     /// Get the name of the grouping of the command
@@ -265,9 +268,10 @@ public:
 
 protected:
     enum CmdType {
-        AlterDoc       = 1,  /// Command change the Document
-        Alter3DView    = 2,  /// Command change the Gui
-        AlterSelection = 4   /// Command change the Selection
+        AlterDoc       = 1,  /**< Command change the Document */
+        Alter3DView    = 2,  /**< Command change the Gui */
+        AlterSelection = 4,  /**< Command change the Selection */
+        ForEdit        = 8   /**< Command is in a special edit mode active */
     };
 
     /** @name Attributes 
@@ -371,7 +375,7 @@ public:
     /// Ignore when language has changed. 
     void languageChange() {}
     const char* className() const
-    { return "MacroCommand"; }
+    { return "Gui::MacroCommand"; }
  
     /** @name Methods to set the properties of the Script Command */
     //@{
@@ -413,7 +417,7 @@ public:
     void removeCommand(Command* pCom);
 
     /// Adds the given command to a given widget
-    bool addTo(const char* Name,QWidget *pcWidget);
+    bool addTo(const char* Name, QWidget* pcWidget);
 
     /** Returns all commands of a special App Module
      *  delivers a vector of all comands in the given application module. When no 
@@ -447,7 +451,7 @@ public:
 
     /// method is OBSOLET use GetModuleCommands() or GetAllCommands()
     const std::map<std::string, Command*>& getCommands() const { return _sCommands; }
-    /// get frequently called by the AppWnd to check the commands are active
+    /// get frequently called by the AppWnd to check the commands are active.
     void testActive(void);
 private:
     /// Destroys all commands in the manager and empties the list.
@@ -562,7 +566,7 @@ protected: \
     virtual void activated(int iMsg);\
     virtual bool isActive(void)\
     {\
-        Gui::MDIView* view = getMainWindow()->activeWindow();\
+        Gui::MDIView* view = Gui::getMainWindow()->activeWindow();\
         return view && view->isDerivedFrom(Gui::View3DInventor::getClassTypeId());\
     }\
 };
