@@ -23,6 +23,7 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <QApplication>
 # include <QMessageBox>
 # include <QWhatsThis>
 #if QT_VERSION >= 0x040200
@@ -48,9 +49,6 @@
 #include "View.h"
 
 #include "DlgParameterImp.h"
-#include "DlgMacroExecuteImp.h"
-#include "DlgMacroRecordImp.h"
-#include "Macro.h"
 #include "DlgPreferencesImp.h"
 #include "DlgCustomizeImp.h"
 #include "Widgets.h"
@@ -59,6 +57,7 @@
 #include "GuiConsole.h"
 #include "WorkbenchManager.h"
 #include "Workbench.h"
+#include "Selection.h"
 
 using Base::Console;
 using Base::Sequencer;
@@ -79,8 +78,9 @@ StdCmdWorkbench::StdCmdWorkbench()
     sToolTipText  = QT_TR_NOOP("Switch between workbenches");
     sWhatsThis    = "Std_Workbench";
     sStatusTip    = QT_TR_NOOP("Switch between workbenches");
-    sPixmap       = "FCIcon";
+    sPixmap       = "freecad";
     iAccel        = 0;
+    eType         = 0;
 }
 
 void StdCmdWorkbench::activated(int i)
@@ -119,18 +119,15 @@ bool StdCmdWorkbench::isActive(void)
 
 Action * StdCmdWorkbench::createAction(void)
 {
-  Action *pcAction;
+    Action *pcAction;
 
-  pcAction = new WorkbenchGroup(this,getMainWindow());
-  pcAction->setText(QObject::tr(sMenuText));
-  pcAction->setToolTip(QObject::tr(sToolTipText));
-  pcAction->setStatusTip(QObject::tr(sStatusTip));
-  pcAction->setWhatsThis(QObject::tr(sWhatsThis));
-  if(sPixmap)
-    pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
-  pcAction->setShortcut(iAccel);
+    pcAction = new WorkbenchGroup(this,getMainWindow());
+    applyCommandData(pcAction);
+    if (sPixmap)
+        pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
+    pcAction->setShortcut(iAccel);
 
-  return pcAction;
+    return pcAction;
 }
 
 //===========================================================================
@@ -148,6 +145,7 @@ StdCmdRecentFiles::StdCmdRecentFiles()
     sWhatsThis    = "Std_RecentFiles";
     sStatusTip    = QT_TR_NOOP("Recent file list");
     iAccel        = 0;
+    eType         = 0;
 }
 
 /**
@@ -166,14 +164,11 @@ void StdCmdRecentFiles::activated(int iMsg)
  */
 Action * StdCmdRecentFiles::createAction(void)
 {
-  RecentFilesAction* pcAction = new RecentFilesAction(this, getMainWindow());
-  pcAction->setObjectName(QLatin1String("recentFiles"));
-  pcAction->setDropDownMenu(true);
-  pcAction->setText(QObject::tr(sMenuText));
-  pcAction->setToolTip(QObject::tr(sToolTipText));
-  pcAction->setStatusTip(QObject::tr(sStatusTip));
-  pcAction->setWhatsThis(QObject::tr(sWhatsThis));
-  return pcAction;
+    RecentFilesAction* pcAction = new RecentFilesAction(this, getMainWindow());
+    pcAction->setObjectName(QLatin1String("recentFiles"));
+    pcAction->setDropDownMenu(true);
+    applyCommandData(pcAction);
+    return pcAction;
 }
 
 //===========================================================================
@@ -190,21 +185,26 @@ StdCmdAbout::StdCmdAbout()
     sToolTipText  = QT_TR_NOOP("About %1");
     sWhatsThis    = "Std_About";
     sStatusTip    = QT_TR_NOOP("About %1");
-    sPixmap       = App::Application::Config()["AppIcon"].c_str();
+    eType         = 0;
 }
 
 Action * StdCmdAbout::createAction(void)
 {
     Action *pcAction;
 
-    QString exe = QString::fromUtf8(App::Application::Config()["ExeName"].c_str());
+    QString exe = QString::fromUtf8(App::GetApplication().getExecutableName());
     pcAction = new Action(this,getMainWindow());
-    pcAction->setText(QObject::tr(sMenuText).arg(exe));
-    pcAction->setToolTip(QObject::tr(sToolTipText).arg(exe));
-    pcAction->setStatusTip(QObject::tr(sStatusTip).arg(exe));
+    pcAction->setText(QCoreApplication::translate(
+        this->className(), sMenuText, 0,
+        QCoreApplication::CodecForTr).arg(exe));
+    pcAction->setToolTip(QCoreApplication::translate(
+        this->className(), sToolTipText, 0,
+        QCoreApplication::CodecForTr).arg(exe));
+    pcAction->setStatusTip(QCoreApplication::translate(
+        this->className(), sStatusTip, 0,
+        QCoreApplication::CodecForTr).arg(exe));
     pcAction->setWhatsThis(QLatin1String(sWhatsThis));
-    if(sPixmap)
-        pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
+    pcAction->setIcon(QApplication::windowIcon());
     pcAction->setShortcut(iAccel);
 
     return pcAction;
@@ -227,10 +227,16 @@ void StdCmdAbout::activated(int iMsg)
 void StdCmdAbout::languageChange()
 {
     if (_pcAction) {
-        QString exe = QString::fromUtf8(App::Application::Config()["ExeName"].c_str());
-        _pcAction->setText(QObject::tr(sMenuText).arg(exe));
-        _pcAction->setToolTip(QObject::tr(sToolTipText).arg(exe));
-        _pcAction->setStatusTip(QObject::tr(sStatusTip).arg(exe));
+        QString exe = QString::fromUtf8(App::GetApplication().getExecutableName());
+        _pcAction->setText(QCoreApplication::translate(
+            this->className(), sMenuText, 0,
+            QCoreApplication::CodecForTr).arg(exe));
+        _pcAction->setToolTip(QCoreApplication::translate(
+            this->className(), sToolTipText, 0,
+            QCoreApplication::CodecForTr).arg(exe));
+        _pcAction->setStatusTip(QCoreApplication::translate(
+            this->className(), sStatusTip, 0,
+            QCoreApplication::CodecForTr).arg(exe));
         _pcAction->setWhatsThis(QLatin1String(sWhatsThis));
     }
 }
@@ -248,6 +254,7 @@ StdCmdAboutQt::StdCmdAboutQt()
   sToolTipText  = QT_TR_NOOP("About Qt");
   sWhatsThis    = "Std_AboutQt";
   sStatusTip    = QT_TR_NOOP("About Qt");
+  eType         = 0;
 }
 
 void StdCmdAboutQt::activated(int iMsg)
@@ -268,6 +275,7 @@ StdCmdTipOfTheDay::StdCmdTipOfTheDay()
   sToolTipText  = QT_TR_NOOP("Tip of the day");
   sWhatsThis    = "Std_TipOfTheDay";
   sStatusTip    = QT_TR_NOOP("Tip of the day");
+  eType         = 0;
 }
 
 void StdCmdTipOfTheDay::activated(int iMsg)
@@ -283,19 +291,19 @@ DEF_STD_CMD(StdCmdWhatsThis);
 StdCmdWhatsThis::StdCmdWhatsThis()
   :Command("Std_WhatsThis")
 {
-  sGroup        = QT_TR_NOOP("Help");
-  sMenuText     = QT_TR_NOOP("&What's This?");
-  sToolTipText  = QT_TR_NOOP("What's This");
-  //	sWhatsThis		= sToolTipText;
-  sWhatsThis    = "Std_WhatsThis";
-  sStatusTip    = QT_TR_NOOP("What's This");
-  iAccel        = Qt::SHIFT+Qt::Key_F1;
-  sPixmap       = "WhatsThis";
+    sGroup        = QT_TR_NOOP("Help");
+    sMenuText     = QT_TR_NOOP("&What's This?");
+    sToolTipText  = QT_TR_NOOP("What's This");
+    sWhatsThis    = "Std_WhatsThis";
+    sStatusTip    = QT_TR_NOOP("What's This");
+    iAccel        = keySequenceToAccel(QKeySequence::WhatsThis);
+    sPixmap       = "WhatsThis";
+    eType         = 0;
 }
 
 void StdCmdWhatsThis::activated(int iMsg)
 {
-  QWhatsThis::enterWhatsThisMode();
+    QWhatsThis::enterWhatsThisMode();
 }
 
 //===========================================================================
@@ -313,11 +321,13 @@ StdCmdDlgParameter::StdCmdDlgParameter()
   sStatusTip    = QT_TR_NOOP("Opens a Dialog to edit the parameters");
   //sPixmap     = "settings";
   iAccel        = 0;
+  eType         = 0;
 }
 
 void StdCmdDlgParameter::activated(int iMsg)
 {
   Gui::Dialog::DlgParameterImp cDlg(getMainWindow());
+  cDlg.resize(QSize(800, 600));
   cDlg.exec();
 }
 
@@ -336,6 +346,7 @@ StdCmdDlgPreferences::StdCmdDlgPreferences()
     sStatusTip    = QT_TR_NOOP("Opens a Dialog to edit the preferences");
     sPixmap     = "preferences-system";
     iAccel        = 0;
+    eType         = 0;
 }
 
 void StdCmdDlgPreferences::activated(int iMsg)
@@ -344,116 +355,6 @@ void StdCmdDlgPreferences::activated(int iMsg)
     cDlg.exec();
 }
 
-//===========================================================================
-// Std_DlgMacroRecord
-//===========================================================================
-DEF_STD_CMD_A(StdCmdDlgMacroRecord);
-
-StdCmdDlgMacroRecord::StdCmdDlgMacroRecord()
-  :Command("Std_DlgMacroRecord")
-{
-  sGroup        = QT_TR_NOOP("Tools");
-  sMenuText     = QT_TR_NOOP("&Macro recording ...");
-  sToolTipText  = QT_TR_NOOP("Opens a dialog to record a macro");
-  sWhatsThis    = "Std_DlgMacroRecord";
-  sStatusTip    = QT_TR_NOOP("Opens a dialog to record a macro");
-  sPixmap       = "macro-record";
-  iAccel        = 0;
-}
-
-void StdCmdDlgMacroRecord::activated(int iMsg)
-{
-  Gui::Dialog::DlgMacroRecordImp cDlg(getMainWindow());
-  cDlg.exec();
-}
-
-bool StdCmdDlgMacroRecord::isActive(void)
-{
-  return ! (getGuiApplication()->macroManager()->isOpen());
-}
-
-//===========================================================================
-// Std_DlgMacroExecute
-//===========================================================================
-DEF_STD_CMD_A(StdCmdDlgMacroExecute);
-
-StdCmdDlgMacroExecute::StdCmdDlgMacroExecute()
-  :Command("Std_DlgMacroExecute")
-{
-  sGroup        = QT_TR_NOOP("Tools");
-  sMenuText     = QT_TR_NOOP("Macros ...");
-  sToolTipText  = QT_TR_NOOP("Opens a dialog to let you execute a recorded macro");
-  sWhatsThis    = "Std_DlgMacroExecute";
-  sStatusTip    = QT_TR_NOOP("Opens a dialog to let you execute a recorded macro");
-  sPixmap       = "accessories-text-editor";
-  iAccel        = 0;
-}
-
-void StdCmdDlgMacroExecute::activated(int iMsg)
-{
-  Gui::Dialog::DlgMacroExecuteImp cDlg(getMainWindow());
-  cDlg.exec();
-}
-
-bool StdCmdDlgMacroExecute::isActive(void)
-{
-  return ! (getGuiApplication()->macroManager()->isOpen());
-}
-
-//===========================================================================
-// Std_DlgMacroExecuteDirect
-//===========================================================================
-DEF_STD_CMD_A(StdCmdDlgMacroExecuteDirect);
-
-StdCmdDlgMacroExecuteDirect::StdCmdDlgMacroExecuteDirect()
-  :Command("Std_DlgMacroExecuteDirect")
-{
-  sGroup        = QT_TR_NOOP("Tools");
-  sMenuText     = QT_TR_NOOP("Execute &editor");
-  sToolTipText  = QT_TR_NOOP("Execute the macro/script in the editor");
-  sWhatsThis    = "Std_DlgMacroExecuteDirect";
-  sStatusTip    = QT_TR_NOOP("Execute the macro/script in the editor");
-  sPixmap       = "macro-execute";
-  iAccel        = 0;
-}
-
-void StdCmdDlgMacroExecuteDirect::activated(int iMsg)
-{
-  doCommand(Command::Gui,"Gui.SendMsgToActiveView(\"Run\")");
-}
-
-bool StdCmdDlgMacroExecuteDirect::isActive(void)
-{
-  return getGuiApplication()->sendHasMsgToActiveView("Run");
-}
-
-//===========================================================================
-// Std_MacroStop
-//===========================================================================
-DEF_STD_CMD_A(StdCmdMacroStop);
-
-StdCmdMacroStop::StdCmdMacroStop()
-  :Command("Std_DlgMacroStop")
-{
-  sGroup        = QT_TR_NOOP("Tools");
-  sMenuText     = QT_TR_NOOP("S&top macro recording");
-  sToolTipText  = QT_TR_NOOP("Stop the macro recording session");
-  sWhatsThis    = "Std_DlgMacroStop";
-  sStatusTip    = QT_TR_NOOP("Stop the macro recording session");
-  sPixmap       = "macro-stop";
-  iAccel        = 0;
-}
-
-void StdCmdMacroStop::activated(int iMsg)
-{
-  getGuiApplication()->macroManager()->commit();
-}
-
-bool StdCmdMacroStop::isActive(void)
-{
-  return getGuiApplication()->macroManager()->isOpen();
-}
-
 //===========================================================================
 // Std_DlgCustomize
 //===========================================================================
@@ -469,6 +370,7 @@ StdCmdDlgCustomize::StdCmdDlgCustomize()
     sStatusTip    = QT_TR_NOOP("Customize toolbars and command bars");
     sPixmap       = "applications-accessories";
     iAccel        = 0;
+    eType         = 0;
 }
 
 void StdCmdDlgCustomize::activated(int iMsg)
@@ -488,41 +390,43 @@ DEF_STD_CMD(StdCmdCommandLine);
 StdCmdCommandLine::StdCmdCommandLine()
   :Command("Std_CommandLine")
 {
-  sGroup        = QT_TR_NOOP("Tools");
-  sMenuText     = QT_TR_NOOP("Start command &line...");
-  sToolTipText  = QT_TR_NOOP("Opens the command line in the console");
-  sWhatsThis    = "Std_CommandLine";
-  sStatusTip    = QT_TR_NOOP("Opens the command line in the console");
-  sPixmap       = "utilities-terminal";
-  iAccel        = 0;
+    sGroup        = QT_TR_NOOP("Tools");
+    sMenuText     = QT_TR_NOOP("Start command &line...");
+    sToolTipText  = QT_TR_NOOP("Opens the command line in the console");
+    sWhatsThis    = "Std_CommandLine";
+    sStatusTip    = QT_TR_NOOP("Opens the command line in the console");
+    sPixmap       = "utilities-terminal";
+    iAccel        = 0;
+    eType         = 0;
 }
 
 void StdCmdCommandLine::activated(int iMsg)
 {
-  bool show = getMainWindow()->isMaximized ();
+    bool show = getMainWindow()->isMaximized ();
 
-  // pop up the Gui command window
-  GUIConsole Wnd;
+    // pop up the Gui command window
+    GUIConsole Wnd;
 
-  getMainWindow()->showMinimized () ;
-  qApp->processEvents();
+    getMainWindow()->showMinimized () ;
+    qApp->processEvents();
 
-  // create temporary console sequencer
-  Base::ConsoleSequencer* seq = new Base::ConsoleSequencer;
-  Base::Interpreter().runCommandLine("Console mode");
-  delete seq;
+    // create temporary console sequencer
+    {
+          Base::ConsoleSequencer seq;
+          Base::Interpreter().runCommandLine("Console mode");
+    }
 
 #ifdef Q_WS_X11
-  // On X11 this may not work. For further information see QWidget::showMaximized
-  //
-  // workaround for X11
-  getMainWindow()->hide();
-  getMainWindow()->show();
+    // On X11 this may not work. For further information see QWidget::showMaximized
+    //
+    // workaround for X11
+    getMainWindow()->hide();
+    getMainWindow()->show();
 #endif
 
-  // pop up the main window
-  show ? getMainWindow()->showMaximized () : getMainWindow()->showNormal () ;
-  qApp->processEvents();
+    // pop up the main window
+    show ? getMainWindow()->showMaximized () : getMainWindow()->showNormal () ;
+    qApp->processEvents();
 }
 
 //===========================================================================
@@ -540,7 +444,8 @@ StdCmdOnlineHelp::StdCmdOnlineHelp()
     sWhatsThis    = "Std_OnlineHelp";
     sStatusTip    = QT_TR_NOOP("Help");
     sPixmap       = "help-browser";
-    iAccel        = Qt::Key_F1;
+    iAccel        = keySequenceToAccel(QKeySequence::HelpContents);
+    eType         = 0;
 }
 
 void StdCmdOnlineHelp::activated(int iMsg)
@@ -563,6 +468,7 @@ StdCmdOnlineHelpPython::StdCmdOnlineHelpPython()
     sWhatsThis    = "Std_OnlineHelpPython";
     sStatusTip    = QT_TR_NOOP("Show the Python documentation");
     sPixmap       = "python";
+    eType         = 0;
 }
 
 void StdCmdOnlineHelpPython::activated(int iMsg)
@@ -597,6 +503,7 @@ StdCmdOnlineHelpWebsite::StdCmdOnlineHelpWebsite()
     sToolTipText  = QT_TR_NOOP("The website where the help is maintained");
     sWhatsThis    = "Std_OnlineHelpWebsite";
     sStatusTip    = QT_TR_NOOP("Help Website");
+    eType         = 0;
 }
 
 void StdCmdOnlineHelpWebsite::activated(int iMsg)
@@ -620,6 +527,7 @@ StdCmdFreeCADWebsite::StdCmdFreeCADWebsite()
     sToolTipText  = QT_TR_NOOP("The FreeCAD website");
     sWhatsThis    = "Std_FreeCADWebsite";
     sStatusTip    = QT_TR_NOOP("FreeCAD Website");
+    eType         = 0;
 }
 
 void StdCmdFreeCADWebsite::activated(int iMsg)
@@ -642,6 +550,7 @@ StdCmdPythonWebsite::StdCmdPythonWebsite()
     sWhatsThis    = "Std_PythonWebsite";
     sStatusTip    = QT_TR_NOOP("Python Website");
     sPixmap       = "python";
+    eType         = 0;
 }
 
 void StdCmdPythonWebsite::activated(int iMsg)
@@ -649,6 +558,62 @@ void StdCmdPythonWebsite::activated(int iMsg)
     OpenURLInBrowser("http://python.org");
 }
 
+//===========================================================================
+// Std_MeasurementSimple
+//===========================================================================
+
+DEF_STD_CMD(StdCmdMeasurementSimple);
+
+StdCmdMeasurementSimple::StdCmdMeasurementSimple()
+  :Command("Std_MeasurementSimple")
+{
+    sGroup        = QT_TR_NOOP("Tools");
+    sMenuText     = QT_TR_NOOP("Mesure distance");
+    sToolTipText  = QT_TR_NOOP("Measures distance between two selected objects");
+    sWhatsThis    = QT_TR_NOOP("Measures distance between two selected objects");
+    sStatusTip    = QT_TR_NOOP("Measures distance between two selected objects");
+    sPixmap       = "view-measurement";
+    eType         = 0;
+}
+
+void StdCmdMeasurementSimple::activated(int iMsg)
+{
+    unsigned int n = getSelection().countObjectsOfType(App::DocumentObject::getClassTypeId());
+ 
+    if (n == 1) {
+        QMessageBox::warning(Gui::getMainWindow(), QObject::tr("Wrong selection"),
+            QObject::tr("Only one object selected. Please select two objects.\n"
+                        "Be aware the point where you click matters."));
+        return;
+    }
+    if (n < 1 || n > 2) {
+        QMessageBox::warning(Gui::getMainWindow(), QObject::tr("Wrong selection"),
+            QObject::tr("Please select two objects.\n"
+                        "Be aware the point where you click matters."));
+        return;
+    }
+
+    std::vector<Gui::SelectionSingleton::SelObj> Sel = getSelection().getSelection();
+
+    std::string name;
+    name += "Dist ";
+    name += Sel[0].FeatName;
+    name += "-";
+    name += Sel[0].SubName;
+    name += " to ";
+    name += Sel[1].FeatName;
+    name += "-";
+    name += Sel[1].SubName;
+
+    openCommand("Insert measurement");
+    doCommand(Doc,"_f = App.activeDocument().addObject(\"App::MeasureDistance\",\"%s\")","Measurement");
+    doCommand(Doc,"_f.Label ='%s'",name.c_str());
+    doCommand(Doc,"_f.P1 = FreeCAD.Vector(%f,%f,%f)",Sel[0].x,Sel[0].y,Sel[0].z);
+    doCommand(Doc,"_f.P2 = FreeCAD.Vector(%f,%f,%f)",Sel[1].x,Sel[1].y,Sel[1].z);
+    updateActive();
+    commitCommand();
+}
+
 namespace Gui {
 
 void CreateStdCommands(void)
@@ -660,10 +625,6 @@ void CreateStdCommands(void)
 
     rcCmdMgr.addCommand(new StdCmdDlgParameter());
     rcCmdMgr.addCommand(new StdCmdDlgPreferences());
-    rcCmdMgr.addCommand(new StdCmdDlgMacroRecord());
-    rcCmdMgr.addCommand(new StdCmdDlgMacroExecute());
-    rcCmdMgr.addCommand(new StdCmdDlgMacroExecuteDirect());
-    rcCmdMgr.addCommand(new StdCmdMacroStop());
     rcCmdMgr.addCommand(new StdCmdDlgCustomize());
     rcCmdMgr.addCommand(new StdCmdCommandLine());
     rcCmdMgr.addCommand(new StdCmdWorkbench());
@@ -675,6 +636,7 @@ void CreateStdCommands(void)
     rcCmdMgr.addCommand(new StdCmdOnlineHelpWebsite());
     rcCmdMgr.addCommand(new StdCmdFreeCADWebsite());
     rcCmdMgr.addCommand(new StdCmdPythonWebsite());
+    rcCmdMgr.addCommand(new StdCmdMeasurementSimple());
     //rcCmdMgr.addCommand(new StdCmdDownloadOnlineHelp());
     rcCmdMgr.addCommand(new StdCmdTipOfTheDay());
     //rcCmdMgr.addCommand(new StdCmdDescription());
