@@ -37,6 +37,7 @@
 #include "Action.h"
 #include "Application.h"
 #include "BitmapFactory.h"
+#include "Control.h"
 #include "FileDialog.h"
 #include "MainWindow.h"
 #include "Tree.h"
@@ -54,6 +55,7 @@
 #include "ViewProviderGeometryObject.h"
 #include "SceneInspector.h"
 #include "DemoMode.h"
+#include "TextureMapping.h"
 
 #include <Base/Console.h>
 #include <Base/Exception.h>
@@ -1099,7 +1101,6 @@ Action * StdViewDockUndockFullscreen::createAction(void)
     fullscr->setToolTip(QCoreApplication::translate(
         this->className(), sToolTipText, 0,
         QCoreApplication::CodecForTr));
-
     fullscr->setStatusTip(QCoreApplication::translate(
         this->className(), sStatusTip, 0,
         QCoreApplication::CodecForTr));
@@ -1281,12 +1282,51 @@ StdCmdViewCreate::StdCmdViewCreate()
 
 void StdCmdViewCreate::activated(int iMsg)
 {
-  getActiveGuiDocument()->createView("View3DIv");
+    getActiveGuiDocument()->createView("View3DIv");
+    getActiveGuiDocument()->getActiveView()->viewAll();
 }
 
 bool StdCmdViewCreate::isActive(void)
 {
-  return (getActiveGuiDocument()!=NULL);
+    return (getActiveGuiDocument()!=NULL);
+}
+
+//===========================================================================
+// Std_ToggleNavigation
+//===========================================================================
+DEF_STD_CMD_A(StdCmdToggleNavigation);
+
+StdCmdToggleNavigation::StdCmdToggleNavigation()
+  : Command("Std_ToggleNavigation")
+{
+    sGroup        = QT_TR_NOOP("Standard-View");
+    sMenuText     = QT_TR_NOOP("Toggle navigation/Edit mode");
+    sToolTipText  = QT_TR_NOOP("Toggle between navigation and edit mode");
+    sStatusTip    = QT_TR_NOOP("Toggle between navigation and edit mode");
+    sWhatsThis    = "Std_ToggleNavigation";
+  //iAccel        = Qt::SHIFT+Qt::Key_Space;
+    iAccel        = Qt::Key_Escape;
+    eType         = Alter3DView;
+}
+
+void StdCmdToggleNavigation::activated(int iMsg)
+{
+    Gui::MDIView* view = Gui::getMainWindow()->activeWindow();
+    if (view && view->isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
+        Gui::View3DInventorViewer* viewer = static_cast<Gui::View3DInventor*>(view)->getViewer();
+        SbBool toggle = viewer->isRedirectedToSceneGraph();
+        viewer->setRedirectToSceneGraph(!toggle);
+    }
+}
+
+bool StdCmdToggleNavigation::isActive(void)
+{
+    Gui::MDIView* view = Gui::getMainWindow()->activeWindow();
+    if (view && view->isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
+        Gui::View3DInventorViewer* viewer = static_cast<Gui::View3DInventor*>(view)->getViewer();
+        return viewer->isEditing();
+    }
+    return false;
 }
 
 //===========================================================================
@@ -1669,8 +1709,8 @@ void StdViewBoxZoom::activated(int iMsg)
     View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
     if ( view ) {
         View3DInventorViewer* viewer = view->getViewer();
-        if (!viewer->isPicking())
-            viewer->startPicking(View3DInventorViewer::BoxZoom);
+        if (!viewer->isSelecting())
+            viewer->startSelection(View3DInventorViewer::BoxZoom);
     }
 }
 
@@ -1771,22 +1811,55 @@ void StdCmdSceneInspector::activated(int iMsg)
     View3DInventor* child = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
     if (child) {
         View3DInventorViewer* viewer = child->getViewer();
-        Gui::Dialog::DlgInspector dlg(getMainWindow());
-        dlg.setNode(viewer->getSceneGraph());
-        dlg.exec();
+        static QPointer<Gui::Dialog::DlgInspector> dlg = 0;
+        if (!dlg)
+            dlg = new Gui::Dialog::DlgInspector(getMainWindow());
+        dlg->setNode(viewer->getSceneGraph());
+        dlg->setAttribute(Qt::WA_DeleteOnClose);
+        dlg->show();
     }
 }
 
+//===========================================================================
+// Std_TextureMapping
+//===========================================================================
+
+DEF_STD_CMD_A(StdCmdTextureMapping);
+
+StdCmdTextureMapping::StdCmdTextureMapping()
+  : Command("Std_TextureMapping")
+{
+    // seting the
+    sGroup        = QT_TR_NOOP("Tools");
+    sMenuText     = QT_TR_NOOP("Texture mapping...");
+    sToolTipText  = QT_TR_NOOP("Texture mapping");
+    sWhatsThis    = "Std_TextureMapping";
+    sStatusTip    = QT_TR_NOOP("Texture mapping");
+    eType         = Alter3DView;
+}
+
+void StdCmdTextureMapping::activated(int iMsg)
+{
+    Gui::Control().showDialog(new Gui::Dialog::TaskTextureMapping);
+}
+
+bool StdCmdTextureMapping::isActive(void)
+{
+    Gui::MDIView* view = getMainWindow()->activeWindow();
+    return view && view->isDerivedFrom(Gui::View3DInventor::getClassTypeId())
+                && (Gui::Control().activeDialog()==0);
+}
+
 DEF_STD_CMD(StdCmdDemoMode);
 
 StdCmdDemoMode::StdCmdDemoMode()
   : Command("Std_DemoMode")
 {
     sGroup        = QT_TR_NOOP("Standard-View");
-    sMenuText     = QT_TR_NOOP("Demo mode...");
-    sToolTipText  = QT_TR_NOOP("Demo mode");
-    sWhatsThis    = QT_TR_NOOP("Demo mode");
-    sStatusTip    = QT_TR_NOOP("Demo mode");
+    sMenuText     = QT_TR_NOOP("View turntable...");
+    sToolTipText  = QT_TR_NOOP("View turntable");
+    sWhatsThis    = QT_TR_NOOP("View turntable");
+    sStatusTip    = QT_TR_NOOP("View turntable");
     eType         = Alter3DView;
 }
 
@@ -1856,7 +1929,9 @@ void CreateViewStdCommands(void)
     rcCmdMgr.addCommand(new StdCmdTreeSelection());
     rcCmdMgr.addCommand(new StdCmdMeasureDistance());
     rcCmdMgr.addCommand(new StdCmdSceneInspector());
+    rcCmdMgr.addCommand(new StdCmdTextureMapping());
     rcCmdMgr.addCommand(new StdCmdDemoMode());
+    rcCmdMgr.addCommand(new StdCmdToggleNavigation());
 }
 
 } // namespace Gui
