@@ -53,6 +53,10 @@ using namespace std;
 
 /* TRANSLATOR Gui::Dialog::DlgDisplayPropertiesImp */
 
+#if 0 // needed for Qt's lupdate utility
+    qApp->translate("QDockWidget", "Display properties");
+#endif
+
 /**
  *  Constructs a DlgDisplayPropertiesImp which is a child of 'parent', with the 
  *  name 'name' and widget flags set to 'f' 
@@ -66,6 +70,8 @@ DlgDisplayPropertiesImp::DlgDisplayPropertiesImp( QWidget* parent, Qt::WFlags fl
     this->setupUi(this);
     textLabel1_3->hide();
     changePlot->hide();
+    buttonLineColor->setModal(false);
+    buttonColor->setModal(false);
 
     std::vector<Gui::ViewProvider*> views = getSelection();
     setDisplayModes(views);
@@ -103,6 +109,15 @@ DlgDisplayPropertiesImp::~DlgDisplayPropertiesImp()
     Gui::Selection().Detach(this);
 }
 
+void DlgDisplayPropertiesImp::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::LanguageChange) {
+        this->retranslateUi(this);
+    }
+    QDialog::changeEvent(e);
+}
+
+/// @cond DOXERR
 void DlgDisplayPropertiesImp::OnChange(Gui::SelectionSingleton::SubjectType &rCaller,
                                        Gui::SelectionSingleton::MessageType Reason)
 {
@@ -121,6 +136,7 @@ void DlgDisplayPropertiesImp::OnChange(Gui::SelectionSingleton::SubjectType &rCa
         setLineTransparency(views);
     }
 }
+/// @endcond
 
 void DlgDisplayPropertiesImp::slotChangedObject(const Gui::ViewProvider& obj,
                                                 const App::Property& prop)
@@ -174,12 +190,12 @@ void DlgDisplayPropertiesImp::slotChangedObject(const Gui::ViewProvider& obj,
             float value = static_cast<const App::PropertyFloat&>(prop).getValue();
             if (prop_name == "PointSize") {
                 bool blocked = spinPointSize->blockSignals(true);
-                spinPointSize->setValue(value);
+                spinPointSize->setValue((int)value);
                 spinPointSize->blockSignals(blocked);
             }
             else if (prop_name == "LineWidth") {
                 bool blocked = spinLineWidth->blockSignals(true);
-                spinLineWidth->setValue(value);
+                spinLineWidth->setValue((int)value);
                 spinLineWidth->blockSignals(blocked);
             }
         }
