@@ -25,11 +25,13 @@
 
 #ifndef _PreComp_
 # include <QRegExp>
+# include <memory>
 #endif
 
 #include "DlgSettings3DViewImp.h"
 #include "NavigationStyle.h"
 #include "PrefWidgets.h"
+#include "ui_MouseButtons.h"
 #include <App/Application.h>
 #include <Base/Console.h>
 #include <Base/Parameter.h>
@@ -66,6 +68,7 @@ void DlgSettings3DViewImp::saveSettings()
     QVariant data = comboNavigationStyle->itemData(comboNavigationStyle->currentIndex(), Qt::UserRole);
     hGrp->SetASCII("NavigationStyle", (const char*)data.toByteArray());
 
+    checkBoxInvertZoom->onSave();
     checkBoxAntiAliasing->onSave();
     CheckBox_CornerCoordSystem->onSave();
     CheckBox_ShowFPS->onSave();
@@ -80,6 +83,7 @@ void DlgSettings3DViewImp::saveSettings()
 
 void DlgSettings3DViewImp::loadSettings()
 {
+    checkBoxInvertZoom->onRestore();
     checkBoxAntiAliasing->onRestore();
     CheckBox_CornerCoordSystem->onRestore();
     CheckBox_ShowFPS->onRestore();
@@ -98,6 +102,28 @@ void DlgSettings3DViewImp::loadSettings()
     if (index > -1) comboNavigationStyle->setCurrentIndex(index);
 }
 
+void DlgSettings3DViewImp::on_mouseButton_clicked()
+{
+    QDialog dlg(this);
+    Ui_MouseButtons ui;
+    ui.setupUi(&dlg);
+
+    QVariant data = comboNavigationStyle->itemData(comboNavigationStyle->currentIndex(), Qt::UserRole);
+    void* instance = Base::Type::createInstanceByName((const char*)data.toByteArray());
+    std::auto_ptr<UserNavigationStyle> ns(static_cast<UserNavigationStyle*>(instance));
+    ui.groupBox->setTitle(ui.groupBox->title()+QString::fromAscii(" ")+comboNavigationStyle->currentText());
+    QString descr;
+    descr = qApp->translate((const char*)data.toByteArray(),ns->mouseButtons(NavigationStyle::SELECTION));
+    ui.selectionLabel->setText(QString::fromAscii("<b>%1</b>").arg(descr));
+    descr = qApp->translate((const char*)data.toByteArray(),ns->mouseButtons(NavigationStyle::PANNING));
+    ui.panningLabel->setText(QString::fromAscii("<b>%1</b>").arg(descr));
+    descr = qApp->translate((const char*)data.toByteArray(),ns->mouseButtons(NavigationStyle::DRAGGING));
+    ui.rotationLabel->setText(QString::fromAscii("<b>%1</b>").arg(descr));
+    descr = qApp->translate((const char*)data.toByteArray(),ns->mouseButtons(NavigationStyle::ZOOMING));
+    ui.zoomingLabel->setText(QString::fromAscii("<b>%1</b>").arg(descr));
+    dlg.exec();
+}
+
 /**
  * Sets the strings of the subwidgets using the current language.
  */
@@ -132,3 +158,4 @@ void DlgSettings3DViewImp::retranslate()
 }
 
 #include "moc_DlgSettings3DViewImp.cpp"
+
