@@ -41,6 +41,20 @@
 
 using namespace Gui::Dialog;
 
+namespace Gui { namespace Dialog {
+typedef std::vector< std::pair<QLatin1String, QString> > GroupMap;
+
+struct GroupMap_find {
+    const QLatin1String& item;
+    GroupMap_find(const QLatin1String& item) : item(item) {}
+    bool operator () (const std::pair<QLatin1String, QString>& elem) const
+    {
+        return elem.first == item;
+    }
+};
+}
+}
+
 /* TRANSLATOR Gui::Dialog::DlgCustomToolbars */
 
 /**
@@ -58,29 +72,38 @@ DlgCustomToolbars::DlgCustomToolbars(DlgCustomToolbars::Type t, QWidget* parent)
     CommandManager & cCmdMgr = Application::Instance->commandManager();
     std::map<std::string,Command*> sCommands = cCmdMgr.getCommands();
 
-    // do a special sort before adding to the tree view
-    QStringList groups;
-    groups << QLatin1String("File") << QLatin1String("Edit")
-           << QLatin1String("View") << QLatin1String("Standard-View")
-           << QLatin1String("Tools") << QLatin1String("Window")
-           << QLatin1String("Help") << QLatin1String("Macros");
+    GroupMap groupMap;
+    groupMap.push_back(std::make_pair(QLatin1String("File"), QString()));
+    groupMap.push_back(std::make_pair(QLatin1String("Edit"), QString()));
+    groupMap.push_back(std::make_pair(QLatin1String("View"), QString()));
+    groupMap.push_back(std::make_pair(QLatin1String("Standard-View"), QString()));
+    groupMap.push_back(std::make_pair(QLatin1String("Tools"), QString()));
+    groupMap.push_back(std::make_pair(QLatin1String("Window"), QString()));
+    groupMap.push_back(std::make_pair(QLatin1String("Help"), QString()));
+    groupMap.push_back(std::make_pair(QLatin1String("Macros"), qApp->translate("Gui::MacroCommand", "Macros")));
+
     for (std::map<std::string,Command*>::iterator it = sCommands.begin(); it != sCommands.end(); ++it) {
-        QString group = QLatin1String(it->second->getGroupName());
-        if (!groups.contains(group))
-            groups << group;
+        QLatin1String group(it->second->getGroupName());
+        QString text = qApp->translate(it->second->className(), it->second->getGroupName());
+        GroupMap::iterator jt;
+        jt = std::find_if(groupMap.begin(), groupMap.end(), GroupMap_find(group));
+        if (jt != groupMap.end())
+            jt->second = text;
+        else
+            groupMap.push_back(std::make_pair(group, text));
     }
 
     int index = 0;
-    for ( QStringList::Iterator It = groups.begin(); It != groups.end(); ++It, ++index ) {
-        categoryBox->addItem(QObject::tr((*It).toAscii()));
-        categoryBox->setItemData(index, QVariant(*It), Qt::UserRole);
+    for (GroupMap::iterator it = groupMap.begin(); it != groupMap.end(); ++it, ++index) {
+        categoryBox->addItem(it->second);
+        categoryBox->setItemData(index, QVariant(it->first), Qt::UserRole);
     }
 
     // fills the combo box with all available workbenches
     QStringList workbenches = Application::Instance->workbenches();
     workbenches.sort();
     index = 0;
-    for ( QStringList::Iterator it = workbenches.begin(); it != workbenches.end(); ++it, ++index ) {
+    for (QStringList::Iterator it = workbenches.begin(); it != workbenches.end(); ++it, ++index) {
         QPixmap px = Application::Instance->workbenchIcon(*it);
         QString mt = Application::Instance->workbenchMenuText(*it);
         if ( px.isNull() )
@@ -167,8 +190,8 @@ void DlgCustomToolbars::on_categoryBox_activated(int index)
     sepitem->setBackgroundColor(0, Qt::lightGray);
     for (std::vector<Command*>::iterator it = aCmds.begin(); it != aCmds.end(); ++it) {
         QTreeWidgetItem* item = new QTreeWidgetItem(commandTreeWidget);
-        item->setText(1, QObject::trUtf8((*it)->getMenuText()));
-        item->setToolTip(1, QObject::trUtf8((*it)->getToolTipText()));
+        item->setText(1, qApp->translate((*it)->className(), (*it)->getMenuText()));
+        item->setToolTip(1, qApp->translate((*it)->className(), (*it)->getToolTipText()));
         item->setData(1, Qt::UserRole, QByteArray((*it)->getName()));
         item->setSizeHint(0, QSize(32, 32));
         item->setBackgroundColor(0, Qt::lightGray);
@@ -219,7 +242,7 @@ void DlgCustomToolbars::importCustomToolbars(const QByteArray& name)
                 if (pCmd) {
                     // command name
                     QTreeWidgetItem* item = new QTreeWidgetItem(toplevel);
-                    item->setText(0, QObject::trUtf8(pCmd->getMenuText()));
+                    item->setText(0, qApp->translate(pCmd->className(), pCmd->getMenuText()));
                     item->setData(0, Qt::UserRole, QByteArray(it2->first.c_str()));
                     if (pCmd->getPixmap())
                         item->setIcon(0, BitmapFactory().pixmap(pCmd->getPixmap()));
@@ -495,6 +518,26 @@ void DlgCustomToolbars::onModifyMacroAction(const QByteArray& macro)
     }
 }
 
+void DlgCustomToolbars::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::LanguageChange) {
+        this->retranslateUi(this);
+        int count = categoryBox->count();
+
+        CommandManager & cCmdMgr = Application::Instance->commandManager();
+        for (int i=0; i<count; i++) {
+            QVariant data = categoryBox->itemData(i, Qt::UserRole);
+            std::vector<Command*> aCmds = cCmdMgr.getGroupCommands(data.toByteArray());
+            if (!aCmds.empty()) {
+                QString text = qApp->translate(aCmds[0]->className(), aCmds[0]->getGroupName());
+                categoryBox->setItemText(i, text);
+            }
+        }
+        on_categoryBox_activated(categoryBox->currentIndex());
+    }
+    QWidget::changeEvent(e);
+}
+
 // -------------------------------------------------------------
 
 /* TRANSLATOR Gui::Dialog::DlgCustomToolbarsImp */
@@ -633,6 +676,13 @@ void DlgCustomToolbarsImp::moveDownCustomCommand(const QString& name, const QByt
     }
 }
 
+void DlgCustomToolbarsImp::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::LanguageChange) {
+    }
+    DlgCustomToolbars::changeEvent(e);
+}
+
 
 /* TRANSLATOR Gui::Dialog::DlgCustomToolBoxbarsImp */
 
@@ -654,4 +704,12 @@ DlgCustomToolBoxbarsImp::~DlgCustomToolBoxbarsImp()
 {
 }
 
+void DlgCustomToolBoxbarsImp::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::LanguageChange) {
+        setWindowTitle( tr( "Toolbox bars" ) );
+    }
+    DlgCustomToolbars::changeEvent(e);
+}
+
 #include "moc_DlgToolbarsImp.cpp"
