@@ -29,6 +29,8 @@
 #include <Base/Matrix.h>
 #include <Base/MatrixPy.h>
 
+#include <App/Document.h>
+
 #include "Document.h"
 #include "ViewProviderExtern.h"
 
@@ -90,6 +92,37 @@ PyObject*  DocumentPy::setPos(PyObject *args)
     } PY_CATCH;
 }
 
+PyObject* DocumentPy::setEdit(PyObject *args)
+{
+    char *psFeatStr;
+    int mod = 0;
+    if (!PyArg_ParseTuple(args, "s|i;Name of the object to edit has to be given!",
+                          &psFeatStr,&mod))     // convert args: Python->C 
+        return NULL;  // NULL triggers exception 
+    App::DocumentObject * obj = getDocumentPtr()->getDocument()->getObject(psFeatStr);
+    if (!obj) {
+        PyErr_Format(PyExc_Exception, "No such object found in document: '%s'", psFeatStr);
+        return 0;
+    }
+    
+    bool ok = getDocumentPtr()->setEdit(getDocumentPtr()->getViewProvider(obj),mod);
+    if (!ok) {
+        PyErr_Format(PyExc_Exception, "Failed to set object '%s' in edit mode", psFeatStr);
+        return 0;
+    }
+
+    Py_Return;
+}
+
+PyObject* DocumentPy::resetEdit(PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ";No arguments allowed"))     // convert args: Python->C 
+        return NULL;  // NULL triggers exception 
+    getDocumentPtr()->resetEdit();
+
+    Py_Return;
+}
+
 PyObject*  DocumentPy::addAnnotation(PyObject *args)
 {
     char *psAnnoName,*psFileName,*psModName=0;
