@@ -149,6 +149,9 @@ void MDIView::closeEvent(QCloseEvent *e)
         // and thus the notification in QMdiSubWindow::closeEvent of
         // other mdi windows to get maximized if this window
         // is maximized will fail.
+        // This odd behaviour is caused by the invocation of 
+        // d->mdiArea->removeSubWindow(parent) which we must let there
+        // because otherwise other parts don't work as they should.
 #if defined (NO_USE_QT_MDI_AREA)
         // avoid flickering
         getMainWindow()->removeWindow(this);
@@ -230,7 +233,10 @@ void MDIView::setCurrentViewMode(ViewMode mode)
         case TopLevel:
             {
                 if (this->currentMode == Child) {
-                    getMainWindow()->removeWindow(this);
+#if !defined (NO_USE_QT_MDI_AREA)
+                    if (qobject_cast<QMdiSubWindow*>(this->parentWidget()))
+#endif
+                        getMainWindow()->removeWindow(this);
                     setWindowFlags(windowFlags() | Qt::Window);
                     setParent(0, Qt::Window | Qt::WindowTitleHint | Qt::WindowSystemMenuHint | 
                                  Qt::WindowMinMaxButtonsHint);
@@ -259,7 +265,10 @@ void MDIView::setCurrentViewMode(ViewMode mode)
         case FullScreen:
             {
                 if (this->currentMode == Child) {
-                    getMainWindow()->removeWindow(this);
+#if !defined (NO_USE_QT_MDI_AREA)
+                    if (qobject_cast<QMdiSubWindow*>(this->parentWidget()))
+#endif
+                        getMainWindow()->removeWindow(this);
                     setWindowFlags(windowFlags() | Qt::Window);
                     setParent(0, Qt::Window);
                     showFullScreen();
