@@ -99,6 +99,7 @@
 #include "GuiInitScript.h"
 
 #include "Document.h"
+#include "MergeDocuments.h"
 #include "ViewProviderExtern.h"
 
 #if defined(Q_OS_WIN32)
@@ -406,6 +407,7 @@ MainWindow::MainWindow(QWidget * parent, Qt::WFlags f)
         MainWindowTabBar* result = new MainWindowTabBar(this);
         result->setDrawBase(true);
         result->setElideMode(Qt::ElideRight);
+        result->hide(); // avoid to show horizontal bar in top left area
         //result->setDocumentMode(_documentMode);
         connect(result, SIGNAL(currentChanged(int)), l, SLOT(tabChanged()));
         l->unusedTabBars << result;
@@ -733,6 +735,12 @@ void MainWindow::addWindow(MDIView* view)
 #endif
 }
 
+/**
+ * Removes the instance of Gui::MDiView from the main window and sends am event
+ * to the parent widget, a QMdiSubWindow to delete itself.
+ * If you want to avoid that the Gui::MDIView instance gets destructed too you
+ * must reparent it afterwards, e.g. set parent to NULL.
+ */
 void MainWindow::removeWindow(Gui::MDIView* view)
 {
     // free all connections
@@ -772,6 +780,11 @@ void MainWindow::removeWindow(Gui::MDIView* view)
                this, SLOT(onWindowDestroyed()));
 #else
     QWidget* parent = view->parentWidget();
+    // The call of 'd->mdiArea->removeSubWindow(parent)' causes the QMdiSubWindow
+    // to loose its parent and thus the notification in QMdiSubWindow::closeEvent
+    // of other mdi windows to get maximized if this window is maximized will fail.
+    // However, we must let it here otherwise deleting MDI child views directly can
+    // cause other problems.
     d->mdiArea->removeSubWindow(parent);
     parent->deleteLater();
 #endif
@@ -1036,10 +1049,11 @@ void MainWindow::closeEvent (QCloseEvent * e)
     if (e->isAccepted()) {
         // Send close event to all non-modal dialogs
         QList<QDialog*> dialogs = this->findChildren<QDialog*>();
-        for (QList<QDialog*>::iterator it = dialogs.begin(); it != dialogs.end(); ++it)
+        for (QList<QDialog*>::iterator it = dialogs.begin(); it != dialogs.end(); ++it) {
             (*it)->close();
+        }
         QList<MDIView*> mdis = this->findChildren<MDIView*>();
-        // Force to close any MDI child view
+        // Force to close any remaining (passive) MDI child views
         for (QList<MDIView*>::iterator it = mdis.begin(); it != mdis.end(); ++it) {
             (*it)->hide();
             (*it)->deleteLater();
@@ -1048,7 +1062,9 @@ void MainWindow::closeEvent (QCloseEvent * e)
         saveWindowSettings();
         delete d->assistant;
         d->assistant = 0;
-        QMainWindow::closeEvent( e );
+
+        /*emit*/ mainWindowClosed();
+        qApp->quit(); // stop the event loop
     }
 }
 
@@ -1080,6 +1096,18 @@ void MainWindow::showMainWindow()
 #endif
 }
 
+void MainWindow::delayedStartup()
+{
+    // processing all command line files
+    App::Application::processCmdLineFiles();
+
+    // Create new document?
+    ParameterGrp::handle hGrp = WindowParameter::getDefaultParameter()->GetGroup("Document");
+    if (hGrp->GetBool("CreateNewDoc", true)) {
+        App::GetApplication().newDocument();
+    }
+}
+
 void MainWindow::appendRecentFile(const QString& filename)
 {
     RecentFilesAction *recent = this->findChild<RecentFilesAction *>
@@ -1269,63 +1297,6 @@ void MainWindow::dragEnterEvent (QDragEnterEvent * e)
         e->ignore();
 }
 
-namespace Gui {
-class MimePersistence : public Base::Persistence
-{
-public:
-    MimePersistence(App::Document* doc)
-    {
-        connectExport = doc->signalExportObjects.connect
-            (boost::bind(&MimePersistence::exportObject, this, _1, _2));
-        connectImport = doc->signalImportObjects.connect
-            (boost::bind(&MimePersistence::importObject, this, _1, _2));
-        document = Gui::Application::Instance->getDocument(doc);
-    }
-    ~MimePersistence()
-    {
-        connectExport.disconnect();
-        connectImport.disconnect();
-    }
-    unsigned int getMemSize (void) const
-    {
-        return 0;
-    }
-    void importObject(const std::vector<App::DocumentObject*>& o, Base::XMLReader & r)
-    {
-        objects = o;
-        Restore(r);
-    }
-    void exportObject(const std::vector<App::DocumentObject*>& o, Base::Writer & w)
-    {
-        objects = o;
-        Save(w);
-    }
-    void Save (Base::Writer & w) const
-    {
-        w.addFile("GuiDocument.xml", this);
-    }
-    void Restore(Base::XMLReader &r)
-    {
-        r.addFile("GuiDocument.xml", this);
-    }
-    void SaveDocFile (Base::Writer & w) const
-    {
-        document->exportObjects(objects, w);
-    }
-    void RestoreDocFile(Base::Reader & r)
-    {
-        document->importObjects(objects, r);
-    }
-
-private:
-    Gui::Document* document;
-    std::vector<App::DocumentObject*> objects;
-    typedef boost::signals::connection Connection;
-    Connection connectExport;
-    Connection connectImport;
-};
-}
-
 QMimeData * MainWindow::createMimeDataFromSelection () const
 {
     std::vector<SelectionSingleton::SelObj> sel = Selection().getCompleteSelection();
@@ -1359,14 +1330,14 @@ QMimeData * MainWindow::createMimeDataFromSelection () const
         mime = QLatin1String("application/x-documentobject");
         Base::ByteArrayOStreambuf buf(res);
         std::ostream str(&buf);
-        MimePersistence mimeView(doc);
+        MergeDocuments mimeView(doc);
         doc->exportObjects(obj, str);
     }
     else {
         mime = QLatin1String("application/x-documentobject-file");
         static Base::FileInfo fi(Base::FileInfo::getTempFileName());
         Base::ofstream str(fi, std::ios::out | std::ios::binary);
-        MimePersistence mimeView(doc);
+        MergeDocuments mimeView(doc);
         doc->exportObjects(obj, str);
         str.close();
         res = fi.filePath().c_str();
@@ -1398,8 +1369,8 @@ void MainWindow::insertFromMimeData (const QMimeData * mimeData)
         Base::ByteArrayIStreambuf buf(res);
         std::istream in(0);
         in.rdbuf(&buf);
-        MimePersistence mimeView(doc);
-        doc->importObjects(in);
+        MergeDocuments mimeView(doc);
+        mimeView.importObjects(in);
     }
     else if (mimeData->hasFormat(QLatin1String("application/x-documentobject-file"))) {
         QByteArray res = mimeData->data(QLatin1String("application/x-documentobject-file"));
@@ -1408,8 +1379,8 @@ void MainWindow::insertFromMimeData (const QMimeData * mimeData)
 
         Base::FileInfo fi((const char*)res);
         Base::ifstream str(fi, std::ios::in | std::ios::binary);
-        MimePersistence mimeView(doc);
-        doc->importObjects(str);
+        MergeDocuments mimeView(doc);
+        mimeView.importObjects(str);
         str.close();
     }
     else if (mimeData->hasUrls()) {
