@@ -283,11 +283,16 @@ private Q_SLOTS:
      * \internal
      */
     void showMainWindow();
+    /**
+     * \internal
+     */
+    void delayedStartup();
 
 Q_SIGNALS:
     void timeEvent();
     void windowStateChanged(MDIView*);
     void workbenchActivated(const QString&);
+    void mainWindowClosed();
 
 private:
     /// some kind of singleton
