@@ -37,11 +37,12 @@
 #endif
 #include <Inventor/sensors/SoTimerSensor.h>
 
+#include <App/Application.h>
 #include "NavigationStyle.h"
 #include "View3DInventorViewer.h"
 #include "Application.h"
 #include "MenuManager.h"
-#include "MouseModel.h"
+#include "MouseSelection.h"
 
 using namespace Gui;
 
@@ -80,7 +81,7 @@ const Base::Type& NavigationStyleEvent::style() const
 
 TYPESYSTEM_SOURCE_ABSTRACT(Gui::NavigationStyle,Base::BaseClass);
 
-NavigationStyle::NavigationStyle() : viewer(0), pcMouseModel(0)
+NavigationStyle::NavigationStyle() : viewer(0), mouseSelection(0)
 {
     PRIVATE(this) = new NavigationStyleP();
     PRIVATE(this)->animsensor = new SoTimerSensor(NavigationStyleP::viewAnimationCB, this);
@@ -138,6 +139,8 @@ void NavigationStyle::initialize()
     this->ctrldown = FALSE;
     this->shiftdown = FALSE;
     this->altdown = FALSE;
+    this->invertZoom = App::GetApplication().GetParameterGroupByPath
+        ("User parameter:BaseApp/Preferences/View")->GetBool("InvertZoom",false);
 }
 
 void NavigationStyle::finalize()
@@ -215,6 +218,10 @@ void NavigationStyle::setCameraOrientation(const SbRotation& rot)
         PRIVATE(this)->focal2 = box.getCenter();
     }
 
+    // avoid to interfere with spinning (fixes #3101462)
+    if (this->isAnimating())
+        this->stopAnimating();
+
     if (PRIVATE(this)->animsensor->isScheduled()) {
         PRIVATE(this)->animsensor->unschedule();
     }
@@ -621,8 +628,8 @@ void NavigationStyle::updateAnimation()
 
 void NavigationStyle::redraw()
 {
-    if (pcMouseModel)
-        pcMouseModel->redraw();
+    if (mouseSelection)
+        mouseSelection->redraw();
 }
 
 SbBool NavigationStyle::handleEventInForeground(const SoEvent* const e)
@@ -675,6 +682,7 @@ void NavigationStyle::startAnimating(const SbVec3f& axis, float velocity)
 {
     if (!isAnimationEnabled()) return;
 
+    this->prevRedrawTime = SbTime::getTimeOfDay();
     this->spinincrement = SbRotation::identity();
     SbRotation rot;
     rot.setValue(axis, velocity);
@@ -693,46 +701,67 @@ void NavigationStyle::stopAnimating(void)
         NavigationStyle::IDLE : NavigationStyle::INTERACT);
 }
 
-void NavigationStyle::startPicking(NavigationStyle::ePickMode mode)
+void NavigationStyle::setZoomInverted(SbBool on)
 {
-    if (pcMouseModel)
+    this->invertZoom = on;
+}
+
+SbBool NavigationStyle::isZoomInverted() const
+{
+    return this->invertZoom;
+}
+
+void NavigationStyle::startSelection(AbstractMouseSelection* mouse)
+{
+    if (!mouse)
         return;
   
+    mouseSelection = mouse;
+    mouseSelection->grabMouseModel(viewer);
+}
+
+void NavigationStyle::startSelection(NavigationStyle::SelectionMode mode)
+{
+    if (mouseSelection)
+        return;
+    if (isSelecting())
+        stopSelection();
+  
     switch (mode)
     {
     case Lasso:
-        pcMouseModel = new PolyPickerMouseModel();
+        mouseSelection = new PolyPickerSelection();
         break;
     case Rectangle:
-        pcMouseModel = new SelectionMouseModel();
+        mouseSelection = new RectangleSelection();
         break;
     case BoxZoom:
-        pcMouseModel = new BoxZoomMouseModel();
+        mouseSelection = new BoxZoomSelection();
         break;
     case Clip:
-        pcMouseModel = new PolyClipMouseModel();
+        mouseSelection = new PolyClipSelection();
         break;
     default:
         break;
     }
 
-    if (pcMouseModel)
-        pcMouseModel->grabMouseModel(viewer);
+    if (mouseSelection)
+        mouseSelection->grabMouseModel(viewer);
 }
 
-void NavigationStyle::stopPicking()
+void NavigationStyle::stopSelection()
 {
     pcPolygon.clear();
-    delete pcMouseModel; 
-    pcMouseModel = 0;
+    delete mouseSelection; 
+    mouseSelection = 0;
 }
 
-SbBool NavigationStyle::isPicking() const
+SbBool NavigationStyle::isSelecting() const
 {
-    return (pcMouseModel ? TRUE : FALSE);
+    return (mouseSelection ? TRUE : FALSE);
 }
 
-const std::vector<SbVec2f>& NavigationStyle::getPickedPolygon(SbBool* clip_inner) const
+const std::vector<SbVec2s>& NavigationStyle::getPolygon(SbBool* clip_inner) const
 {
     if (clip_inner)
         *clip_inner = this->clipInner;
@@ -842,20 +871,21 @@ SbBool NavigationStyle::processEvent(const SoEvent * const ev)
 {
     // If we're in picking mode then all events must be redirected to the
     // appropriate mouse model.
-    if (pcMouseModel) {
-        int hd=pcMouseModel->handleEvent(ev,viewer->getViewportRegion());
-        if (hd==AbstractMouseModel::Continue||hd==AbstractMouseModel::Restart) {
+    if (mouseSelection) {
+        int hd=mouseSelection->handleEvent(ev,viewer->getViewportRegion());
+        if (hd==AbstractMouseSelection::Continue||
+            hd==AbstractMouseSelection::Restart) {
             return TRUE;
         }
-        else if (hd==AbstractMouseModel::Finish) {
-            pcPolygon = pcMouseModel->getPolygon();
-            clipInner = pcMouseModel->isInner();
-            delete pcMouseModel; pcMouseModel = 0;
+        else if (hd==AbstractMouseSelection::Finish) {
+            pcPolygon = mouseSelection->getPositions();
+            clipInner = mouseSelection->isInner();
+            delete mouseSelection; mouseSelection = 0;
             return NavigationStyle::processSoEvent(ev);
         }
-        else if (hd==AbstractMouseModel::Cancel) {
+        else if (hd==AbstractMouseSelection::Cancel) {
             pcPolygon.clear();
-            delete pcMouseModel; pcMouseModel = 0;
+            delete mouseSelection; mouseSelection = 0;
             return NavigationStyle::processSoEvent(ev);
         }
     }
@@ -933,15 +963,14 @@ void NavigationStyle::openPopupMenu(const SbVec2s& position)
     if (used && subMenuGroup.actions().indexOf(used) >= 0 && used->isChecked()) {
         QByteArray type = used->data().toByteArray();
         QWidget* widget = viewer->getWidget();
-        if (widget) {
+        while (widget && !widget->inherits("Gui::View3DInventor"))
             widget = widget->parentWidget();
-            if (widget) {
-                // this is the widget where the viewer is embedded
-                Base::Type style = Base::Type::fromName((const char*)type);
-                if (style != this->getTypeId()) {
-                    QEvent* event = new NavigationStyleEvent(style);
-                    QApplication::postEvent(widget, event);
-                }
+        if (widget) {
+            // this is the widget where the viewer is embedded
+            Base::Type style = Base::Type::fromName((const char*)type);
+            if (style != this->getTypeId()) {
+                QEvent* event = new NavigationStyleEvent(style);
+                QApplication::postEvent(widget, event);
             }
         }
     }
@@ -953,6 +982,8 @@ TYPESYSTEM_SOURCE_ABSTRACT(Gui::UserNavigationStyle,Gui::NavigationStyle);
 
 // ----------------------------------------------------------------------------------
 
+/* TRANSLATOR Gui::InventorNavigationStyle */
+
 TYPESYSTEM_SOURCE(Gui::InventorNavigationStyle, Gui::UserNavigationStyle);
 
 InventorNavigationStyle::InventorNavigationStyle()
@@ -963,6 +994,22 @@ InventorNavigationStyle::~InventorNavigationStyle()
 {
 }
 
+const char* InventorNavigationStyle::mouseButtons(ViewerMode mode)
+{
+    switch (mode) {
+    case NavigationStyle::SELECTION:
+        return QT_TR_NOOP("Press CTRL and left mouse button");
+    case NavigationStyle::PANNING:
+        return QT_TR_NOOP("Press middle mouse button");
+    case NavigationStyle::DRAGGING:
+        return QT_TR_NOOP("Press left mouse button");
+    case NavigationStyle::ZOOMING:
+        return QT_TR_NOOP("Scroll middle mouse button");
+    default:
+        return "No description";
+    }
+}
+
 SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
 {
     // Events when in "ready-to-seek" mode are ignored, except those
@@ -1060,14 +1107,14 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
                 float ratio = vp.getViewportAspectRatio();
                 SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
                 this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
-                this->panningmove = FALSE;
+                this->lockrecenter = FALSE;
             }
             else if (!press && ev->wasShiftDown() &&
                 (this->currentmode != NavigationStyle::SELECTION)) {
                 SbTime tmp = (ev->getTime() - this->centerTime);
                 float dci = (float)QApplication::doubleClickInterval()/1000.0f;
                 // is it just a left click?
-                if (tmp.getValue() < dci && !this->panningmove) {
+                if (tmp.getValue() < dci && !this->lockrecenter) {
                     if (!this->seekToPoint(pos)) {
                         panToCenter(panningplane, posn);
                         this->interactiveCountDec();
@@ -1091,10 +1138,12 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
             else if (viewer->isEditing() && (this->currentmode == NavigationStyle::SPINNING)) {
                 processed = TRUE;
             }
+            this->lockrecenter = TRUE;
             break;
         case SoMouseButtonEvent::BUTTON2:
             // If we are in edit mode then simply ignore the RMB events
             // to pass the event to the base class.
+            this->lockrecenter = TRUE;
             if (!viewer->isEditing()) {
                 // If we are in zoom or pan mode ignore RMB events otherwise
                 // the canvas doesn't get any release events 
@@ -1114,13 +1163,13 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
                 float ratio = vp.getViewportAspectRatio();
                 SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
                 this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
-                this->panningmove = FALSE;
+                this->lockrecenter = FALSE;
             }
             else {
                 SbTime tmp = (ev->getTime() - this->centerTime);
                 float dci = (float)QApplication::doubleClickInterval()/1000.0f;
                 // is it just a middle click?
-                if (tmp.getValue() < dci && !this->panningmove) {
+                if (tmp.getValue() < dci && !this->lockrecenter) {
                     if (!this->seekToPoint(pos)) {
                         panToCenter(panningplane, posn);
                         this->interactiveCountDec();
@@ -1131,11 +1180,17 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
             this->button3down = press;
             break;
         case SoMouseButtonEvent::BUTTON4:
-            zoom(viewer->getCamera(), 0.05f);
+            if (this->invertZoom)
+                zoom(viewer->getCamera(), -0.05f);
+            else
+                zoom(viewer->getCamera(), 0.05f);
             processed = TRUE;
             break;
         case SoMouseButtonEvent::BUTTON5:
-            zoom(viewer->getCamera(), -0.05f);
+            if (this->invertZoom)
+                zoom(viewer->getCamera(), 0.05f);
+            else
+                zoom(viewer->getCamera(), -0.05f);
             processed = TRUE;
             break;
         default:
@@ -1145,13 +1200,13 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
 
     // Mouse Movement handling
     if (type.isDerivedFrom(SoLocation2Event::getClassTypeId())) {
+        this->lockrecenter = TRUE;
         const SoLocation2Event * const event = (const SoLocation2Event *) ev;
         if (this->currentmode == NavigationStyle::ZOOMING) {
             this->zoomByCursor(posn, prevnormalized);
             processed = TRUE;
         }
         else if (this->currentmode == NavigationStyle::PANNING) {
-            this->panningmove = TRUE;
             float ratio = vp.getViewportAspectRatio();
             panCamera(viewer->getCamera(), ratio, this->panningplane, posn, prevnormalized);
             processed = TRUE;
@@ -1250,6 +1305,8 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
 
 // ----------------------------------------------------------------------------------
 
+/* TRANSLATOR Gui::CADNavigationStyle */
+
 TYPESYSTEM_SOURCE(Gui::CADNavigationStyle, Gui::UserNavigationStyle);
 
 CADNavigationStyle::CADNavigationStyle() : lockButton1(FALSE)
@@ -1260,6 +1317,22 @@ CADNavigationStyle::~CADNavigationStyle()
 {
 }
 
+const char* CADNavigationStyle::mouseButtons(ViewerMode mode)
+{
+    switch (mode) {
+    case NavigationStyle::SELECTION:
+        return QT_TR_NOOP("Press left mouse button");
+    case NavigationStyle::PANNING:
+        return QT_TR_NOOP("Press middle mouse button");
+    case NavigationStyle::DRAGGING:
+        return QT_TR_NOOP("Press left and middle mouse button");
+    case NavigationStyle::ZOOMING:
+        return QT_TR_NOOP("Scroll middle mouse button");
+    default:
+        return "No description";
+    }
+}
+
 SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
 {
 #if 0
@@ -1355,6 +1428,7 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
         // SoDebugError::postInfo("processSoEvent", "button = %d", button);
         switch (button) {
         case SoMouseButtonEvent::BUTTON1:
+            this->lockrecenter = TRUE;
             this->button1down = press;
 #if 0 // disable to avoid interferences where this key combination is used, too
             if (press && ev->wasShiftDown() &&
@@ -1363,14 +1437,14 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
                 float ratio = vp.getViewportAspectRatio();
                 SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
                 this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
-                this->panningmove = FALSE;
+                this->lockrecenter = FALSE;
             }
             else if (!press && ev->wasShiftDown() &&
                 (this->currentmode != NavigationStyle::SELECTION)) {
                 SbTime tmp = (ev->getTime() - this->centerTime);
                 float dci = (float)QApplication::doubleClickInterval()/1000.0f;
                 // is it just a left click?
-                if (tmp.getValue() < dci && !this->panningmove) {
+                if (tmp.getValue() < dci && !this->lockrecenter) {
                     if (!this->seekToPoint(pos)) {
                         panToCenter(panningplane, posn);
                         this->interactiveCountDec();
@@ -1414,6 +1488,7 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
         case SoMouseButtonEvent::BUTTON2:
             // If we are in edit mode then simply ignore the RMB events
             // to pass the event to the base class.
+            this->lockrecenter = TRUE;
             if (!viewer->isEditing()) {
                 // If we are in zoom or pan mode ignore RMB events otherwise
                 // the canvas doesn't get any release events 
@@ -1435,13 +1510,13 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
                 float ratio = vp.getViewportAspectRatio();
                 SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
                 this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
-                this->panningmove = FALSE;
+                this->lockrecenter = FALSE;
             }
             else {
                 SbTime tmp = (ev->getTime() - this->centerTime);
                 float dci = (float)QApplication::doubleClickInterval()/1000.0f;
                 // is it just a middle click?
-                if (tmp.getValue() < dci && !this->panningmove) {
+                if (tmp.getValue() < dci && !this->lockrecenter) {
                     if (!this->seekToPoint(pos)) {
                         panToCenter(panningplane, posn);
                         this->interactiveCountDec();
@@ -1452,11 +1527,17 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
             this->button3down = press;
             break;
         case SoMouseButtonEvent::BUTTON4:
-            zoom(viewer->getCamera(), 0.05f);
+            if (this->invertZoom)
+                zoom(viewer->getCamera(), -0.05f);
+            else
+                zoom(viewer->getCamera(), 0.05f);
             processed = TRUE;
             break;
         case SoMouseButtonEvent::BUTTON5:
-            zoom(viewer->getCamera(), -0.05f);
+            if (this->invertZoom)
+                zoom(viewer->getCamera(), 0.05f);
+            else
+                zoom(viewer->getCamera(), -0.05f);
             processed = TRUE;
             break;
         default:
@@ -1466,13 +1547,13 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
 
     // Mouse Movement handling
     if (type.isDerivedFrom(SoLocation2Event::getClassTypeId())) {
+        this->lockrecenter = TRUE;
         const SoLocation2Event * const event = (const SoLocation2Event *) ev;
         if (this->currentmode == NavigationStyle::ZOOMING) {
             this->zoomByCursor(posn, prevnormalized);
             processed = TRUE;
         }
         else if (this->currentmode == NavigationStyle::PANNING) {
-            this->panningmove = TRUE;
             float ratio = vp.getViewportAspectRatio();
             panCamera(viewer->getCamera(), ratio, this->panningplane, posn, prevnormalized);
             processed = TRUE;
