@@ -46,7 +46,7 @@ class SbSphereSheetProjector;
 namespace Gui {
 
 class View3DInventorViewer;
-class AbstractMouseModel;
+class AbstractMouseSelection;
 
 /**
  * @author Werner Mayer
@@ -83,7 +83,7 @@ public:
         SELECTION
     };
 
-    enum ePickMode {
+    enum SelectionMode {
         Lasso       = 0,  /**< Select objects using a lasso. */
         Rectangle   = 1,  /**< Select objects using a rectangle. */
         BoxZoom     = 2,  /**< Perform a box zoom. */
@@ -104,6 +104,9 @@ public:
     void stopAnimating(void);
     SbBool isAnimating(void) const;
 
+    void setZoomInverted(SbBool);
+    SbBool isZoomInverted() const;
+
     void updateAnimation();
     void redraw();
 
@@ -118,10 +121,11 @@ public:
     void setPopupMenuEnabled(const SbBool on);
     SbBool isPopupMenuEnabled(void) const;
 
-    void startPicking(ePickMode = Lasso);
-    void stopPicking();
-    SbBool isPicking() const;
-    const std::vector<SbVec2f>& getPickedPolygon(SbBool* clip_inner=0) const;
+    void startSelection(AbstractMouseSelection*);
+    void startSelection(SelectionMode = Lasso);
+    void stopSelection();
+    SbBool isSelecting() const;
+    const std::vector<SbVec2s>& getPolygon(SbBool* clip_inner=0) const;
 
 protected:
     void initialize();
@@ -172,15 +176,16 @@ protected:
     SbPlane panningplane;
     SbTime prevRedrawTime;
     SbTime centerTime;
-    SbBool panningmove;
+    SbBool lockrecenter;
     SbBool menuenabled;
     SbBool ctrldown, shiftdown, altdown;
     SbBool button1down, button2down, button3down;
+    SbBool invertZoom;
 
     /** @name Mouse model */
     //@{
-    AbstractMouseModel* pcMouseModel;
-    std::vector<SbVec2f> pcPolygon;
+    AbstractMouseSelection* mouseSelection;
+    std::vector<SbVec2s> pcPolygon;
     SbBool clipInner;
     //@}
 
@@ -213,6 +218,7 @@ class GuiExport UserNavigationStyle : public NavigationStyle {
 public:
     UserNavigationStyle(){}
     ~UserNavigationStyle(){}
+    virtual const char* mouseButtons(ViewerMode) = 0;
 };
 
 class GuiExport InventorNavigationStyle : public UserNavigationStyle {
@@ -223,6 +229,7 @@ class GuiExport InventorNavigationStyle : public UserNavigationStyle {
 public:
     InventorNavigationStyle();
     ~InventorNavigationStyle();
+    const char* mouseButtons(ViewerMode);
 
 protected:
     SbBool processSoEvent(const SoEvent * const ev);
@@ -236,6 +243,7 @@ class GuiExport CADNavigationStyle : public UserNavigationStyle {
 public:
     CADNavigationStyle();
     ~CADNavigationStyle();
+    const char* mouseButtons(ViewerMode);
 
 protected:
     SbBool processSoEvent(const SoEvent * const ev);
