@@ -29,7 +29,10 @@
 #include "ui_Placement.h"
 #include <Gui/DockWindowManager.h>
 #include <Gui/Application.h>
+#include <Gui/Document.h>
 #include <Gui/Selection.h>
+#include <Gui/ViewProvider.h>
+#include <App/Document.h>
 #include <App/PropertyGeo.h>
 #include <Base/Console.h>
 
@@ -39,54 +42,22 @@ namespace Gui { namespace Dialog {
 class find_placement
 {
 public:
+    find_placement(const std::string& name) : propertyname(name)
+    {
+    }
     bool operator () (const std::pair<std::string, App::Property*>& elem) const
     {
-        if (elem.first == "Placement") {
+        if (elem.first == propertyname) {
             return elem.second->isDerivedFrom
                 (Base::Type::fromName("App::PropertyPlacement"));
         }
 
         return false;
     }
+
+    std::string propertyname;
 };
 
-void applyPlacement(const Base::Placement& p)
-{
-    std::vector<App::DocumentObject*> sel = Gui::Selection().getObjectsOfType
-        (App::DocumentObject::getClassTypeId());
-    if (!sel.empty()) {
-        for (std::vector<App::DocumentObject*>::iterator it=sel.begin();it!=sel.end();++it) {
-            std::map<std::string,App::Property*> props;
-            (*it)->getPropertyMap(props);
-            // search for the placement property
-            std::map<std::string,App::Property*>::iterator jt;
-            jt = std::find_if(props.begin(), props.end(), find_placement());
-            if (jt != props.end()) {
-                //static_cast<App::PropertyPlacement*>(jt->second)->setValue(p);
-                Base::Vector3d pos = p.getPosition();
-                const Base::Rotation& rt = p.getRotation();
-                QString cmd = QString::fromAscii(
-                    "App.getDocument(\"%1\").%2.Placement="
-                    "App.Placement("
-                    "App.Vector(%3,%4,%5),"
-                    "App.Rotation(%6,%7,%8,%9))\n")
-                    .arg(QLatin1String((*it)->getDocument()->getName()))
-                    .arg(QLatin1String((*it)->getNameInDocument()))
-                    .arg(pos.x,0,'g',6)
-                    .arg(pos.y,0,'g',6)
-                    .arg(pos.z,0,'g',6)
-                    .arg(rt[0],0,'g',6)
-                    .arg(rt[1],0,'g',6)
-                    .arg(rt[2],0,'g',6)
-                    .arg(rt[3],0,'g',6);
-                Application::Instance->runPythonCode((const char*)cmd.toAscii());
-            }
-        }
-    }
-    else {
-        Base::Console().Warning("No object selected.\n");
-    }
-}
 }
 }
 
@@ -95,14 +66,17 @@ void applyPlacement(const Base::Placement& p)
 Placement::Placement(QWidget* parent, Qt::WFlags fl)
   : Gui::LocationDialog(parent, fl)
 {
+    propertyName = "Placement"; // default name
     ui = new Ui_PlacementComp(this);
+    ui->applyPlacementChange->hide();
+
     ui->angle->setSuffix(QString::fromUtf8(" \xc2\xb0"));
     ui->yawAngle->setSuffix(QString::fromUtf8(" \xc2\xb0"));
     ui->pitchAngle->setSuffix(QString::fromUtf8(" \xc2\xb0"));
     ui->rollAngle->setSuffix(QString::fromUtf8(" \xc2\xb0"));
 
     // create a signal mapper in order to have one slot to perform the change
-    QSignalMapper* signalMapper = new QSignalMapper(this);
+    signalMapper = new QSignalMapper(this);
     connect(this, SIGNAL(directionChanged()), signalMapper, SLOT(map()));
     signalMapper->setMapping(this, 0);
 
@@ -126,36 +100,147 @@ void Placement::showDefaultButtons(bool ok)
 {
     ui->oKButton->setVisible(ok);
     ui->closeButton->setVisible(ok);
+    ui->applyButton->setVisible(ok);
+}
+
+void Placement::applyPlacement(const Base::Placement& p, bool incremental, bool data)
+{
+    Gui::Document* document = Application::Instance->activeDocument();
+    if (!document) return;
+
+    std::vector<App::DocumentObject*> sel = Gui::Selection().getObjectsOfType
+        (App::DocumentObject::getClassTypeId(), document->getDocument()->getName());
+    if (!sel.empty()) {
+        if (data) {
+            document->openCommand("Placement");
+            for (std::vector<App::DocumentObject*>::iterator it=sel.begin();it!=sel.end();++it) {
+                std::map<std::string,App::Property*> props;
+                (*it)->getPropertyMap(props);
+                // search for the placement property
+                std::map<std::string,App::Property*>::iterator jt;
+                jt = std::find_if(props.begin(), props.end(), find_placement(this->propertyName));
+                if (jt != props.end()) {
+                    Base::Placement cur = static_cast<App::PropertyPlacement*>(jt->second)->getValue();
+                    if (incremental)
+                        cur = p * cur;
+                    else
+                        cur = p;
+
+                    Base::Vector3d pos = cur.getPosition();
+                    const Base::Rotation& rt = cur.getRotation();
+                    QString cmd = QString::fromAscii(
+                        "App.getDocument(\"%1\").%2.Placement="
+                        "App.Placement("
+                        "App.Vector(%3,%4,%5),"
+                        "App.Rotation(%6,%7,%8,%9))\n")
+                        .arg(QLatin1String((*it)->getDocument()->getName()))
+                        .arg(QLatin1String((*it)->getNameInDocument()))
+                        .arg(pos.x,0,'g',6)
+                        .arg(pos.y,0,'g',6)
+                        .arg(pos.z,0,'g',6)
+                        .arg(rt[0],0,'g',6)
+                        .arg(rt[1],0,'g',6)
+                        .arg(rt[2],0,'g',6)
+                        .arg(rt[3],0,'g',6);
+                    Application::Instance->runPythonCode((const char*)cmd.toAscii());
+                }
+            }
+
+            document->commitCommand();
+        }
+        // apply transformation only on view matrix not on placement property
+        else {
+            for (std::vector<App::DocumentObject*>::iterator it=sel.begin();it!=sel.end();++it) {
+                std::map<std::string,App::Property*> props;
+                (*it)->getPropertyMap(props);
+                // search for the placement property
+                std::map<std::string,App::Property*>::iterator jt;
+                jt = std::find_if(props.begin(), props.end(), find_placement(this->propertyName));
+                if (jt != props.end()) {
+                    Base::Placement cur = static_cast<App::PropertyPlacement*>(jt->second)->getValue();
+                    if (incremental)
+                        cur = p * cur;
+                    else
+                        cur = p;
+
+                    Gui::ViewProvider* vp = document->getViewProvider(*it);
+                    if (vp) vp->setTransformation(cur.toMatrix());
+                }
+            }
+        }
+    }
+    else {
+        Base::Console().Warning("No object selected.\n");
+    }
 }
 
 void Placement::onPlacementChanged(int)
 {
-    // If there listeners to the 'placementChanged' signal we rely
+    // If there are listeners to the 'placementChanged' signal we rely
     // on that the listener updates any placement. If no listeners
     // are connected the placement is applied to all selected objects
     // automatically.
-    if (ui->applyPlacementChange->isChecked()) {
-        Base::Placement plm = this->getPlacement();
-        if (receivers(SIGNAL(placementChanged(QVariant))) > 0) {
-            QVariant data = QVariant::fromValue<Base::Placement>(plm);
-            /*emit*/ placementChanged(data);
-        }
-        else {
-            applyPlacement(plm);
-        }
+    bool incr = ui->applyIncrementalPlacement->isChecked();
+    Base::Placement plm = this->getPlacement();
+    if (receivers(SIGNAL(placementChanged(QVariant, bool, bool))) > 0) {
+        QVariant data = QVariant::fromValue<Base::Placement>(plm);
+        /*emit*/ placementChanged(data, incr, false);
+    }
+    else {
+        applyPlacement(plm, incr, false);
     }
-}
-
-void Placement::on_applyPlacementChange_toggled(bool on)
-{
-    ui->applyButton->setDisabled(on);
-    if (on) onPlacementChanged(0);
 }
 
 void Placement::on_applyIncrementalPlacement_toggled(bool on)
 {
     if (on) {
-        this->pm = getPlacementData();
+        this->ref = getPlacementData();
+        on_resetButton_clicked();
+    }
+    else {
+        Base::Placement p = getPlacementData();
+        p = p * this->ref;
+        setPlacementData(p);
+        onPlacementChanged(0);
+    }
+}
+
+void Placement::reject()
+{
+    Base::Placement plm;
+    if (receivers(SIGNAL(placementChanged(QVariant, bool, bool))) > 0) {
+        QVariant data = QVariant::fromValue<Base::Placement>(plm);
+        /*emit*/ placementChanged(data, true, false);
+    }
+    else {
+        applyPlacement(plm, true, false);
+    }
+    QDialog::reject();
+}
+
+void Placement::accept()
+{
+    on_applyButton_clicked();
+    QDialog::accept();
+}
+
+void Placement::on_applyButton_clicked()
+{
+    // If there are listeners to the 'placementChanged' signal we rely
+    // on that the listener updates any placement. If no listeners
+    // are connected the placement is applied to all selected objects
+    // automatically.
+    bool incr = ui->applyIncrementalPlacement->isChecked();
+    Base::Placement plm = this->getPlacement();
+    if (receivers(SIGNAL(placementChanged(QVariant, bool, bool))) > 0) {
+        QVariant data = QVariant::fromValue<Base::Placement>(plm);
+        /*emit*/ placementChanged(data, incr, true);
+    }
+    else {
+        applyPlacement(plm, incr, true);
+    }
+
+    if (ui->applyIncrementalPlacement->isChecked()) {
         QList<QDoubleSpinBox*> sb = this->findChildren<QDoubleSpinBox*>();
         for (QList<QDoubleSpinBox*>::iterator it = sb.begin(); it != sb.end(); ++it) {
             (*it)->blockSignals(true);
@@ -163,36 +248,6 @@ void Placement::on_applyIncrementalPlacement_toggled(bool on)
             (*it)->blockSignals(false);
         }
     }
-    else {
-        Base::Placement p = getPlacementData();
-        this->pm = p * this->pm;
-        this->blockSignals(true);
-        setPlacementData(this->pm);
-        this->blockSignals(false);
-    }
-}
-
-void Placement::accept()
-{
-    if (!ui->applyPlacementChange->isChecked())
-        on_applyButton_clicked();
-    QDialog::accept();
-}
-
-void Placement::on_applyButton_clicked()
-{
-    // If there listeners to the 'placementChanged' signal we rely
-    // on that the listener updates any placement. If no listeners
-    // are connected the placement is applied to all selected objects
-    // automatically.
-    Base::Placement plm = this->getPlacement();
-    if (receivers(SIGNAL(placementChanged(QVariant))) > 0) {
-        QVariant data = QVariant::fromValue<Base::Placement>(plm);
-        /*emit*/ placementChanged(data);
-    }
-    else {
-        applyPlacement(plm);
-    }
 }
 
 void Placement::on_resetButton_clicked()
@@ -221,13 +276,12 @@ Base::Vector3f Placement::getDirection() const
 
 void Placement::setPlacement(const Base::Placement& p)
 {
-    this->pm = p;
-    setPlacementData(this->pm);
+    setPlacementData(p);
 }
 
 void Placement::setPlacementData(const Base::Placement& p)
 {
-
+    signalMapper->blockSignals(true);
     ui->xPos->setValue(p.getPosition().x);
     ui->yPos->setValue(p.getPosition().y);
     ui->zPos->setValue(p.getPosition().z);
@@ -239,6 +293,7 @@ void Placement::setPlacementData(const Base::Placement& p)
     ui->rollAngle->setValue(R);
 
     // check if the user-defined direction is already there
+    bool newitem = true;
     double angle;
     Base::Vector3d axis;
     p.getRotation().getValue(axis, angle);
@@ -250,26 +305,28 @@ void Placement::setPlacementData(const Base::Placement& p)
             const Base::Vector3f val = data.value<Base::Vector3f>();
             if (val == dir) {
                 ui->direction->setCurrentIndex(i);
-                return;
+                newitem = false;
+                break;
             }
         }
     }
 
-    // add a new item before the very last item
-    QString display = QString::fromAscii("(%1,%2,%3)")
-        .arg(dir.x)
-        .arg(dir.y)
-        .arg(dir.z);
-    ui->direction->insertItem(ui->direction->count()-1, display,
-        QVariant::fromValue<Base::Vector3f>(dir));
-    ui->direction->setCurrentIndex(ui->direction->count()-2);
+    if (newitem) {
+        // add a new item before the very last item
+        QString display = QString::fromAscii("(%1,%2,%3)")
+            .arg(dir.x)
+            .arg(dir.y)
+            .arg(dir.z);
+        ui->direction->insertItem(ui->direction->count()-1, display,
+            QVariant::fromValue<Base::Vector3f>(dir));
+        ui->direction->setCurrentIndex(ui->direction->count()-2);
+    }
+    signalMapper->blockSignals(false);
 }
 
 Base::Placement Placement::getPlacement() const
 {
     Base::Placement p = getPlacementData();
-    if (ui->applyIncrementalPlacement->isChecked())
-        p = p * this->pm;
     return p;
 }
 
@@ -310,6 +367,8 @@ void Placement::changeEvent(QEvent *e)
 
 // ----------------------------------------------
 
+/* TRANSLATOR Gui::Dialog::DockablePlacement */
+
 DockablePlacement::DockablePlacement(QWidget* parent, Qt::WFlags fl) : Placement(parent, fl)
 {
     Gui::DockWindowManager* pDockMgr = Gui::DockWindowManager::instance();
@@ -341,16 +400,19 @@ void DockablePlacement::reject()
 
 // ----------------------------------------------
 
+/* TRANSLATOR Gui::Dialog::TaskPlacement */
+
 TaskPlacement::TaskPlacement()
 {
+    this->setButtonPosition(TaskPlacement::South);
     widget = new Placement();
     widget->showDefaultButtons(false);
     taskbox = new Gui::TaskView::TaskBox(QPixmap(), widget->windowTitle(),true, 0);
     taskbox->groupLayout()->addWidget(widget);
 
     Content.push_back(taskbox);
-    connect(widget, SIGNAL(placementChanged(const QVariant &)),
-            this, SLOT(slotPlacementChanged(const QVariant &)));
+    connect(widget, SIGNAL(placementChanged(const QVariant &, bool, bool)),
+            this, SLOT(slotPlacementChanged(const QVariant &, bool, bool)));
 }
 
 TaskPlacement::~TaskPlacement()
@@ -358,18 +420,35 @@ TaskPlacement::~TaskPlacement()
     // automatically deleted in the sub-class
 }
 
-void TaskPlacement::slotPlacementChanged(const QVariant & p)
+void TaskPlacement::setPropertyName(const QString& name)
 {
-    // If there listeners to the 'placementChanged' signal we rely
+    widget->propertyName = (const char*)name.toLatin1();
+}
+
+QDialogButtonBox::StandardButtons TaskPlacement::getStandardButtons() const
+{ 
+    return QDialogButtonBox::Ok|
+           QDialogButtonBox::Cancel|
+           QDialogButtonBox::Apply;
+}
+
+void TaskPlacement::setPlacement(const Base::Placement& p)
+{
+    widget->setPlacement(p);
+}
+
+void TaskPlacement::slotPlacementChanged(const QVariant & p, bool incr, bool data)
+{
+    // If there are listeners to the 'placementChanged' signal we rely
     // on that the listener updates any placement. If no listeners
     // are connected the placement is applied to all selected objects
     // automatically.
-    if (receivers(SIGNAL(placementChanged(QVariant))) > 0) {
-        /*emit*/ placementChanged(p);
+    if (receivers(SIGNAL(placementChanged(QVariant, bool, bool))) > 0) {
+        /*emit*/ placementChanged(p, incr, data);
     }
     else {
         Base::Placement plm = p.value<Base::Placement>();
-        applyPlacement(plm);
+        widget->applyPlacement(plm, incr, data);
     }
 }
 
@@ -379,9 +458,17 @@ bool TaskPlacement::accept()
     return (widget->result() == QDialog::Accepted);
 }
 
-void TaskPlacement::setPlacement(const Base::Placement& p)
+bool TaskPlacement::reject()
 {
-    widget->setPlacement(p);
+    widget->reject();
+    return (widget->result() == QDialog::Rejected);
+}
+
+void TaskPlacement::clicked(int id)
+{
+    if (id == QDialogButtonBox::Apply) {
+        widget->on_applyButton_clicked();
+    }
 }
 
 #include "moc_Placement.cpp"
