@@ -51,7 +51,7 @@ using namespace Gui::DockWnd;
 using namespace Gui::PropertyEditor;
 
 
-/* TRANSLATOR Gui::DockWnd::PropertyView */
+/* TRANSLATOR Gui::PropertyView */
 
 PropertyView::PropertyView(QWidget *parent)
   : QWidget(parent)
@@ -67,9 +67,11 @@ PropertyView::PropertyView(QWidget *parent)
     pLayout->addWidget(tabs, 0, 0);
 
     propertyEditorView = new Gui::PropertyEditor::PropertyEditor();
-    tabs->addTab(propertyEditorView, trUtf8("View"));
+    propertyEditorView->setAutomaticDocumentUpdate(false);
+    tabs->addTab(propertyEditorView, tr("View"));
     propertyEditorData = new Gui::PropertyEditor::PropertyEditor();
-    tabs->addTab(propertyEditorData, trUtf8("Data"));
+    propertyEditorData->setAutomaticDocumentUpdate(true);
+    tabs->addTab(propertyEditorData, tr("Data"));
 }
 
 PropertyView::~PropertyView()
