@@ -141,11 +141,11 @@ void InteractiveInterpreter::setPrompt()
 /**
  * Compile a command and determine whether it is incomplete.
  * 
- * The source string may contain \n characters.
+ * The source string may contain line feeds and/or carriage returns. \n
  * Return value / exceptions raised:
- * \item Return a code object if the command is complete and valid
- * \item Return None if the command is incomplete
- * \item Raise SyntaxError, ValueError or OverflowError if the command is a
+ * - Return a code object if the command is complete and valid
+ * - Return None if the command is incomplete
+ * - Raise SyntaxError, ValueError or OverflowError if the command is a
  * syntax error (OverflowError and ValueError can be produced by
  * malformed literals).
  */
@@ -173,13 +173,13 @@ PyObject* InteractiveInterpreter::compile(const char* source) const
 /**
  * Compile a command and determine whether it is incomplete.
  * 
- * The source string may contain \n characters.
+ * The source string may contain line feeds and/or carriage returns. \n
  * Return value:
- * \item Return  1 if the command is incomplete
- * \item Return  0 if the command is complete and valid
- * \item Return -1 if the command is a syntax error 
- * (OverflowError and ValueError can be produced by
- * malformed literals).
+ * - Return  1 if the command is incomplete
+ * - Return  0 if the command is complete and valid
+ * - Return -1 if the command is a syntax error
+ * .
+ * (OverflowError and ValueError can be produced by malformed literals).
  */
 int InteractiveInterpreter::compileCommand(const char* source) const
 {
@@ -210,15 +210,15 @@ int InteractiveInterpreter::compileCommand(const char* source) const
  *
  * One several things can happen:
  *
- * \item The input is incorrect; compile() raised an exception (SyntaxError or OverflowError).  
- * A syntax traceback will be printed by calling Python's PyErr_Print() method to the redirected stderr.
+ * - The input is incorrect; compile() raised an exception (SyntaxError or OverflowError).  
+ *   A syntax traceback will be printed by calling Python's PyErr_Print() method to the redirected stderr.
  *
- * \item The input is incomplete, and more input is required; compile() returned 'None'. 
- * Nothing happens.
+ * - The input is incomplete, and more input is required; compile() returned 'None'. 
+ *   Nothing happens.
  *
- * \item The input is complete; compile() returned a code object.  The code is executed by calling 
- * runCode() (which also handles run-time exceptions, except for SystemExit).
- * 
+ * - The input is complete; compile() returned a code object.  The code is executed by calling 
+ *   runCode() (which also handles run-time exceptions, except for SystemExit).
+ * .
  * The return value is True if the input is incomplete, False in the other cases (unless
  * an exception is raised). The return value can be used to decide whether to use sys.ps1 
  * or sys.ps2 to prompt the next line.
@@ -621,7 +621,8 @@ void PythonConsole::runSource(const QString& line)
         // launch the command now
         incomplete = d->interpreter->push(line.toUtf8());
         setFocus(); // if focus was lost
-    } catch (const Base::SystemExitException&) {
+    }
+    catch (const Base::SystemExitException&) {
         int ret = QMessageBox::question(this, tr("System exit"), tr("The application is still running.\nDo you want to exit without saving your data?"),
         QMessageBox::Yes, QMessageBox::No|QMessageBox::Escape|QMessageBox::Default);
         if (ret == QMessageBox::Yes) {
@@ -629,11 +630,17 @@ void PythonConsole::runSource(const QString& line)
         } else {
             PyErr_Clear();
         }
-    } catch (const Py::Exception&) {
+    }
+    catch (const Py::Exception&) {
         QMessageBox::critical(this, tr("Python console"), tr("Unhandled PyCXX exception."));
-    } catch (const Base::Exception&) {
+    }
+    catch (const Base::Exception&) {
         QMessageBox::critical(this, tr("Python console"), tr("Unhandled FreeCAD exception."));
-    } catch (...) {
+    }
+    catch (const std::exception&) {
+        QMessageBox::critical(this, tr("Python console"), tr("Unhandled std C++ exception."));
+    }
+    catch (...) {
         QMessageBox::critical(this, tr("Python console"), tr("Unhandled unknown C++ exception."));
     }
 
@@ -973,34 +980,42 @@ void PythonConsole::overrideCursor(const QString& txt)
 
 void PythonConsole::contextMenuEvent ( QContextMenuEvent * e )
 {
-    QMenu* menu = new QMenu(this);
+    QMenu menu(this);
     QAction *a;
 
-    a = menu->addAction(QTextEdit::tr("&Copy"), this, SLOT(copy()), Qt::CTRL+Qt::Key_C);
+    a = menu.addAction(QTextEdit::tr("&Copy"), this, SLOT(copy()), Qt::CTRL+Qt::Key_C);
     a->setEnabled(textCursor().hasSelection());
 
-    a = menu->addAction(tr("&Copy command"), this, SLOT(onCopyCommand()));
+    a = menu.addAction(tr("&Copy command"), this, SLOT(onCopyCommand()));
     a->setEnabled(textCursor().hasSelection());
 
-    a = menu->addAction(tr("&Copy history"), this, SLOT(onCopyHistory()));
+    a = menu.addAction(tr("&Copy history"), this, SLOT(onCopyHistory()));
     a->setEnabled(!d->history.isEmpty());
 
-    a = menu->addAction( tr("Save history as..."), this, SLOT(onSaveHistoryAs()));
+    a = menu.addAction( tr("Save history as..."), this, SLOT(onSaveHistoryAs()));
     a->setEnabled(!d->history.isEmpty());
 
-    menu->addSeparator();
-    a = menu->addAction(QTextEdit::tr("&Paste"), this, SLOT(paste()), Qt::CTRL+Qt::Key_V);
+    menu.addSeparator();
+    a = menu.addAction(QTextEdit::tr("&Paste"), this, SLOT(paste()), Qt::CTRL+Qt::Key_V);
     const QMimeData *md = QApplication::clipboard()->mimeData();
     a->setEnabled(md && canInsertFromMimeData(md));
 
-    a = menu->addAction(QTextEdit::tr("Select All"), this, SLOT(selectAll()), Qt::CTRL+Qt::Key_A);
+    a = menu.addAction(QTextEdit::tr("Select All"), this, SLOT(selectAll()), Qt::CTRL+Qt::Key_A);
     a->setEnabled(!document()->isEmpty());
 
-    menu->addSeparator();
-    menu->addAction( tr("Insert file name..."), this, SLOT(onInsertFileName()));
+    menu.addSeparator();
+    menu.addAction( tr("Insert file name..."), this, SLOT(onInsertFileName()));
+    menu.addSeparator();
 
-    menu->exec(e->globalPos());
-    delete menu;
+    QAction* wrap = menu.addAction(tr("Word wrap"));
+    wrap->setCheckable(true);
+    wrap->setChecked(this->wordWrapMode() != QTextOption::NoWrap);
+
+    QAction* exec = menu.exec(e->globalPos());
+    if (exec == wrap) {
+        this->setWordWrapMode(wrap->isChecked()
+            ? QTextOption::WrapAtWordBoundaryOrAnywhere : QTextOption::NoWrap);
+    }
 }
 
 void PythonConsole::onSaveHistoryAs()
@@ -1175,6 +1190,8 @@ const QStringList& ConsoleHistory::values() const
 
 // -----------------------------------------------------
 
+/* TRANSLATOR Gui::PythonInputField */
+
 PythonInputField::PythonInputField(QWidget* parent)
   : QWidget(parent)
 {
