@@ -65,7 +65,8 @@
 #if QT_VERSION >= 0x040200
 #include <QDesktopServices>
 #endif
-#include <QDesktopWidget> 
+#include <QDesktopWidget>
+#include <QDialogButtonBox>
 #include <QDockWidget>
 #include <qdrawutil.h>
 #include <qevent.h>
@@ -73,6 +74,8 @@
 #include <QFileIconProvider>
 #include <qfontdatabase.h>
 #include <qfontdialog.h>
+#include <QGraphicsRectItem>
+#include <QGraphicsSvgItem>
 #include <QGroupBox>
 #include <QHeaderView>
 #include <qimage.h>
@@ -94,6 +97,7 @@
 #include <qpalette.h>
 #include <qpixmap.h>
 #include <QPixmapCache>
+#include <QPlainTextEdit>
 #include <QPrinter>
 #include <QPrintDialog>
 #include <QProgressBar>
