@@ -28,13 +28,18 @@
 # include <string>
 # include <boost/signals.hpp>
 # include <boost/bind.hpp>
+# include <QApplication>
 # include <QString>
 # include <QStatusBar>
 #endif
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
+#include "Application.h"
+#include "Document.h"
 #include "Selection.h"
+#include "SelectionFilter.h"
 #include "SelectionObjectPy.h"
+#include "View3DInventor.h"
 #include <Base/Exception.h>
 #include <Base/Console.h>
 #include <Base/Interpreter.h>
@@ -456,6 +461,36 @@ bool SelectionSingleton::setPreselect(const char* pDocName, const char* pObjectN
     if (DocName != "")
         rmvPreselect();
 
+    if(ActiveGate)
+    {
+        App::Document* pDoc = getDocument(pDocName);
+
+        if (pDoc) {
+            if(pObjectName){
+                App::DocumentObject* pObject = pDoc->getObject(pObjectName);
+                if(! ActiveGate->allow(pDoc,pObject,pSubName)){
+                    snprintf(buf,512,"Not allowed: %s.%s.%s ",pDocName
+                                                       ,pObjectName
+                                                       ,pSubName
+                                                       );
+
+                    if (getMainWindow()){
+                        getMainWindow()->statusBar()->showMessage(QString::fromAscii(buf),3000);
+                        Gui::MDIView* mdi = Gui::Application::Instance->activeDocument()->getActiveView();
+                        if (mdi && mdi->isDerivedFrom(View3DInventor::getClassTypeId())) {
+                            static_cast<View3DInventor*>(mdi)->setCursor(Qt::ForbiddenCursor);
+                        }
+                    }
+                    return false;
+                }
+
+            }else
+                return ActiveGate->allow(pDoc,0,0);
+        }else
+            return false;
+
+    }
+
     DocName = pDocName;
     FeatName= pObjectName;
     SubName = pSubName;
@@ -481,9 +516,13 @@ bool SelectionSingleton::setPreselect(const char* pDocName, const char* pObjectN
                                                        ,Chng.pSubName
                                                        ,x,y,z);
 
-    if (getMainWindow())
+    if (getMainWindow()){
         getMainWindow()->statusBar()->showMessage(QString::fromAscii(buf),3000);
-
+        Gui::MDIView* mdi = Gui::Application::Instance->activeDocument()->getActiveView();
+        if (mdi && mdi->isDerivedFrom(View3DInventor::getClassTypeId())) {
+            static_cast<View3DInventor*>(mdi)->setCursor(Qt::ArrowCursor);
+        }
+    }
 
     Notify(Chng);
     signalSelectionChanged(Chng);
@@ -498,6 +537,9 @@ void SelectionSingleton::setPreselectCoord( float x, float y, float z)
 {
     static char buf[513];
 
+    // if nothing is in preselect ignore
+    if(!CurrentPreselection.pObjectName) return;
+
     CurrentPreselection.x = x;
     CurrentPreselection.y = y;
     CurrentPreselection.z = z;
@@ -541,6 +583,13 @@ void SelectionSingleton::rmvPreselect()
     hy = 0;
     hz = 0;
 
+    if (getMainWindow()){
+        Gui::MDIView* mdi = Gui::Application::Instance->activeDocument()->getActiveView();
+        if (mdi && mdi->isDerivedFrom(View3DInventor::getClassTypeId())) {
+            static_cast<View3DInventor*>(mdi)->setCursor(Qt::ArrowCursor);
+        }
+    }
+
     //Base::Console().Log("Sel : Rmv preselect \n");
 }
 
@@ -549,6 +598,25 @@ const SelectionChanges &SelectionSingleton::getPreselection(void) const
     return CurrentPreselection;
 }
 
+// add a SelectionGate to control what is selectable
+void SelectionSingleton::addSelectionGate(Gui::SelectionGate *gate)
+{
+    if(ActiveGate)
+        rmvSelectionGate();
+    
+    ActiveGate = gate;
+
+}
+
+// remove the active SelectionGate
+void SelectionSingleton::rmvSelectionGate(void)
+{
+    if (ActiveGate)
+        delete ActiveGate;
+    ActiveGate=0;
+}
+
+
 App::Document* SelectionSingleton::getDocument(const char* pDocName) const
 {
     if (pDocName)
@@ -563,7 +631,7 @@ bool SelectionSingleton::addSelection(const char* pDocName, const char* pObjectN
     if (isSelected(pDocName, pObjectName, pSubName))
         return true;
 
-    _SelObj temp;
+     _SelObj temp;
 
     temp.pDoc = getDocument(pDocName);
 
@@ -572,6 +640,22 @@ bool SelectionSingleton::addSelection(const char* pDocName, const char* pObjectN
             temp.pObject = temp.pDoc->getObject(pObjectName);
         else
             temp.pObject = 0;
+        
+        // check for a Selection Gate
+        if(ActiveGate)
+        {
+            if(! ActiveGate->allow(temp.pDoc,temp.pObject,pSubName)){
+                if (getMainWindow()){
+                    getMainWindow()->statusBar()->showMessage(QString::fromAscii("Selection not allowed by filter"),5000);
+                    Gui::MDIView* mdi = Gui::Application::Instance->activeDocument()->getActiveView();
+                    if (mdi && mdi->isDerivedFrom(View3DInventor::getClassTypeId())) {
+                        static_cast<View3DInventor*>(mdi)->setCursor(Qt::ForbiddenCursor);
+                    }
+                }
+                QApplication::beep();
+                return false;
+            }
+        }
 
         temp.DocName  = pDocName;
         temp.FeatName = pObjectName?pObjectName:"";
@@ -813,8 +897,13 @@ void SelectionSingleton::slotRenamedObject(const App::DocumentObject& Obj)
  */
 SelectionSingleton::SelectionSingleton()
 {
+    ActiveGate = 0;
     App::GetApplication().signalDeletedObject.connect(boost::bind(&Gui::SelectionSingleton::slotDeletedObject, this, _1));
     App::GetApplication().signalRenamedObject.connect(boost::bind(&Gui::SelectionSingleton::slotRenamedObject, this, _1));
+    CurrentPreselection.pDocName = 0;
+    CurrentPreselection.pObjectName = 0;
+    CurrentPreselection.pSubName = 0;
+
 }
 
 /**
@@ -886,6 +975,14 @@ PyMethodDef SelectionSingleton::Methods[] = {
      "addObserver(Object) -- Install an observer\n"},
     {"removeObserver",      (PyCFunction) SelectionSingleton::sRemSelObserver, 1,
      "removeObserver(Object) -- Uninstall an observer\n"},
+    {"addSelectionGate",      (PyCFunction) SelectionSingleton::saddSelectionGate, 1,
+    "addSelectionGate(String) -- activate the selection gate.\n"
+    "The selection gate will prohibit all selections which do not match the\n"
+    "the given selection filter string. Examples strings are:\n"
+    "'SELECT Part::Feature SUB Edge',\n"
+    "'SELECT Robot::RobotObject'\n"},
+    {"removeSelectionGate",      (PyCFunction) SelectionSingleton::sremoveSelectionGate, 1,
+     "removeSelectionGate() -- remove the active slection gate\n"},
     {NULL, NULL, 0, NULL}		/* Sentinel */
 };
 
@@ -968,7 +1065,7 @@ PyObject *SelectionSingleton::sGetSelection(PyObject * /*self*/, PyObject *args,
     try {
         Py::List list;
         for (std::vector<SelectionSingleton::SelObj>::iterator it = sel.begin(); it != sel.end(); ++it) {
-            list.append(Py::Object(it->pObject->getPyObject()));
+            list.append(Py::asObject(it->pObject->getPyObject()));
         }
         return Py::new_reference_to(list);
     }
@@ -989,7 +1086,7 @@ PyObject *SelectionSingleton::sGetSelectionEx(PyObject * /*self*/, PyObject *arg
     try {
         Py::List list;
         for (std::vector<SelectionObject>::iterator it = sel.begin(); it != sel.end(); ++it) {
-            list.append(Py::Object(new SelectionObjectPy(new SelectionObject(*it)), true));
+            list.append(Py::asObject(it->getPyObject()));
         }
         return Py::new_reference_to(list);
     }
@@ -1019,3 +1116,28 @@ PyObject *SelectionSingleton::sRemSelObserver(PyObject * /*self*/, PyObject *arg
         Py_Return;
     } PY_CATCH;
 }
+
+PyObject *SelectionSingleton::saddSelectionGate(PyObject * /*self*/, PyObject *args, PyObject * /*kwd*/)
+{
+    char* filter;
+    if (!PyArg_ParseTuple(args, "s",&filter))
+        return NULL;                             // NULL triggers exception 
+
+    PY_TRY {
+        Selection().addSelectionGate(new SelectionFilterGate(filter));
+    } PY_CATCH;
+
+    Py_Return;
+}
+
+PyObject *SelectionSingleton::sremoveSelectionGate(PyObject * /*self*/, PyObject *args, PyObject * /*kwd*/)
+{
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;                             // NULL triggers exception 
+
+    PY_TRY {
+        Selection().rmvSelectionGate();
+    } PY_CATCH;
+
+    Py_Return;
+}
