@@ -38,12 +38,16 @@
 #include "SoFCInteractiveElement.h"
 #include "SoFCVectorizeSVGAction.h"
 #include "SoFCVectorizeU3DAction.h"
+#include "SoAxisCrossKit.h"
 #include "SoTextLabel.h"
+#include "SoNavigationDragger.h"
+#include "Inventor/SoDrawingGrid.h"
 
 #include "propertyeditor/PropertyItem.h"
 #include "NavigationStyle.h"
 
 using namespace Gui;
+using namespace Gui::Inventor;
 using namespace Gui::PropertyEditor;
 
 static SbBool init_done = FALSE;
@@ -55,6 +59,7 @@ SbBool Gui::SoFCDB::isInitialized(void)
 
 void Gui::SoFCDB::init()
 {
+    RotTransDragger                 ::initClass();
     SoGLRenderActionElement         ::initClass();
     SoFCInteractiveElement          ::initClass();
     SoGLWidgetElement               ::initClass();
@@ -75,15 +80,22 @@ void Gui::SoFCDB::init()
     SoFCHighlightColorAction        ::initClass();
     SoFCDocumentObjectAction        ::initClass();
     SoGLSelectAction                ::initClass();
+    SoVisibleFaceAction             ::initClass();
+    SoBoxSelectionRenderAction      ::initClass();
     SoFCVectorizeSVGAction          ::initClass();
     SoFCVectorizeU3DAction          ::initClass();
+    SoSkipBoundingGroup             ::initClass();
     SoTextLabel                     ::initClass();
     SoStringLabel                   ::initClass();
-
+    TranslateManip                  ::initClass();
+    SoShapeScale                    ::initClass();
+    SoAxisCrossKit                  ::initClass();
+    SoDrawingGrid                   ::initClass();
 
     PropertyItem                    ::init();
     PropertySeparatorItem           ::init();
     PropertyStringItem              ::init();
+    PropertyFontItem                ::init();
     PropertyIntegerItem             ::init();
     PropertyIntegerConstraintItem   ::init();
     PropertyFloatItem               ::init();
@@ -98,6 +110,7 @@ void Gui::SoFCDB::init()
     PropertyColorItem               ::init();
     PropertyFileItem                ::init();
     PropertyPathItem                ::init();
+    PropertyTransientFileItem       ::init();
 
     NavigationStyle                 ::init();
     UserNavigationStyle             ::init();
