@@ -39,9 +39,11 @@
 # include <Inventor/actions/SoGLRenderAction.h>
 # include <Inventor/bundles/SoMaterialBundle.h>
 # include <Inventor/elements/SoLazyElement.h>
+# include <Inventor/nodes/SoSurroundScale.h>
 # include <Inventor/misc/SoState.h>
 #endif
 
+#include <Inventor/draggers/SoTranslate2Dragger.h>
 #include <Inventor/elements/SoFontNameElement.h>
 #include <Inventor/elements/SoFontSizeElement.h>
 #include <Inventor/elements/SoCullElement.h>
@@ -81,7 +83,7 @@ s="""
 
 App.ActiveDocument.addObject("App::InventorObject","iv").Buffer=s
 
-\encode
+\endcode
 */
 
 SO_NODE_SOURCE(SoTextLabel);
@@ -373,3 +375,26 @@ void SoStringLabel::GLRender(SoGLRenderAction *action)
 
     state->pop();
 }
+
+// ------------------------------------------------------
+
+SO_NODE_SOURCE(TranslateManip);
+
+void
+TranslateManip::initClass()
+{
+    SO_NODE_INIT_CLASS(TranslateManip, SoTransformManip,
+                       "TransformManip");
+}
+
+TranslateManip::TranslateManip()
+{
+    SO_NODE_CONSTRUCTOR(TranslateManip);
+
+    SoTranslate2Dragger *myDrag = new SoTranslate2Dragger;
+    setDragger(myDrag);
+}
+
+TranslateManip::~TranslateManip()
+{
+}
