@@ -125,13 +125,29 @@ void TaskBox::hideGroupBox()
     setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 }
 
+namespace Gui { namespace TaskView {
+class TaskIconLabel : public iisIconLabel {
+public:
+    TaskIconLabel(const QIcon &icon, 
+                  const QString &title,
+                  QWidget *parent = 0)
+      : iisIconLabel(icon, title, parent) {
+    }
+    void setTitle(const QString &text) {
+        myText = text;
+        update();
+    }
+};
+}
+}
+
 void TaskBox::actionEvent (QActionEvent* e)
 {
     QAction *action = e->action();
     switch (e->type()) {
     case QEvent::ActionAdded:
         {
-            iisIconLabel *label = new iisIconLabel(
+            TaskIconLabel *label = new TaskIconLabel(
                 action->icon(), action->text(), this);
             this->addIconLabel(label);
             connect(label,SIGNAL(clicked()),action,SIGNAL(triggered()));
@@ -139,7 +155,13 @@ void TaskBox::actionEvent (QActionEvent* e)
         }
     case QEvent::ActionChanged:
         {
-            // cannot change anything
+            // update label when action changes
+            QBoxLayout* bl = myGroup->groupLayout();
+            int index = this->actions().indexOf(action);
+            if (index < 0) break;
+            QWidgetItem* item = static_cast<QWidgetItem*>(bl->itemAt(index));
+            TaskIconLabel* label = static_cast<TaskIconLabel*>(item->widget());
+            label->setTitle(action->text());
             break;
         }
     case QEvent::ActionRemoved:
@@ -172,13 +194,16 @@ TaskView::TaskView(QWidget *parent)
     this->setMinimumWidth(200);
 
     Gui::Selection().Attach(this);
+
+    connectApplicationActiveDocument = 
     App::GetApplication().signalActiveDocument.connect
         (boost::bind(&Gui::TaskView::TaskView::slotActiveDocument, this, _1));
 }
 
 TaskView::~TaskView()
 {
-     Gui::Selection().Detach(this);
+    connectApplicationActiveDocument.disconnect();
+    Gui::Selection().Detach(this);
 }
 
 void TaskView::slotActiveDocument(const App::Document& doc)
@@ -187,26 +212,23 @@ void TaskView::slotActiveDocument(const App::Document& doc)
 }
 
 
+/// @cond DOXERR
 void TaskView::OnChange(Gui::SelectionSingleton::SubjectType &rCaller,
                         Gui::SelectionSingleton::MessageType Reason)
 {
     std::string temp;
 
-    if (Reason.Type == SelectionChanges::AddSelection) {
-        //Gui::Document* pDoc = Application::Instance->getDocument(Reason.pDocName);
-        //Gui::ViewProvider *pVP = pDoc->getViewProviderByName(Reason.pObjectName);
-    }
-    else if (Reason.Type == SelectionChanges::ClrSelection) {
-    }
-    else if (Reason.Type == SelectionChanges::SetSelection) {
-    }
-    else if (Reason.Type == SelectionChanges::RmvSelection) {
-    }
+    if (Reason.Type == SelectionChanges::AddSelection ||
+        Reason.Type == SelectionChanges::ClrSelection || 
+        Reason.Type == SelectionChanges::SetSelection ||
+        Reason.Type == SelectionChanges::RmvSelection) {
 
-    if(!ActiveDialog)
-        updateWatcher();
+        if(!ActiveDialog)
+            updateWatcher();
+    }
 
 }
+/// @endcond
 
 void TaskView::showDialog(TaskDialog *dlg)
 {
@@ -235,10 +257,20 @@ void TaskView::showDialog(TaskDialog *dlg)
 
     const std::vector<QWidget*>& cont = dlg->getDialogContent();
 
-    taskPanel->addWidget(ActiveCtrl);
+    // give to task dialog to customize the button box
+    dlg->modifyStandardButtons(ActiveCtrl->buttonBox);
 
-    for (std::vector<QWidget*>::const_iterator it=cont.begin();it!=cont.end();++it){
-        taskPanel->addWidget(*it);
+    if (dlg->buttonPosition() == TaskDialog::North) {
+        taskPanel->addWidget(ActiveCtrl);
+        for (std::vector<QWidget*>::const_iterator it=cont.begin();it!=cont.end();++it){
+            taskPanel->addWidget(*it);
+        }
+    }
+    else {
+        for (std::vector<QWidget*>::const_iterator it=cont.begin();it!=cont.end();++it){
+            taskPanel->addWidget(*it);
+        }
+        taskPanel->addWidget(ActiveCtrl);
     }
 
     if (!dlg->needsFullSpace())
@@ -275,18 +307,16 @@ void TaskView::removeDialog(void)
 
 void TaskView::updateWatcher(void)
 {
-    // add all widghets for all watcher to the task view
-    for(std::vector<TaskWatcher*>::iterator it=ActiveWatcher.begin();it!=ActiveWatcher.end();++it){
-
+    // add all widgets for all watcher to the task view
+    for (std::vector<TaskWatcher*>::iterator it=ActiveWatcher.begin();it!=ActiveWatcher.end();++it) {
         bool match = (*it)->shouldShow();
         std::vector<QWidget*> &cont = (*it)->getWatcherContent();
-        for(std::vector<QWidget*>::iterator it2=cont.begin();it2!=cont.end();++it2){
+        for (std::vector<QWidget*>::iterator it2=cont.begin();it2!=cont.end();++it2) {
             if(match)
                 (*it2)->show();
             else
                 (*it2)->hide();
         }
-
     }
 }
 
@@ -302,29 +332,31 @@ void TaskView::addTaskWatcher(const std::vector<TaskWatcher*> &Watcher)
 
 void TaskView::addTaskWatcher(void)
 {
-    // add all widghets for all watcher to the task view
-    for(std::vector<TaskWatcher*>::iterator it=ActiveWatcher.begin();it!=ActiveWatcher.end();++it){
+    // add all widgets for all watcher to the task view
+    for (std::vector<TaskWatcher*>::iterator it=ActiveWatcher.begin();it!=ActiveWatcher.end();++it){
         std::vector<QWidget*> &cont = (*it)->getWatcherContent();
-        for(std::vector<QWidget*>::iterator it2=cont.begin();it2!=cont.end();++it2){
+        for (std::vector<QWidget*>::iterator it2=cont.begin();it2!=cont.end();++it2){
            taskPanel->addWidget(*it2);
            (*it2)->show();
         }
     }
-    taskPanel->addStretch();
+
+    if (!ActiveWatcher.empty())
+        taskPanel->addStretch();
     updateWatcher();
 }
 
 void TaskView::removeTaskWatcher(void)
 {
-    // remove all widghets
-    for(std::vector<TaskWatcher*>::iterator it=ActiveWatcher.begin();it!=ActiveWatcher.end();++it){
+    // remove all widgets
+    for (std::vector<TaskWatcher*>::iterator it=ActiveWatcher.begin();it!=ActiveWatcher.end();++it) {
         std::vector<QWidget*> &cont = (*it)->getWatcherContent();
-        for(std::vector<QWidget*>::iterator it2=cont.begin();it2!=cont.end();++it2){
+        for (std::vector<QWidget*>::iterator it2=cont.begin();it2!=cont.end();++it2) {
             (*it2)->hide();
             taskPanel->removeWidget(*it2);
         }
-
     }
+
     taskPanel->removeStretch();
 }
 
