@@ -27,6 +27,7 @@
 #include <map>
 #include <string>
 #include <vector>
+#include <boost/signals.hpp>
 
 #include <Gui/iisTaskPanel/include/iisTaskPanel>
 #include <Gui/Selection.h>
@@ -37,12 +38,13 @@ class Property;
 }
 
 namespace Gui {
-class Control;
+class ControlSingleton;
 namespace DockWnd{
 class CombiView;
 }
 namespace TaskView {
 
+typedef boost::signals::connection Connection;
 class TaskEditControl;
 class TaskDialog;
 
@@ -101,6 +103,7 @@ public:
                           Gui::SelectionSingleton::MessageType Reason);
 
     friend class Gui::DockWnd::CombiView;
+    friend class Gui::ControlSingleton;
 
     void addTaskWatcher(const std::vector<TaskWatcher*> &Watcher);
     void addTaskWatcher(void);
@@ -117,7 +120,7 @@ protected:
     void updateWatcher(void);
     /// used by Gui::Contol to register Dialogs
     void showDialog(TaskDialog *dlg);
-    // removes the runing dialog after accept() or reject() from the TaskView
+    // removes the running dialog after accept() or reject() from the TaskView
     void removeDialog(void);
 
     void slotActiveDocument(const App::Document&);
@@ -127,6 +130,8 @@ protected:
     iisTaskPanel* taskPanel;
     TaskDialog *ActiveDialog;
     TaskEditControl *ActiveCtrl;
+
+    Connection connectApplicationActiveDocument;
 };
 
 } //namespace TaskView
