@@ -228,6 +228,19 @@ TextEditor::~TextEditor()
     delete d;
 }
 
+void TextEditor::insertFromMimeData(const QMimeData *source)
+{
+    // do not want to have HTML text here
+    if (source->hasHtml()) {
+        QMimeData mime;
+        mime.setText(source->text());
+        TextEdit::insertFromMimeData(&mime);
+    }
+    else {
+        TextEdit::insertFromMimeData(source);
+    }
+}
+
 void TextEditor::keyPressEvent (QKeyEvent * e)
 {
     if ( e->key() == Qt::Key_Tab ) {
