@@ -51,13 +51,13 @@
 #include "Application.h"
 #include "MainWindow.h"
 
-
 using namespace Gui;
 
 QPixmap*  TreeWidget::documentPixmap = 0;
 const int TreeWidget::DocumentType = 1000;
 const int TreeWidget::ObjectType = 1001;
 
+
 /* TRANSLATOR Gui::TreeWidget */
 TreeWidget::TreeWidget(QWidget* parent)
     : QTreeWidget(parent), fromOutside(false)
@@ -78,6 +78,11 @@ TreeWidget::TreeWidget(QWidget* parent)
     this->relabelObjectAction->setShortcut(Qt::Key_F2);
     connect(this->relabelObjectAction, SIGNAL(triggered()),
             this, SLOT(onRelabelObject()));
+    this->finishEditingAction = new QAction(this);
+    this->finishEditingAction->setText(tr("Finish editing"));
+    this->finishEditingAction->setStatusTip(tr("Finish editing object"));
+    connect(this->finishEditingAction, SIGNAL(triggered()),
+            this, SLOT(onFinishEditing()));
 
     // Setup connections
     Application::Instance->signalNewDocument.connect(boost::bind(&TreeWidget::slotNewDocument, this, _1));
@@ -122,16 +127,17 @@ TreeWidget::~TreeWidget()
 void TreeWidget::contextMenuEvent (QContextMenuEvent * e)
 {
     // ask workbenches and view provider, ...
-    MenuItem* view = new MenuItem;
-    Gui::Application::Instance->setupContextMenu("Tree", view);
+    MenuItem view;
+    Gui::Application::Instance->setupContextMenu("Tree", &view);
 
     QMenu contextMenu;
     QMenu subMenu;
+    QMenu editMenu;
     QActionGroup subMenuGroup(&subMenu);
     subMenuGroup.setExclusive(true);
     connect(&subMenuGroup, SIGNAL(triggered(QAction*)),
             this, SLOT(onActivateDocument(QAction*)));
-    MenuManager::getInstance()->setupContextMenu(view, contextMenu);
+    MenuManager::getInstance()->setupContextMenu(&view, contextMenu);
 
     // get the current item
     this->contextItem = itemAt(e->pos());
@@ -157,6 +163,22 @@ void TreeWidget::contextMenuEvent (QContextMenuEvent * e)
         if (!contextMenu.actions().isEmpty())
             contextMenu.addSeparator();
         contextMenu.addAction(this->relabelObjectAction);
+
+        // if only one item is selected setup the edit menu
+        if (this->selectedItems().size() == 1) {
+            objitem->object()->setupContextMenu(&editMenu, this, SLOT(onStartEditing()));
+            QList<QAction*> editAct = editMenu.actions();
+            if (!editAct.isEmpty()) {
+                QAction* topact = contextMenu.actions().front();
+                for (QList<QAction*>::iterator it = editAct.begin(); it != editAct.end(); ++it)
+                    contextMenu.insertAction(topact, *it);
+                QAction* first = editAct.front();
+                contextMenu.setDefaultAction(first);
+                if (objitem->object()->isEditing())
+                    contextMenu.insertAction(topact, this->finishEditingAction);
+                contextMenu.insertSeparator(topact);
+            }
+        }
     }
 
     // add a submenu to active a document if two or more exist
@@ -178,7 +200,6 @@ void TreeWidget::contextMenuEvent (QContextMenuEvent * e)
         active = subMenuGroup.checkedAction();
         subMenu.addActions(subMenuGroup.actions());
     }
-    delete view;
     if (contextMenu.actions().count() > 0)
         contextMenu.exec(QCursor::pos());
 }
@@ -221,6 +242,35 @@ void TreeWidget::onRelabelObject()
         editItem(item);
 }
 
+void TreeWidget::onStartEditing()
+{
+    QAction* action = qobject_cast<QAction*>(sender());
+    if (action) {
+        if (this->contextItem && this->contextItem->type() == ObjectType) {
+            DocumentObjectItem* objitem = static_cast<DocumentObjectItem*>
+                (this->contextItem);
+            int edit = action->data().toInt();
+            App::DocumentObject* obj = objitem->object()->getObject();
+            if (!obj) return;
+            Gui::Document* doc = Gui::Application::Instance->getDocument(obj->getDocument());
+            doc->setEdit(objitem->object(), edit);
+        }
+    }
+}
+
+void TreeWidget::onFinishEditing()
+{
+    if (this->contextItem && this->contextItem->type() == ObjectType) {
+        DocumentObjectItem* objitem = static_cast<DocumentObjectItem*>
+            (this->contextItem);
+        App::DocumentObject* obj = objitem->object()->getObject();
+        if (!obj) return;
+        Gui::Document* doc = Gui::Application::Instance->getDocument(obj->getDocument());
+        doc->resetEdit();
+        doc->getDocument()->recompute();
+    }
+}
+
 void TreeWidget::onActivateDocument(QAction* active)
 {
     // activate the specified document
@@ -245,18 +295,18 @@ Qt::DropActions TreeWidget::supportedDropActions () const
 
 void TreeWidget::mouseDoubleClickEvent (QMouseEvent * event)
 {
-    QTreeWidgetItem* Item = itemAt(event->pos());
-    if (!Item) return;
-    if (Item->type() == TreeWidget::DocumentType) {
-        QTreeWidget::mouseDoubleClickEvent(event);
-        const Gui::Document* doc = static_cast<DocumentItem*>(Item)->document();
+    QTreeWidgetItem* item = itemAt(event->pos());
+    if (!item) return;
+    if (item->type() == TreeWidget::DocumentType) {
+        //QTreeWidget::mouseDoubleClickEvent(event);
+        const Gui::Document* doc = static_cast<DocumentItem*>(item)->document();
         if (!doc) return;
         MDIView *view = doc->getActiveView();
         if (!view) return;
         getMainWindow()->setActiveWindow(view);
     }
-    else if (Item->type() == TreeWidget::ObjectType) {
-        if (!(static_cast<DocumentObjectItem*>(Item)->object())->doubleClicked())
+    else if (item->type() == TreeWidget::ObjectType) {
+        if (!(static_cast<DocumentObjectItem*>(item)->object())->doubleClicked())
             QTreeWidget::mouseDoubleClickEvent(event);
     }
 }
@@ -532,6 +582,7 @@ void TreeWidget::changeEvent(QEvent *e)
 {
     if (e->type() == QEvent::LanguageChange) {
         this->headerItem()->setText(0, tr("Labels & Attributes"));
+        this->rootItem->setText(0, tr("Application"));
     }
 
     QTreeWidget::changeEvent(e);
@@ -739,33 +790,33 @@ void DocumentItem::slotChangeObject(const Gui::ViewProviderDocumentObject& view)
     std::string objectName = obj->getNameInDocument();
     std::map<std::string, DocumentObjectItem*>::iterator it = ObjectMap.find(objectName);
     if (it != ObjectMap.end()) {
-
-        // is the object a group?
-        if (obj->getTypeId().isDerivedFrom(App::DocumentObjectGroup::getClassTypeId())) {
+         // use new grouping style
+#        if 1
             std::set<QTreeWidgetItem*> children;
-            std::vector<App::DocumentObject*> group = static_cast<App::DocumentObjectGroup*>(obj)->Group.getValues();
+            std::vector<App::DocumentObject*> group = view.claimChildren();
             for (std::vector<App::DocumentObject*>::iterator jt = group.begin(); jt != group.end(); ++jt) {
-                const char* internalName = (*jt)->getNameInDocument();
-                if (internalName) {
-                    std::map<std::string, DocumentObjectItem*>::iterator kt = ObjectMap.find(internalName);
-                    if (kt != ObjectMap.end()) {
-                        children.insert(kt->second);
-                        QTreeWidgetItem* parent = kt->second->parent();
-                        if (parent && parent != it->second) {
-                            int index = parent->indexOfChild(kt->second);
-                            parent->takeChild(index);
-                            it->second->addChild(kt->second);
+                if(*jt){
+                    const char* internalName = (*jt)->getNameInDocument();
+                    if (internalName) {
+                        std::map<std::string, DocumentObjectItem*>::iterator kt = ObjectMap.find(internalName);
+                        if (kt != ObjectMap.end()) {
+                            children.insert(kt->second);
+                            QTreeWidgetItem* parent = kt->second->parent();
+                            if (parent && parent != it->second) {
+                                int index = parent->indexOfChild(kt->second);
+                                parent->takeChild(index);
+                                it->second->addChild(kt->second);
+                            }
+                        }
+                        else {
+                            Base::Console().Warning("Gui::DocumentItem::slotChangedObject(): Cannot reparent unknown object.\n");
                         }
                     }
                     else {
-                        Base::Console().Warning("DocumentItem::slotChangedObject: Cannot reparent unknown object.\n");
+                        Base::Console().Warning("Gui::DocumentItem::slotChangedObject(): Group references unknown object.\n");
                     }
                 }
-                else {
-                    Base::Console().Warning("DocumentItem::slotChangedObject: Group references unknown object.\n");
-                }
             }
-
             // move all children which are not part of the group anymore to this item
             int count = it->second->childCount();
             for (int i=0; i < count; i++) {
@@ -775,14 +826,54 @@ void DocumentItem::slotChangeObject(const Gui::ViewProviderDocumentObject& view)
                     this->addChild(child);
                 }
             }
-            this->treeWidget()->expandItem(it->second);
-        }
+            //this->treeWidget()->expandItem(it->second);
+         // old grouping style here
+#        else 
 
+            // is the object a group?
+            if (obj->getTypeId().isDerivedFrom(App::DocumentObjectGroup::getClassTypeId())) {
+                std::set<QTreeWidgetItem*> children;
+                std::vector<App::DocumentObject*> group = static_cast<App::DocumentObjectGroup*>(obj)->Group.getValues();
+                for (std::vector<App::DocumentObject*>::iterator jt = group.begin(); jt != group.end(); ++jt) {
+                    const char* internalName = (*jt)->getNameInDocument();
+                    if (internalName) {
+                        std::map<std::string, DocumentObjectItem*>::iterator kt = ObjectMap.find(internalName);
+                        if (kt != ObjectMap.end()) {
+                            children.insert(kt->second);
+                            QTreeWidgetItem* parent = kt->second->parent();
+                            if (parent && parent != it->second) {
+                                int index = parent->indexOfChild(kt->second);
+                                parent->takeChild(index);
+                                it->second->addChild(kt->second);
+                            }
+                        }
+                        else {
+                            Base::Console().Warning("DocumentItem::slotChangedObject: Cannot reparent unknown object.\n");
+                        }
+                    }
+                    else {
+                        Base::Console().Warning("DocumentItem::slotChangedObject: Group references unknown object.\n");
+                    }
+                }
+
+                // move all children which are not part of the group anymore to this item
+                int count = it->second->childCount();
+                for (int i=0; i < count; i++) {
+                    QTreeWidgetItem* child = it->second->child(i);
+                    if (children.find(child) == children.end()) {
+                        it->second->takeChild(i);
+                        this->addChild(child);
+                    }
+                }
+                this->treeWidget()->expandItem(it->second);
+            }
+        // end of grouping style switch
+#       endif 
         // set the text label
         std::string displayName = obj->Label.getValue();
         it->second->setText(0, QString::fromUtf8(displayName.c_str()));
     } else {
-        Base::Console().Warning("DocumentItem::slotChangedObject: Cannot change unknown object.\n");
+        Base::Console().Warning("Gui::DocumentItem::slotChangedObject(): Cannot change unknown object.\n");
     }
 }
 
@@ -954,13 +1045,16 @@ DocumentObjectItem::DocumentObjectItem(Gui::ViewProviderDocumentObject* pcViewPr
 {
     setFlags(flags()|Qt::ItemIsEditable);
     // Setup connections
-    pcViewProvider->signalChangeIcon.connect(boost::bind(&DocumentObjectItem::slotChangeIcon, this));
-    pcViewProvider->signalChangeToolTip.connect(boost::bind(&DocumentObjectItem::slotChangeToolTip, this, _1));
-    pcViewProvider->signalChangeStatusTip.connect(boost::bind(&DocumentObjectItem::slotChangeStatusTip, this, _1));
+    connectIcon = pcViewProvider->signalChangeIcon.connect(boost::bind(&DocumentObjectItem::slotChangeIcon, this));
+    connectTool = pcViewProvider->signalChangeToolTip.connect(boost::bind(&DocumentObjectItem::slotChangeToolTip, this, _1));
+    connectStat = pcViewProvider->signalChangeStatusTip.connect(boost::bind(&DocumentObjectItem::slotChangeStatusTip, this, _1));
 }
 
 DocumentObjectItem::~DocumentObjectItem()
 {
+    connectIcon.disconnect();
+    connectTool.disconnect();
+    connectStat.disconnect();
 }
 
 Gui::ViewProviderDocumentObject* DocumentObjectItem::object() const
@@ -1102,5 +1196,7 @@ void DocumentObjectItem::slotChangeStatusTip(const QString& tip)
     this->setStatusTip(0, tip);
 }
 
+
+
 #include "moc_Tree.cpp"
 
