@@ -35,6 +35,7 @@
 # include <QPainter>
 # include <QPrinter>
 # include <QPrintDialog>
+# include <QStackedWidget>
 # include <QTimer>
 # include <QUrl>
 # include <Inventor/actions/SoWriteAction.h>
@@ -49,6 +50,7 @@
 # include <Inventor/fields/SoSFString.h>
 # include <Inventor/fields/SoSFColor.h>
 #endif
+# include <QStackedWidget>
 
 #include <Base/Exception.h>
 #include <Base/Console.h>
@@ -79,11 +81,21 @@
 
 using namespace Gui;
 
+void GLOverlayWidget::paintEvent(QPaintEvent* ev)
+{
+    QPainter paint(this);
+    paint.drawImage(0,0,image);
+    paint.end();
+}
+
+/* TRANSLATOR Gui::View3DInventor */
+
 TYPESYSTEM_SOURCE_ABSTRACT(Gui::View3DInventor,Gui::BaseView);
 
 View3DInventor::View3DInventor(Gui::Document* pcDocument, QWidget* parent, Qt::WFlags wflags)
     : MDIView(pcDocument, parent, wflags), _viewerPy(0)
 {
+    stack = new QStackedWidget(this);
     // important for highlighting 
     setMouseTracking(true);
     // accept drops on the window, get handled in dropEvent, dragEnterEvent   
@@ -96,7 +108,8 @@ View3DInventor::View3DInventor(Gui::Document* pcDocument, QWidget* parent, Qt::W
     // create the inventor widget and set the defaults
 #if !defined (NO_USE_QT_MDI_AREA)
     _viewer = new View3DInventorViewer(0);
-    setCentralWidget(_viewer->getWidget());
+    stack->addWidget(_viewer->getWidget());
+    setCentralWidget(stack);
 #else
     _viewer = new View3DInventorViewer(this);
 #endif
@@ -217,17 +230,17 @@ void View3DInventor::OnChange(ParameterGrp::SubjectType &rCaller,ParameterGrp::M
     }
     else if (strcmp(Reason,"EnablePreselection") == 0) {
         const ParameterGrp& rclGrp = ((ParameterGrp&)rCaller);
-        SoFCEnableHighlightAction cAct(rclGrp.GetBool("EnablePreselection", false));
+        SoFCEnableHighlightAction cAct(rclGrp.GetBool("EnablePreselection", true));
         cAct.apply(_viewer->getSceneGraph());
     }
     else if (strcmp(Reason,"EnableSelection") == 0) {
         const ParameterGrp& rclGrp = ((ParameterGrp&)rCaller);
-        SoFCEnableSelectionAction cAct(rclGrp.GetBool("EnableSelection", false));
+        SoFCEnableSelectionAction cAct(rclGrp.GetBool("EnableSelection", true));
         cAct.apply(_viewer->getSceneGraph());
     }
     else if (strcmp(Reason,"HighlightColor") == 0) {
         float transparency;
-        SbColor highlightColor(0.1f, 0.1f, 0.8f);
+        SbColor highlightColor(0.8f, 0.1f, 0.1f);
         unsigned long highlight = (unsigned long)(highlightColor.getPackedValue());
         highlight = rGrp.GetUnsigned("HighlightColor", highlight);
         highlightColor.setPackedValue((uint32_t)highlight, transparency);
@@ -237,7 +250,7 @@ void View3DInventor::OnChange(ParameterGrp::SubjectType &rCaller,ParameterGrp::M
     }
     else if (strcmp(Reason,"SelectionColor") == 0) {
         float transparency;
-        SbColor selectionColor(0.1f, 0.5f, 0.1f);
+        SbColor selectionColor(0.1f, 0.8f, 0.1f);
         unsigned long selection = (unsigned long)(selectionColor.getPackedValue());
         selection = rGrp.GetUnsigned("SelectionColor", selection);
         selectionColor.setPackedValue((uint32_t)selection, transparency);
@@ -251,6 +264,10 @@ void View3DInventor::OnChange(ParameterGrp::SubjectType &rCaller,ParameterGrp::M
         Base::Type type = Base::Type::fromName(model.c_str());
         _viewer->setNavigationType(type);
     }
+    else if (strcmp(Reason,"InvertZoom") == 0) {
+        bool on = rGrp.GetBool("InvertZoom", false);
+        _viewer->navigationStyle()->setZoomInverted(on);
+    }
     else if (strcmp(Reason,"EyeDistance") == 0) {
         _viewer->setStereoOffset(rGrp.GetFloat("EyeDistance",65.0));
     }
@@ -277,10 +294,10 @@ void View3DInventor::OnChange(ParameterGrp::SubjectType &rCaller,ParameterGrp::M
             _viewer->setCameraType(SoPerspectiveCamera::getClassTypeId());
     }
     else {
-        unsigned long col1 = rGrp.GetUnsigned("BackgroundColor",0);
-        unsigned long col2 = rGrp.GetUnsigned("BackgroundColor2",2139082239); // default color (purple)
-        unsigned long col3 = rGrp.GetUnsigned("BackgroundColor3",ULONG_MAX); // default color (white)
-        unsigned long col4 = rGrp.GetUnsigned("BackgroundColor4",ULONG_MAX); // default color (white)
+        unsigned long col1 = rGrp.GetUnsigned("BackgroundColor",3940932863UL);
+        unsigned long col2 = rGrp.GetUnsigned("BackgroundColor2",859006463UL); // default color (dark blue)
+        unsigned long col3 = rGrp.GetUnsigned("BackgroundColor3",2880160255UL); // default color (blue/grey)
+        unsigned long col4 = rGrp.GetUnsigned("BackgroundColor4",1869583359UL); // default color (blue/grey)
         float r1,g1,b1,r2,g2,b2,r3,g3,b3,r4,g4,b4;
         r1 = ((col1 >> 24) & 0xff) / 255.0; g1 = ((col1 >> 16) & 0xff) / 255.0; b1 = ((col1 >> 8) & 0xff) / 255.0;
         r2 = ((col2 >> 24) & 0xff) / 255.0; g2 = ((col2 >> 16) & 0xff) / 255.0; b2 = ((col2 >> 8) & 0xff) / 255.0;
@@ -327,6 +344,7 @@ void View3DInventor::print()
     printer.setFullPage(true);
     QPrintDialog dlg(&printer, this);
     if (dlg.exec() == QDialog::Accepted) {
+        Gui::WaitCursor wc;
         print(&printer);
     }
 }
@@ -335,6 +353,7 @@ void View3DInventor::printPdf()
 {
     QString filename = QFileDialog::getSaveFileName(this, tr("Export PDF"), QString(), tr("PDF file (*.pdf)"));
     if (!filename.isEmpty()) {
+        Gui::WaitCursor wc;
         QPrinter printer(QPrinter::HighResolution);
         printer.setOutputFormat(QPrinter::PdfFormat);
         printer.setOutputFileName(filename);
@@ -451,7 +470,7 @@ bool View3DInventor::onMsg(const char* pMsg, const char** ppReturn)
         return true;
     }
     else if(strcmp("ViewLeft",pMsg) == 0 ) {
-        _viewer->setCameraOrientation(SbRotation(0.5, 0.5, 0.5, 0.5));
+        _viewer->setCameraOrientation(SbRotation(-0.5, 0.5, 0.5, -0.5));
         _viewer->viewAll();
         return true;
     }
@@ -462,7 +481,7 @@ bool View3DInventor::onMsg(const char* pMsg, const char** ppReturn)
         return true;
     }
     else if(strcmp("ViewRight",pMsg) == 0 ) {
-        _viewer->setCameraOrientation(SbRotation(-0.5, 0.5, 0.5, -0.5));
+        _viewer->setCameraOrientation(SbRotation(0.5, 0.5, 0.5, 0.5));
         _viewer->viewAll();
         return true;
     }
@@ -473,7 +492,7 @@ bool View3DInventor::onMsg(const char* pMsg, const char** ppReturn)
     }
     else if(strcmp("ViewAxo",pMsg) == 0 ) {
         _viewer->setCameraOrientation(SbRotation
-            (-0.353553f, -0.146447f, -0.353553f, 0.853553f));
+            (-0.353553f, -0.146447f, -0.353553f, -0.853553f));
         _viewer->viewAll();
         return true;
     }
@@ -633,11 +652,33 @@ bool View3DInventor::hasClippingPlane() const
     return _viewer->hasClippingPlane();
 }
 
+void View3DInventor::setOverlayWidget(GLOverlayWidget* widget)
+{
+    removeOverlayWidget();
+    QGLWidget* w = static_cast<QGLWidget*>(_viewer->getGLWidget());
+    QImage img = w->grabFrameBuffer();
+    widget->setImage(img);
+    stack->addWidget(widget);
+    stack->setCurrentIndex(1);
+}
+
+void View3DInventor::removeOverlayWidget()
+{
+    stack->setCurrentIndex(0);
+    QWidget* overlay = stack->widget(1);
+    if (overlay) stack->removeWidget(overlay);
+}
+
 void View3DInventor::setCursor(const QCursor& aCursor)
 {
     _viewer->getWidget()->setCursor(aCursor);
 }
 
+void View3DInventor::setCursor(Qt::CursorShape aCursor)
+{
+    _viewer->getWidget()->setCursor(aCursor);
+}
+
 void View3DInventor::dump(const char* filename)
 {
     SoGetPrimitiveCountAction action;
@@ -816,4 +857,5 @@ void View3DInventor::customEvent(QEvent * e)
     }
 }
 
+
 #include "moc_View3DInventor.cpp"
