@@ -34,7 +34,6 @@
 # include <GL/gl.h>
 # endif
 # include <Inventor/SbBox.h>
-# include <Inventor/SbTesselator.h>
 # include <Inventor/actions/SoGetBoundingBoxAction.h>
 # include <Inventor/actions/SoHandleEventAction.h> 
 # include <Inventor/actions/SoToVRML2Action.h>
@@ -74,6 +73,7 @@
 # include <Inventor/SoPickedPoint.h>
 # include <Inventor/VRMLnodes/SoVRMLGroup.h>
 # include <QEventLoop>
+# include <QKeyEvent>
 # include <QMessageBox>
 # include <QTimer>
 #endif
@@ -87,6 +87,7 @@
 #include <zipios++/gzipoutputstream.h>
 
 #include "View3DInventorViewer.h"
+#include "ViewProviderDocumentObject.h"
 #include "SoFCBackgroundGradient.h"
 #include "SoFCColorBar.h"
 #include "SoFCColorLegend.h"
@@ -94,6 +95,7 @@
 #include "SoFCOffscreenRenderer.h"
 #include "SoFCSelection.h"
 #include "SoFCInteractiveElement.h"
+#include "SoFCBoundingBox.h"
 #include "Selection.h"
 #include "SoFCSelectionAction.h"
 #include "SoFCVectorizeU3DAction.h"
@@ -103,6 +105,9 @@
 #include "NavigationStyle.h"
 #include "ViewProvider.h"
 
+#include <Inventor/draggers/SoCenterballDragger.h>
+
+
 //#define FC_LOGGING_CB
 
 #define new DEBUG_CLIENTBLOCK
@@ -110,11 +115,13 @@
 using namespace Gui;
 
 /** \defgroup View3D 3D Viewer
+ *  \ingroup GUI
  *
  * The 3D Viewer is one of the major components in a CAD/CAE systems.
  * Therefore an overview and some remarks to the FreeCAD 3D viewing system.
  * 
  * \section overview Overview
+ * \todo Overview and complements for the 3D Viewer
  */
 
 // *************************************************************************
@@ -125,13 +132,9 @@ SOQT_OBJECT_ABSTRACT_SOURCE(View3DInventorViewer);
 
 View3DInventorViewer::View3DInventorViewer (QWidget *parent, const char *name, 
                                             SbBool embed, Type type, SbBool build) 
-  : inherited (parent, name, embed, type, build), inEdit(0),navigation(0),
+  : inherited (parent, name, embed, type, build), editViewProvider(0),navigation(0),
     editing(FALSE), redirected(FALSE)
 {
-    // set the layout for the flags
-    _flaglayout = new FlagLayout(3);
-    this->getGLWidget()->setLayout(_flaglayout);
-  
     Gui::Selection().Attach(this);
 
     // Coin should not clear the pixel-buffer, so the background image
@@ -180,15 +183,31 @@ View3DInventorViewer::View3DInventorViewer (QWidget *parent, const char *name,
     cam->nearDistance = 0;
     cam->farDistance = 10;
 
+	// dragger
+	//SoSeparator * draggSep = new SoSeparator();
+	//SoScale *scale = new SoScale();
+	//scale->scaleFactor = SbVec3f  (0.2,0.2,0.2);
+	//draggSep->addChild(scale);
+	//SoCenterballDragger *dragger = new SoCenterballDragger();
+	//dragger->center = SbVec3f  (0.8,0.8,0);
+	////dragger->rotation = SbRotation(rrot[0],rrot[1],rrot[2],rrot[3]);
+	//draggSep->addChild(dragger);
+
+
     this->foregroundroot->addChild(cam);
     this->foregroundroot->addChild(lm);
     this->foregroundroot->addChild(bc);
+    //this->foregroundroot->addChild(draggSep);
 
     // set the ViewProvider root
-    SoSelection* selectionRoot = new SoSelection();
-    selectionRoot->addSelectionCallback(View3DInventorViewer::selectCB, this);
-    selectionRoot->addDeselectionCallback(View3DInventorViewer::deselectCB, this);
-    pcViewProviderRoot = selectionRoot;
+    //SoSelection* selectionRoot = new SoSelection();
+    //selectionRoot->addSelectionCallback(View3DInventorViewer::selectCB, this);
+    //selectionRoot->addDeselectionCallback(View3DInventorViewer::deselectCB, this);
+    //pcViewProviderRoot = selectionRoot;
+    // NOTE: For every mouse click event the SoSelection searches for the picked
+    // point which causes a certain slow-down because for all objects the primitives
+    // must be created. Using an SoSeparator avoids this drawback.
+    pcViewProviderRoot = new SoSeparator();
 
     // increase refcount before passing it to setScenegraph(), to avoid
     // premature destruction
@@ -210,6 +229,10 @@ View3DInventorViewer::View3DInventorViewer (QWidget *parent, const char *name,
     pcViewProviderRoot->addChild(cb);
 #endif
 
+    // Set our own render action which show a bounding box if
+    // the SoFCSelection::BOX style is set
+    this->setGLRenderAction(new SoBoxSelectionRenderAction);
+
     // set the transperency and antialiasing settings
 //  getGLRenderAction()->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_BLEND);
     getGLRenderAction()->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_SORTED_TRIANGLE_BLEND);
@@ -217,7 +240,7 @@ View3DInventorViewer::View3DInventorViewer (QWidget *parent, const char *name,
 
     // Settings
     setSeekTime(0.4f);
-    if ( isSeekValuePercentage() == false )
+    if (isSeekValuePercentage() == false)
         setSeekValueAsPercentage(true);
     setSeekDistance(100);
     setViewing(false);
@@ -265,6 +288,7 @@ void View3DInventorViewer::initialize()
     this->axiscrossSize = 10;
 }
 
+/// @cond DOXERR
 void View3DInventorViewer::OnChange(Gui::SelectionSingleton::SubjectType &rCaller,
                                     Gui::SelectionSingleton::MessageType Reason)
 {
@@ -276,6 +300,12 @@ void View3DInventorViewer::OnChange(Gui::SelectionSingleton::SubjectType &rCalle
         cAct.apply(pcViewProviderRoot);
     }
 }
+/// @endcond
+
+SbBool View3DInventorViewer::hasViewProvider(ViewProvider* pcProvider) const
+{
+    return _ViewProviderSet.find(pcProvider) != _ViewProviderSet.end();
+}
 
 /// adds an ViewProvider to the view, e.g. from a feature
 void View3DInventorViewer::addViewProvider(ViewProvider* pcProvider)
@@ -292,8 +322,8 @@ void View3DInventorViewer::addViewProvider(ViewProvider* pcProvider)
 
 void View3DInventorViewer::removeViewProvider(ViewProvider* pcProvider)
 {
-    if (this->inEdit == pcProvider)
-        resetEdit();
+    if (this->editViewProvider == pcProvider)
+        resetEditingViewProvider();
 
     SoSeparator* root = pcProvider->getRoot();
     if (root) pcViewProviderRoot->removeChild(root);
@@ -305,31 +335,37 @@ void View3DInventorViewer::removeViewProvider(ViewProvider* pcProvider)
     _ViewProviderSet.erase(pcProvider);
 }
 
-bool View3DInventorViewer::setEdit(Gui::ViewProvider* p, int ModNum)
+SbBool View3DInventorViewer::setEditingViewProvider(Gui::ViewProvider* p, int ModNum)
 {
     if (_ViewProviderSet.find(p) == _ViewProviderSet.end())
         return false;
-    if (this->inEdit)
+    if (this->editViewProvider)
         return false; // only one view provider is editable at a time
-    bool ok = p->setEdit(ModNum);
+    bool ok = p->startEditing(ModNum);
     if (ok) {
-        this->inEdit = p;
-        addEventCallback(SoEvent::getClassTypeId(), Gui::ViewProvider::eventCallback,this->inEdit);
+        this->editViewProvider = p;
+        addEventCallback(SoEvent::getClassTypeId(), Gui::ViewProvider::eventCallback,this->editViewProvider);
     }
 
     return ok;
 }
 
 /// reset from edit mode
-void View3DInventorViewer::resetEdit(void)
+void View3DInventorViewer::resetEditingViewProvider()
 {
-    if (this->inEdit){
-        this->inEdit->unsetEdit();
-        removeEventCallback(SoEvent::getClassTypeId(), Gui::ViewProvider::eventCallback,this->inEdit);
-        this->inEdit = 0;
+    if (this->editViewProvider){
+        this->editViewProvider->finishEditing();
+        removeEventCallback(SoEvent::getClassTypeId(), Gui::ViewProvider::eventCallback,this->editViewProvider);
+        this->editViewProvider = 0;
     }
 }
 
+/// reset from edit mode
+SbBool View3DInventorViewer::isEditingViewProvider() const
+{
+    return this->editViewProvider ? true : false;
+}
+
 void View3DInventorViewer::clearBuffer(void * userdata, SoAction * action)
 {
     if (action->isOfType(SoGLRenderAction::getClassTypeId())) {
@@ -381,7 +417,7 @@ void View3DInventorViewer::setGradientBackgroudColor(const SbColor& fromColor,
 
 void View3DInventorViewer::setEnabledFPSCounter(bool on)
 {
-#if defined (FC_OS_LINUX) || defined(FC_OS_CYGWIN) || defined(FC_OS_MACOSX)
+#if defined (FC_OS_LINUX) || defined(FC_OS_CYGWIN) || defined(FC_OS_MACOSX) || defined(FC_OS_BSD)
     setenv("COIN_SHOW_FPS_COUNTER", (on?"1":"0"), 1);
 #else
     on ? _putenv ("COIN_SHOW_FPS_COUNTER=1") : _putenv ("COIN_SHOW_FPS_COUNTER=0");
@@ -644,56 +680,64 @@ void View3DInventorViewer::saveGraphic(int pagesize, int eBackgroundType, SoVect
     va->endPage();
 }
 
-void View3DInventorViewer::startPicking(View3DInventorViewer::ePickMode mode)
+void View3DInventorViewer::startSelection(View3DInventorViewer::SelectionMode mode)
 {
-    navigation->startPicking(NavigationStyle::ePickMode(mode));
+    navigation->startSelection(NavigationStyle::SelectionMode(mode));
 }
 
-void View3DInventorViewer::stopPicking()
+void View3DInventorViewer::stopSelection()
 {
-    navigation->stopPicking();
+    navigation->stopSelection();
 }
 
-bool View3DInventorViewer::isPicking() const
+bool View3DInventorViewer::isSelecting() const
 {
-    return navigation->isPicking();
+    return navigation->isSelecting();
 }
 
-const std::vector<SbVec2f>& View3DInventorViewer::getPickedPolygon(SbBool* clip_inner) const
+const std::vector<SbVec2s>& View3DInventorViewer::getPolygon(SbBool* clip_inner) const
 {
-    return navigation->getPickedPolygon(clip_inner);
+    return navigation->getPolygon(clip_inner);
 }
 
-void View3DInventorViewer::tessCB(void * v0, void * v1, void * v2, void * cbdata)
+std::vector<SbVec2f> View3DInventorViewer::getGLPolygon(const std::vector<SbVec2s>& pnts) const
 {
-    int * vtx0 = (int *)v0; 
-    int * vtx1 = (int *)v1; 
-    int * vtx2 = (int *)v2;
+    const SbViewportRegion& vp = this->getViewportRegion();
+    const SbVec2s& sz = vp.getWindowSize();
+    short w,h; sz.getValue(w,h);
+    const SbVec2s& sp = vp.getViewportSizePixels();
+    const SbVec2s& op = vp.getViewportOriginPixels();
+    const SbVec2f& siz = vp.getViewportSize();
+    float dX, dY; siz.getValue(dX, dY);
+    float fRatio = vp.getViewportAspectRatio();
 
-    std::vector<int>* array = (std::vector<int> *)cbdata;
-    array->push_back(*vtx0);
-    array->push_back(*vtx1);
-    array->push_back(*vtx2);
-    array->push_back(-1);
-}
+    std::vector<SbVec2f> poly;
+    for (std::vector<SbVec2s>::const_iterator it = pnts.begin(); it != pnts.end(); ++it) {
+        SbVec2s loc = *it - op;
+        SbVec2f pos((float)loc[0]/(float)sp[0], (float)loc[1]/(float)sp[1]);
+        float pX,pY; pos.getValue(pX,pY);
 
-std::vector<int> View3DInventorViewer::tessellate(const std::vector<SbVec2f>& polygon) const
-{
-    std::vector<int> indices(polygon.size());
-    std::vector<int> face_indices;
+        // now calculate the real points respecting aspect ratio information
+        //
+        if (fRatio > 1.0f) {
+            pX = (pX - 0.5f*dX) * fRatio + 0.5f*dX;
+            pos.setValue(pX,pY);
+        }
+        else if (fRatio < 1.0f) {
+            pY = (pY - 0.5f*dY) / fRatio + 0.5f*dY;
+            pos.setValue(pX,pY);
+        }
 
-    SbTesselator tessellator(tessCB, &face_indices);
-    tessellator.beginPolygon();
-
-    int index = 0;
-    for (std::vector<SbVec2f>::const_iterator it = polygon.begin(); it != polygon.end(); ++it, index++) {
-        indices[index] = index;
-        tessellator.addVertex(SbVec3f((*it)[0], (*it)[1], 0.0f), &(indices[index]));
+        poly.push_back(pos);
     }
 
-    // run the triangulation now
-    tessellator.endPolygon();
-    return face_indices;
+    return poly;
+}
+
+std::vector<SbVec2f> View3DInventorViewer::getGLPolygon(SbBool* clip_inner) const
+{
+    const std::vector<SbVec2s>& pnts = navigation->getPolygon(clip_inner);
+    return getGLPolygon(pnts);
 }
 
 bool View3DInventorViewer::dumpToFile(const char* filename, bool binary) const
@@ -856,16 +900,20 @@ void View3DInventorViewer::actualRedraw(void)
     glDepthRange(0.1,1.0);
 
     // draw lines for the flags
-    int ct = _flaglayout->count();
-    SbViewVolume vv = getCamera()->getViewVolume(getGLAspectRatio());
-    for (int i=0; i<ct;i++) {
-        Flag* flag = qobject_cast<Flag*>(_flaglayout->itemAt(i)->widget());
-        if (flag) {
-            SbVec3f pt = flag->getOrigin();
-            vv.projectToScreen(pt, pt);
-            int tox = (int)(pt[0] * size[0]);
-            int toy = (int)((1.0f-pt[1]) * size[1]);
-            flag->drawLine(tox, toy);
+    if (_flaglayout) {
+        // it can happen that the GL widget gets replaced internally by SoQt which
+        // causes to destroy the FlagLayout instance
+        int ct = _flaglayout->count();
+        SbViewVolume vv = getCamera()->getViewVolume(getGLAspectRatio());
+        for (int i=0; i<ct;i++) {
+            Flag* flag = qobject_cast<Flag*>(_flaglayout->itemAt(i)->widget());
+            if (flag) {
+                SbVec3f pt = flag->getOrigin();
+                vv.projectToScreen(pt, pt);
+                int tox = (int)(pt[0] * size[0]);
+                int toy = (int)((1.0f-pt[1]) * size[1]);
+                flag->drawLine(tox, toy);
+            }
         }
     }
 
@@ -943,6 +991,21 @@ void View3DInventorViewer::printDimension()
         getMainWindow()->setPaneText(2, QLatin1String(""));
 }
 
+void View3DInventorViewer::selectAll()
+{
+    std::vector<App::DocumentObject*> objs;
+    for (std::set<ViewProvider*>::iterator it = _ViewProviderSet.begin(); it != _ViewProviderSet.end(); ++it) {
+         if ((*it)->getTypeId().isDerivedFrom(ViewProviderDocumentObject::getClassTypeId())) {
+             ViewProviderDocumentObject* vp = static_cast<ViewProviderDocumentObject*>(*it);
+             App::DocumentObject* obj = vp->getObject();
+             if (obj) objs.push_back(obj);
+         }
+    }
+
+    if (!objs.empty())
+        Gui::Selection().setSelection(objs.front()->getDocument()->getName(), objs);
+}
+
 /*!
   As ProgressBar has no chance to control the incoming Qt events of SoQt we need to override
   SoQtViewer::processEvent() to prevent the scenegraph from being selected or deselected
@@ -950,6 +1013,13 @@ void View3DInventorViewer::printDimension()
  */
 void View3DInventorViewer::processEvent(QEvent * event)
 {
+    if (event->type() == QEvent::KeyPress) {
+        QKeyEvent* ke = static_cast<QKeyEvent*>(event);
+        if (ke->matches(QKeySequence::SelectAll)) {
+            selectAll();
+            return;
+        }
+    }
     if (!Base::Sequencer().isRunning() ||
         !Base::Sequencer().isBlocking())
         inherited::processEvent(event);
@@ -957,8 +1027,12 @@ void View3DInventorViewer::processEvent(QEvent * event)
 
 SbBool View3DInventorViewer::processSoEvent(const SoEvent * const ev)
 {
-    if (isRedirectedToSceneGraph())
-        return SoQtRenderArea::processSoEvent(ev);
+    if (isRedirectedToSceneGraph()) {
+        SbBool processed = SoQtRenderArea::processSoEvent(ev);
+        if (!processed)
+            processed = navigation->processEvent(ev);
+        return processed;
+    }
     if (ev->getTypeId().isDerivedFrom(SoKeyboardEvent::getClassTypeId())) {
         // filter out 'Q' and 'ESC' keys
         const SoKeyboardEvent * const ke = static_cast<const SoKeyboardEvent *>(ev);
@@ -1211,9 +1285,29 @@ void View3DInventorViewer::boxZoom(const SbBox2s& box)
 
 void View3DInventorViewer::viewAll()
 {
+    // in the scene graph we may have objects which we want to exlcude
+    // when doing a fit all. Such objects must be part of the group
+    // SoSkipBoundingGroup.
+    SoSearchAction sa;
+    sa.setType(SoSkipBoundingGroup::getClassTypeId());
+    sa.setInterest(SoSearchAction::ALL);
+    sa.apply(this->getSceneGraph());
+    const SoPathList & pathlist = sa.getPaths();
+    for (int i = 0; i < pathlist.getLength(); i++ ) {
+        SoPath * path = pathlist[i];
+        SoSkipBoundingGroup * group = static_cast<SoSkipBoundingGroup*>(path->getTail());
+        group->mode = SoSkipBoundingGroup::EXCLUDE_BBOX;
+    }
+
     // call the default implementation first to make sure everything is visible
     SoQtViewer::viewAll();
-    navigation->viewAll();
+
+    for (int i = 0; i < pathlist.getLength(); i++ ) {
+        SoPath * path = pathlist[i];
+        SoSkipBoundingGroup * group = static_cast<SoSkipBoundingGroup*>(path->getTail());
+        group->mode = SoSkipBoundingGroup::INCLUDE_BBOX;
+    }
+    //navigation->viewAll();
 }
 
 void View3DInventorViewer::viewSelection()
@@ -1726,6 +1820,17 @@ static unsigned char cross_mask_bitmap[] = {
 // Set cursor graphics according to mode.
 void View3DInventorViewer::setCursorRepresentation(int modearg)
 {
+    // There is a synchronization problem between Qt and SoQt which
+    // happens when popping up a context-menu. In this case the
+    // Qt::WA_UnderMouse attribute is resetted and never set again
+    // even if the mouse is still in the canvas. Thus, the cursor
+    // won't be changed as long as the user doesn't leave and enter
+    // the canvas. To fix this we explicitly set Qt::WA_UnderMouse
+    // if the mouse is inside the canvas.
+    QWidget* w = this->getGLWidget();
+    if (w && w->rect().contains(QCursor::pos()))
+        w->setAttribute(Qt::WA_UnderMouse);
+
     if (!this->isCursorEnabled()) {
         this->setComponentCursor(SoQtCursor::getBlankCursor());
         return;
@@ -1858,6 +1963,11 @@ std::vector<ViewProvider*> View3DInventorViewer::getViewProvidersOfType(const Ba
 
 void View3DInventorViewer::addFlag(Flag* item, FlagLayout::Position pos)
 {
+    if (!_flaglayout) {
+        _flaglayout = new FlagLayout(3);
+        this->getGLWidget()->setLayout(_flaglayout);
+    }
+
     item->setParent(this->getGLWidget());
     _flaglayout->addWidget(item, pos);
     item->show();
