@@ -34,6 +34,7 @@
 
 #include <Gui/Selection.h>
 #include <Gui/Flag.h>
+#include <QPointer>
 
 
 class SoSeparator;
@@ -49,7 +50,6 @@ namespace Gui {
 
 class ViewProvider;
 class SoFCBackgroundGradient;
-class AbstractMouseModel;
 class NavigationStyle;
 
 
@@ -69,7 +69,7 @@ public:
         Transparent = 3,  /**< Transparent background  */
     };
     /// Pick modes for picking points in the scene
-    enum ePickMode {
+    enum SelectionMode {
         Lasso       = 0,  /**< Select objects using a lasso. */
         Rectangle   = 1,  /**< Select objects using a rectangle. */
         BoxZoom     = 2,  /**< Perform a box zoom. */
@@ -121,6 +121,7 @@ public:
 
     /** @name Handling of view providers */
     //@{
+    SbBool hasViewProvider(ViewProvider*) const;
     /// adds an ViewProvider to the view, e.g. from a feature
     void addViewProvider(ViewProvider*);
     /// remove a ViewProvider
@@ -130,9 +131,11 @@ public:
     /// get all view providers of given type
     std::vector<ViewProvider*> getViewProvidersOfType(const Base::Type& typeId) const;
     /// set the ViewProvider in special edit mode
-    bool setEdit(Gui::ViewProvider* p, int ModNum=0);
+    SbBool setEditingViewProvider(Gui::ViewProvider* p, int ModNum=0);
+    /// return whether a view provider is edited
+    SbBool isEditingViewProvider() const;
     /// reset from edit mode
-    void resetEdit(void);
+    void resetEditingViewProvider();
     //@}
 
     /** @name Making pictures */
@@ -155,13 +158,14 @@ public:
      */
     bool dumpToFile(const char* filename, bool binary) const;
 
-    /** @name Picking methods */
+    /** @name Selection methods */
     //@{
-    void startPicking( ePickMode = Lasso );
-    void stopPicking();
-    bool isPicking() const;
-    const std::vector<SbVec2f>& getPickedPolygon(SbBool* clip_inner=0) const;
-    std::vector<int> tessellate(const std::vector<SbVec2f>&) const;
+    void startSelection(SelectionMode = Lasso);
+    void stopSelection();
+    bool isSelecting() const;
+    std::vector<SbVec2f> getGLPolygon(SbBool* clip_inner=0) const;
+    std::vector<SbVec2f> getGLPolygon(const std::vector<SbVec2s>&) const;
+    const std::vector<SbVec2s>& getPolygon(SbBool* clip_inner=0) const;
     //@}
 
     /** @name Edit methods */
@@ -264,6 +268,7 @@ protected:
     virtual SbBool processSoEvent(const SoEvent * const ev);
     SbBool processSoEventBase(const SoEvent * const ev);
     void printDimension();
+    void selectAll();
 
     static void clearBuffer(void * userdata, SoAction * action);
     static void setGLWidget(void * userdata, SoAction * action);
@@ -275,11 +280,10 @@ protected:
 private:
     static void selectCB(void * closure, SoPath * p);
     static void deselectCB(void * closure, SoPath * p);
-    static void tessCB(void * v0, void * v1, void * v2, void * cbdata);
 
 private:
     std::set<ViewProvider*> _ViewProviderSet;
-    ViewProvider* inEdit;
+    ViewProvider* editViewProvider;
     SoFCBackgroundGradient *pcBackGround;
     SoSeparator * backgroundroot;
     SoSeparator * foregroundroot;
@@ -307,7 +311,7 @@ public:
     void addFlag(Flag*, FlagLayout::Position);
 
 private:
-    FlagLayout* _flaglayout;
+    QPointer<FlagLayout> _flaglayout;
 
     // friends
     friend class NavigationStyle;
