@@ -36,6 +36,7 @@
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
 #include <Base/Console.h>
+#include <Base/Exception.h>
 #include <Base/Matrix.h>
 #include <App/PropertyGeo.h>
 
@@ -60,7 +61,7 @@ using namespace Gui;
 PROPERTY_SOURCE_ABSTRACT(Gui::ViewProvider, App::PropertyContainer)
 
 ViewProvider::ViewProvider() 
-  : pcAnnotation(0), pyViewObject(0), _iActualMode(-1), _updateData(true)
+  : pcAnnotation(0), pyViewObject(0), _iActualMode(-1), _iEditMode(-1), _updateData(true)
 {
     pcRoot = new SoSeparator();
     pcRoot->ref();
@@ -88,12 +89,35 @@ ViewProvider::~ViewProvider()
         pcAnnotation->unref();
 }
 
+bool ViewProvider::startEditing(int ModNum)
+{
+    bool ok = setEdit(ModNum);
+    if (ok) _iEditMode = ModNum;
+    return ok;
+}
+
+int ViewProvider::getEditingMode() const
+{
+    return _iEditMode;
+}
+
+bool ViewProvider::isEditing() const
+{
+    return getEditingMode() > -1;
+}
+
+void ViewProvider::finishEditing()
+{
+    unsetEdit(_iEditMode);
+    _iEditMode = -1;
+}
+
 bool ViewProvider::setEdit(int ModNum)
 {
     return true;
 }
 
-void ViewProvider::unsetEdit(void)
+void ViewProvider::unsetEdit(int ModNum)
 {
 }
 
@@ -133,40 +157,47 @@ void ViewProvider::eventCallback(void * ud, SoEventCallback * node)
 
     SoPickedPoint* pp = rp.getPickedPoint();
 
-    // Keyboard events
-    if (ev->getTypeId().isDerivedFrom(SoKeyboardEvent::getClassTypeId())) {
-        SoKeyboardEvent * ke = (SoKeyboardEvent *)ev;
-        switch (ke->getKey()) {
-        case SoKeyboardEvent::ESCAPE:
-            Gui::Application::Instance->activeDocument()->resetEdit();
-            break;
-        default:
+    try {
+        // Keyboard events
+        if (ev->getTypeId().isDerivedFrom(SoKeyboardEvent::getClassTypeId())) {
+            SoKeyboardEvent * ke = (SoKeyboardEvent *)ev;
+            switch (ke->getKey()) {
+            case SoKeyboardEvent::ESCAPE:
+                Gui::Application::Instance->activeDocument()->resetEdit();
+                break;
+            default:
+                // call the virtual method
+                if (self->keyPressed (ke->getKey()))
+                    node->setHandled();
+                break;
+            }
+        }
+        // switching the mouse buttons
+        else if (ev->getTypeId().isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
+
+            const SoMouseButtonEvent * const event = (const SoMouseButtonEvent *) ev;
+            const int button = event->getButton();
+            const SbBool press = event->getState() == SoButtonEvent::DOWN ? TRUE : FALSE;
+
             // call the virtual method
-            if (self->keyPressed (ke->getKey()))
+            if (self->mouseButtonPressed(button,press,point,norm,pp))
+                node->setHandled();
+        }
+        // Mouse Movement handling
+        else if (ev->getTypeId().isDerivedFrom(SoLocation2Event::getClassTypeId())) {
+            if (self->mouseMove(point,norm,pp))
                 node->setHandled();
-            break;
         }
     }
-    // switching the mouse buttons
-    else if (ev->getTypeId().isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
-
-        const SoMouseButtonEvent * const event = (const SoMouseButtonEvent *) ev;
-        const int button = event->getButton();
-        const SbBool press = event->getState() == SoButtonEvent::DOWN ? TRUE : FALSE;
-
-        // call the virtual method
-        if (self->mouseButtonPressed(button,press,point,norm,pp))
-            node->setHandled();
+    catch (const Base::Exception& e) {
+        Base::Console().Error("Unhandled exception in ViewProvider::eventCallback: %s\n", e.what());
     }
-    // Mouse Movement handling
-    else if (ev->getTypeId().isDerivedFrom(SoLocation2Event::getClassTypeId())) {
-        if (self->mouseMove(point,norm,pp))
-            node->setHandled();
+    catch (const std::exception& e) {
+        Base::Console().Error("Unhandled std exception in ViewProvider::eventCallback: %s\n", e.what());
+    }
+    catch (...) {
+        Base::Console().Error("Unhandled unknown C++ exception in ViewProvider::eventCallback");
     }
-
-    // clean up
-    //if (Point)
-    //    delete Point;
 }
 
 SoSeparator* ViewProvider::getAnnotation(void)
@@ -247,8 +278,8 @@ std::vector<std::string> ViewProvider::getDisplayMaskModes() const
 }
 
 /**
- * If you add new viewing modes in @ref getModes() then you need to reimplement
- * also setMode() to handle these new modes by setting the appropriate display
+ * If you add new viewing modes in @ref getDisplayModes() then you need to reimplement
+ * also seDisplaytMode() to handle these new modes by setting the appropriate display
  * mode.
  */
 void ViewProvider::setDisplayMode(const char* ModeName)
