@@ -41,6 +41,8 @@ class SbMatrix;
 class SoEventCallback;
 class SoPickedPoint;
 class QString;
+class QMenu;
+class QObject;
 
 
 namespace Base {
@@ -62,7 +64,13 @@ class ViewProviderPy;
 class ObjectItem;
 
 
-
+/** General interface for all visual stuff in FreeCAD
+  * This class is used to generate and handle all around 
+  * visualizing and presenting objects from the FreeCAD 
+  * App layer to the user. This class and its descendents 
+  * have to be implemented for any object type in order to 
+  * show them in the 3DView and TreeView.
+  */
 class GuiExport ViewProvider : public App::PropertyContainer
 {
     PROPERTY_HEADER(Gui::ViewProvider);
@@ -84,10 +92,23 @@ public:
     virtual SoSeparator* getBackRoot(void) const {return 0;}
     virtual void select(SoPath*) {}
     virtual void deselect(SoPath*) {}
+
     /** @name Methods used by the Tree
+      * If you want to take control over the 
+      * appearance of your object in the tree you
+      * can reimplemnt this methods.
      */
     //@{
+    /// deliver the icon shown in the tree view
     virtual QIcon getIcon(void) const;
+    /** deliver the children belonging to this object
+      * this method is used to deliver the objects to 
+      * the tree framework which should be grouped under its 
+      * label. Obvious is the usage in the group but it can
+      * be used for any kind of grouping needed for a special 
+      * purpose.
+      */
+    virtual std::vector<App::DocumentObject*> claimChildren(void)const{return std::vector<App::DocumentObject*>();}
     //@}
 
     /** @name Signals of the view provider */
@@ -139,11 +160,21 @@ public:
      * you can handle most of the events in the viewer by yourself
      */
     //@{
-    //virtual bool edit(void){return false;}
+protected:
+    enum EditMode {Default = 0,
+                   Transform = 1
+    };
     /// is called by the document when the provider goes in edit mode
-    virtual bool setEdit(int ModNum = 0);
+    virtual bool setEdit(int ModNum);
     /// is called when you loose the edit mode
-    virtual void unsetEdit(void);
+    virtual void unsetEdit(int ModNum);
+    /// return the edit mode or -1 if nothing is being edited
+    int getEditingMode() const;
+
+public:
+    bool startEditing(int ModNum = 0);
+    bool isEditing() const;
+    void finishEditing();
     //@}
 
     /** @name Task panel 
@@ -166,9 +197,8 @@ public:
     virtual bool mouseButtonPressed(int Button, bool pressed, const SbVec3f &pos,
                                     const SbVec3f &norm, const SoPickedPoint* pp)
     { return false; }
-
-    //virtual const char* getEditModeName(void){return 0;}
-
+    /// set up the context-menu with the supported edit modes
+    virtual void setupContextMenu(QMenu*, QObject*, const char*) {}
 
     /** @name direct handling methods
      *  This group of methods is to direct influence the 
@@ -224,6 +254,7 @@ protected:
 
 private:
     int _iActualMode;
+    int _iEditMode;
     std::string _sCurrentMode;
     std::map<std::string, int> _sDisplayMaskModes;
     bool _updateData;
