@@ -31,7 +31,10 @@ class SoText2;
 class SoAsciiText;
 class SoBaseColor;
 class SoTranslation;
+class SoTransform;
 class SoRotationXYZ;
+class SoImage;
+class SoCoordinate3;
 
 namespace Gui
 {
@@ -50,6 +53,7 @@ public:
     App::PropertyEnumeration    Justification;
     App::PropertyFloat          FontSize;
     App::PropertyString         FontName;
+    App::PropertyFloat          LineSpacing;
     App::PropertyAngle          Rotation;
     App::PropertyEnumeration    RotationAxis;
 
@@ -73,6 +77,61 @@ private:
     static const char* RotationAxisEnums[];
 };
 
+/**
+ * This is a different implementation of an annotation object which uses an
+ * SoImage node instead of an SoText2 or SoAsciiText node.
+ * This approach gives a bit more flexibility since it can render arbitrary
+ * annotations.
+ */
+class GuiExport ViewProviderAnnotationLabel : public ViewProviderDocumentObject
+{
+    PROPERTY_HEADER(Gui::ViewProviderAnnotationLabel);
+
+public:
+    /// Constructor
+    ViewProviderAnnotationLabel(void);
+    virtual ~ViewProviderAnnotationLabel();
+
+    // Display properties
+    App::PropertyColor          TextColor;
+    App::PropertyColor          BackgroundColor;
+    App::PropertyEnumeration    Justification;
+    App::PropertyFloat          FontSize;
+    App::PropertyFont           FontName;
+
+    void attach(App::DocumentObject *);
+    void updateData(const App::Property*);
+    std::vector<std::string> getDisplayModes(void) const;
+    void setDisplayMode(const char* ModeName);
+
+    /** @name Edit methods */
+    //@{
+    bool doubleClicked(void);
+    void setupContextMenu(QMenu*, QObject*, const char*);
+protected:
+    bool setEdit(int ModNum);
+    void unsetEdit(int ModNum);
+    //@}
+
+protected:
+    void onChanged(const App::Property* prop);
+    void drawImage(const std::vector<std::string>&);
+
+private:
+    static void dragStartCallback(void * data, SoDragger * d);
+    static void dragFinishCallback(void * data, SoDragger * d);
+    static void dragMotionCallback(void * data, SoDragger * d);
+
+private:
+    SoCoordinate3    * pCoords;
+    SoImage          * pImage;
+    SoBaseColor      * pColor;
+    SoTranslation    * pBaseTranslation;
+    SoTransform      * pTextTranslation;
+
+    static const char* JustificationEnums[];
+};
+
 } //namespace Gui
 
 
