@@ -31,6 +31,7 @@
 
 class SoMaterial;
 class SoDrawStyle;
+class SoType;
 
 namespace App
 {
@@ -80,6 +81,12 @@ public:
     /// Get the python wrapper for that ViewProvider
     PyObject* getPyObject();
 
+    /** @name Restoring view provider from document load */
+    //@{
+    virtual void startRestoring();
+    virtual void finishRestoring();
+    //@}
+
 protected:
     /// Gets called by the container whenever a property has been changed
     virtual void onChanged(const App::Property* prop);
@@ -90,7 +97,7 @@ protected:
      * to an object. The method returns after the first front root node 
      * matches. If no front root node matches, 0 is returned.
      */
-    SoSeparator* findFrontRootOfType(const SoType& type) const;
+    SoNode* findFrontRootOfType(const SoType& type) const;
 
 protected:
     App::DocumentObject *pcObject;
