@@ -24,6 +24,7 @@
 #include "PreCompiled.h"
 
 #ifndef _PreComp_
+# include <sstream>
 # include <QPixmap>
 # include <Inventor/nodes/SoDrawStyle.h>
 # include <Inventor/nodes/SoMaterial.h>
@@ -52,6 +53,7 @@
 #include "Application.h"
 #include "BitmapFactory.h"
 #include "Document.h"
+#include <App/DocumentObjectPy.h>
 #include <App/GeoFeature.h>
 #include <App/PropertyGeo.h>
 #include <Base/Console.h>
@@ -74,9 +76,9 @@ ViewProviderPythonFeatureImp::~ViewProviderPythonFeatureImp()
 QIcon ViewProviderPythonFeatureImp::getIcon() const
 {
     // default icon
-    static QPixmap px = BitmapFactory().pixmap("Tree_Python");
+    //static QPixmap px = BitmapFactory().pixmap("Tree_Python");
 
-    // Run the getDisplayModes method of the proxy object.
+    // Run the getIcon method of the proxy object.
     Base::PyGILStateLocker lock;
     try {
         App::Property* proxy = object->getPropertyByName("Proxy");
@@ -107,12 +109,42 @@ QIcon ViewProviderPythonFeatureImp::getIcon() const
         Base::Console().Error("ViewProviderPythonFeature::getIcon: %s\n", e.what());
     }
 
-    return px;
+    return QIcon();
+}
+
+std::vector<App::DocumentObject*> ViewProviderPythonFeatureImp::claimChildren() const 
+{
+    std::vector<App::DocumentObject*> children;
+    Base::PyGILStateLocker lock;
+    try {
+        App::Property* proxy = object->getPropertyByName("Proxy");
+        if (proxy && proxy->getTypeId() == App::PropertyPythonObject::getClassTypeId()) {
+            Py::Object vp = static_cast<App::PropertyPythonObject*>(proxy)->getValue();
+            if (vp.hasAttr(std::string("claimChildren"))) {
+                Py::Callable method(vp.getAttr(std::string("claimChildren")));
+                Py::Tuple args(0);
+                Py::List list(method.apply(args));
+                for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+                    PyObject* item = (*it).ptr();
+                    if (PyObject_TypeCheck(item, &(App::DocumentObjectPy::Type))) {
+                        App::DocumentObject* obj = static_cast<App::DocumentObjectPy*>(item)->getDocumentObjectPtr();
+                        children.push_back(obj);
+                    }
+                }
+            }
+        }
+    }
+    catch (Py::Exception&) {
+        Base::PyException e; // extract the Python error text
+        Base::Console().Error("ViewProviderPythonFeature::claimChildren: %s\n", e.what());
+    }
+
+    return children;
 }
 
 void ViewProviderPythonFeatureImp::attach(App::DocumentObject *pcObject)
 {
-    // Run the getDisplayModes method of the proxy object.
+    // Run the attach method of the proxy object.
     Base::PyGILStateLocker lock;
     try {
         App::Property* proxy = object->getPropertyByName("Proxy");
@@ -128,7 +160,8 @@ void ViewProviderPythonFeatureImp::attach(App::DocumentObject *pcObject)
     }
     catch (Py::Exception&) {
         Base::PyException e; // extract the Python error text
-        Base::Console().Error("ViewProviderPythonFeature::attach: %s\n", e.what());
+        const char* name = object->getObject()->Label.getValue();
+        Base::Console().Error("ViewProviderPythonFeature::attach (%s): %s\n", name, e.what());
     }
 }
 
@@ -154,13 +187,14 @@ void ViewProviderPythonFeatureImp::updateData(const App::Property* prop)
     }
     catch (Py::Exception&) {
         Base::PyException e; // extract the Python error text
-        Base::Console().Error("ViewProviderPythonFeature::updateData: %s\n", e.what());
+        const char* name = object->getObject()->Label.getValue();
+        Base::Console().Error("ViewProviderPythonFeature::updateData (%s): %s\n", name, e.what());
     }
 }
 
 void ViewProviderPythonFeatureImp::onChanged(const App::Property* prop)
 {
-    // Run the execute method of the proxy object.
+    // Run the onChanged method of the proxy object.
     Base::PyGILStateLocker lock;
     try {
         App::Property* proxy = object->getPropertyByName("Proxy");
@@ -178,13 +212,30 @@ void ViewProviderPythonFeatureImp::onChanged(const App::Property* prop)
     }
     catch (Py::Exception&) {
         Base::PyException e; // extract the Python error text
-        Base::Console().Error("ViewProviderPythonFeature::onChanged: %s\n", e.what());
+        const char* name = object->getObject()->Label.getValue();
+        Base::Console().Error("ViewProviderPythonFeature::onChanged (%s): %s\n", name, e.what());
+    }
+}
+
+void ViewProviderPythonFeatureImp::startRestoring()
+{
+}
+
+void ViewProviderPythonFeatureImp::finishRestoring()
+{
+    App::Property* proxy = object->getPropertyByName("Proxy");
+    if (proxy && proxy->getTypeId() == App::PropertyPythonObject::getClassTypeId()) {
+        Py::Object vp = static_cast<App::PropertyPythonObject*>(proxy)->getValue();
+        if (vp.ptr() == Py::_None()) {
+            object->show();
+            static_cast<App::PropertyPythonObject*>(proxy)->setValue(Py::Int(1));
+        }
     }
 }
 
 const char* ViewProviderPythonFeatureImp::getDefaultDisplayMode() const
 {
-    // Run the getDisplayModes method of the proxy object.
+    // Run the getDefaultDisplayMode method of the proxy object.
     Base::PyGILStateLocker lock;
     static std::string mode;
     try {
@@ -202,7 +253,8 @@ const char* ViewProviderPythonFeatureImp::getDefaultDisplayMode() const
     }
     catch (Py::Exception&) {
         Base::PyException e; // extract the Python error text
-        Base::Console().Error("ViewProviderPythonFeature::getDefaultDisplayMode: %s\n", e.what());
+        const char* name = object->getObject()->Label.getValue();
+        Base::Console().Error("ViewProviderPythonFeature::getDefaultDisplayMode (%s): %s\n", name, e.what());
     }
 
     return 0;
@@ -231,7 +283,8 @@ std::vector<std::string> ViewProviderPythonFeatureImp::getDisplayModes(void) con
     }
     catch (Py::Exception&) {
         Base::PyException e; // extract the Python error text
-        Base::Console().Error("ViewProviderPythonFeature::getDisplayModes: %s\n", e.what());
+        const char* name = object->getObject()->Label.getValue();
+        Base::Console().Error("ViewProviderPythonFeature::getDisplayModes (%s): %s\n", name, e.what());
     }
 
     return modes;
@@ -239,7 +292,7 @@ std::vector<std::string> ViewProviderPythonFeatureImp::getDisplayModes(void) con
 
 std::string ViewProviderPythonFeatureImp::setDisplayMode(const char* ModeName)
 {
-    // Run the getDisplayModes method of the proxy object.
+    // Run the setDisplayMode method of the proxy object.
     Base::PyGILStateLocker lock;
     try {
         App::Property* proxy = object->getPropertyByName("Proxy");
@@ -256,7 +309,8 @@ std::string ViewProviderPythonFeatureImp::setDisplayMode(const char* ModeName)
     }
     catch (Py::Exception&) {
         Base::PyException e; // extract the Python error text
-        Base::Console().Error("ViewProviderPythonFeature::setDisplayMode: %s\n", e.what());
+        const char* name = object->getObject()->Label.getValue();
+        Base::Console().Error("ViewProviderPythonFeature::setDisplayMode (%s): %s\n", name, e.what());
     }
 
     return ModeName;
