@@ -47,12 +47,15 @@ public:
 
     // Returns the icon
     QIcon getIcon() const;
+    std::vector<App::DocumentObject*> claimChildren() const;
 
     /** @name Update data methods*/
     //@{
     void attach(App::DocumentObject *pcObject);
     void updateData(const App::Property*);
     void onChanged(const App::Property* prop);
+    void startRestoring();
+    void finishRestoring();
     //@}
 
     /** @name Display methods */
@@ -89,7 +92,14 @@ public:
 
     // Returns the icon
     QIcon getIcon() const {
-        return imp->getIcon();
+        QIcon icon = imp->getIcon();
+        if (icon.isNull())
+            icon = ViewProviderT::getIcon();
+        return icon;
+    }
+
+    std::vector<App::DocumentObject*> claimChildren() const {
+        return imp->claimChildren();
     }
 
     /** @name Update data methods*/
@@ -105,6 +115,16 @@ public:
     }
     //@}
 
+    /** @name Restoring view provider from document load */
+    //@{
+    virtual void startRestoring() {
+        imp->startRestoring();
+    }
+    virtual void finishRestoring() {
+        imp->finishRestoring();
+    }
+    //@}
+
     /** @name Display methods */
     //@{
     /// get the default display mode
