@@ -25,10 +25,12 @@
 #ifndef _PreComp_
 # include <QColorDialog>
 # include <QDesktopWidget>
+# include <QDialogButtonBox>
 # include <QDrag>
 # include <QKeyEvent>
 # include <QMimeData>
 # include <QPainter>
+# include <QPlainTextEdit>
 # include <QStylePainter>
 # include <QToolTip>
 #endif
@@ -259,14 +261,30 @@ void CheckListDialog::accept ()
 
 // ------------------------------------------------------------------------------
 
+namespace Gui {
+struct ColorButtonP
+{
+    QColor col;
+    QPointer<QColorDialog> cd;
+    bool allowChange;
+    bool drawFrame;
+    bool modal;
+
+    ColorButtonP() : cd(0), allowChange(true), drawFrame(true), modal(true)
+    {
+    }
+};
+}
+
 /**
  * Constructs a colored button called \a name with parent \a parent.
  */
-ColorButton::ColorButton( QWidget* parent )
-    : QPushButton( parent ), _allowChange(true), _drawFrame(true)
+ColorButton::ColorButton(QWidget* parent)
+    : QPushButton(parent)
 {
-    _col = palette().color(QPalette::Active,QPalette::Midlight);
-    connect( this, SIGNAL( clicked() ), SLOT( onChooseColor() ));
+    d = new ColorButtonP();
+    d->col = palette().color(QPalette::Active,QPalette::Midlight);
+    connect(this, SIGNAL(clicked()), SLOT(onChooseColor()));
 }
 
 /**
@@ -274,14 +292,15 @@ ColorButton::ColorButton( QWidget* parent )
  */
 ColorButton::~ColorButton()
 {
+    delete d;
 }
 
 /** 
  * Sets the color \a c to the button. 
  */
-void ColorButton::setColor( const QColor& c )
+void ColorButton::setColor(const QColor& c)
 {
-    _col = c;
+    d->col = c;
     update();
 }
 
@@ -290,27 +309,37 @@ void ColorButton::setColor( const QColor& c )
  */
 QColor ColorButton::color() const
 {
-    return _col;
+    return d->col;
 }
 
 void ColorButton::setAllowChangeColor(bool ok)
 {
-    _allowChange = ok;
+    d->allowChange = ok;
 }
 
 bool ColorButton::allowChangeColor() const
 {
-    return _allowChange;
+    return d->allowChange;
 }
 
 void ColorButton::setDrawFrame(bool ok)
 {
-    _drawFrame = ok;
+    d->drawFrame = ok;
 }
 
 bool ColorButton::drawFrame() const
 {
-    return _drawFrame;
+    return d->drawFrame;
+}
+
+void ColorButton::setModal(bool b)
+{
+    d->modal = b;
+}
+
+bool ColorButton::isModal() const
+{
+    return d->modal;
 }
 
 /**
@@ -326,13 +355,14 @@ void ColorButton::paintEvent (QPaintEvent * e)
     QColor pen = palette().color(group,QPalette::ButtonText);
     {
         QPainter paint(this);
-        paint.setPen( pen );
+        paint.setPen(pen);
 
-        if (_drawFrame) {
-            paint.setBrush(QBrush(_col));
+        if (d->drawFrame) {
+            paint.setBrush(QBrush(d->col));
             paint.drawRect(5, 5, width()-10, height()-10);
-        } else {
-            paint.fillRect(5, 5, width()-10, height()-10, QBrush(_col));
+        }
+        else {
+            paint.fillRect(5, 5, width()-10, height()-10, QBrush(d->col));
         }
     }
 
@@ -352,13 +382,34 @@ void ColorButton::paintEvent (QPaintEvent * e)
  */
 void ColorButton::onChooseColor()
 {
-    if (!_allowChange)
+    if (!d->allowChange)
         return;
-    QColor c = QColorDialog::getColor( _col, this );
-    if (c.isValid()) {
-        setColor( c );
-        changed();
+#if QT_VERSION >= 0x040500
+    if (d->modal) {
+#endif
+        QColor c = QColorDialog::getColor(d->col, this);
+        if (c.isValid()) {
+            setColor(c);
+            changed();
+        }
+#if QT_VERSION >= 0x040500
     }
+    else {
+        if (d->cd.isNull()) {
+            d->cd = new QColorDialog(d->col, this);
+            d->cd->setAttribute(Qt::WA_DeleteOnClose);
+            connect(d->cd, SIGNAL(currentColorChanged(const QColor &)),
+                    this, SLOT(onColorChosen(const QColor&)));
+        }
+        d->cd->show();
+    }
+#endif
+}
+
+void ColorButton::onColorChosen(const QColor& c)
+{
+    setColor(c);
+    changed();
 }
 
 // ------------------------------------------------------------------------------
@@ -661,4 +712,80 @@ void StatusWidget::adjustPosition(QWidget* w)
     move(p);
 }
 
+// --------------------------------------------------------------------
+
+LabelEditor::LabelEditor (QWidget * parent)
+  : QWidget(parent)
+{
+    QHBoxLayout *layout = new QHBoxLayout(this);
+    layout->setMargin(0);
+    layout->setSpacing(2);
+
+    lineEdit = new QLineEdit(this);
+    layout->addWidget(lineEdit);
+
+    connect(lineEdit, SIGNAL(textChanged(const QString &)),
+            this, SIGNAL(textChanged(const QString &)));
+
+    button = new QPushButton(QLatin1String("..."), this);
+    button->setFixedWidth(2*button->fontMetrics().width(QLatin1String(" ... ")));
+    layout->addWidget(button);
+
+    connect(button, SIGNAL(clicked()), this, SLOT(changeText()));
+
+    setFocusProxy(lineEdit);
+}
+
+LabelEditor::~LabelEditor()
+{
+}
+
+QString LabelEditor::text() const
+{
+    return lineEdit->text();
+}
+
+void LabelEditor::setText(const QString& s)
+{
+    lineEdit->setText(s);
+}
+
+void LabelEditor::changeText()
+{
+    QDialog dlg(this);
+    QVBoxLayout* hboxLayout = new QVBoxLayout(&dlg);
+    QDialogButtonBox* buttonBox = new QDialogButtonBox(&dlg);
+    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Close);
+
+    QPlainTextEdit *edit = new QPlainTextEdit(&dlg);
+    edit->setPlainText(this->lineEdit->text());
+    
+    hboxLayout->addWidget(edit);
+    hboxLayout->addWidget(buttonBox);
+    connect(buttonBox, SIGNAL(accepted()), &dlg, SLOT(accept()));
+    connect(buttonBox, SIGNAL(rejected()), &dlg, SLOT(reject()));
+    if (dlg.exec() == QDialog::Accepted) {
+        this->lineEdit->setText(edit->toPlainText());
+    }
+}
+
+/**
+ * Sets the browse button's text to \a txt.
+ */
+void LabelEditor::setButtonText(const QString& txt)
+{
+    button->setText(txt);
+    int w1 = 2*button->fontMetrics().width(txt);
+    int w2 = 2*button->fontMetrics().width(QLatin1String(" ... "));
+    button->setFixedWidth((w1 > w2 ? w1 : w2));
+}
+
+/**
+ * Returns the browse button's text.
+ */
+QString LabelEditor::buttonText() const
+{
+    return button->text();
+}
+
 #include "moc_Widgets.cpp"
