@@ -37,18 +37,17 @@ class PrefCheckBox;
 class CommandViewItemPrivate;
 
 /**
- * This class allows to drag one or more items which
- * correspond to a Command object. The dragged items you
- * can drop onto a @ref Gui::CustomToolBar object
+ * This class allows to drag one or more items which correspond to a Command object.
+ * The dragged items can be dropped onto the @ref Gui::PythonConsole.
  * @see CommandViewItem, Command
- * \author Werner Mayer
+ * @author Werner Mayer
  */
 class CommandIconView : public QListWidget
 {
   Q_OBJECT
 
 public:
-  CommandIconView ( QWidget * parent = 0 );
+  CommandIconView (QWidget * parent = 0);
   virtual ~CommandIconView ();
 
 protected:
@@ -116,39 +115,43 @@ private:
  */
 class GuiExport ColorButton : public QPushButton
 {
-  Q_OBJECT
+    Q_OBJECT
 
-  Q_PROPERTY( QColor color READ color WRITE setColor )
-  Q_PROPERTY( bool allowChangeColor READ allowChangeColor WRITE setAllowChangeColor )
-  Q_PROPERTY( bool drawFrame READ drawFrame WRITE setDrawFrame )
+    Q_PROPERTY( QColor color READ color WRITE setColor )
+    Q_PROPERTY( bool allowChangeColor READ allowChangeColor WRITE setAllowChangeColor )
+    Q_PROPERTY( bool drawFrame READ drawFrame WRITE setDrawFrame )
 
 public:
-  ColorButton( QWidget* parent = 0 );
-  ~ColorButton();
+    ColorButton(QWidget* parent = 0);
+    ~ColorButton();
 
-  void setColor( const QColor& );
-  QColor color() const;
+    void setColor(const QColor&);
+    QColor color() const;
 
-  void setAllowChangeColor(bool);
-  bool allowChangeColor() const;
+    void setAllowChangeColor(bool);
+    bool allowChangeColor() const;
 
-  void setDrawFrame(bool);
-  bool drawFrame() const;
+    void setDrawFrame(bool);
+    bool drawFrame() const;
+
+    void setModal(bool);
+    bool isModal() const;
 
 public Q_SLOTS:
-  virtual void onChooseColor();
+    void onChooseColor();
+
+private Q_SLOTS:
+    void onColorChosen(const QColor&);
 
 Q_SIGNALS:
-  /** Emits this signal when color has changed */
-  void changed();
+    /** Emits this signal when color has changed */
+    void changed();
 
 protected:
-  void paintEvent ( QPaintEvent* );
+    void paintEvent (QPaintEvent*);
 
 private:
-  QColor _col;
-  bool _allowChange;
-  bool _drawFrame;
+    struct ColorButtonP *d;
 };
 
 // ------------------------------------------------------------------------------
@@ -280,6 +283,45 @@ protected:
 private:
     QLabel* label;
 };
+
+// ----------------------------------------------------------------------
+
+class GuiExport LabelEditor : public QWidget
+{
+    Q_OBJECT
+
+    Q_PROPERTY(QString  text        READ text        WRITE setText      )
+    Q_PROPERTY(QString  buttonText  READ buttonText  WRITE setButtonText)
+
+public:
+    LabelEditor (QWidget * parent = 0);
+    virtual ~LabelEditor();
+
+    /** 
+    * Returns the filename.
+    */
+    QString text() const;
+
+    /**
+    * Returns the button's text.
+    */
+    QString buttonText() const;
+
+public Q_SLOTS:
+    virtual void setText(const QString &);
+    virtual void setButtonText (const QString &);
+
+Q_SIGNALS:
+    void textChanged(const QString &);
+
+private Q_SLOTS:
+    void changeText();
+
+private:
+    QLineEdit *lineEdit;
+    QPushButton *button;
+};
+
 } // namespace Gui
 
 #endif // GUI_WIDGETS_H
