@@ -35,6 +35,7 @@
 #include "ToolBoxManager.h"
 #include "Window.h"
 #include "Selection.h"
+#include "MainWindow.h"
 #include <Gui/CombiView.h>
 #include <Gui/TaskView/TaskView.h>
 #include <Gui/TaskView/TaskWatcher.h>
@@ -47,25 +48,21 @@
 using namespace Gui;
 
 /** \defgroup workbench Workbench Framework
- *
- * FreeCAD provides the possibility to have one or more workbenches for a module. A workbench changes the appearance of the main window
- * in that way that it defines toolbars, items in the toolbox, menus or the context menu and dockable windows that are shown to the user.
- *
- * The idea behind this concept is that the user should see only the functions that are required for the task that he is doing at this
- * moment and not to show dozens of unneeded functions which the user never uses.
- *
- * \section stepbystep Step by step
- * Here follows a short description of how an own workbench can be added to a module.
- *
- * \subsection newClass Inherit either from Workbench or StdWorkbench
- * First you have to subclass either Workbench or StdWorkbench and reimplement the methods setupMenuBar(), setupToolBars(), setupCommandBars()
- * and setupDockWindows().
- *
- * The difference between both classes is that these methods of Workbench are pure virtual while StdWorkbench defines already the standard menus and
- * toolbars, such as the 'File', 'Edit', ..., 'Help' menus with their common functions.
- *
- * If your class derives from Workbench then you have to define your menus, toolbars and toolbox items from scratch while deriving from StdWorkbench
- * you have the possibility to add your preferred functions or even remove some unneeded functions.
+    \ingroup GUI
+
+    FreeCAD provides the possibility to have one or more workbenches for a module.
+    A workbench changes the appearance of the main window in that way that it defines toolbars, items in the toolbox, menus or the context menu and dockable windows that are shown to the user.
+    The idea behind this concept is that the user should see only the functions that are required for the task that he is doing at this moment and not to show dozens of unneeded functions which the user never uses.
+
+    \section stepbystep Step by step
+    Here follows a short description of how an own workbench can be added to a module.
+
+    \subsection newClass Inherit either from Workbench or StdWorkbench
+    First you have to subclass either \ref Gui::Workbench "Workbench" or \ref Gui::StdWorkbench "StdWorkbench" and reimplement the methods \ref Gui::Workbench::setupMenuBar() "setupMenuBar()", \ref Gui::Workbench::setupToolBars() "setupToolBars()", \ref Gui::Workbench::setupCommandBars() "setupCommandBars()" and \ref Gui::Workbench::setupDockWindows() "setupDockWindows()".
+
+    The difference between both classes is that these methods of %Workbench are pure virtual while StdWorkbench defines already the standard menus and toolbars, such as the 'File', 'Edit', ..., 'Help' menus with their common functions.
+
+    If your class derives from %Workbench then you have to define your menus, toolbars and toolbox items from scratch while deriving from StdWorkbench you have the possibility to add your preferred functions or even remove some unneeded functions.
  * \code
  *
  * class MyWorkbench : public StdWorkbench
@@ -199,6 +196,7 @@ using namespace Gui;
  * \endcode
  */
 
+/// @namespace Gui @class Workbench
 TYPESYSTEM_SOURCE_ABSTRACT(Gui::Workbench, Base::BaseClass)
 
 Workbench::Workbench()
@@ -350,13 +348,15 @@ void Workbench::addTaskWatcher(std::vector<Gui::TaskView::TaskWatcher*> &Watcher
     pcCombiView->getTaskPanel()->addTaskWatcher(Watcher); 
 }
 
-
 void Workbench::removeTaskWatcher(void)
 {
-   Gui::DockWnd::CombiView* pcCombiView = qobject_cast<Gui::DockWnd::CombiView*>
+    Gui::DockWnd::CombiView* pcCombiView = qobject_cast<Gui::DockWnd::CombiView*>
         (Gui::DockWindowManager::instance()->getDockWindow("Combo View"));
 
-   pcCombiView->getTaskPanel()->removeTaskWatcher();
+    std::vector<Gui::TaskView::TaskWatcher*> watcher;
+    pcCombiView->getTaskPanel()->removeTaskWatcher();
+    // make sure to delete the old watchers
+    pcCombiView->getTaskPanel()->addTaskWatcher(watcher); 
 }
 
 // --------------------------------------------------------------------
@@ -432,7 +432,8 @@ MenuItem* StdWorkbench::setupMenuBar() const
     file->setCommand("&File");
     *file << "Std_New" << "Std_Open" << "Separator" << "Std_CloseActiveWindow"
           << "Std_CloseAllWindows" << "Separator" << "Std_Save" << "Std_SaveAs"
-          << "Separator" << "Std_Import" << "Std_Export" << "Std_ProjectInfo" 
+          << "Separator" << "Std_Import" << "Std_Export" 
+          << "Std_MergeProjects" << "Std_ProjectInfo" 
           << "Separator" << "Std_Print" << "Std_PrintPdf"
           << "Separator" << "Std_RecentFiles" << "Separator" << "Std_Quit";
 
@@ -476,16 +477,18 @@ MenuItem* StdWorkbench::setupMenuBar() const
     view->setCommand("&View");
     *view << "Std_ViewCreate" << "Std_OrthographicCamera" << "Std_PerspectiveCamera" << "Separator" 
           << stdviews << "Std_FreezeViews" << "Separator" << view3d << zoom
-          << "Std_ViewDockUndockFullscreen" << "Std_ToggleClipPlane" << "Separator" << visu
-          << "Std_ToggleVisibility" << "Std_SetAppearance" << "Std_RandomColor" << "Separator" 
+          << "Std_ViewDockUndockFullscreen" << "Std_ToggleClipPlane"
+          << "Std_TextureMapping" << "Separator" << visu
+          << "Std_ToggleVisibility" << "Std_ToggleNavigation"
+          << "Std_SetAppearance" << "Std_RandomColor" << "Separator" 
           << "Std_MeasureDistance" << "Separator" 
           << "Std_Workbench" << "Std_ToolBarMenu" << "Std_DockViewMenu" << "Separator" 
-          << "Std_ViewStatusBar" << "Std_UserInterface";
+          << "Std_ViewStatusBar";
 
     // Tools
     MenuItem* tool = new MenuItem( menuBar );
     tool->setCommand("&Tools");
-    *tool << "Std_CommandLine" << "Std_DlgParameter" << "Separator"
+    *tool << "Std_DlgParameter" << "Separator"
           << "Std_ViewScreenShot" << "Std_SceneInspector" << "Std_DemoMode" 
           << "Separator" << "Std_DlgCustomize";
 
@@ -562,7 +565,7 @@ ToolBarItem* StdWorkbench::setupCommandBars() const
     ToolBarItem* macro = new ToolBarItem( root );
     macro->setCommand("Special Ops");
     *macro << "Std_DlgParameter" << "Std_DlgPreferences" << "Std_DlgMacroRecord" << "Std_MacroStopRecord" 
-           << "Std_DlgMacroExecute" << "Std_DlgCustomize" << "Std_CommandLine";
+           << "Std_DlgMacroExecute" << "Std_DlgCustomize";
 
     return root;
 }
@@ -584,6 +587,56 @@ DockWindowItems* StdWorkbench::setupDockWindows() const
 
 // --------------------------------------------------------------------
 
+TYPESYSTEM_SOURCE(Gui::BlankWorkbench, Gui::Workbench)
+
+BlankWorkbench::BlankWorkbench()
+  : Workbench()
+{
+}
+
+BlankWorkbench::~BlankWorkbench()
+{
+}
+
+void BlankWorkbench::activated()
+{
+    QList<QDockWidget*> dw = getMainWindow()->findChildren<QDockWidget*>();
+    for (QList<QDockWidget*>::iterator it = dw.begin(); it != dw.end(); ++it)
+        (*it)->toggleViewAction()->setVisible(false);
+    getMainWindow()->statusBar()->hide();
+}
+
+void BlankWorkbench::deactivated()
+{
+    getMainWindow()->statusBar()->show();
+}
+
+void BlankWorkbench::setupContextMenu(const char* recipient,MenuItem* item) const
+{
+}
+
+MenuItem* BlankWorkbench::setupMenuBar() const
+{
+    return new MenuItem();
+}
+
+ToolBarItem* BlankWorkbench::setupToolBars() const
+{
+    return new ToolBarItem();
+}
+
+ToolBarItem* BlankWorkbench::setupCommandBars() const
+{
+    return new ToolBarItem();
+}
+
+DockWindowItems* BlankWorkbench::setupDockWindows() const
+{
+    return new DockWindowItems();
+}
+
+// --------------------------------------------------------------------
+
 TYPESYSTEM_SOURCE(Gui::NoneWorkbench, Gui::StdWorkbench)
 
 NoneWorkbench::NoneWorkbench()
