@@ -26,6 +26,7 @@
 #ifndef _PreComp_
 # include <algorithm>
 # include <QComboBox>
+# include <QFontDatabase>
 # include <QLayout>
 # include <QLocale>
 # include <QPixmap>
@@ -45,6 +46,7 @@
 #include <Gui/ViewProviderDocumentObject.h>
 #include <Gui/Widgets.h>
 #include <Gui/Placement.h>
+#include <Gui/FileDialog.h>
 
 #include "PropertyItem.h"
 
@@ -356,6 +358,55 @@ QVariant PropertyStringItem::editorData(QWidget *editor) const
 
 // --------------------------------------------------------------------
 
+TYPESYSTEM_SOURCE(Gui::PropertyEditor::PropertyFontItem, Gui::PropertyEditor::PropertyItem);
+
+PropertyFontItem::PropertyFontItem()
+{
+}
+
+QVariant PropertyFontItem::value(const App::Property* prop) const
+{
+    assert(prop && prop->getTypeId().isDerivedFrom(App::PropertyFont::getClassTypeId()));
+
+    std::string value = static_cast<const App::PropertyFont*>(prop)->getValue();
+    return QVariant(QString::fromUtf8(value.c_str()));
+}
+
+void PropertyFontItem::setValue(const QVariant& value)
+{
+    if (!value.canConvert(QVariant::String))
+        return;
+    QString val = value.toString();
+    QString data = QString::fromAscii("\"%1\"").arg(val);
+    setPropertyValue(data);
+}
+
+QWidget* PropertyFontItem::createEditor(QWidget* parent, const QObject* receiver, const char* method) const
+{
+    QComboBox *cb = new QComboBox(parent);
+    cb->setFrame(false);
+    QObject::connect(cb, SIGNAL(activated(const QString&)), receiver, method);
+    return cb;
+}
+
+void PropertyFontItem::setEditorData(QWidget *editor, const QVariant& data) const
+{
+    QComboBox *cb = qobject_cast<QComboBox*>(editor);
+    QFontDatabase fdb;
+    QStringList familyNames = fdb.families(QFontDatabase::Any);
+    cb->addItems(familyNames);
+    int index = familyNames.indexOf(data.toString());
+    cb->setCurrentIndex(index);
+}
+
+QVariant PropertyFontItem::editorData(QWidget *editor) const
+{
+    QComboBox *cb = qobject_cast<QComboBox*>(editor);
+    return QVariant(cb->currentText());
+}
+
+// --------------------------------------------------------------------
+
 TYPESYSTEM_SOURCE(Gui::PropertyEditor::PropertySeparatorItem, Gui::PropertyEditor::PropertyItem);
 
 QWidget* PropertySeparatorItem::createEditor(QWidget* parent, const QObject* receiver, const char* method) const
@@ -482,9 +533,7 @@ QVariant PropertyFloatItem::toString(const QVariant& prop) const
     QString data = QLocale::system().toString(value, 'f', 2);
     const std::vector<App::Property*>& props = getPropertyData();
     if (!props.empty()) {
-        if (props.front()->getTypeId().isDerivedFrom(App::PropertyAngle::getClassTypeId()))
-            data += QString::fromUtf8(" \xc2\xb0");
-        else if (props.front()->getTypeId().isDerivedFrom(App::PropertyDistance::getClassTypeId())) {
+        if (props.front()->getTypeId().isDerivedFrom(App::PropertyDistance::getClassTypeId())) {
             QString unit = Base::UnitsApi::getPrefUnitOf(Base::Length);
             unit.prepend(QLatin1String(" "));
             data += unit;
@@ -494,6 +543,16 @@ QVariant PropertyFloatItem::toString(const QVariant& prop) const
             unit.prepend(QLatin1String(" "));
             data += unit;
         }
+        else if (props.front()->getTypeId().isDerivedFrom(App::PropertySpeed::getClassTypeId())) {
+            //QString unit = Base::UnitsApi::getPrefUnitOf(Base::Acceleration);
+            //unit.prepend(QLatin1String(" "));
+            //data += unit;
+        }
+        else if (props.front()->getTypeId().isDerivedFrom(App::PropertyAcceleration::getClassTypeId())) {
+            QString unit = Base::UnitsApi::getPrefUnitOf(Base::Acceleration);
+            unit.prepend(QLatin1String(" "));
+            data += unit;
+        }
     }
 
     return QVariant(data);
@@ -532,9 +591,6 @@ void PropertyFloatItem::setEditorData(QWidget *editor, const QVariant& data) con
     const std::vector<App::Property*>& prop = getPropertyData();
     if (prop.empty())
         return;
-    else if (prop.front()->getTypeId().isDerivedFrom(App::PropertyAngle::getClassTypeId())) {
-        sb->setSuffix(QString::fromUtf8(" \xc2\xb0"));
-    }
     else if (prop.front()->getTypeId().isDerivedFrom(App::PropertyDistance::getClassTypeId())) {
         QString unit = Base::UnitsApi::getPrefUnitOf(Base::Length);
         unit.prepend(QLatin1String(" "));
@@ -546,6 +602,18 @@ void PropertyFloatItem::setEditorData(QWidget *editor, const QVariant& data) con
         unit.prepend(QLatin1String(" "));
         sb->setSuffix(unit);
     }
+    else if (prop.front()->getTypeId().isDerivedFrom(App::PropertySpeed::getClassTypeId())) {
+        //sb->setMinimum(0.0);
+        //QString unit = Base::UnitsApi::getPrefUnitOf(Base::Acceleration);
+        //unit.prepend(QLatin1String(" "));
+        //sb->setSuffix(unit);
+    }
+    else if (prop.front()->getTypeId().isDerivedFrom(App::PropertyAcceleration::getClassTypeId())) {
+        sb->setMinimum(0.0);
+        QString unit = Base::UnitsApi::getPrefUnitOf(Base::Acceleration);
+        unit.prepend(QLatin1String(" "));
+        sb->setSuffix(unit);
+    }
 }
 
 QVariant PropertyFloatItem::editorData(QWidget *editor) const
@@ -630,8 +698,25 @@ PropertyAngleItem::PropertyAngleItem()
 
 void PropertyAngleItem::setEditorData(QWidget *editor, const QVariant& data) const
 {
+    const App::PropertyFloatConstraint::Constraints* c = 0;
+    const std::vector<App::Property*>& items = getPropertyData();
+    if (!items.empty()) {
+        App::PropertyAngle* prop = static_cast<App::PropertyAngle*>(items[0]);
+        c = prop->getConstraints();
+    }
+
     QDoubleSpinBox *sb = qobject_cast<QDoubleSpinBox*>(editor);
-    sb->setRange((double)INT_MIN, (double)INT_MAX);
+    if (c) {
+        sb->setMinimum(c->LowerBound);
+        sb->setMaximum(c->UpperBound);
+        sb->setSingleStep(c->StepSize);
+    }
+    else {
+        sb->setMinimum((double)INT_MIN);
+        sb->setMaximum((double)INT_MAX);
+        sb->setSingleStep(1.0);
+    }
+
     sb->setValue(data.toDouble());
     sb->setSuffix(QString::fromUtf8(" \xc2\xb0"));
 }
@@ -905,8 +990,11 @@ namespace Gui { namespace PropertyEditor {
 class PlacementEditor : public Gui::LabelButton
 {
 public:
-    PlacementEditor(QWidget * parent = 0) : LabelButton(parent), _task(0)
+    PlacementEditor(const QString& name, QWidget * parent = 0)
+        : LabelButton(parent), _task(0)
     {
+        propertyname = name;
+        propertyname.replace(QLatin1String(" "), QLatin1String(""));
     }
     ~PlacementEditor()
     {
@@ -927,10 +1015,11 @@ protected:
         }
         if (!_task) {
             _task = task;
-            connect(task, SIGNAL(placementChanged(const QVariant &)),
+            connect(task, SIGNAL(placementChanged(const QVariant &, bool, bool)),
                     this, SLOT(setValue(const QVariant&)));
         }
         task->setPlacement(value().value<Base::Placement>());
+        task->setPropertyName(propertyname);
         Gui::Control().showDialog(task);
     }
     void showValue(const QVariant& d)
@@ -952,6 +1041,7 @@ protected:
     }
 
     Gui::Dialog::TaskPlacement* _task;
+    QString propertyname;
 };
 }
 }
@@ -1125,7 +1215,7 @@ void PropertyPlacementItem::setValue(const QVariant& value)
 
 QWidget* PropertyPlacementItem::createEditor(QWidget* parent, const QObject* receiver, const char* method) const
 {
-    PlacementEditor *pe = new PlacementEditor(parent);
+    PlacementEditor *pe = new PlacementEditor(this->propertyName(), parent);
     QObject::connect(pe, SIGNAL(valueChanged(const QVariant &)), receiver, method);
     return pe;
 }
@@ -1235,27 +1325,26 @@ PropertyStringListItem::PropertyStringListItem()
 {
 }
 
-QWidget* PropertyStringListItem::createEditor(QWidget* parent, const QObject* /*receiver*/, const char* /*method*/) const
+QWidget* PropertyStringListItem::createEditor(QWidget* parent, const QObject* receiver, const char* method) const
 {
-    QComboBox *cb = new QComboBox(parent);
-    cb->setFrame(false);
-    return cb;
+    Gui::LabelEditor* le = new Gui::LabelEditor(parent);
+    le->setAutoFillBackground(true);
+    QObject::connect(le, SIGNAL(textChanged(const QString&)), receiver, method);
+    return le;
 }
 
 void PropertyStringListItem::setEditorData(QWidget *editor, const QVariant& data) const
 {
-    QComboBox *cb = qobject_cast<QComboBox*>(editor);
-    cb->setEditable(true);
-    cb->insertItems(0, data.toStringList());
-    cb->setCurrentIndex(0);
+    Gui::LabelEditor *le = qobject_cast<Gui::LabelEditor*>(editor);
+    QStringList list = data.toStringList();
+    le->setText(list.join(QChar::fromAscii('\n')));
 }
 
 QVariant PropertyStringListItem::editorData(QWidget *editor) const
 {
-    QComboBox *cb = qobject_cast<QComboBox*>(editor);
-    QStringList list;
-    for (int i=0; i<cb->count(); i++)
-        list << cb->itemText(i);
+    Gui::LabelEditor *le = qobject_cast<Gui::LabelEditor*>(editor);
+    QString complete = le->text();
+    QStringList list = complete.split(QChar::fromAscii('\n'));
     return QVariant(list);
 }
 
@@ -1379,12 +1468,17 @@ QVariant PropertyFileItem::value(const App::Property* prop) const
 {
     assert(prop && prop->getTypeId().isDerivedFrom(App::PropertyFile::getClassTypeId()));
 
-    std::string value = ((App::PropertyFile*)prop)->getValue();
+    std::string value = static_cast<const App::PropertyFile*>(prop)->getValue();
     return QVariant(QString::fromUtf8(value.c_str()));
 }
 
-void PropertyFileItem::setValue(const QVariant& /*value*/)
+void PropertyFileItem::setValue(const QVariant& value)
 {
+    if (!value.canConvert(QVariant::String))
+        return;
+    QString val = value.toString();
+    QString data = QString::fromAscii("\"%1\"").arg(val);
+    setPropertyValue(data);
 }
 
 QVariant PropertyFileItem::toolTip(const App::Property* prop) const
@@ -1392,6 +1486,26 @@ QVariant PropertyFileItem::toolTip(const App::Property* prop) const
     return value(prop);
 }
 
+QWidget* PropertyFileItem::createEditor(QWidget* parent, const QObject* receiver, const char* method) const
+{
+    Gui::FileChooser *fc = new Gui::FileChooser(parent);
+    fc->setAutoFillBackground(true);
+    QObject::connect(fc, SIGNAL(fileNameSelected(const QString&)), receiver, method);
+    return fc;
+}
+
+void PropertyFileItem::setEditorData(QWidget *editor, const QVariant& data) const
+{
+    Gui::FileChooser *fc = qobject_cast<Gui::FileChooser*>(editor);
+    fc->setFileName(data.toString());
+}
+
+QVariant PropertyFileItem::editorData(QWidget *editor) const
+{
+    Gui::FileChooser *fc = qobject_cast<Gui::FileChooser*>(editor);
+    return QVariant(fc->fileName());
+}
+
 // --------------------------------------------------------------------
 
 TYPESYSTEM_SOURCE(Gui::PropertyEditor::PropertyPathItem, Gui::PropertyEditor::PropertyItem);
@@ -1400,13 +1514,96 @@ PropertyPathItem::PropertyPathItem()
 {
 }
 
-QVariant PropertyPathItem::value(const App::Property* /*prop*/) const
+QVariant PropertyPathItem::value(const App::Property* prop) const
 {
-    return QVariant();
+    assert(prop && prop->getTypeId().isDerivedFrom(App::PropertyPath::getClassTypeId()));
+
+    std::string value = static_cast<const App::PropertyPath*>(prop)->getValue().string();
+    return QVariant(QString::fromUtf8(value.c_str()));
 }
 
-void PropertyPathItem::setValue(const QVariant& /*value*/)
+void PropertyPathItem::setValue(const QVariant& value)
 {
+    if (!value.canConvert(QVariant::String))
+        return;
+    QString val = value.toString();
+    QString data = QString::fromAscii("\"%1\"").arg(val);
+    setPropertyValue(data);
+}
+
+QVariant PropertyPathItem::toolTip(const App::Property* prop) const
+{
+    return value(prop);
+}
+
+QWidget* PropertyPathItem::createEditor(QWidget* parent, const QObject* receiver, const char* method) const
+{
+    Gui::FileChooser *fc = new Gui::FileChooser(parent);
+    fc->setAutoFillBackground(true);
+    QObject::connect(fc, SIGNAL(fileNameSelected(const QString&)), receiver, method);
+    return fc;
+}
+
+void PropertyPathItem::setEditorData(QWidget *editor, const QVariant& data) const
+{
+    Gui::FileChooser *fc = qobject_cast<Gui::FileChooser*>(editor);
+    fc->setFileName(data.toString());
+}
+
+QVariant PropertyPathItem::editorData(QWidget *editor) const
+{
+    Gui::FileChooser *fc = qobject_cast<Gui::FileChooser*>(editor);
+    return QVariant(fc->fileName());
+}
+
+// --------------------------------------------------------------------
+
+TYPESYSTEM_SOURCE(Gui::PropertyEditor::PropertyTransientFileItem, Gui::PropertyEditor::PropertyItem);
+
+PropertyTransientFileItem::PropertyTransientFileItem()
+{
+}
+
+QVariant PropertyTransientFileItem::value(const App::Property* prop) const
+{
+    assert(prop && prop->getTypeId().isDerivedFrom(App::PropertyFileIncluded::getClassTypeId()));
+
+    std::string value = static_cast<const App::PropertyFileIncluded*>(prop)->getValue();
+    return QVariant(QString::fromUtf8(value.c_str()));
+}
+
+void PropertyTransientFileItem::setValue(const QVariant& value)
+{
+    if (!value.canConvert(QVariant::String))
+        return;
+    QString val = value.toString();
+    QString data = QString::fromAscii("\"%1\"").arg(val);
+    setPropertyValue(data);
+}
+
+QVariant PropertyTransientFileItem::toolTip(const App::Property* prop) const
+{
+    return value(prop);
+}
+
+QWidget* PropertyTransientFileItem::createEditor(QWidget* parent, const QObject* receiver, const char* method) const
+{
+    Gui::FileChooser *fc = new Gui::FileChooser(parent);
+    fc->setAutoFillBackground(true);
+    QObject::connect(fc, SIGNAL(fileNameSelected(const QString&)), receiver, method);
+    return fc;
+}
+
+void PropertyTransientFileItem::setEditorData(QWidget *editor, const QVariant& data) const
+{
+    Gui::FileChooser *fc = qobject_cast<Gui::FileChooser*>(editor);
+    fc->setFileName(data.toString());
+}
+
+QVariant PropertyTransientFileItem::editorData(QWidget *editor) const
+{
+    Gui::FileChooser *fc = qobject_cast<Gui::FileChooser*>(editor);
+    return QVariant(fc->fileName());
 }
 
 // --------------------------------------------------------------------
