@@ -117,6 +117,26 @@ protected:
     PropertyStringItem();
 };
 
+/**
+ * Change a font property.
+ * \author Werner Mayer
+ */
+class GuiExport PropertyFontItem: public PropertyItem
+{
+    TYPESYSTEM_HEADER();
+
+    virtual QWidget* createEditor(QWidget* parent, const QObject* receiver, const char* method) const;
+    virtual void setEditorData(QWidget *editor, const QVariant& data) const;
+    virtual QVariant editorData(QWidget *editor) const;
+
+protected:
+    virtual QVariant value(const App::Property*) const;
+    virtual void setValue(const QVariant&);
+
+protected:
+    PropertyFontItem();
+};
+
 /**
  * Dummy property to separate groups of properties.
  * \author Werner Mayer
@@ -427,6 +447,10 @@ class GuiExport PropertyFileItem: public PropertyItem
 {
     TYPESYSTEM_HEADER();
 
+    virtual QWidget* createEditor(QWidget* parent, const QObject* receiver, const char* method) const;
+    virtual void setEditorData(QWidget *editor, const QVariant& data) const;
+    virtual QVariant editorData(QWidget *editor) const;
+
 protected:
     virtual QVariant value(const App::Property*) const;
     virtual void setValue(const QVariant&);
@@ -444,12 +468,38 @@ class GuiExport PropertyPathItem: public PropertyItem
 {
     TYPESYSTEM_HEADER();
 
+    virtual QWidget* createEditor(QWidget* parent, const QObject* receiver, const char* method) const;
+    virtual void setEditorData(QWidget *editor, const QVariant& data) const;
+    virtual QVariant editorData(QWidget *editor) const;
+
 protected:
     virtual QVariant value(const App::Property*) const;
     virtual void setValue(const QVariant&);
 
 protected:
     PropertyPathItem();
+    virtual QVariant toolTip(const App::Property*) const;
+};
+
+/**
+ * Show path of included file.
+ * \author Werner Mayer
+ */
+class GuiExport PropertyTransientFileItem: public PropertyItem
+{
+    TYPESYSTEM_HEADER();
+
+    virtual QWidget* createEditor(QWidget* parent, const QObject* receiver, const char* method) const;
+    virtual void setEditorData(QWidget *editor, const QVariant& data) const;
+    virtual QVariant editorData(QWidget *editor) const;
+
+protected:
+    virtual QVariant value(const App::Property*) const;
+    virtual void setValue(const QVariant&);
+
+protected:
+    PropertyTransientFileItem();
+    virtual QVariant toolTip(const App::Property*) const;
 };
 
 class PropertyItemEditorFactory : public QItemEditorFactory
