@@ -34,6 +34,8 @@
 #include <Inventor/Qt/SoQt.h>
 #if defined(Q_OS_WIN)
 #include <windows.h>
+#elif defined(Q_WS_X11)
+#include <QX11EmbedWidget>
 #endif
 // FreeCAD Base header
 #include <Base/Exception.h>
@@ -167,6 +169,53 @@ FreeCADGui_setupWithoutGUI(PyObject * /*self*/, PyObject *args)
     return Py_None;
 }
 
+static PyObject *
+FreeCADGui_embedToWindow(PyObject * /*self*/, PyObject *args)
+{
+    char* pointer;
+    if (!PyArg_ParseTuple(args, "s", &pointer))
+        return NULL;
+
+    QWidget* widget = Gui::getMainWindow();
+    if (!widget) {
+        PyErr_SetString(PyExc_Exception, "No main window");
+        return 0;
+    }
+
+    std::string pointer_str = pointer;
+    std::stringstream str(pointer_str);
+
+#if defined(Q_OS_WIN)
+    void* window = 0;
+    str >> window;
+    WId winid = (WId)window;
+
+    LONG oldLong = GetWindowLong(winid, GWL_STYLE);
+    SetWindowLong(winid, GWL_STYLE,
+    oldLong | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
+    //SetWindowLong(widget->winId(), GWL_STYLE,
+    //    WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
+    SetParent(widget->winId(), winid);
+
+    QEvent embeddingEvent(QEvent::EmbeddingControl);
+    QApplication::sendEvent(widget, &embeddingEvent);
+#elif defined(Q_WS_X11)
+    WId winid;
+    str >> winid;
+
+    QX11EmbedWidget* x11 = new QX11EmbedWidget();
+    widget->setParent(x11);
+    x11->embedInto(winid);
+    x11->show();
+#else
+    PyErr_SetString(PyExc_NotImplementedError, "Not implemented for this platform");
+    return 0;
+#endif
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 struct PyMethodDef FreeCADGui_methods[] = { 
     {"showMainWindow",FreeCADGui_showMainWindow,METH_VARARGS,
      "showMainWindow() -- Show the main window\n"
@@ -177,6 +226,8 @@ struct PyMethodDef FreeCADGui_methods[] = {
     {"setupWithoutGUI",FreeCADGui_setupWithoutGUI,METH_VARARGS,
      "setupWithoutGUI() -- Uses this module without starting\n"
      "an event loop or showing up any GUI\n"},
+    {"embedToWindow",FreeCADGui_embedToWindow,METH_VARARGS,
+     "embedToWindow() -- Embeds the main window into another window\n"},
     {NULL, NULL}  /* sentinel */
 };
 
@@ -192,7 +243,8 @@ QWidget* setupMainWindow()
         Gui::MainWindow *mw = new Gui::MainWindow();
         QIcon icon = qApp->windowIcon();
         if (icon.isNull())
-            mw->setWindowIcon(Gui::BitmapFactory().pixmap(App::Application::Config()["AppIcon"].c_str()));
+            qApp->setWindowIcon(Gui::BitmapFactory().pixmap(App::Application::Config()["AppIcon"].c_str()));
+        mw->setWindowIcon(qApp->windowIcon());
         QString appName = qApp->applicationName();
         if (!appName.isEmpty())
             mw->setWindowTitle(appName);
@@ -249,7 +301,8 @@ PyMODINIT_FUNC initFreeCADGui()
     try {
         Base::Interpreter().loadModule("FreeCAD");
         App::Application::Config()["AppIcon"] = "freecad";
-        App::Application::Config()["ConsoleBanner"] = "(c) Juergen Riegel, Werner Mayer 2001-2009\n";
+        App::Application::Config()["SplashPicture"] = "freecadsplash";
+        App::Application::Config()["ConsoleBanner"] = "\xc2\xa9 Juergen Riegel, Werner Mayer, Yorik van Havre 2001-2011\n";
         Gui::Application::initApplication();
         Py_InitModule("FreeCADGui", FreeCADGui_methods);
     }
