@@ -27,7 +27,7 @@
 #   undef _PreComp_
 #endif
 
-#ifdef FC_LINUX
+#ifdef FC_OS_LINUX
 #   include <unistd.h>
 #endif
 
@@ -38,6 +38,7 @@
 #include <cstdio>
 #include <QApplication>
 #include <QMessageBox>
+#include <QLocale>
 
 // FreeCAD header
 #include <Base/Console.h>
@@ -52,11 +53,11 @@
 
 // If you stumble here, run the target "BuildExtractRevision" on Windows systems or the Python script "SubWCRev.py" on Linux based systems
 // which builds src/Build/Version.h. Or create your own from src/Build/Version.h.in!
-#include "../Build/Version.h"
+#include <Build/Version.h>
 
 void PrintInitHelp(void);
 
-const char sBanner[] = "\xc2\xa9 Juergen Riegel, Werner Mayer 2001-2010\n"\
+const char sBanner[] = "\xc2\xa9 Juergen Riegel, Werner Mayer, Yorik van Havre 2001-2011\n"\
 "  #####                 ####  ###   ####  \n" \
 "  #                    #      # #   #   # \n" \
 "  #     ##  #### ####  #     #   #  #   # \n" \
@@ -144,12 +145,18 @@ std::string ProgramOptions::err;
 
 int main( int argc, char ** argv )
 {
-    // Make sure that we use '.' as decimal point
 #if defined(FC_OS_LINUX)
+    // Make sure to setup the Qt locale system before setting LANG and LC_ALL to C.
+    // which is needed to use the system locale settings.
+    (void)QLocale::system();
+    // Make sure that we use '.' as decimal point. See also
+    // http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=559846
     putenv("LANG=C");
     putenv("LC_ALL=C");
+    putenv("PYTHONPATH=");
 #else
     setlocale(LC_NUMERIC, "C");
+    _putenv("PYTHONPATH=");
 #endif
 
     // Name and Version of the Application
