@@ -44,6 +44,18 @@
 
 using namespace CompleteGui;
 
+#if 0 // needed for Qt's lupdate utility
+    qApp->translate("Workbench", "Ske&tch");
+    qApp->translate("Workbench", "&Drawing");
+    qApp->translate("Workbench", "&Raytracing");
+    qApp->translate("Workbench", "&Drafting");
+    qApp->translate("Workbench", "Sketch based");
+    qApp->translate("Workbench", "Parametric");
+    qApp->translate("Workbench", "Object appearence");
+    qApp->translate("Workbench", "Wire Tools");
+#endif
+
+/// @namespace CompleteGui @class Workbench
 TYPESYSTEM_SOURCE(CompleteGui::Workbench, Gui::StdWorkbench)
 
 Workbench::Workbench()
@@ -74,8 +86,18 @@ void Workbench::setupContextMenu(const char* recipient,Gui::MenuItem* item) cons
     else if (strcmp(recipient,"Tree") == 0)
     {
         if ( Gui::Selection().countObjectsOfType(App::DocumentObject::getClassTypeId()) > 0 )
+            {
+            Gui::MenuItem* DraftContext = new Gui::MenuItem();
+            DraftContext->setCommand("Draft tools");
+
+            *DraftContext << "Draft_ApplyStyle" << "Draft_ToggleDisplayMode"
+                          << "Draft_AddToGroup";
+
             *item << "Std_SetAppearance" << "Std_ToggleVisibility" 
-                  << "Std_RandomColor" << "Separator" << "Std_Delete";
+                  << "Std_RandomColor" << "Separator" << "Std_Delete"
+                  << DraftContext;
+            }
+
     }
 }
 
@@ -87,16 +109,17 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     // File
     Gui::MenuItem* file = new Gui::MenuItem( menuBar );
-    file->setCommand(QT_TR_NOOP("&File"));
+    file->setCommand("&File");
     *file << "Std_New" << "Std_Open" << "Separator" << "Std_CloseActiveWindow"
           << "Std_CloseAllWindows" << "Separator" << "Std_Save" << "Std_SaveAs"
-          << "Separator" << "Std_Import" << "Std_Export" << "Std_ProjectInfo" 
-        //<< "Separator" << "Std_Print" << "Std_PrintPdf"
+          << "Separator" << "Std_Import" << "Std_Export"
+          << "Std_MergeProjects" << "Std_ProjectInfo" 
+          << "Separator" << "Std_Print" << "Std_PrintPdf"
           << "Separator" << "Std_RecentFiles" << "Separator" << "Std_Quit";
 
     // Edit
     Gui::MenuItem* edit = new Gui::MenuItem( menuBar );
-    edit->setCommand(QT_TR_NOOP("&Edit"));
+    edit->setCommand("&Edit");
     *edit << "Std_Undo" << "Std_Redo" << "Separator" << "Std_Cut" << "Std_Copy"
           << "Std_Paste" << "Std_DuplicateSelection" << "Separator"
           << "Std_Refresh" << "Std_SelectAll" << "Std_Delete" << "Std_Placement"
@@ -104,7 +127,7 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     // Standard views
     Gui::MenuItem* stdviews = new Gui::MenuItem;
-    stdviews->setCommand(QT_TR_NOOP("Standard views"));
+    stdviews->setCommand("Standard views");
     *stdviews << "Std_ViewFitAll" << "Std_ViewFitSelection" << "Std_ViewAxo"
               << "Separator" << "Std_ViewFront" << "Std_ViewRight"
               << "Std_ViewTop" << "Separator" << "Std_ViewRear" 
@@ -112,46 +135,51 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     // stereo
     Gui::MenuItem* view3d = new Gui::MenuItem;
-    view3d->setCommand(QT_TR_NOOP("&Stereo"));
+    view3d->setCommand("&Stereo");
     *view3d << "Std_ViewIvStereoRedGreen" << "Std_ViewIvStereoQuadBuff" 
             << "Std_ViewIvStereoInterleavedRows" << "Std_ViewIvStereoInterleavedColumns" 
             << "Std_ViewIvStereoOff" << "Separator" << "Std_ViewIvIssueCamPos";
 
     // zoom
     Gui::MenuItem* zoom = new Gui::MenuItem;
-    zoom->setCommand(QT_TR_NOOP("&Zoom"));
+    zoom->setCommand("&Zoom");
     *zoom << "Std_ViewZoomIn" << "Std_ViewZoomOut" << "Separator" << "Std_ViewBoxZoom";
 
     // Visibility
     Gui::MenuItem* visu = new Gui::MenuItem;
-    visu->setCommand(QT_TR_NOOP("Visibility"));
+    visu->setCommand("Visibility");
     *visu << "Std_ToggleVisibility" << "Std_ShowSelection" << "Std_HideSelection"
-          << "Separator" << "Std_ToggleObjects" << "Std_ShowObjects" << "Std_HideObjects";
+          << "Separator" << "Std_ToggleObjects" << "Std_ShowObjects" << "Std_HideObjects"
+          << "Separator" << "Std_ToggleSelectability";
 
     // View
     Gui::MenuItem* view = new Gui::MenuItem( menuBar );
-    view->setCommand(QT_TR_NOOP("&View"));
+    view->setCommand("&View");
     *view << "Std_ViewCreate" << "Std_OrthographicCamera" << "Std_PerspectiveCamera" << "Separator" 
           << stdviews << "Std_FreezeViews" << "Separator" << view3d << zoom
-          << "Std_ViewDockUndockFullscreen" << "Std_ToggleClipPlane" << "Separator" << visu
-          << "Std_SetAppearance" << "Std_ToggleVisibility" << "Std_RandomColor" << "Separator" 
-          //<< "Std_MeasureDistance" << "Separator" 
+          << "Std_ViewDockUndockFullscreen" << "Std_ToggleClipPlane"
+          << "Std_TextureMapping" << "Separator" << visu
+          << "Std_ToggleVisibility" << "Std_ToggleNavigation"
+          << "Std_SetAppearance" << "Std_RandomColor" << "Separator" 
+          << "Std_MeasureDistance" << "Separator" 
           << "Std_Workbench" << "Std_ToolBarMenu" << "Std_DockViewMenu" << "Separator" 
-          << "Std_ViewStatusBar" << "Std_UserInterface";
+          << "Std_ViewStatusBar";
 
     // Tools
     Gui::MenuItem* tool = new Gui::MenuItem( menuBar );
-    tool->setCommand(QT_TR_NOOP("&Tools"));
-    *tool << "Std_CommandLine" << "Std_DlgParameter" << "Separator" << "Std_DlgMacroRecord"
-          << "Std_MacroStopRecord" << "Std_DlgMacroExecute" << "Std_DlgMacroExecuteDirect" 
-          << "Separator" << "Std_ViewScreenShot" << "Separator" << "Std_DlgCustomize";
+    tool->setCommand("&Tools");
+    *tool << "Std_DlgParameter" << "Separator"
+          << "Std_DlgMacroRecord" << "Std_MacroStopRecord"
+          << "Std_DlgMacroExecute" << "Std_DlgMacroExecuteDirect" 
+          << "Separator" << "Std_ViewScreenShot" << "Std_SceneInspector"
+          << "Std_DemoMode" << "Separator" << "Std_DlgCustomize";
 
     // Mesh ****************************************************************************************************
     Gui::MenuItem* mesh = new Gui::MenuItem( menuBar );
 
     // submenu analyze
     Gui::MenuItem* analyze = new Gui::MenuItem();
-    analyze->setCommand(QT_TR_NOOP("Analyze"));
+    analyze->setCommand("Analyze");
     *analyze << "Mesh_Evaluation" 
              << "Mesh_EvaluateFacet" 
              << "Mesh_CurvatureInfo" 
@@ -161,15 +189,16 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     // submenu boolean
     Gui::MenuItem* boolean = new Gui::MenuItem();
-    boolean->setCommand(QT_TR_NOOP("Boolean"));
+    boolean->setCommand("Boolean");
     *boolean << "Mesh_Union" 
              << "Mesh_Intersection" 
              << "Mesh_Difference";
 
-    mesh->setCommand(QT_TR_NOOP("&Meshes"));
+    mesh->setCommand("&Meshes");
     *mesh << "Mesh_Import" 
           << "Mesh_Export" 
           << "Mesh_FromGeometry" 
+          << "MeshPart_Mesher"
           << "Separator"
           << analyze 
           << "Mesh_HarmonizeNormals" 
@@ -178,10 +207,13 @@ Gui::MenuItem* Workbench::setupMenuBar() const
           << "Mesh_FillupHoles" 
           << "Mesh_FillInteractiveHole" 
           << "Mesh_RemoveComponents"
-          << "Mesh_RemoveCompByHand" 
+          << "Mesh_RemoveCompByHand"
+          << "Mesh_AddFacet"
+          << "Mesh_Smoothing"
           << "Separator" 
           << "Mesh_BuildRegularSolid" 
           << boolean << "Separator" 
+          << "Mesh_PolySelect"
           << "Mesh_PolyCut"
           << "Mesh_PolySplit" 
           << "Mesh_PolySegm" 
@@ -191,11 +223,11 @@ Gui::MenuItem* Workbench::setupMenuBar() const
     // Part ****************************************************************************************************
 
     Gui::MenuItem* part = new Gui::MenuItem(menuBar);
-    part->setCommand(QT_TR_NOOP("&Part"));
+    part->setCommand("&Part");
 
     // submenu boolean
     Gui::MenuItem* para = new Gui::MenuItem();
-    para->setCommand(QT_TR_NOOP("Parametric"));
+    para->setCommand("Parametric");
     *para << "Part_Box"
           << "Part_Cylinder"
           << "Part_Sphere"
@@ -210,16 +242,38 @@ Gui::MenuItem* Workbench::setupMenuBar() const
           << "Part_Boolean"
           << "Part_Extrude"
           << "Part_Revolve"
+          << "Part_Mirror"
           << "Part_Fillet"
+          << "PartDesign_Chamfer"
           << "Separator"
           << "Part_ShapeInfo";
 
 #   ifdef COMPLETE_SHOW_SKETCHER
     if (mgr.getCommandByName("Sketcher_NewSketch")) {
         Gui::MenuItem* sketch = new Gui::MenuItem(menuBar);
-        sketch->setCommand(QT_TR_NOOP("Ske&tch"));
+        sketch->setCommand("Ske&tch");
+        Gui::MenuItem* geom = new Gui::MenuItem();
+        geom->setCommand("Sketcher geoms");
+        *geom /*<< "Sketcher_CreatePoint"*/
+              /*<< "Sketcher_CreateArc"*/
+              /*<< "Sketcher_CreateCircle"*/
+              << "Sketcher_CreateLine"
+              << "Sketcher_CreatePolyline"
+              << "Sketcher_CreateBox"
+              /*<< "Sketcher_CreateText"*/
+              /*<< "Sketcher_CreateDraftLine"*/;
+        Gui::MenuItem* cons = new Gui::MenuItem();
+        cons->setCommand("Sketcher constraints");
+        *cons /*<< "Sketcher_ConstrainLock"*/
+              << "Sketcher_ConstrainCoincident"
+              << "Sketcher_ConstrainVertical"
+              << "Sketcher_ConstrainHorizontal"
+              << "Sketcher_ConstrainDistance"
+              << "Sketcher_ConstrainParallel";
         *sketch 
             << "Sketcher_NewSketch"
+            << geom
+            << cons
             << "Separator" 
             << "PartDesign_Pad" 
             << "PartDesign_Fillet"
@@ -231,7 +285,7 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     Gui::MenuItem* drawing = new Gui::MenuItem(menuBar);
 
-    drawing->setCommand(QT_TR_NOOP("&Drawing"));
+    drawing->setCommand("&Drawing");
     *drawing
         << "Drawing_Open" 
         << "Separator" 
@@ -244,7 +298,7 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     Gui::MenuItem* raytracing = new Gui::MenuItem(menuBar);
 
-    raytracing->setCommand(QT_TR_NOOP("&Raytracing"));
+    raytracing->setCommand("&Raytracing");
     *raytracing 
         << "Raytracing_WriteView" 
         << "Raytracing_WriteCamera" 
@@ -255,15 +309,26 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 #   ifdef COMPLETE_USE_DRAFTING
     if (mgr.getCommandByName("Draft_Line")) {
         Gui::MenuItem* Drafting = new Gui::MenuItem(menuBar);
+        Drafting->setCommand("&Drafting");
+
+        Gui::MenuItem* DraftContext = new Gui::MenuItem();
+        DraftContext->setCommand("Object appearence");
+
+        *DraftContext << "Draft_ApplyStyle" << "Draft_ToggleDisplayMode";
+
+        Gui::MenuItem* DraftWireTools = new Gui::MenuItem();
+        DraftWireTools->setCommand("Wire Tools");
+
+        *DraftWireTools << "Draft_WireToBSpline" << "Draft_AddPoint" << "Draft_DelPoint";
 
-        Drafting->setCommand(QT_TR_NOOP("&Drafting"));
         *Drafting 
-            << "Draft_SelectPlane"
             << "Draft_Line"
             << "Draft_Wire"
             << "Draft_Circle"
             << "Draft_Arc"
             << "Draft_Rectangle"
+            << "Draft_Polygon"
+            << "Draft_BSpline"
             << "Draft_Text"
             << "Draft_Dimension"
             << "Separator" 
@@ -273,9 +338,11 @@ Gui::MenuItem* Workbench::setupMenuBar() const
             << "Draft_Trimex" 
             << "Draft_Upgrade" 
             << "Draft_Downgrade" 
-            << "Draft_Scale" 
-            << "Separator" 
-            << "Draft_ApplyStyle"
+            << "Draft_Scale"
+            << "Draft_Edit"
+            << "Draft_Drawing"
+            << DraftWireTools
+            << DraftContext
         ;
     }
 #   endif
@@ -285,7 +352,7 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     // Windows
     Gui::MenuItem* wnd = new Gui::MenuItem( menuBar );
-    wnd->setCommand(QT_TR_NOOP("&Windows"));
+    wnd->setCommand("&Windows");
     *wnd << "Std_ActivateNextWindow" << "Std_ActivatePrevWindow" << "Separator"
          << "Std_TileWindows" << "Std_CascadeWindows"
          << "Std_ArrangeIcons" << "Separator" << "Std_WindowsMenu" << "Std_Windows";
@@ -297,13 +364,13 @@ Gui::MenuItem* Workbench::setupMenuBar() const
 
     // Help
     Gui::MenuItem* helpWebsites = new Gui::MenuItem;
-    helpWebsites->setCommand(QT_TR_NOOP("&Online-help"));
+    helpWebsites->setCommand("&Online-help");
     *helpWebsites << "Std_OnlineHelpWebsite" 
                   << "Std_FreeCADWebsite" 
                   << "Std_PythonWebsite";
 
     Gui::MenuItem* help = new Gui::MenuItem( menuBar );
-    help->setCommand(QT_TR_NOOP("&Help"));
+    help->setCommand("&Help");
     *help << "Std_OnlineHelp" 
           << "Std_OnlineHelpPython" 
           << "Std_PythonHelp"
@@ -327,7 +394,7 @@ Gui::ToolBarItem* Workbench::setupToolBars() const
 
     // File
     Gui::ToolBarItem* file = new Gui::ToolBarItem( root );
-    file->setCommand(QT_TR_NOOP("File"));
+    file->setCommand("File");
     *file << "Std_New" 
           << "Std_Open" 
           << "Std_Save" 
@@ -347,20 +414,20 @@ Gui::ToolBarItem* Workbench::setupToolBars() const
 
     // Macro
     Gui::ToolBarItem* macro = new Gui::ToolBarItem( root );
-    macro->setCommand(QT_TR_NOOP("Macro"));
+    macro->setCommand("Macro");
     *macro << "Std_DlgMacroRecord" << "Std_MacroStopRecord" << "Std_DlgMacroExecute"
            << "Std_DlgMacroExecuteDirect";
 
     // View
     Gui::ToolBarItem* view = new Gui::ToolBarItem( root );
-    view->setCommand(QT_TR_NOOP("View"));
+    view->setCommand("View");
     *view << "Std_ViewFitAll" << "Separator" << "Std_ViewAxo" << "Separator" << "Std_ViewFront" 
           << "Std_ViewRight" << "Std_ViewTop" << "Separator" << "Std_ViewRear" << "Std_ViewLeft" 
           << "Std_ViewBottom";
 
     // Part Design
     Gui::ToolBarItem* part_design = new Gui::ToolBarItem( root );
-    part_design->setCommand(QT_TR_NOOP("Part design"));
+    part_design->setCommand("Part design");
     *part_design
         << "Part_Box"
         << "Part_Cylinder"
@@ -377,7 +444,9 @@ Gui::ToolBarItem* Workbench::setupToolBars() const
         << "Separator"
         << "Part_Extrude"
         << "Part_Revolve"
+        << "Part_Mirror"
         << "Part_Fillet"
+        << "PartDesign_Chamfer"
     ;
 
     // Sketch based
@@ -385,7 +454,7 @@ Gui::ToolBarItem* Workbench::setupToolBars() const
 #   ifdef COMPLETE_SHOW_SKETCHER
     if (mgr.getCommandByName("Sketcher_NewSketch")) {
         Gui::ToolBarItem* sketch_based = new Gui::ToolBarItem( root );
-        sketch_based->setCommand(QT_TR_NOOP("Sketch based"));
+        sketch_based->setCommand("Sketch based");
         *sketch_based 
             << "Sketcher_NewSketch"
             << "Separator"
@@ -397,7 +466,7 @@ Gui::ToolBarItem* Workbench::setupToolBars() const
 
     // Drawing
     Gui::ToolBarItem* drawing = new Gui::ToolBarItem( root );
-    drawing->setCommand(QT_TR_NOOP("Drawings"));
+    drawing->setCommand("Drawings");
     *drawing << "Drawing_Open" 
              << "Separator" 
              << "Drawing_NewA3Landscape"  
@@ -406,7 +475,7 @@ Gui::ToolBarItem* Workbench::setupToolBars() const
 
     // Raytracing
     Gui::ToolBarItem* raytracing = new Gui::ToolBarItem( root );
-    raytracing->setCommand(QT_TR_NOOP("Raytracing"));
+    raytracing->setCommand("Raytracing");
     *raytracing << "Raytracing_WriteView" 
                 << "Raytracing_WriteCamera" 
                 << "Raytracing_WritePart";
@@ -415,27 +484,30 @@ Gui::ToolBarItem* Workbench::setupToolBars() const
 #   ifdef COMPLETE_USE_DRAFTING
     if (mgr.getCommandByName("Draft_Line")) {
         Gui::ToolBarItem* Drafting = new Gui::ToolBarItem( root );
-        Drafting->setCommand(QT_TR_NOOP("Drafting"));
+        Drafting->setCommand("Drafting");
         *Drafting
-            << "Draft_SelectPlane"
             << "Draft_Line"
             << "Draft_Wire"
             << "Draft_Circle"
             << "Draft_Arc"
             << "Draft_Rectangle"
+            << "Draft_Polygon"
+            << "Draft_BSpline"
             << "Draft_Text"
             << "Draft_Dimension"
             << "Separator" 
             << "Draft_Move" 
             << "Draft_Rotate" 
             << "Draft_Offset" 
-
             << "Draft_Trimex" 
             << "Draft_Upgrade" 
             << "Draft_Downgrade" 
-            << "Draft_Scale" 
-            << "Separator" 
-            << "Draft_ApplyStyle"
+            << "Draft_Scale"
+            << "Draft_Edit"
+            << "Draft_Drawing"
+            << "Draft_WireToBSpline"
+            << "Draft_AddPoint"
+            << "Draft_DelPoint"
         ;
     }
 #   endif
