@@ -30,39 +30,13 @@ This is the GUI part of the Draft module.
 Report to Draft.py for info
 '''
 
-import FreeCAD, FreeCADGui, os
+import FreeCAD, FreeCADGui, os, Draft
 
 try:
 	from PyQt4 import QtCore,QtGui,QtSvg	
 except:
 	FreeCAD.Console.PrintMessage("Error: Python-qt4 package must be installed on your system to use the Draft module.")
 
-def draftPath():
-	"returns the current Draft module path"
-	path1 = FreeCAD.ConfigGet("AppHomePath") + "Mod/Draft"
-	path2 = FreeCAD.ConfigGet("UserAppData") + "Mod/Draft"
-	if os.path.exists(path2): return path2
-	else: return path1
-
-def findicons():
-	"loads the icon file"
-	filepath = draftPath() + os.sep + "icons.svg"
-	iconmap = QtGui.QPixmap()
-	if not iconmap.load(filepath):
-		# If loading by plug-in fails do it the conventional way
-		file=QtCore.QFile(filepath)
-		if not file.open(QtCore.QFile.ReadOnly):
-			raise Exception("Cannot open file %s" % (filepath))
-		ba=file.readAll()
-		render=QtSvg.QSvgRenderer(ba)
-		image = QtGui.QImage(render.viewBox().size(),QtGui.QImage.Format_ARGB32_Premultiplied)
-		image.fill(0x00000000)
-		painter=QtGui.QPainter(image)
-		render.render(painter)
-		painter.end()
-		iconmap = QtGui.QPixmap.fromImage(image)
-	return iconmap
-	
 def getMainWindow():
 	"returns the main window"
 	# using QtGui.qApp.activeWindow() isn't very reliable because if another
@@ -74,9 +48,37 @@ def getMainWindow():
 			return i
 	raise Exception("No main window found")
 
+class todo:
+	''' static todo class, delays execution of functions.  Use todo.delay
+	to schedule geometry manipulation that would crash coin if done in the
+	event callback'''
+
+	'''List of (function, argument) pairs to be executed by
+	QTcore.QTimer.singleShot(0,doTodo).'''
+	itinerary = []
+
+	@staticmethod
+	def doTasks():
+		for f, arg in todo.itinerary:
+			try:
+                                # print "debug: executing",f
+                                f(arg)
+			except:
+				wrn = "[Draft.todo] Unexpected error:" + sys.exc_info()[0]
+				FreeCAD.Console.PrintWarning (wrn)
+		todo.itinerary = []
+
+	@staticmethod
+	def delay (f, arg):
+                # print "debug: delaying",f
+		if todo.itinerary == []:
+			QtCore.QTimer.singleShot(0, todo.doTasks)
+		todo.itinerary.append((f,arg))
+
 def translate(context,text):
         "convenience function for Qt translator"
-        return QtGui.QApplication.translate(context, text, None, QtGui.QApplication.UnicodeUTF8)
+        return QtGui.QApplication.translate(context, text, None,
+                                            QtGui.QApplication.UnicodeUTF8)
 
 #---------------------------------------------------------------------------
 # Customized widgets
@@ -88,6 +90,11 @@ class DraftDockWidget(QtGui.QWidget):
 		QtGui.QDockWidget.__init__(self,parent)
 	def resizeEvent(self,event):
 		self.emit(QtCore.SIGNAL("resized()"))
+        def changeEvent(self, event):
+                if event.type() == QtCore.QEvent.LanguageChange:
+                        self.emit(QtCore.SIGNAL("retranslate()"))
+                else:
+                        QtGui.QWidget.changeEvent(self,event)
 
 class DraftLineEdit(QtGui.QLineEdit):
 	"custom QLineEdit widget that has the power to catch Escape keypress"
@@ -102,7 +109,7 @@ class DraftLineEdit(QtGui.QLineEdit):
 			self.emit(QtCore.SIGNAL("down()"))
 		elif (event.key() == QtCore.Qt.Key_Z) and QtCore.Qt.ControlModifier:
 			self.emit(QtCore.SIGNAL("undo()"))
-		else:
+                else:
 			QtGui.QLineEdit.keyPressEvent(self, event)
 
 class toolBar:
@@ -119,88 +126,129 @@ class toolBar:
 				self.params = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
 				paramcolor = self.params.GetUnsigned("color")>>8
 				paramlinewidth = self.params.GetInt("linewidth")
+                                paramfontsize = self.params.GetFloat("textheight")
 				paramconstr = self.params.GetUnsigned("constructioncolor")>>8
-				icons = findicons()
 				self.constrMode = False
 				draftToolbar.setObjectName("draftToolbar")
 				self.state = None
 				self.textbuffer = []
 				self.draftToolbar = draftToolbar
-
-				def _pushButton (name, layout, hide=True, icon=None, width=60):
-					button = QtGui.QPushButton(draftToolbar)
-					#button.setGeometry(geometry)
-					button.setObjectName(name)
-                                        button.setMaximumSize(QtCore.QSize(width,22))
-					if hide: button.hide()
-                                        if icon:
-                                                button.setIcon(QtGui.QIcon(icons.copy(QtCore.QRect(icon[0],icon[1],64,64))))
-                                                button.setIconSize(QtCore.QSize(16, 16))
-                                        layout.addWidget(button)
-					return button
-
-                                def _label (name, layout, hide=True):
-                                        label = QtGui.QLabel(draftToolbar)
-                                        label.setObjectName(name)
-                                        if hide: label.hide()
-                                        layout.addWidget(label)
-                                        return label
-
-                                def _lineedit (name, layout, hide=True, width=None):
-                                        lineedit = DraftLineEdit(draftToolbar)
-                                        lineedit.setObjectName(name)
-                                        if hide: lineedit.hide()
-                                        if not width: width = 800
-                                        lineedit.setMaximumSize(QtCore.QSize(width,18))
-                                        layout.addWidget(lineedit)
-                                        return lineedit
-
+                                self.crossedViews = []
+                                
                                 self.layout = QtGui.QBoxLayout(QtGui.QBoxLayout.LeftToRight,draftToolbar)
                                 self.layout.setDirection(QtGui.QBoxLayout.LeftToRight)
                                 self.layout.setObjectName("layout")
 
-                                self.promptlabel = _label("promptlabel",self.layout)
-				self.cmdlabel = _label("cmdlabel",self.layout, hide=False)
+				def _pushButton (name, hide=True, icon=None, width=66):
+					button = QtGui.QPushButton(draftToolbar)
+					#button.setGeometry(geometry)
+					button.setObjectName(name)
+                                        button.setMaximumSize(QtCore.QSize(width,26))
+					if hide: button.hide()
+                                        if icon:
+                                                button.setIcon(QtGui.QIcon(':/icons/'+icon+'.svg'))
+                                                button.setIconSize(QtCore.QSize(16, 16))
+                                        self.layout.addWidget(button)
+					return button
+
+                                def _label (name, hide=True):
+                                        label = QtGui.QLabel(draftToolbar)
+                                        label.setObjectName(name)
+                                        if hide: label.hide()
+                                        self.layout.addWidget(label)
+                                        return label
+
+                                def _lineedit (name, hide=True, width=None):
+                                        lineedit = DraftLineEdit(draftToolbar)
+                                        lineedit.setObjectName(name)
+                                        if hide: lineedit.hide()
+                                        if not width: width = 800
+                                        lineedit.setMaximumSize(QtCore.QSize(width,22))
+                                        self.layout.addWidget(lineedit)
+                                        return lineedit
+
+                                def _spinbox (name, val=None, vmax=None, hide=True, double=False, size=None):
+                                        if double:
+                                                sbox = QtGui.QDoubleSpinBox(draftToolbar)
+                                                sbox.setDecimals(2)
+                                        else:
+                                                sbox = QtGui.QSpinBox(draftToolbar)
+                                        sbox.setObjectName(name)
+                                        if val: sbox.setValue(val)
+                                        if vmax: sbox.setMaximum(vmax)
+                                        if size: sbox.setMaximumSize(QtCore.QSize(size[0],size[1]))
+                                        if hide: sbox.hide()
+                                        self.layout.addWidget(sbox)
+                                        return sbox
+
+                                def _combo (name, hide=True, editable=True, width=None):
+                                        cbox = QtGui.QComboBox(draftToolbar)
+                                        cbox.setObjectName(name)
+                                        if editable: cbox.setEditable(True)
+                                        if hide: cbox.hide()
+                                        if width: cbox.setMinimumSize(QtCore.QSize(width,22))
+                                        self.layout.addWidget(cbox)
+                                        return cbox
+
+                                def _checkbox (name, checked=True, hide=True):
+                                        chk = QtGui.QCheckBox(draftToolbar)
+                                        chk.setChecked(checked)
+                                        chk.setObjectName(name)
+                                        if hide: chk.hide()
+                                        self.layout.addWidget(chk)
+                                        return chk
+
+                                # command
+
+                                self.promptlabel = _label("promptlabel", hide=False)
+				self.cmdlabel = _label("cmdlabel", hide=False)
 				boldtxt = QtGui.QFont()
 				boldtxt.setWeight(75)
 				boldtxt.setBold(True)
 				self.cmdlabel.setFont(boldtxt)
 
-				self.labelx = _label("labelx", self.layout)
-				self.xValue = _lineedit("xValue",self.layout,width=60)
+                                # point
+
+				self.labelx = _label("labelx")
+				self.xValue = _lineedit("xValue", width=60)
 				self.xValue.setText("0.00")
-				self.labely = _label("labely",self.layout)
-				self.yValue = _lineedit("yValue",self.layout,width=60)
+                                #self.xValue.setInputMask("0.00")
+				self.labely = _label("labely")
+				self.yValue = _lineedit("yValue", width=60)
 				self.yValue.setText("0.00")
-				self.labelz = _label("labelz",self.layout)
-				self.zValue = _lineedit("zValue",self.layout,width=60)
+				self.labelz = _label("labelz")
+				self.zValue = _lineedit("zValue", width=60)
 				self.zValue.setText("0.00")
 
-				self.offsetLabel = _label("offsetlabel",self.layout)
-				self.offsetValue = _lineedit("offsetValue",self.layout,width=60)
+                                # options
+
+                                self.numFaces = _spinbox("numFaces",3)
+
+				self.offsetLabel = _label("offsetlabel")
+				self.offsetValue = _lineedit("offsetValue", width=60)
 				self.offsetValue.setText("0.00")
 
-				self.isRelative = QtGui.QCheckBox(draftToolbar)
-				self.isRelative.setChecked(True)
-				self.isRelative.setObjectName("isRelative")
-                                self.layout.addWidget(self.isRelative)
-				self.isRelative.hide()
-                                
-				self.undoButton = _pushButton("undoButton",self.layout,icon=(64,64))
-				self.finishButton = _pushButton("finishButton",self.layout,icon=(448,64))
-				self.closeButton = _pushButton("closeButton",self.layout,icon=(512,64))
-                                
-				self.xyButton = _pushButton("xyButton",self.layout)
-				self.xzButton = _pushButton("xzButton",self.layout)
-				self.yzButton = _pushButton("yzButton",self.layout)
-				self.currentViewButton = _pushButton("view",self.layout)
-				self.resetPlaneButton = _pushButton("none",self.layout)
-
-				self.labelRadius = _label("labelRadius",self.layout)
-				self.radiusValue = _lineedit("radiusValue",self.layout,width=60)
+                                self.labelRadius = _label("labelRadius")
+				self.radiusValue = _lineedit("radiusValue", width=60)
 				self.radiusValue.setText("0.00")
 
-				self.textValue = _lineedit("textValue",self.layout)
+				self.isRelative = _checkbox("isRelative",checked=True)
+
+                                self.hasFill = _checkbox("hasFill",checked=self.params.GetBool("fillmode"))
+
+                                self.continueCmd = _checkbox("continueCmd",checked=False)
+                                
+				self.undoButton = _pushButton("undoButton", icon='Draft_Rotate')
+				self.finishButton = _pushButton("finishButton", icon='Draft_Finish')
+				self.closeButton = _pushButton("closeButton", icon='Draft_Lock')
+                                
+				self.xyButton = _pushButton("xyButton")
+				self.xzButton = _pushButton("xzButton")
+				self.yzButton = _pushButton("yzButton")
+				self.currentViewButton = _pushButton("view")
+				self.resetPlaneButton = _pushButton("none")
+
+				self.textValue = _lineedit("textValue")
 
                                 self.isCopy = QtGui.QCheckBox(draftToolbar)
 				self.isCopy.setChecked(False)
@@ -208,45 +256,16 @@ class toolBar:
                                 self.layout.addWidget(self.isCopy)
 				self.isCopy.hide()
 
-                                self.labelPage = _label("labelPage",self.layout)
-                                self.pageBox = QtGui.QComboBox(draftToolbar)
-                                self.pageBox.setEditable(True)
-                                self.pageBox.setObjectName("pageBox")
-                                self.pageBox.addItem("Add New")
-                                self.layout.addWidget(self.pageBox)
-                                self.pageBox.hide()
-                                self.labelScale = _label("labelScale",self.layout)
-                                self.scaleBox = QtGui.QComboBox(draftToolbar)
-                                self.scaleBox.setEditable(True)
-                                self.scaleBox.setObjectName("scaleBox")
-                                self.scaleBox.addItem("5")
-                                self.scaleBox.addItem("10")
-                                self.scaleBox.addItem("20")
-                                self.scaleBox.addItem("50")
-                                self.layout.addWidget(self.scaleBox)
-                                self.scaleBox.hide()
-                                self.labelMargin = _label("labelMargin",self.layout)
-                                self.marginValue = QtGui.QSpinBox(draftToolbar)
-				self.marginValue.setObjectName("marginValue")
-                                self.marginValue.setMaximum(999)
-				self.marginValue.setValue(50)
-                                self.layout.addWidget(self.marginValue)
-				self.marginValue.hide()
-                                self.labelLWMod = _label("labelLWMod",self.layout)
-                                self.LWModValue = QtGui.QSpinBox(draftToolbar)
-				self.LWModValue.setObjectName("LWModValue")
-                                self.LWModValue.setMaximum(999)
-				self.LWModValue.setValue(100)
-                                self.layout.addWidget(self.LWModValue)
-				self.LWModValue.hide()
-                                self.pageButton = _pushButton("pageButton",self.layout,icon=(640,128))
+                                # spacer
 
                                 spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding,
                                                                QtGui.QSizePolicy.Minimum)
                                 self.layout.addItem(spacerItem)
 
-                                self.wplabel = _label("wplabel",self.layout, hide=False)
-                                defaultWP = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft").GetInt("defaultWP")
+                                # settings buttons
+
+                                self.wplabel = _pushButton("wplabel", icon='Draft_SelectPlane',hide=False,width=120)
+                                defaultWP = self.params.GetInt("defaultWP")
                                 if defaultWP == 1:
                                         self.wplabel.setText("Top")
                                 elif defaultWP == 2:
@@ -255,28 +274,30 @@ class toolBar:
                                         self.wplabel.setText("Side")
                                 else:
                                         self.wplabel.setText("None")
-                                self.constrButton = _pushButton("constrButton", self.layout, hide=False, icon=(640,64),width=22)
+                                self.constrButton = _pushButton("constrButton", hide=False, icon='Draft_Construction',width=22)
 				self.constrButton.setCheckable(True)
 				self.constrButton.setChecked(False)
 				self.constrColor = QtGui.QColor(paramconstr)
-                                style = "#constrButton:Checked {background-color: "
-                                style += self.getDefaultColor("constr",rgb=True)+" }"
-				draftToolbar.setStyleSheet(style)
-				
-				self.colorButton = _pushButton("colorButton",self.layout, hide=False,width=22)
+				self.colorButton = _pushButton("colorButton",hide=False,width=22)
 				self.color = QtGui.QColor(paramcolor)
 				self.colorPix = QtGui.QPixmap(16,16)
 				self.colorPix.fill(self.color)
 				self.colorButton.setIcon(QtGui.QIcon(self.colorPix))
+                                self.facecolorButton = _pushButton("facecolorButton",hide=False,width=22)
+				self.facecolor = QtGui.QColor(204,204,204)
+				self.facecolorPix = QtGui.QPixmap(16,16)
+				self.facecolorPix.fill(self.facecolor)
+				self.facecolorButton.setIcon(QtGui.QIcon(self.facecolorPix))
+				self.widthButton = _spinbox("widthButton",val=paramlinewidth,hide=False,size=(50,22))
+                                self.widthButton.setSuffix("px")
+                                self.fontsizeButton = _spinbox("fontsizeButton",val=paramfontsize,hide=False,double=True,size=(50,22))
+				self.applyButton = _pushButton("applyButton", hide=False, icon='Draft_Apply',width=22)
 
-				self.widthButton = QtGui.QSpinBox(draftToolbar)
-                                self.widthButton.setMaximumSize(QtCore.QSize(50,22))
-				self.widthButton.setObjectName("widthButton")
-				self.widthButton.setValue(paramlinewidth)
-                                self.layout.addWidget(self.widthButton)
-
-				self.applyButton = _pushButton("applyButton", self.layout, hide=False, icon=(384,64),width=22)
-				
+                                style = "#pageWpButton:Checked {background-color:rgb(255,0,0)} "
+                                style += "#constrButton:Checked {background-color: "
+                                style += self.getDefaultColor("constr",rgb=True)+" }"
+				draftToolbar.setStyleSheet(style)
+                                
 				self.sourceCmd=None
 
 				self.retranslateUi(draftToolbar)
@@ -313,19 +334,20 @@ class toolBar:
 				QtCore.QObject.connect(self.zValue,QtCore.SIGNAL("undo()"),self.undoSegment)
 				QtCore.QObject.connect(self.radiusValue,QtCore.SIGNAL("escaped()"),self.finish)
 				QtCore.QObject.connect(self.draftToolbar,QtCore.SIGNAL("resized()"),self.relocate)
+                                QtCore.QObject.connect(self.draftToolbar,QtCore.SIGNAL("retranslate()"),self.retranslateUi)
+                                QtCore.QObject.connect(self.wplabel,QtCore.SIGNAL("pressed()"),self.selectplane)
 				QtCore.QObject.connect(self.colorButton,QtCore.SIGNAL("pressed()"),self.getcol)
+                                QtCore.QObject.connect(self.facecolorButton,QtCore.SIGNAL("pressed()"),self.getfacecol)
 				QtCore.QObject.connect(self.widthButton,QtCore.SIGNAL("valueChanged(int)"),self.setwidth)
+                                QtCore.QObject.connect(self.fontsizeButton,QtCore.SIGNAL("valueChanged(double)"),self.setfontsize)
 				QtCore.QObject.connect(self.applyButton,QtCore.SIGNAL("pressed()"),self.apply)
 				QtCore.QObject.connect(self.constrButton,QtCore.SIGNAL("toggled(bool)"),self.toggleConstrMode)
-                                QtCore.QObject.connect(self.pageButton,QtCore.SIGNAL("pressed()"),self.drawPage)
-				QtCore.QMetaObject.connectSlotsByName(draftToolbar)
 
 #---------------------------------------------------------------------------
 # language tools
 #---------------------------------------------------------------------------
 				
-			def retranslateUi(self, draftToolbar):
-                                
+			def retranslateUi(self, draftToolbar=self):
                                 self.promptlabel.setText(translate("draft", "active command:"))
 				self.cmdlabel.setText(translate("draft", "None"))
 				self.cmdlabel.setToolTip(translate("draft", "Active Draft command"))
@@ -337,14 +359,17 @@ class toolBar:
 				self.zValue.setToolTip(translate("draft", "Z coordinate of next point"))
 				self.labelRadius.setText(translate("draft", "Radius"))
 				self.radiusValue.setToolTip(translate("draft", "Radius of Circle"))
-				self.isRelative.setText(translate("draft", "Relative"))
+				self.isRelative.setText(translate("draft", "&Relative"))
 				self.isRelative.setToolTip(translate("draft", "Coordinates relative to last point or absolute (SPACE)"))
-				self.finishButton.setText(translate("draft", "Finish"))
+                                self.hasFill.setText(translate("draft", "&Filled"))
+                                self.hasFill.setToolTip(translate("draft", "Check this if the object should appear as filled, otherwise it will appear as wireframe (F)"))
+				self.finishButton.setText(translate("draft", "F&inish"))
 				self.finishButton.setToolTip(translate("draft", "Finishes the current line without closing (F)"))
-				self.undoButton.setText(translate("draft", "Undo"))
+				self.undoButton.setText(translate("draft", "&Undo"))
 				self.undoButton.setToolTip(translate("draft", "Undo the last segment (CTRL+Z)"))
-				self.closeButton.setText(translate("draft", "Close"))
+				self.closeButton.setText(translate("draft", "&Close"))
 				self.closeButton.setToolTip(translate("draft", "Finishes and closes the current line (C)"))
+                                self.numFaces.setToolTip(translate("draft", "Number of sides"))
                                 self.offsetLabel.setText(translate("draft", "Offset"))
 				self.xyButton.setText(translate("draft", "XY"))
 				self.xyButton.setToolTip(translate("draft", "Select XY plane"))
@@ -356,22 +381,16 @@ class toolBar:
 				self.currentViewButton.setToolTip(translate("draft", "Select plane perpendicular to the current view"))
 				self.resetPlaneButton.setText(translate("draft", "None"))
 				self.resetPlaneButton.setToolTip(translate("draft", "Do not project points to a drawing plane"))
-				self.widthButton.setSuffix(translate("draft", "px"))
-				self.isCopy.setText(translate("draft", "Copy"))
+				self.isCopy.setText(translate("draft", "&Copy"))
 				self.isCopy.setToolTip(translate("draft", "If checked, objects will be copied instead of moved (C)"))
-				self.colorButton.setToolTip(translate("draft", "Current line color for new objects"))
-				self.widthButton.setToolTip(translate("draft", "Current line width for new objects"))
+				self.colorButton.setToolTip(translate("draft", "Line Color"))
+                                self.continueCmd.setToolTip(translate("draft", "If checked, command will not finish until you press the command button again"))
+                                self.continueCmd.setText(translate("draft", "&Continue"))
+                                self.facecolorButton.setToolTip(translate("draft", "Face Color"))
+				self.widthButton.setToolTip(translate("draft", "Line Width"))
+                                self.fontsizeButton.setToolTip(translate("draft", "Font Size"))
 				self.applyButton.setToolTip(translate("draft", "Apply to selected objects"))
 				self.constrButton.setToolTip(translate("draft", "Toggles Construction Mode"))
-                                self.wplabel.setToolTip(translate("draft", "Current working plane"))
-                                self.labelPage.setText(translate("draft", "Page"))
-                                self.labelScale.setText(translate("draft", "Scale"))
-                                self.labelMargin.setText(translate("draft", "Margin"))
-                                self.labelLWMod.setText(translate("draft", "LW Mod"))
-                                self.pageBox.setToolTip(translate("draft", "Page to draw to. If selecting Add New, you can edit the name"))
-                                self.scaleBox.setToolTip(translate("draft", "Scale factor to apply. Drawing page is always in millimeters. So if you draw in meters, a scale of 10 means 1:100"))
-                                self.marginValue.setToolTip(translate("draft", "The value of the margin (offset between the (0,0) point and the page border)"))
-                                self.LWModValue.setToolTip(translate("draft", "Linewidth scale modifier. Lower values mean thicker lines."))
 
 #---------------------------------------------------------------------------
 # Interface modes
@@ -390,6 +409,7 @@ class toolBar:
 			def lineUi(self):
 				self.cmdlabel.setText(translate("draft", "Line"))
 				self.isRelative.show()
+                                self.hasFill.show()
 				self.finishButton.show()
 				self.closeButton.show()
 				self.undoButton.show()
@@ -399,6 +419,7 @@ class toolBar:
 				self.cmdlabel.setText(translate("draft", "Circle"))
 				self.pointUi()
 				self.labelx.setText(translate("draft", "Center X"))
+				self.hasFill.show()
 
 			def arcUi(self):
 				self.cmdlabel.setText(translate("draft", "Arc"))
@@ -427,7 +448,9 @@ class toolBar:
 				self.xValue.hide()
 				self.yValue.hide()
 				self.zValue.hide()
+                                self.numFaces.hide()
 				self.isRelative.hide()
+                                self.hasFill.hide()
 				self.finishButton.hide()
 				self.undoButton.hide()
 				self.closeButton.hide()
@@ -442,15 +465,7 @@ class toolBar:
 				self.radiusValue.hide()
 				self.isCopy.hide()
 				self.textValue.hide()
-                                self.pageBox.hide()
-                                self.labelPage.hide()
-                                self.scaleBox.hide()
-                                self.labelScale.hide()
-                                self.marginValue.hide()
-                                self.labelMargin.hide()
-                                self.pageButton.hide()
-                                self.labelLWMod.hide()
-                                self.LWModValue.hide()
+                                self.continueCmd.hide()
 
 			def radiusUi(self):
 				self.labelx.hide()
@@ -462,6 +477,7 @@ class toolBar:
 				self.labelRadius.setText(translate("draft", "Radius"))
 				self.labelRadius.show()
 				self.radiusValue.show()
+                                self.hasFill.show()
 
 			def textUi(self):
 				self.labelx.hide()
@@ -505,16 +521,17 @@ class toolBar:
 				self.labelx.setText(translate("draft", "Pick Object"))
 				self.labelx.show()
 
-                        def pageUi(self):
-				self.labelPage.show()
-                                self.pageBox.show()
-                                self.labelScale.show()
-                                self.scaleBox.show()
-                                self.marginValue.show()
-                                self.labelMargin.show()
-                                self.pageButton.show()
-                                self.labelLWMod.show()
-                                self.LWModValue.show()
+                        def editUi(self):
+				self.labelx.hide()
+				self.labely.hide()
+				self.labelz.hide()
+				self.xValue.hide()
+				self.yValue.hide()
+				self.zValue.hide()
+                                self.numFaces.hide()
+				self.isRelative.hide()
+                                self.hasFill.hide()
+				self.finishButton.show()
 
 			def relocate(self):
 				"relocates the right-aligned buttons depending on the toolbar size"
@@ -544,28 +561,41 @@ class toolBar:
 					if (i.Type == "App::Annotation"):
 						i.ViewObject.TextColor=col
 					else:
-						i.ViewObject.LineColor = col
+                                                if "LineColor" in i.ViewObject.PropertiesList:
+                                                        i.ViewObject.LineColor = col
+                                                if "PointColor" in i.ViewObject.PropertiesList:
+                                                        i.ViewObject.PointColor = col
+
+                        def getfacecol(self):
+				"opens a color picker dialog"
+				self.facecolor=QtGui.QColorDialog.getColor()
+				self.facecolorPix.fill(self.facecolor)
+				self.facecolorButton.setIcon(QtGui.QIcon(self.facecolorPix))
+                                r = float(self.facecolor.red()/255.0)
+				g = float(self.facecolor.green()/255.0)
+				b = float(self.facecolor.blue()/255.0)
+				col = (r,g,b,0.0)
+                                for i in FreeCADGui.Selection.getSelection():
+                                        if "ShapeColor" in i.ViewObject.PropertiesList:
+						i.ViewObject.ShapeColor = col
 					
 			def setwidth(self,val):
 				if self.params.GetBool("saveonexit"):
 					self.params.SetInt("linewidth",int(val))
-				lw = float(val)
 				for i in FreeCADGui.Selection.getSelection():
-					if not (i.Type == "App::Annotation"):
-						i.ViewObject.LineWidth = lw
+					if "LineWidth" in i.ViewObject.PropertiesList:
+						i.ViewObject.LineWidth = float(val)
+
+                        def setfontsize(self,val):
+				if self.params.GetBool("saveonexit"):
+					self.params.SetFloat("textheight",float(val))
+				for i in FreeCADGui.Selection.getSelection():
+					if "FontSize" in i.ViewObject.PropertiesList:
+						i.ViewObject.FontSize = float(val)
 
 			def apply(self):
-				r = float(self.color.red()/255.0)
-				g = float(self.color.green()/255.0)
-				b = float(self.color.blue()/255.0)
-				lw = float(self.widthButton.value())
-				col = (r,g,b,0.0)
 				for i in FreeCADGui.Selection.getSelection():
-					if (i.Type == "App::Annotation"):
-						i.ViewObject.TextColor=col
-					else:
-						i.ViewObject.LineColor = col
-						i.ViewObject.LineWidth = lw	
+					Draft.formatObject(i)	
 					
 			def checkx(self):
 				if self.yValue.isEnabled():
@@ -649,17 +679,29 @@ class toolBar:
 				checks for special characters in the entered coords that mut be
 				treated as shortcuts
 				'''
-				if txt.endsWith(" "):
+                                spec = False
+				if txt.endsWith(" ") or txt.endsWith("r"):
 					self.isRelative.setChecked(not self.isRelative.isChecked())
-					for i in [self.xValue,self.yValue,self.zValue]:
-						if (i.text() == txt): i.setText("")
-				if txt.endsWith("f"):
-					if self.finishButton.isVisible(): self.finish()
-				if txt.endsWith("c"):
-					if self.closeButton.isVisible(): self.closeLine()
-					elif self.isCopy.isVisible():
-						self.isCopy.setChecked(not self.isCopy.isChecked())
-
+                                        spec = True
+                                if txt.endsWith("i"):
+                                        if self.hasFill.isVisible():
+                                                self.hasFill.setChecked(not self.hasFill.isChecked())
+                                        spec = True
+                                if txt.endsWith("f"):
+                                        if self.finishButton.isVisible():
+                                                self.finish()
+                                        spec = True
+                                if txt.endsWith("c"):
+                                        if self.closeButton.isVisible():
+                                                self.closeLine()
+                                        elif self.isCopy.isVisible():
+                                                self.isCopy.setChecked(not self.isCopy.isChecked())
+                                        elif self.continueCmd.isVisible():
+                                                self.continueCmd.setChecked(not self.continueCmd.isChecked())
+                                        spec = True
+                                if spec:
+                                        for i in [self.xValue,self.yValue,self.zValue]:
+                                                if (i.text() == txt): i.setText("")
 			def sendText(self):
 				'''
 				this function sends the entered text to the active draft command
@@ -720,6 +762,10 @@ class toolBar:
 					r = float(self.color.red()/255.0)
 					g = float(self.color.green()/255.0)
 					b = float(self.color.blue()/255.0)
+                                elif type == "face":
+					r = float(self.facecolor.red()/255.0)
+					g = float(self.facecolor.green()/255.0)
+					b = float(self.facecolor.blue()/255.0)
 				elif type == "constr":
 					color = QtGui.QColor(self.params.GetUnsigned("constructioncolor")>>8)
 					r = color.red()/255.0
@@ -733,9 +779,17 @@ class toolBar:
 
 			def cross(self,on=True):
 				if on:
-					self.app.setOverrideCursor(QtCore.Qt.CrossCursor)
+					# self.app.setOverrideCursor(QtCore.Qt.CrossCursor)
+                                        if not self.crossedViews:
+                                                mw = getMainWindow()
+                                                self.crossedViews = mw.findChildren(QtGui.QFrame,"SoQtWidgetName")
+                                        for w in self.crossedViews:
+                                                w.setCursor(QtCore.Qt.CrossCursor)
 				else:
-					self.app.restoreOverrideCursor()
+					# self.app.restoreOverrideCursor()
+                                        for w in self.crossedViews:
+                                                w.unsetCursor()
+                                        self.crossedViews = []
 
 			def toggleConstrMode(self,checked):
 				self.draftToolbar.setStyleSheet("#constrButton:Checked {background-color: "+self.getDefaultColor("constr",rgb=True)+" }")
@@ -744,47 +798,33 @@ class toolBar:
                         def drawPage(self):
                                 self.sourceCmd.draw()
 
+                        def changePage(self,index):
+                                pagename = str(self.pageBox.itemText(index))
+                                vobj = FreeCADGui.ActiveDocument.getObject(pagename)
+                                if vobj:
+                                        self.scaleBox.setEditText(str(vobj.HintScale))
+                                        self.marginXValue.setValue(float(vobj.HintOffsetX))
+                                        self.marginYValue.setValue(float(vobj.HintOffsetY))
+
+                        def selectplane(self):
+                                FreeCADGui.runCommand("Draft_SelectPlane")
+
+                        def popupMenu(self,mlist):
+                                "pops up a menu filled with the given list"
+                                self.groupmenu = QtGui.QMenu()
+                                for i in mlist:
+                                        self.groupmenu.addAction(i)
+                                pos = getMainWindow().cursor().pos()
+                                self.groupmenu.popup(pos)
+                                QtCore.QObject.connect(self.groupmenu,QtCore.SIGNAL("triggered(QAction *)"),self.popupTriggered)
+
+                        def popupTriggered(self,action):
+                                self.sourceCmd.proceed(str(action.text()))
+
 #---------------------------------------------------------------------------
 # Initialization
 #---------------------------------------------------------------------------
-
-		# adding command icons to FreeCAD
-		iconfile = findicons()
-		icons = {}
-		icons['Draft_line']=iconfile.copy(QtCore.QRect(0,0,64,64))
-		icons['Draft_wire']=iconfile.copy(QtCore.QRect(64,0,64,64))
-		icons['Draft_rectangle']=iconfile.copy(QtCore.QRect(128,0,64,64))
-		icons['Draft_circle']=iconfile.copy(QtCore.QRect(196,0,64,64))
-		icons['Draft_arc']=iconfile.copy(QtCore.QRect(256,0,64,64))
-		icons['Draft_text']=iconfile.copy(QtCore.QRect(320,0,64,64))
-		icons['Draft_dimension']=iconfile.copy(QtCore.QRect(448,0,64,64))
-		icons['Draft_move']=iconfile.copy(QtCore.QRect(0,64,64,64))
-		icons['Draft_rotate']=iconfile.copy(QtCore.QRect(64,64,64,64))
-		icons['Draft_offset']=iconfile.copy(QtCore.QRect(128,64,64,64))
-		icons['Draft_trimex']=iconfile.copy(QtCore.QRect(196,64,64,64))
-		icons['Draft_upgrade']=iconfile.copy(QtCore.QRect(256,64,64,64))
-		icons['Draft_downgrade']=iconfile.copy(QtCore.QRect(320,64,64,64))
-		icons['Draft_apply']=iconfile.copy(QtCore.QRect(384,64,64,64))
-		icons['Draft_finish']=iconfile.copy(QtCore.QRect(448,64,64,64))
-		icons['Draft_lock']=iconfile.copy(QtCore.QRect(512,64,64,64))
-		icons['Draft_scale']=iconfile.copy(QtCore.QRect(576,64,64,64))
-		icons['Draft_selectPlane']=iconfile.copy(QtCore.QRect(576,0,64,64))
-                icons['Draft_sendToDrawing']=iconfile.copy(QtCore.QRect(640,0,64,64))
-		icons['preferences-draft']=iconfile.copy(QtCore.QRect(384,0,64,64))
-		for name,icon in icons.iteritems():
-			ba = QtCore.QByteArray()
-			bu = QtCore.QBuffer(ba)
-			icon.save(bu,'XPM')
-			FreeCADGui.addIcon(name,str(ba))
-
-		# loads a translation engine
-		languages = {"English":"en","French":"fr","German":"de","Italian":"it"}
-		ln = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/General").GetString("Language")
-		if ln in languages:
-			self.translator = QtCore.QTranslator()
-			self.translator.load("draft_"+languages[ln],draftPath()+os.sep+"Languages")
-			QtGui.QApplication.installTranslator(self.translator)
-			
+	
 		# create the draft Toolbar
 		self.mw = getMainWindow()
 		self.draftWidget = QtGui.QDockWidget()
@@ -799,3 +839,8 @@ class toolBar:
 		self.draftWidget.setVisible(False)
 		self.draftWidget.toggleViewAction().setVisible(False)
 
+        def changeEvent(self, event):
+                if event.type() == QtCore.QEvent.LanguageChange:
+                        print "Language changed!"
+                        self.ui.retranslateUi(self)
+
