@@ -27,7 +27,7 @@ __url__ = ["http://free-cad.sourceforge.net"]
 
 "this file contains generic geometry functions for manipulating Part shapes"
 
-import FreeCAD, Part, fcvec, math, cmath
+import FreeCAD, Part, fcvec, math, cmath, FreeCADGui
 from FreeCAD import Vector
 
 NORM = Vector(0,0,1) # provisory normal direction for all geometry ops.
@@ -49,17 +49,24 @@ def vec(edge):
 		return None
 
 def edg(p1,p2):
-	"edge(Vector,Vector) -- returns an edge from 2 vectors"
+	"edg(Vector,Vector) -- returns an edge from 2 vectors"
 	if isinstance(p1,FreeCAD.Vector) and isinstance(p2,FreeCAD.Vector):
 		if fcvec.equals(p1,p2): return None
 		else: return Part.Line(p1,p2).toShape()
 
+def getVerts(shape):
+        "getVerts(shape) -- returns a list containing vectors of each vertex of the shape"
+        p = []
+        for v in shape.Vertexes:
+                p.append(v.Point)
+        return p
+
 def v1(edge):
 	"v1(edge) -- returns the first point of an edge"
 	return edge.Vertexes[0].Point
 
 def isPtOnEdge(pt,edge) :
-	''' Tests if a point is on an edge'''
+	'''isPtOnEdge(Vector,edge) -- Tests if a point is on an edge'''
 	if isinstance(edge.Curve,Part.Line) :
 		orig = edge.Vertexes[0].Point
 		if fcvec.isNull(pt.sub(orig).cross(vec(edge))) :
@@ -94,21 +101,46 @@ def isPtOnEdge(pt,edge) :
 
 	
 # edge functions *****************************************************************
+
+def findEdge(anEdge,aList):
+        '''findEdge(anEdge,aList): returns True if anEdge is found in aList of edges'''
+        for e in range(len(aList)):
+                if str(anEdge.Curve) == str(aList[e].Curve):
+                        if fcvec.equals(anEdge.Vertexes[0].Point,aList[e].Vertexes[0].Point):
+                                if fcvec.equals(anEdge.Vertexes[-1].Point,aList[e].Vertexes[-1].Point):
+                                        return(e)
+        return None
 	
 
-def findIntersection(edge1,edge2,infinite1=False,infinite2=False) :
+def findIntersection(edge1,edge2,infinite1=False,infinite2=False,ex1=False,ex2=False) :
 	'''findIntersection(edge1,edge2,infinite1=False,infinite2=False):
-	returns a list containing the intersection point(s) of 2 edges'''
+	returns a list containing the intersection point(s) of 2 edges.
+        You can also feed 4 points instead of edge1 and edge2'''
+
+        pt1 = None
 	
-	if isinstance(edge1.Curve,Part.Line) and isinstance(edge2.Curve,Part.Line) :
-		
-		# deals with 2 lines
-		
-		pt1, pt2, pt3, pt4 = [edge1.Vertexes[0].Point,edge1.Vertexes[1].Point,
-							  edge2.Vertexes[0].Point,edge2.Vertexes[1].Point]
-							  
-		if fcvec.isNull(pt2.sub(pt1).cross(pt3.sub(pt1)).cross(pt2.sub(pt4).cross(pt3.sub(pt4)))) :
-			vec1 = vec(edge1) ; vec2 = vec(edge2)
+        if isinstance(edge1,FreeCAD.Vector) and isinstance(edge2,FreeCAD.Vector):
+                # we got points directly
+                pt1 = edge1
+                pt2 = edge2
+                pt3 = infinite1
+                pt4 = infinite2
+                infinite1 = ex1
+                infinite2 = ex2
+
+        elif isinstance(edge1.Curve,Part.Line) and isinstance(edge2.Curve,Part.Line) :	
+		# we have 2 edges	
+		pt1, pt2, pt3, pt4 = [edge1.Vertexes[0].Point,
+                                      edge1.Vertexes[1].Point,
+                                      edge2.Vertexes[0].Point,
+                                      edge2.Vertexes[1].Point]
+
+        if pt1:
+                #we have 2 straight lines		  
+		if fcvec.isNull(pt2.sub(pt1).cross(pt3.sub(pt1)).cross(pt2.sub(pt4).cross(pt3.sub(pt4)))):
+			vec1 = pt2.sub(pt1) ; vec2 = pt4.sub(pt3)
+                        if fcvec.isNull(vec1) or fcvec.isNull(vec2):
+                                return []
 			vec1.normalize()  ; vec2.normalize()
 			cross = vec1.cross(vec2)
 			if not fcvec.isNull(cross) :
@@ -262,8 +294,6 @@ def mirror (point, edge):
 	else:
 		return None
 
-
-
 def findClosest(basepoint,pointslist):
 	'''
 	findClosest(vector,list)
@@ -309,9 +339,13 @@ def getBoundary(shape):
 		if lut[e.hashCode()] == 1: bound.append(e)
 	return bound
 
-	
 def sortEdges(lEdges, aVertex=None):
 	"an alternative, more accurate version of Part.__sortEdges__"
+
+        for e in lEdges:
+                if not isinstance(e.Curve,Part.Line):
+                        print "Warning: sortedges cannot treat wired containing curves yet."
+                        return lEdges
 	
 	def isSameVertex(V1, V2):
 		''' Test if vertexes have same coordinates with precision 10E(-precision)'''
@@ -349,7 +383,11 @@ def sortEdges(lEdges, aVertex=None):
 		if result[0] != 0 :
 			del lEdges[result[1]]
 			next = sortEdges(lEdges, result[3].Vertexes[-((-result[2])^1)])
-			olEdges += [result[3]] + next
+                        if isSameVertex(aVertex,result[3].Vertexes[0]):
+                                olEdges += [result[3]] + next
+                        else:
+                                newedge = Part.Line(aVertex.Point,result[3].Vertexes[0].Point).toShape()
+                                olEdges += [newedge] + next
 			return olEdges
 		else :
 			return []
@@ -417,6 +455,11 @@ def findPerpendicular(point,edgeslist,force=None):
 	The function will return a list	[vector_from_point_to_closest_edge,edge_index]
 	or None if no perpendicular vector could be found.
 	'''
+        if not isinstance(edgeslist,list):
+                try:
+                        edgeslist = edgeslist.Edges
+                except:
+                        return None
 	if (force == None):
 		valid = None
 		for edge in edgeslist:
@@ -432,6 +475,7 @@ def findPerpendicular(point,edgeslist,force=None):
 		dist = findDistance(point,edge)
 		if dist: return [dist,force]
 		else: return None
+        return None
 
 def offset(edge,vector):
 	'''
@@ -451,6 +495,99 @@ def offset(edge,vector):
 		newrad = Vector.add(rad,vector).Length
 		return Part.Circle(edge.Curve.Center,NORM,newrad).toShape()
 
+def isReallyClosed(wire):
+        "checks if a wire is really closed"
+        if len(wire.Edges) == len(wire.Vertexes): return True
+        v1 = wire.Vertexes[0].Point
+        v2 = wire.Vertexes[-1].Point
+        if fcvec.equals(v1,v2): return True
+        return False
+
+def getNormal(shape):
+        "finds the normal of a shape, if possible"
+        n = Vector(0,0,1)
+        if shape.ShapeType == "Face":
+                n = shape.normalAt(0.5,0.5)
+        elif shape.ShapeType == "Edge":
+                if isinstance(shape.Curve,Part.Circle):
+                        n = shape.Curve.Axis
+        else:
+                for e in shape.Edges:
+                        if isinstance(e.Curve,Part.Circle):
+                                n = e.Curve.Axis
+                                break
+                        e1 = vec(shape.Edges[0])
+                        for i in range(1,len(shape.Edges)):
+                                e2 = vec(shape.Edges[i])
+                                if 0.1 < abs(e1.getAngle(e2)) < 1.56:
+                                        n = e1.cross(e2).normalize()
+                                        break
+        vdir = FreeCADGui.ActiveDocument.ActiveView.getViewDirection()
+        if n.getAngle(vdir) < 0.78: n = fcvec.neg(n)
+        return n
+
+def offsetWire(wire,dvec,bind=False):
+        '''
+        offsetWire(wire,vector,[bind]): offsets the given wire along the
+        given vector. The vector will be applied at the first vertex of
+        the wire. If bind is True (and the shape is open), the original
+        wire and the offsetted one are bound by 2 edges, forming a face.
+        '''
+        edges = sortEdges(wire.Edges)
+        norm = getNormal(wire)
+        closed = isReallyClosed(wire)
+        nedges = []
+        for i in range(len(edges)):
+                curredge = edges[i]
+                delta = dvec
+                if i != 0:
+                        angle = fcvec.angle(vec(edges[0]),vec(curredge),norm)
+                        delta = fcvec.rotate(delta,angle,norm)
+                nedge = offset(curredge,delta)
+                nedges.append(nedge)
+        nedges = connect(nedges,closed)
+        if bind and not closed:
+                e1 = Part.Line(edges[0].Vertexes[0].Point,nedges[0].Vertexes[0].Point).toShape()
+                e2 = Part.Line(edges[-1].Vertexes[-1].Point,nedges[-1].Vertexes[-1].Point).toShape()
+                alledges = edges.extend(nedges)
+                alledges = alledges.extend([e1,e2])
+                w = Part.Wire(alledges)
+                return w
+        else:
+                return nedges
+
+def connect(edges,closed=False):
+        '''connects the edges in the given list by their intersections'''
+        nedges = []
+        for i in range(len(edges)):
+                curr = edges[i]
+                if i > 0:
+                        prev = edges[i-1]
+                else:
+                        if closed:
+                                prev = edges[-1]
+                        else:
+                                prev = None
+                if i < (len(edges)-1):
+                        next = edges[i+1]
+                else:
+                        if closed: next = edges[0]
+                        else:
+                                next = None
+                if prev:
+                        v1 = findIntersection(curr,prev,True,True)[0]
+                else:
+                        v1 = curr.Vertexes[0].Point
+                if next:
+                        v2 = findIntersection(curr,next,True,True)[0]
+                else:
+                        v2 = curr.Vertexes[-1].Point
+                if isinstance(curr.Curve,Part.Line):
+                        nedges.append(Part.Line(v1,v2).toShape())
+                elif isinstance(curr.Curve,Part.Circle):
+                        nedges.append(Part.Arc(v1,findMidPoint(curr),v2))
+        return Part.Wire(nedges)
+
 def findDistance(point,edge,strict=False):
 	'''
 	findDistance(vector,edge,[strict]) - Returns a vector from the point to its
@@ -540,9 +677,107 @@ def findClosestCircle(point,circles):
 			dist = c.Center.sub(point).Length
 			closest = c
 	return closest
-	
+
+def isCoplanar(faces):
+        "checks if all faces in the given list are coplanar"
+        if len(faces) < 2:
+                return True
+        base =faces[0].normalAt(.5,.5)
+        for i in range(1,len(faces)):
+                normal = faces[i].normalAt(.5,.5)
+                if (normal.getAngle(base) > .0001) and (normal.getAngle(base) < 3.1415):
+                        return False
+        return True
+
+def findWires(edges):
+        '''finds connected edges in the list, and returns a list of lists containing edges
+        that can be connected'''
+        def verts(shape):
+                return [shape.Vertexes[0].Point,shape.Vertexes[-1].Point]
+        def group(shapes):
+                shapesIn = shapes[:]
+                shapesOut = [shapesIn.pop()]
+                changed = False
+                for s in shapesIn:
+                        if len(s.Vertexes) < 2:
+                                continue
+                        else:
+                                clean = True
+                                for v in verts(s):
+                                        for i in range(len(shapesOut)):
+                                                if clean and (v in verts(shapesOut[i])):
+                                                        shapesOut[i] = Part.Wire(shapesOut[i].Edges+s.Edges)
+                                                        changed = True
+                                                        clean = False
+                                if clean:
+                                        shapesOut.append(s)
+                return(changed,shapesOut)
+        working = True
+        edgeSet = edges
+        while working:
+                result = group(edgeSet)
+                working = result[0]
+                edgeSet = result[1]
+        return result[1]
+
+def getTangent(edge,frompoint=None):
+        '''
+        returns the tangent to an edge. If from point is given, it is used to
+        calculate the tangent (only useful for an arc of course).
+        '''
+        if isinstance(edge.Curve,Part.Line):
+                return vec(edge)
+        elif isinstance(edge.Curve,Part.Circle):
+                if not frompoint:
+                        v1 = edge.Vertexes[0].Point.sub(edge.Curve.Center)
+                else:
+                        v1 = frompoint.sub(edge.Curve.Center)
+                return v1.cross(edge.Curve.Axis)
+        return None
+   
 # circle functions *********************************************************
 
+def getBoundaryAngles(angle,alist):
+        '''returns the 2 closest angles from the list that
+        encompass the given angle'''
+        negs = True
+        while negs:
+                negs = False
+                for i in range(len(alist)):
+                        if alist[i] < 0:
+                                alist[i] = 2*math.pi + alist[i]
+                                negs = True
+                if angle < 0:
+                        angle = 2*math.pi + angle
+                        negs = True
+        lower = None
+        for a in alist:
+                if a < angle:
+                        if lower == None:
+                                lower = a
+                        else:
+                                if a > lower:
+                                        lower = a
+        if lower == None:
+                lower = 0
+                for a in alist:
+                        if a > lower:
+                                lower = a
+        higher = None
+        for a in alist:
+                if a > angle:
+                        if higher == None:
+                                higher = a
+                        else:
+                                if a < higher:
+                                        higher = a
+        if higher == None:
+                higher = 2*math.pi
+                for a in alist:
+                        if a < higher:
+                                higher = a
+        return (lower,higher)
+                        
 
 def circleFrom2tan1pt(tan1, tan2, point):
 	"circleFrom2tan1pt(edge, edge, Vector)"
