@@ -79,9 +79,12 @@ def dist(u, v):
 	return u.sub(v).Length
 
 def angle(u,v=Vector(1,0,0),normal=Vector(0,0,1)):
-	'''angle(Vector,[Vector],[Vector]) - returns the angle in radians between the two vectors.
-	If only one is given, angle is between the vector and the horizontal East direction.
-	If a third vector is given, it is the normal used to determine the sign of the angle.
+	'''
+        angle(Vector,[Vector],[Vector]) - returns the angle
+        in radians between the two vectors. If only one is given,
+        angle is between the vector and the horizontal East direction.
+	If a third vector is given, it is the normal used to determine
+        the sign of the angle.
 	'''
 	typecheck ([(u,Vector), (v,Vector)], "angle")
 	ll = u.Length*v.Length
@@ -116,10 +119,7 @@ def rotate(u,angle,axis=Vector(0,0,1)):
 	If axis is omitted, the rotation is made on the xy plane.'''
 	typecheck ([(u,Vector), (angle,(int,long,float)), (axis,Vector)], "rotate")
 
-	# special case optimizations removed for clarity KEC
-	#if rotangle == 0: return u
-	#if equals(axis,Vector(0,0,1)): return rotate2D(u,rotangle)
-	#if equals(axis,Vector(0,0,-1)): return rotate2D(u,-rotangle)
+	if angle == 0: return u
 
 	l=axis.Length
 	x=axis.x/l
@@ -166,61 +166,59 @@ def isColinear(vlist):
 		if angle(vlist[i].sub(vlist[0]),first) != 0:
 			return False
 	return True
-		
 
-'''
-Obsolete/useless function.  Complete deletion seems appropriate - KEC
+def rounded(v):
+        "returns a rounded vector"
+        p = precision()
+        return Vector(round(v.x,p),round(v.y,p),round(v.z,p))
 
-def new(first, other):
-	"new(Vector,Vector) - creates a new vector from first one to second one"
-	typecheck ([(first,Vector), (other,Vector)]);
-	return Vector(other.x-first.x, other.y-first.y, other.z-first.z)
+def getPlaneRotation(u,v,w=None):
+        "returns a rotation matrix defining the (u,v,w) coordinates system"
+        if not w: w = u.cross(v)
+        typecheck([(u,Vector), (v,Vector), (w,Vector)], "getPlaneRotation")
+        m = FreeCAD.Matrix(
+                u.x,v.x,w.x,0,
+                u.y,v.y,w.y,0,
+                u.z,v.z,w.z,0,
+                0.0,0.0,0.0,1.0)
+        return m
 
-def add(first, other):
-	"add(Vector,Vector) - adds two vectors. OBSOLETE - use Vector.add()"
-	typecheck ([(first,Vector), (other,Vector)]);
-	return Vector(first.x+other.x, first.y+other.y, first.z+other.z)
+def reorient(u,ref):
+        '''Checks the orientation of u and reorients on
+        what we consider positive axis direction'''
+        if ref == "x":
+                if u.x < 0:
+                        return neg(u)
+                elif u.x == 0:
+                        if u.y > 0:
+                                return neg(u)
+                        elif u.y == 0:
+                                if u.z > 0: return neg(u)
+        elif ref == "y":
+                if u.y < 0:
+                        return neg(u)
+                elif u.y == 0:
+                        if u.x < 0:
+                                return neg(u)
+                        elif u.x == 0:
+                                if u.z < 0: return neg(u)
+        elif ref == "z":
+                if u.z < 0:
+                        return neg(u)
+                elif u.z == 0:
+                        if u.y > 0:
+                                return neg(u)
+                        elif u.y == 0:
+                                if u.x < 0: return neg(u)
+        return u
 
-def sub(first, other):
-	' ' '
-	sub(Vector,Vector) - subtracts second vector from first one.
-	OBSOLETE - use Vector.sub()
-	' ' '
-	typecheck ([(first,Vector), (other,Vector)])
-	return Vector(first.x-other.x, first.y-other.y, first.z-other.z)
-
-def length(first):
-	"lengh(Vector) - gives vector length - OBSOLETE! use Vector.Length instead!"
-	typecheck([(first,Vector)])
-	return math.sqrt(first.x*first.x + first.y*first.y + first.z*first.z)
-
--- normalized was mainly used idiomatically to for scaling a vector.
--- replaced with scaleTo and Vector.normalize()
-def normalized(first):
-	"normalized(Vector) - returns a unit vector"
-	typecheck ([(first,Vector)])
-	v = Vector(first)
-	v.normalize
-	return v
-
--- only single argument version was used.  Eliminated, but could be renamed crossz with second arg removed
-def crossproduct(u, v=Vector(0,0,1)):
-	' ' '
-	crossproduct(Vector,Vector) - returns the cross product of both vectors.
-	If only one is given, cross product is made with vertical axis,
-	thus returning its perpendicular in XY plane
-	' ' '
-	typecheck ([(u,Vector), (v,Vector)])
-	return u.cross(v)
-
-def dotproduct(u, v):
-	"dotproduct(Vector,Vector) - returns the dot product of both vectors"
-	typecheck ([(u,Vector), (v,Vector)])
-	return u.dot(v)
-
-def rounded(vector,pr=precision):
-	' ' 'rounded(vector,int) - returns a rounded vector to given precision, or is
-	precision is not specified, built-in precision is used.' ' '
-	if isinstance(vector,Vector) and isinstance(pr,int):
-		return Vector(round(vector.x,pr),round(vector.y,pr),round(vector.z,pr))
-'''
+def removeDoubles(vlist):
+        "removes consecutive doubles from a list of vectors"
+        typecheck ([(vlist,list)], "removeDoubles");
+        nlist = []
+        if len(vlist) < 2: return vlist
+        for i in range(len(vlist)-1):
+                if not equals(vlist[i],vlist[i+1]):
+                        nlist.append(vlist[i])
+        nlist.append(vlist[-1])
+        return nlist
