@@ -1,4 +1,6 @@
 
+# -*- coding: utf8 -*-
+
 #***************************************************************************
 #*                                                                         *
 #*   Copyright (c) 2009 Yorik van Havre <yorik@gmx.fr>                     *
@@ -41,13 +43,13 @@ texts, colors,layers (from groups)
 import FreeCAD, os, Part, math, re, string
 from draftlibs import fcvec, dxfColorMap, dxfLibrary, fcgeo
 from draftlibs.dxfReader import readDXF
-from Draft import Dimension, DimensionViewProvider
+from Draft import Dimension, ViewProviderDimension
 from FreeCAD import Vector
 
 try: import FreeCADGui
 except: gui = False
 else: gui = True
-try: draftui = FreeCADGui.activeWorkbench().draftToolBar.ui
+try: draftui = FreeCADGui.draftToolBar.ui
 except: draftui = None
 
 pythonopen = open # to distinguish python built-in open function from the one declared here
@@ -83,7 +85,7 @@ def calcBulge(v1,bulge,v2):
 	chord = v2.sub(v1)
 	sagitta = (bulge * chord.Length)/2
 	startpoint = v1.add(fcvec.scale(chord,0.5))
-	perp = chord.cross(Vector(1,0,0))
+	perp = chord.cross(Vector(0,0,1))
 	if not fcvec.isNull(perp): perp.normalize()
 	endpoint = fcvec.scale(perp,sagitta)
 	return startpoint.add(endpoint)
@@ -126,8 +128,10 @@ class fcformat:
 		self.dxf = drawing
 		params = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
 		self.paramtext = params.GetBool("dxftext")
-		self.paramlayouts = params.GetBool("dxflayouts")
+                self.paramstarblocks = params.GetBool("dxfstarblocks")
+		self.dxflayout = params.GetBool("dxflayouts")
 		self.paramstyle = params.GetInt("dxfstyle")
+                self.join = params.GetBool("joingeometry")
 		bparams = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/View")
 
 		if self.paramstyle > 1:
@@ -329,6 +333,37 @@ def drawCircle(circle):
 	except: warn(circle)
 	return None
 
+def drawSolid(solid):
+        "returns a Part shape from a dxf solid"
+        p4 = None
+        p1x = rawValue(solid,10)
+        p1y = rawValue(solid,20)
+        p1z = rawValue(solid,30) or 0
+        p2x = rawValue(solid,11)
+        p2y = rawValue(solid,21)
+        p2z = rawValue(solid,31) or p1z
+        p3x = rawValue(solid,12)
+        p3y = rawValue(solid,22)
+        p3z = rawValue(solid,32) or p1z
+        p4x = rawValue(solid,13)
+        p4y = rawValue(solid,23)
+        p4z = rawValue(solid,33) or p1z
+        p1 = FreeCAD.Vector(p1x,p1y,p1z)
+        p2 = FreeCAD.Vector(p2x,p2y,p2z)
+        p3 = FreeCAD.Vector(p3x,p3y,p3z)
+        if p4x != None: p4 = FreeCAD.Vector(p4x,p4y,p4z)
+        if p4 and (p4 != p3) and (p4 != p2) and (p4 != p1):
+                try:
+                        return Part.Face(Part.makePolygon([p1,p2,p3,p4,p1]))
+                except:
+                        warn(solid)
+        else:
+                try:
+                        return Part.Face(Part.makePolygon([p1,p2,p3,p1]))
+                except:
+                        warn(solid)
+        return None
+
 def drawBlock(blockref):
 	"returns a shape from a dxf block reference"
 	shapes = []
@@ -348,8 +383,18 @@ def drawBlock(blockref):
 		s = drawCircle(circle)
 		if s: shapes.append(s)
 	for insert in blockref.entities.get_type('insert'):
+                print "found insert ",insert.block
 		s = drawInsert(insert)
 		if s: shapes.append(s)
+        for solid in blockref.entities.get_type('solid'):
+		s = drawSolid(solid)
+		if s: shapes.append(s)
+        for text in blockref.entities.get_type('text'):
+                if fmt.dxflayout or (not rawValue(text,67)):
+                        addText(text)
+        for text in blockref.entities.get_type('mtext'):
+                if fmt.dxflayout or (not rawValue(text,67)):
+                        addText(text)
 	try: shape = Part.makeCompound(shapes)
 	except: warn(blockref)
 	if shape:
@@ -365,6 +410,10 @@ def drawInsert(insert):
 		for b in drawing.blocks.data:
 			if b.name == insert.block:
 				shape = drawBlock(b)
+        if fmt.paramtext:
+                attrs = attribs(insert)
+                for a in attrs:
+                        addText(a,attrib=True)
 	if shape:
 		pos = FreeCAD.Vector(insert.loc[0],insert.loc[1],insert.loc[2])
 		rot = math.radians(insert.rotation)
@@ -377,14 +426,61 @@ def drawInsert(insert):
 		return shape
 	return None
 
-def addObject(shape,name,layer):
+def attribs(insert):
+        "checks if an insert has attributes, and returns the values if yes"
+        atts = []
+        if rawValue(insert,66) != 1: return []
+        index = None
+        for i in range(len(drawing.entities.data)):
+                if drawing.entities.data[i] == insert:
+                        index = i
+                        break
+        if index == None: return []
+        j = index+1
+        while True:
+                ent = drawing.entities.data[j]
+                print str(ent)
+                if str(ent) == 'seqend':
+                        return atts
+                elif str(ent) == 'attrib':
+                        atts.append(ent)
+                        j += 1
+
+def addObject(shape,name="Shape",layer=None):
 	"adds a new object to the document with passed arguments"
 	newob=doc.addObject("Part::Feature",name)
-	lay=locateLayer(layer)
-	lay.addObject(newob)
+        if layer:
+                lay=locateLayer(layer)
+                lay.addObject(newob)
 	newob.Shape = shape
 	return newob
 
+def addText(text,attrib=False):
+        "adds a new text to the document"
+        if attrib:
+                lay = locateLayer(rawValue(text,8))
+                val = rawValue(text,1)
+                pos = FreeCAD.Vector(rawValue(text,10),rawValue(text,20),rawValue(text,30))
+                hgt = rawValue(text,40)
+        else:
+                lay = locateLayer(text.layer)
+                val = text.value
+                pos = FreeCAD.Vector(text.loc[0],text.loc[1],text.loc[2])
+                hgt = text.height
+        if val:
+                newob=doc.addObject("App::Annotation","Text")
+                lay.addObject(newob)
+                val = re.sub('{([^!}]([^}]|\n)*)}', '', val)
+                val = re.sub('%%d','°',val)
+                val = re.sub('%%c','Ø',val)
+                val = val.decode("Latin1").encode("Latin1")
+                newob.LabelText = val
+                newob.Position = pos
+                if gui:
+                        newob.ViewObject.FontSize=float(hgt)
+                        newob.ViewObject.DisplayMode = "World"
+                        fmt.formatObject(newob,text,textmode=True)
+
 def processdxf(document,filename):
 	"this does the translation of the dxf contents into FreeCAD Part objects"
 
@@ -402,17 +498,23 @@ def processdxf(document,filename):
 
 	# getting config parameters
 
+        global fmt
 	fmt = fcformat(drawing)
+        shapes = []
 
 	# drawing lines
 
 	lines = drawing.entities.get_type("line")
 	if lines: FreeCAD.Console.PrintMessage("drawing "+str(len(lines))+" lines...\n")
 	for line in lines:
-		shape = drawLine(line)
-		if shape:
-			newob = addObject(shape,"Line",line.layer)
-			if gui: fmt.formatObject(newob,line)
+                if fmt.dxflayout or (not rawValue(line,67)):
+                        shape = drawLine(line)
+                        if shape:
+                                if fmt.join:
+                                        shapes.append(shape)
+                                else:
+                                        newob = addObject(shape,"Line",line.layer)
+                                        if gui: fmt.formatObject(newob,line)
 						
 	# drawing polylines
 
@@ -420,114 +522,145 @@ def processdxf(document,filename):
 	polylines.extend(drawing.entities.get_type("polyline"))
 	if polylines: FreeCAD.Console.PrintMessage("drawing "+str(len(polylines))+" polylines...\n")
 	for polyline in polylines:
-		shape = drawPolyline(polyline)
-		if shape:
-			newob = addObject(shape,"Polyline",polyline.layer)
-			if gui: fmt.formatObject(newob,polyline)
+                if fmt.dxflayout or (not rawValue(polyline,67)):
+                        shape = drawPolyline(polyline)
+                        if shape:
+                                if fmt.join:
+                                        shapes.append(shape)
+                                else:
+                                        newob = addObject(shape,"Polyline",polyline.layer)
+                                        if gui: fmt.formatObject(newob,polyline)
 				
 	# drawing arcs
 
 	arcs = drawing.entities.get_type("arc")
 	if arcs: FreeCAD.Console.PrintMessage("drawing "+str(len(arcs))+" arcs...\n")
 	for arc in arcs:
-		shape = drawArc(arc)
-		if shape:
-			newob = addObject(shape,"Arc",arc.layer)
-			if gui: fmt.formatObject(newob,arc)
+                if fmt.dxflayout or (not rawValue(arc,67)):
+                        shape = drawArc(arc)
+                        if shape:
+                                if fmt.join:
+                                        shapes.append(shape)
+                                else:
+                                        newob = addObject(shape,"Arc",arc.layer)
+                                        if gui: fmt.formatObject(newob,arc)
+
+        # joining lines, polylines and arcs if needed
+
+        if fmt.join and shapes:
+                FreeCAD.Console.PrintMessage("Joining geometry...\n")
+                edges = []
+                for s in shapes:
+                        edges.extend(s.Edges)
+                shapes = fcgeo.findWires(edges)
+                for s in shapes:
+                        newob = addObject(s)
 
 	# drawing circles
 
 	circles = drawing.entities.get_type("circle")
 	if circles: FreeCAD.Console.PrintMessage("drawing "+str(len(circles))+" circles...\n")
 	for circle in circles:
-		shape = drawCircle(circle)
-		if shape:
-			newob = addObject(shape,"Circle",circle.layer)
-			if gui: fmt.formatObject(newob,circle)
+                if fmt.dxflayout or (not rawValue(circle,67)):
+                        shape = drawCircle(circle)
+                        if shape:
+                                newob = addObject(shape,"Circle",circle.layer)
+                                if gui: fmt.formatObject(newob,circle)
+
+        # drawing solids
+
+	solids = drawing.entities.get_type("solid")
+	if solids: FreeCAD.Console.PrintMessage("drawing "+str(len(circles))+" solids...\n")
+	for solid in solids:
+                if fmt.dxflayout or (not rawValue(solid,67)):
+                        shape = drawSolid(solid)
+                        if shape:
+                                newob = addObject(shape,"Solid",solid.layer)
+                                if gui: fmt.formatObject(newob,solid)
 
 	# drawing texts
 
-	print fmt.paramtext
-
 	if fmt.paramtext:
 		texts = drawing.entities.get_type("mtext")
 		texts.extend(drawing.entities.get_type("text"))
 		if texts: FreeCAD.Console.PrintMessage("drawing "+str(len(texts))+" texts...\n")
 		for text in texts:
-				lay=locateLayer(text.layer)
-				newob=doc.addObject("App::Annotation","Text")
-				lay.addObject(newob)
-				newob.LabelText = re.sub('{([^!}]([^}]|\n)*)}', '', text.value) 
-				newob.Position=FreeCAD.Vector(text.loc[0],text.loc[1],text.loc[2])
-				if gui:
-					newob.ViewObject.FontSize=float(text.height)
-					newob.ViewObject.DisplayMode = "World"
-					fmt.formatObject(newob,text,textmode=True)
+                        if fmt.dxflayout or (not rawValue(text,67)):
+                                addText(text)
 					
 	else: FreeCAD.Console.PrintMessage("skipping texts...\n")
 
 	# drawing dims
 
-	dims = drawing.entities.get_type("dimension")
 	if fmt.paramtext:
+                dims = drawing.entities.get_type("dimension")
 		FreeCAD.Console.PrintMessage("drawing "+str(len(dims))+" dimensions...\n")
 		for dim in dims:
-			try:
-				layer = rawValue(dim,8)
-				x1 = float(rawValue(dim,10))
-				y1 = float(rawValue(dim,20))
-				z1 = float(rawValue(dim,30))
-				x2 = float(rawValue(dim,13))
-				y2 = float(rawValue(dim,23))
-				z2 = float(rawValue(dim,33))
-				x3 = float(rawValue(dim,14))
-				y3 = float(rawValue(dim,24))
-				z3 = float(rawValue(dim,34))
-				d = rawValue(dim,70)
-				if d: align = int(d)
-				else: align = 0
-				d = rawValue(dim,50)
-				if d: angle = float(d)
-				else: angle = 0
-			except:
-				warn(dim)
-			else:
-				lay=locateLayer(layer)
-				pt = FreeCAD.Vector(x1,y1,z1)
-				p1 = FreeCAD.Vector(x2,y2,z2)
-				p2 = FreeCAD.Vector(x3,y3,z3)
-				if align == 0:
-					if angle == 0:
-						p2 = FreeCAD.Vector(x3,y2,z2)
-					else:
-						p2 = FreeCAD.Vector(x2,y3,z2)
+                        if fmt.dxflayout or (not rawValue(dim,67)):
+                                try:
+                                        layer = rawValue(dim,8)
+                                        x1 = float(rawValue(dim,10))
+                                        y1 = float(rawValue(dim,20))
+                                        z1 = float(rawValue(dim,30))
+                                        x2 = float(rawValue(dim,13))
+                                        y2 = float(rawValue(dim,23))
+                                        z2 = float(rawValue(dim,33))
+                                        x3 = float(rawValue(dim,14))
+                                        y3 = float(rawValue(dim,24))
+                                        z3 = float(rawValue(dim,34))
+                                        d = rawValue(dim,70)
+                                        if d: align = int(d)
+                                        else: align = 0
+                                        d = rawValue(dim,50)
+                                        if d: angle = float(d)
+                                        else: angle = 0
+                                except:
+                                        warn(dim)
+                                else:
+                                        lay=locateLayer(layer)
+                                        pt = FreeCAD.Vector(x1,y1,z1)
+                                        p1 = FreeCAD.Vector(x2,y2,z2)
+                                        p2 = FreeCAD.Vector(x3,y3,z3)
+                                        if align == 0:
+                                                if angle == 0:
+                                                        p2 = FreeCAD.Vector(x3,y2,z2)
+                                                else:
+                                                        p2 = FreeCAD.Vector(x2,y3,z2)
 
-				newob = doc.addObject("App::FeaturePython","Dimension")
-				lay.addObject(newob)
-				Dimension(newob)
-				DimensionViewProvider(newob.ViewObject)
-				newob.Start = p1
-				newob.End = p2
-				newob.Dimline = pt
-				if gui:
-					dim.layer = layer
-					dim.color_index = 256
-					fmt.formatObject (newob,dim)
+                                        newob = doc.addObject("App::FeaturePython","Dimension")
+                                        lay.addObject(newob)
+                                        Dimension(newob)
+                                        ViewProviderDimension(newob.ViewObject)
+                                        newob.Start = p1
+                                        newob.End = p2
+                                        newob.Dimline = pt
+                                        if gui:
+                                                dim.layer = layer
+                                                dim.color_index = 256
+                                                fmt.formatObject (newob,dim)
 						
 	else: FreeCAD.Console.PrintMessage("skipping dimensions...\n")
 
 	# drawing blocks
 
 	inserts = drawing.entities.get_type("insert")
+        if not fmt.paramstarblocks:
+                FreeCAD.Console.PrintMessage("skipping *blocks...\n")
+                newinserts = []
+                for i in inserts:
+                        if fmt.dxflayout or (not rawValue(i,67)):
+                                if i.block[0] != '*':
+                                        newinserts.append(i)
+                inserts = newinserts
 	if inserts:
-		FreeCAD.Console.PrintMessage("drawing "+str(len(dims))+" blocks...\n")
+		FreeCAD.Console.PrintMessage("drawing "+str(len(inserts))+" blocks...\n")
 		blockrefs = drawing.blocks.data
 		for ref in blockrefs:
 			drawBlock(ref)
 		for insert in inserts:
 			shape = drawInsert(insert)
 			if shape:
-				newob = addObject(shape,"Block",insert.layer)
+				newob = addObject(shape,"Block."+insert.block,insert.layer)
 				if gui: fmt.formatObject(newob,insert)
 
 	# finishing
@@ -548,13 +681,17 @@ def open(filename):
 	"called when freecad opens a file"
 	docname = os.path.splitext(os.path.basename(filename))[0]
 	doc = FreeCAD.newDocument(docname)
-	doc.Label = decodeName(docname[:-4])
+	doc.Label = decodeName(docname)
 	processdxf(doc,filename)
+        return doc
 
 def insert(filename,docname):
 	"called when freecad imports a file"
 	groupname = os.path.splitext(os.path.basename(filename))[0]
-	doc=FreeCAD.getDocument(docname)
+	try:
+		doc=FreeCAD.getDocument(docname)
+	except:
+		doc=FreeCAD.newDocument(docname)
 	importgroup = doc.addObject("App::DocumentObjectGroup",groupname)
 	importgroup.Label = decodeName(groupname)
 	processdxf(doc,filename)
@@ -610,16 +747,16 @@ def getWire(wire):
 		else:
 			bul = 0.0
 			points.append((v1.x,v1.y,v1.z,None,None,bul))
-	if not wire.isClosed():
+	if not fcgeo.isReallyClosed(wire):
 		v = edges[-1].Vertexes[-1].Point
 		points.append(fcvec.tup(v))
+        print "wire verts: ",points
 	return points
 
 def getBlock(obj):
 	"returns a dxf block with the contents of the object"
 	block = dxfLibrary.Block(name=obj.Name,layer=getGroup(obj,exportList))
-	writeShape(obj,block)
-	
+	writeShape(obj,block)	
 	return block
 
 def writeShape(ob,dxfobject):
@@ -627,9 +764,16 @@ def writeShape(ob,dxfobject):
 	processededges = []
 	for wire in ob.Shape.Wires: # polylines
 		for e in wire.Edges: processededges.append(e.hashCode())
-		dxfobject.append(dxfLibrary.PolyLine(getWire(wire), [0.0,0.0,0.0],
-					       int(wire.isClosed()), color=getACI(ob),
-					       layer=getGroup(ob,exportList)))
+                if (len(wire.Edges) == 1) and isinstance(wire.Edges[0].Curve,Part.Circle):
+                        center = fcvec.tup(wire.Edges[0].Curve.Center)
+                        radius = wire.Edges[0].Curve.Radius
+                        dxfobject.append(dxfLibrary.Circle(center, radius,
+                                                           color=getACI(ob),
+                                                           layer=getGroup(ob,exportList)))
+                else:
+                        dxfobject.append(dxfLibrary.PolyLine(getWire(wire), [0.0,0.0,0.0],
+                                                             int(fcgeo.isReallyClosed(wire)), color=getACI(ob),
+                                                             layer=getGroup(ob,exportList)))
 	if len(processededges) < len(ob.Shape.Edges): # lone edges
 		loneedges = []
 		for e in ob.Shape.Edges:
@@ -650,7 +794,7 @@ def writeShape(ob,dxfobject):
 				dxfobject.append(dxfLibrary.PolyLine(points, [0.0,0.0,0.0],
 								     0, color=getACI(ob),
 								     layer=getGroup(ob,exportList)))
-			elif (len(edge.Vertexes) == 1): # circles
+			elif (len(edge.Vertexes) == 1) and (isinstance(edge.Curve,Part.Circle)): # circles
 				center = fcvec.tup(edge.Curve.Center)
 				radius = ob.Shape.Edges[0].Curve.Radius
 				dxfobject.append(dxfLibrary.Circle(center, radius,
@@ -674,31 +818,30 @@ def export(objectslist,filename):
 	global exportList
 	exportList = objectslist
 	dxf = dxfLibrary.Drawing()
+        if (len(exportList) == 1) and (exportList[0].isDerivedFrom("Drawing::FeaturePage")):
+                exportPage(exportList[0],filename)
+                return
 	for ob in exportList:
 		print "processing ",ob.Name
-		if isinstance(ob,Part.Feature):
-			if ob.Shape.ShapeType == 'Compound':
-				if (len(ob.Shape.Wires) == 1):
-					print "open wire found"
-					if (len(ob.Shape.Wires[0].Edges) == len(ob.Shape.Edges)):
-						wire = ob.Shape.Wires[0]
-						dxf.append(dxfLibrary.PolyLine(getWire(wire),
-									       [0.0,0.0,0.0],
-									       int(wire.isClosed()),
-									       color=getACI(ob),
-									       layer=getGroup(ob,exportList)))
-					else:
-						print "creating block - 1 wire"
-						block = getBlock(ob)
-						dxf.blocks.append(block)
-						dxf.append(dxfLibrary.Insert(name=ob.Name.upper()))
-				else:
-					print "creating block"
-					block = getBlock(ob)
-					dxf.blocks.append(block)
-					dxf.append(dxfLibrary.Insert(name=ob.Name.upper()))
-			else:
-				writeShape(ob,dxf)
+		if ob.isDerivedFrom("Part::Feature"):
+                        if not ob.Shape.isNull():
+                                if ob.Shape.ShapeType == 'Compound':
+                                        if (len(ob.Shape.Wires) == 1):
+                                                # only one wire in this compound, no lone edge -> polyline
+                                                if (len(ob.Shape.Wires[0].Edges) == len(ob.Shape.Edges)):
+                                                        writeShape(ob,dxf)
+                                                else:
+                                                        # 1 wire + lone edges -> block
+                                                        block = getBlock(ob)
+                                                        dxf.blocks.append(block)
+                                                        dxf.append(dxfLibrary.Insert(name=ob.Name.upper()))
+                                        else:
+                                                # all other cases: block
+                                                block = getBlock(ob)
+                                                dxf.blocks.append(block)
+                                                dxf.append(dxfLibrary.Insert(name=ob.Name.upper()))
+                                else:
+                                        writeShape(ob,dxf)
 				
 		elif (ob.Type == "App::Annotation"):
 
@@ -717,22 +860,60 @@ def export(objectslist,filename):
 							   style='STANDARD',
 							   layer=getGroup(ob,exportList)))
 
-		elif (ob.Type == "App::FeaturePython"):
-			if 'Dimline' in ob.PropertiesList:
-				p1 = fcvec.tup(ob.Start)
-				p2 = fcvec.tup(ob.End)
-				base = Part.Line(ob.Start,ob.End).toShape()
-				proj = fcgeo.findDistance(ob.Dimline,base)
-				if not proj:
-					pbase = fcvec.tup(ob.End)
-				else:
-					pbase = fcvec.tup(ob.End.add(fcvec.neg(proj)))
-				dxf.append(dxfLibrary.Dimension(pbase,p1,p2,color=getACI(ob),
-								layer=getGroup(ob,exportList)))
+		elif 'Dimline' in ob.PropertiesList:
+                        p1 = fcvec.tup(ob.Start)
+                        p2 = fcvec.tup(ob.End)
+                        base = Part.Line(ob.Start,ob.End).toShape()
+                        proj = fcgeo.findDistance(ob.Dimline,base)
+                        if not proj:
+                                pbase = fcvec.tup(ob.End)
+                        else:
+                                pbase = fcvec.tup(ob.End.add(fcvec.neg(proj)))
+                        dxf.append(dxfLibrary.Dimension(pbase,p1,p2,color=getACI(ob),
+                                                        layer=getGroup(ob,exportList)))
 					
 	dxf.saveas(filename)
 	FreeCAD.Console.PrintMessage("successfully exported "+filename+"\r\n")
 
+def exportPage(page,filename):
+        "special export for pages"
+        import importSVG
+        tempdoc = importSVG.open(page.PageResult)
+        tempobj = tempdoc.Objects
+        export(tempobj,filename)
+        FreeCAD.closeDocument(tempdoc.Name)
+        
+def exportPage2(page,filename):
+        "special export for pages, using inkscape"
+        print "Exporting page object..."
+        import tempfile
+        params = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
+        inkscapepath = params.GetString("inkscapepath")
+        ps2dxfpath = params.GetString("ps2dxfpath")
+        if not inkscapepath:
+                if os.name == "posix":
+                        p= "/usr/bin/inkscape"
+                        if os.path.exists(p):
+                                inkscapepath = p
+                        else:
+                                print "Inkscape not found! aborting..."
+        if not ps2dxfpath:
+                if os.name == "posix":
+                        p= "/usr/share/inkscape/extensions/ps2dxf.sh"
+                        if os.path.exists(p):
+                                ps2dxfpath = p
+                        else:
+                                print "Ps2dxf script not found! aborting..."
+        if inkscapepath and ps2dxfpath:
+                pst = tempfile.NamedTemporaryFile(suffix=".ps")
+                os.system(inkscapepath+" "+page.PageResult+" --export-ps="+pst.name)
+                os.system(ps2dxfpath+" "+pst.name+" > "+filename)
+                FreeCAD.Console.PrintMessage("successfully exported "+filename+"\r\n")
+                pst.close()
+              
+        # inkscape test.svg --export-ps=test.ps | /usr/share/inkscape/extensips2dxf.sh test.ps > test.dxf
+        
+
 
 
 
