@@ -31,14 +31,14 @@ paths, lines, arcs and rects.
 Bezier curves are skipped.
 '''
 
-import xml.sax, string, FreeCAD, os, Part, math
+import xml.sax, string, FreeCAD, os, Part, math, re, Draft
 from draftlibs import fcvec, fcgeo
 from FreeCAD import Vector
 
 try: import FreeCADGui
 except: gui = False
 else: gui = True
-try: draftui = FreeCADGui.activeWorkbench().draftToolBar.ui
+try: draftui = FreeCADGui.draftToolBar.ui
 except: draftui = None
 
 pythonopen = open
@@ -229,7 +229,7 @@ def getrgb(color):
 	g = str(hex(int(color[1]*255)))[2:].zfill(2)
 	b = str(hex(int(color[2]*255)))[2:].zfill(2)
 	return "#"+r+g+b
-	
+
 class svgHandler(xml.sax.ContentHandler):
 	"this handler parses the svg files and creates freecad objects"
 
@@ -237,6 +237,9 @@ class svgHandler(xml.sax.ContentHandler):
 		"retrieving Draft parameters"
 		params = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
 		self.style = params.GetInt("svgstyle")
+                self.count = 0
+                self.transform = None
+                self.grouptransform = []
 	
 		if gui and draftui:
 			r = float(draftui.color.red()/255.0)
@@ -262,6 +265,11 @@ class svgHandler(xml.sax.ContentHandler):
 	def startElement(self, name, attrs):
 
 		# reorganizing data into a nice clean dictionary
+
+                self.count += 1
+
+                print "processing element ",self.count,": ",name
+                print "existing group transform: ", self.grouptransform
 		
 		data = {}
 		for (keyword,content) in attrs.items():
@@ -291,7 +299,6 @@ class svgHandler(xml.sax.ContentHandler):
 		self.color = None
 		self.width = None
 		self.text = None
-		self.translate = None
 		
 		if 'fill' in data:
 			if data['fill'][0] != 'none':
@@ -303,19 +310,61 @@ class svgHandler(xml.sax.ContentHandler):
 			if data['stroke-width'] != 'none':
 				self.width = getsize(data['stroke-width'])
 		if 'transform' in data:
-			t = data['transform'][0]
-			t = t.split('(')
-			t.append(data['transform'][1].replace(')',''))
-			if t[0] == "translate":
-				self.translate = Vector(float(t[1]),-float(t[2]),0)
-
+                        m = self.getMatrix(data['transform'])
+                        if name == "g":
+                                self.grouptransform.append(m)
+                        else:
+                                self.transform = m
+                else:
+                        if name == "g":
+                                self.grouptransform.append(FreeCAD.Matrix())
+                                
+                        '''
+                        print "existing grouptransform: ",self.grouptransform
+                        print "existing transform: ",self.transform
+			if "translate" in tr:
+                                i0 = tr.index("translate")
+                                print "getting translate ",tr
+                                if "translate" in self.transform:
+                                        self.transform['translate'] = self.transform['translate'].add(Vector(float(tr[i0+1]),-float(tr[i0+2]),0))
+                                else:
+                                        self.transform['translate'] = Vector(float(tr[i0+1]),-float(tr[i0+2]),0)
+                                if "translate" in self.grouptransform:
+                                        print "adding to group ",self.grouptransform['translate']
+                                        self.transform['translate'] = self.grouptransform['translate'].add(self.transform['translate'])
+                        else:
+                                if "translate" in self.grouptransform:
+                                        print "adding to group ",self.grouptransform['translate']
+                                        self.transform['translate'] = self.grouptransform['translate']
+                        if "scale" in tr:
+                                i0 = tr.index("scale")
+                                if "scale" in self.transform:
+                                        self.transform['scale'] = self.transform['scale'].add(Vector(float(tr[i0+1]),float(tr[i0+2]),0))
+                                else:
+                                        print tr
+                                        self.transform['scale'] = Vector(float(tr[i0+1]),float(tr[i0+2]),0)
+                                if "scale" in self.grouptransform:
+                                        self.transform['scale'] = self.transform['scale'].add(self.grouptransform['scale'])
+                        else:
+                                if "scale" in self.grouptransform:
+                                        self.transform['scale'] = self.grouptransform['scale']
+                        '''
+ 
 		if (self.style == 1):
 			self.color = self.col
 			self.width = self.lw
 
+                pathname = None
+                if 'id' in data:
+                        pathname = data['id'][0]
+                        print "name: ",pathname
+                        
 		# processing paths
-
+                        
 		if name == "path":
+                        print data
+                        
+                        if not pathname: pathname = 'Path'
 
 			path = []
 			point = []
@@ -404,8 +453,8 @@ class svgHandler(xml.sax.ContentHandler):
 					if path:
 						sh = Part.Wire(path)
 						if self.fill: sh = Part.Face(sh)
-						if self.translate: sh.translate(self.translate)
-						obj = self.doc.addObject("Part::Feature","Path")
+                                                sh = self.applyTrans(sh)
+						obj = self.doc.addObject("Part::Feature",pathname)
 						obj.Shape = sh
 						self.format(obj)
 						path = []
@@ -423,10 +472,11 @@ class svgHandler(xml.sax.ContentHandler):
 						currentvec = lastvec.add(Vector(point[0],-point[1],0))
 					else:
 						currentvec = Vector(point[0],-point[1],0)
-					seg = Part.Line(lastvec,currentvec).toShape()
-					print "line ",lastvec,currentvec
-					lastvec = currentvec
-					path.append(seg)
+                                        if not fcvec.equals(lastvec,currentvec):
+                                                seg = Part.Line(lastvec,currentvec).toShape()
+                                                print "line ",lastvec,currentvec
+                                                lastvec = currentvec
+                                                path.append(seg)
 					point = []
 				elif (len(point)==1) and (command=="horizontal"):
 					if relative:
@@ -453,14 +503,14 @@ class svgHandler(xml.sax.ContentHandler):
 						currentvec = lastvec.add(Vector(point[-2],-point[-1],0))
 					else:
 						currentvec = Vector(point[-2],-point[-1],0)
-					chord = fcvec.new(lastvec,currentvec)
-					perp = fcvec.crossproduct(chord)
+					chord = currentvec.sub(lastvec)
+					perp = chord.cross(Vector(0,0,-1))
 					chord = fcvec.scale(chord,.5)
 					if chord.Length > point[0]: a = 0
 					else: a = math.sqrt(point[0]**2-chord.Length**2)
 					s = point[0] - a
 					perp = fcvec.scale(perp,s/perp.Length)
-					midpoint = fcvec.add(lastvec,fcvec.add(chord,perp))
+					midpoint = lastvec.add(chord.add(perp))
 					seg = Part.Arc(lastvec,midpoint,currentvec).toShape()
 					lastvec = currentvec
 					path.append(seg)
@@ -473,8 +523,8 @@ class svgHandler(xml.sax.ContentHandler):
 					if path:
 						sh = Part.Wire(path)
 						if self.fill: sh = Part.Face(sh)
-						if self.translate: sh.translate(self.translate)
-						obj = self.doc.addObject("Part::Feature","Path")
+                                                sh = self.applyTrans(sh)
+						obj = self.doc.addObject("Part::Feature",pathname)
 						obj.Shape = sh
 						self.format(obj)
 						path = []
@@ -495,14 +545,15 @@ class svgHandler(xml.sax.ContentHandler):
 			if path:
 				sh = Part.Wire(path)
 				if self.fill: sh = Part.Face(sh)
-				if self.translate: sh.translate(self.translate)
-				obj = self.doc.addObject("Part::Feature","Path")
+                                sh = self.applyTrans(sh)
+				obj = self.doc.addObject("Part::Feature",pathname)
 				obj.Shape = sh
 				self.format(obj)
 
 		# processing rects
 
 		if name == "rect":
+                        if not pathname: pathname = 'Rectangle'
 			p1 = Vector(data['x'],-data['y'],0)
 			p2 = Vector(data['x']+data['width'],-data['y'],0)
 			p3 = Vector(data['x']+data['width'],-data['y']-data['height'],0)
@@ -514,41 +565,131 @@ class svgHandler(xml.sax.ContentHandler):
 			edges.append(Part.Line(p4,p1).toShape())
 			sh = Part.Wire(edges)
 			if self.fill: sh = Part.Face(sh)
-			if self.translate: sh.translate(self.translate)
-			obj = self.doc.addObject("Part::Feature","Rectangle")
+                        sh = self.applyTrans(sh)
+			obj = self.doc.addObject("Part::Feature",pathname)
 			obj.Shape = sh
 			self.format(obj)
 				     
-#		processing lines
+                # processing lines
 
 		if name == "line":
-			p1 = Vector(data['x1'],-data['y1'],0)
-			p2 = Vector(data['x2'],-data['y2'],0)
+                        if not pathname: pathname = 'Line'
+			p1 = Vector(float(data['x1'][0]),-float(data['y1'][0]),0)
+			p2 = Vector(float(data['x2'][0]),-float(data['y2'][0]),0)
 			sh = Part.Line(p1,p2).toShape()
-			if self.translate: sh.translate(self.translate)
-			obj = self.doc.addObject("Part::Feature","Line")
+                        sh = self.applyTrans(sh)
+			obj = self.doc.addObject("Part::Feature",pathname)
 			obj.Shape = sh
 			self.format(obj)
 
-		if (name == "text") or (name == "tspan"):
-			self.x = data['x']
-			self.y = data['y']
+                # processing circles
+
+                if name == "circle":
+                        if not pathname: pathname = 'Circle'
+                        c = Vector(float(data['cx'][0]),-float(data['cy'][0]),0)
+                        r = float(data['r'][0])
+                        sh = Part.makeCircle(r)
+                        if self.fill:
+                                sh = Part.Wire([sh])
+                                sh = Part.Face(sh)
+                        sh.translate(c)
+                        sh = self.applyTrans(sh)
+			obj = self.doc.addObject("Part::Feature",pathname)
+			obj.Shape = sh
+			self.format(obj)
+
+                # processing texts
+
+		if (name in ["text","tspan"]):
+                        print "processing a text"
+			if 'x' in data:
+                                self.x = data['x']
+                        else:
+                                self.x = 0
+                        if 'y' in data:
+                                self.y = data['y']
+                        else:
+                                self.y = 0
 			if 'font-size' in data:
 				if data['font-size'] != 'none':
 					self.text = getsize(data['font-size'])
+                        else:
+                                self.text = 1
 
+                print "done processing element ",self.count
+                
 	def characters(self,content):
 		if self.text:
-			obj=self.doc.addObject("App::Annotation","Text")
+                        print "reading characters", str(content)
+			obj=self.doc.addObject("App::Annotation",'Text')
 			obj.LabelText = content.encode('latin1')
-			obj.Position = Vector(self.x,-self.y,0)
-			if self.translate: obj.Position = obj.Position.add(self.translate)
+			vec = Vector(self.x,-self.y,0)
+                        if self.transform:
+                                vec = self.translateVec(vec,self.transform)
+                                print "own transform: ",self.transform, vec
+                        for i in range(len(self.grouptransform)):
+                                #vec = self.translateVec(vec,self.grouptransform[-i-1])
+                                vec = self.grouptransform[-i-1].multiply(vec)
+                        print "applying vector: ",vec
+                        obj.Position = vec
 			if gui:
 				obj.ViewObject.FontSize = int(self.text)
 				if self.fill: obj.ViewObject.TextColor = self.fill
 				else: obj.ViewObject.TextColor = (0.0,0.0,0.0,0.0)
-			self.text = None
-			
+
+        def endElement(self, name):
+                if not name in ["tspan"]:
+                        self.transform = None
+                        self.text = None
+                if name == "g":
+                        print "closing group"
+                        self.grouptransform.pop()
+
+        def applyTrans(self,sh):
+                if self.transform:
+                        print "applying transform: ",self.transform
+                        sh = sh.transformGeometry(self.transform)
+                for i in range(len(self.grouptransform)):
+                        sh = sh.transformGeometry(self.grouptransform[-i-1])
+                '''
+                if 'scale' in self.transform:
+                        print "applying scale factor: ",self.transform['scale']
+                        m = FreeCAD.Matrix()
+                        m.scale(self.transform['scale'])
+                        sh = sh.transformGeometry(m)
+                if 'translate' in self.transform:
+                        print "applying translation: ",self.transform['translate']
+                        sh.translate(self.transform['translate'])
+                self.transform = {}
+                '''
+                return sh
+
+        def translateVec(self,vec,mat):
+                v = Vector(mat.A14,mat.A24,mat.A34)
+                return vec.add(v)
+
+        def getMatrix(self,tr):
+                "returns a FreeCAD matrix from a svg transform attribute"
+                s = ""
+                for l in tr:
+                        s += l
+                        s += " "
+                s=s.replace("("," ")
+                s=s.replace(")"," ")
+                s = s.strip()
+                tr = s.split()
+                m = FreeCAD.Matrix()
+                for i in range(len(tr)):
+                        if tr[i] == 'translate':
+                                vec = Vector(float(tr[i+1]),-float(tr[i+2]),0)
+                                m.move(vec)
+                        elif tr[i] == 'scale':
+                                vec = Vector(float(tr[i+1]),float(tr[i+2]),0)
+                                m.scale(vec)
+                        #elif tr[i] == 'rotate':
+                        #        m.rotateZ(float(tr[i+1]))
+                print "generating transformation: ",m
+                return m
 			
 def decodeName(name):
 	"decodes encoded strings"
@@ -562,6 +703,29 @@ def decodeName(name):
 			decodedName = name
 	return decodedName
 
+def getContents(filename,tag,stringmode=False):
+        "gets the contents of all the occurences of the given tag in the given file"
+        result = {}
+        if stringmode:
+                contents = filename
+        else:
+                f = pythonopen(filename)
+                contents = ''
+                for line in f: contents += line
+                f.close()
+        contents = contents.replace('\n','_linebreak')
+        searchpat = '<'+tag+'.*?</'+tag+'>'
+        tags = re.findall(searchpat,contents)
+        for t in tags:
+                tagid = re.findall('id="(.*?)"',t)
+                if tagid:
+                        tagid = tagid[0]
+                else:
+                        tagid = 'none'
+                res = t.replace('_linebreak','\n')
+                result[tagid] = res
+        return result
+
 def open(filename):
 	docname=os.path.split(filename)[1]
 	doc=FreeCAD.newDocument(docname)
@@ -569,11 +733,17 @@ def open(filename):
 	parser = xml.sax.make_parser()
 	parser.setContentHandler(svgHandler())
 	parser._cont_handler.doc = doc
-	parser.parse(pythonopen(filename))
+        f = pythonopen(filename)
+	parser.parse(f)
+        f.close()
 	doc.recompute()
+        return doc
 
 def insert(filename,docname):
-	doc=FreeCAD.getDocument(docname)
+	try:
+		doc=FreeCAD.getDocument(docname)
+	except:
+		doc=FreeCAD.newDocument(docname)
 	parser = xml.sax.make_parser()
 	parser.setContentHandler(svgHandler())
 	parser._cont_handler.doc = doc
@@ -589,7 +759,7 @@ def export(exportList,filename):
 	maxx = 0
 	maxy = 0
 	for ob in exportList:
-		if isinstance(ob,Part.Feature):
+		if ob.isDerivedFrom("Part::Feature"):
 			for v in ob.Shape.Vertexes:
 				if v.Point.x < minx: minx = v.Point.x
 				if v.Point.x > maxx: maxx = v.Point.x
@@ -618,119 +788,10 @@ def export(exportList,filename):
 
 	# writing paths
 	for ob in exportList:
-		if isinstance(ob,Part.Feature):
-			fill = 'none'
-			stroke = 'none'
-			width = 'none'
-			name = ob.Name
-			if ob.Shape.Faces:
-				if gui: fill = getrgb(ob.ViewObject.ShapeColor)
-				else: fill = '#000000'
-			if gui:
-				stroke = getrgb(ob.ViewObject.LineColor)
-				width = ob.ViewObject.LineWidth/100
-			if len(ob.Shape.Vertexes) > 1:
-				svg.write('<path id="' + name + '" ')
-				edges = fcgeo.sortEdges(ob.Shape.Edges)
-				v = edges[0].Vertexes[0].Point
-				svg.write('d="M '+ str(v.x-minx) +' '+ str(-v.y+boty) + ' ')
-				for e in edges:
-					if isinstance(e.Curve,Part.Line):
-						v = e.Vertexes[-1].Point
-						svg.write('L '+ str(v.x-minx) +' '+ str(-v.y+boty) + ' ')
-					elif isinstance(e.Curve,Part.Circle):
-						r = e.Curve.Radius
-						v = e.Vertexes[-1].Point
-						svg.write('A '+ str(r) + ' '+ str(r) +' 0 0 0 '+ str(v.x-minx) +' ')
-						svg.write(str(-v.y+boty) + ' ')
-				if fill != "none":
-					svg.write('Z')
-				svg.write('" ')
-			else:
-				cen = ob.Shape.Edges[0].Curve.Center
-				rad = ob.Shape.Edges[0].Curve.Radius
-				svg.write('<circle cx="'+str(cen.x-minx))
-				svg.write('" cy="'+str(-cen.y+boty))
-				svg.write('" r="'+str(rad)+'" ')
-			svg.write('fill="' + fill + '" stroke="' + stroke + '" ')
-			svg.write('stroke-width="' + str(width) + ' px" ')
-			svg.write('style="stroke-width:'+ str(width))
-			svg.write(';stroke-miterlimit:4;stroke-dasharray:none"')
-			svg.write('/>\n')
-					
-		elif (ob.Type == "App::Annotation"):
-			fill = getrgb(ob.ViewObject.TextColor)
-			height = str(ob.ViewObject.FontSize)
-			x = str(ob.Position.x-minx)
-			y = str(-ob.Position.y+boty)
-			name = ob.Name
-			svg.write('<text id="' + name + '" fill="'+ fill +'" font-size="'+ height)
-			svg.write('" x="' + x + '" y="' + y + '">\n')
-			for line in ob.LabelText:
-				svg.write(line)
-			svg.write('</text>\n')
-
-		elif (ob.Type == "App::FeaturePython"):
-			if 'Dimline' in ob.PropertiesList:
-				stroke = 'none'
-				width = 'none'
-				name = ob.Name
-				height = 0.2
-				dmax = 0
-				if gui:
-					stroke = getrgb(ob.ViewObject.LineColor)
-					width = ob.ViewObject.LineWidth/100
-					height = ob.ViewObject.FontSize
-					dmax = ob.ViewObject.ExtLines
-				p1 = ob.Start
-				p4 = ob.End
-				base = Part.Line(p1,p4).toShape()
-				proj = fcgeo.findDistance(ob.Dimline,base)
-				if not proj:
-					p2 = p1
-					p3 = p4
-				else:
-					p2 = p1.add(fcvec.neg(proj))
-					p3 = p4.add(fcvec.neg(proj))
-					if dmax and (proj.Length > dmax):
-						p1 = p2.add(fcvec.scale(proj.normalize(),dmax))
-						p4 = p3.add(fcvec.scale(proj.normalize(),dmax))
-				midpoint = p2.add(fcvec.scale(p3.sub(p2),0.5))
-				angle = fcvec.angle(p3.sub(p2))
-				if (angle > math.pi/2) or (angle < -math.pi/2): angle = angle+math.pi
-				offset = fcvec.rotate(FreeCAD.Vector(height*.2,0,0),-angle+math.pi/2)
-				tbase = midpoint.add(offset)
-				svg.write('<g id="'+name+'"> ')
-				svg.write('<path ')
-				svg.write('d="M '+str(p1.x-minx)+' '+str(-p1.y+boty)+' ')
-				svg.write('L '+str(p2.x-minx)+' '+str(-p2.y+boty)+' ')
-				svg.write('L '+str(p3.x-minx)+' '+str(-p3.y+boty)+' ')
-				svg.write('L '+str(p4.x-minx)+' '+str(-p4.y+boty)+'" ')
-				svg.write('fill="none" stroke="' + stroke + '" ')
-				svg.write('stroke-width="' + str(width) + ' px" ')
-				svg.write('style="stroke-width:'+ str(width))
-				svg.write(';stroke-miterlimit:4;stroke-dasharray:none"')
-				svg.write('/>\n')
-				svg.write('<circle cx="'+str(p2.x-minx))
-				svg.write('" cy="'+str(-p2.y+boty))
-				svg.write('" r="'+str(height/10)+'" ')
-				svg.write('fill="'+ stroke +'" stroke="none" ')
-				svg.write('style="stroke-miterlimit:4;stroke-dasharray:none"/>\n')
-				svg.write('<circle cx="'+str(p3.x-minx))
-				svg.write('" cy="'+str(-p3.y+boty))
-				svg.write('" r="'+str(height/10)+'" ')
-				svg.write('fill="#000000" stroke="none" ')
-				svg.write('style="stroke-miterlimit:4;stroke-dasharray:none"/>\n')
-				svg.write('<text id="' + name + '" fill="'+ stroke +'" font-size="'+ str(height)+'" ')
-				svg.write('style="text-anchor:middle;text-align:center" ')
-				svg.write('transform="rotate('+str(math.degrees(angle)))
-				svg.write(' '+str(tbase.x-minx)+' '+str(-tbase.y+boty)+')" ')
-				svg.write('x="' + str(tbase.x-minx) + '" y="' + str(-tbase.y+boty) + '">\n')
-				svg.write("%.2f" % p3.sub(p2).Length)
-				svg.write('</text>\n')
-				svg.write('</g>\n')
-
-
+                svg.write('<g transform="translate('+str(-minx)+','+str(-miny+(2*margin))+') scale(1,-1)">\n')
+                svg.write(Draft.getSVG(ob))
+                svg.write('</g>\n')
+                
 	# closing
 	svg.write('</svg>')
 	svg.close()
