@@ -30,6 +30,7 @@
 #include <App/DocumentObject.h>
 #include <App/PropertyStandard.h>
 #include <App/PropertyGeo.h>
+#include <App/FeaturePython.h>
 
 
 class PyObjectBase;
@@ -70,7 +71,10 @@ public:
     }
 };
 
+typedef App::FeaturePythonT<FeatureView> FeatureViewPython;
+
 } //namespace Drawing
 
 
+
 #endif
