@@ -30,6 +30,7 @@
 #include <App/DocumentObject.h>
 #include <App/PropertyLinks.h>
 #include "FeatureView.h"
+#include <App/FeaturePython.h>
 
 
 class PyObjectBase;
@@ -56,6 +57,7 @@ public:
     App::PropertyLink   Source;
     App::PropertyVector Direction;
     App::PropertyBool   ShowHiddenLines;
+    App::PropertyBool   ShowSmoothLines;
  
 
     /** @name methods overide Feature */
@@ -70,6 +72,9 @@ public:
     }
 };
 
+typedef App::FeaturePythonT<FeatureViewPart> FeatureViewPartPython;
+
+
 } //namespace Drawing
 
 
