@@ -27,6 +27,7 @@
 # ifdef FC_OS_WIN32
 #  include <windows.h>
 # endif
+# include <QTimer>
 #endif
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
@@ -59,8 +60,9 @@ ViewProviderDrawingPage::ViewProviderDrawingPage()
   : view(0)
 {
     sPixmap = "Page";
-    ADD_PROPERTY(HintScale,("10"));
-    ADD_PROPERTY(HintOffset,("10"));
+    ADD_PROPERTY(HintScale,(10.0));
+    ADD_PROPERTY(HintOffsetX,(10.0));
+    ADD_PROPERTY(HintOffsetY,(10.0));
 }
 
 ViewProviderDrawingPage::~ViewProviderDrawingPage()
@@ -75,14 +77,14 @@ void ViewProviderDrawingPage::attach(App::DocumentObject *pcFeat)
 
 void ViewProviderDrawingPage::setDisplayMode(const char* ModeName)
 {
-    ViewProviderDocumentObject::setDisplayMode( ModeName );
+    ViewProviderDocumentObject::setDisplayMode(ModeName);
 }
 
 std::vector<std::string> ViewProviderDrawingPage::getDisplayModes(void) const
 {
     // get the modes of the father
     std::vector<std::string> StrList = ViewProviderDocumentObject::getDisplayModes();
-
+    StrList.push_back("Drawing");
     return StrList;
 }
 
@@ -90,39 +92,36 @@ void ViewProviderDrawingPage::updateData(const App::Property* prop)
 {
     Gui::ViewProviderDocumentObjectGroup::updateData(prop);
     if (prop->getTypeId() == App::PropertyFileIncluded::getClassTypeId()) {
-        if (std::string(getPageObject()->PageResult.getValue()) != ""){
-            if (!view){
-                view = new DrawingView(Gui::getMainWindow());
-                view->load(QString::fromUtf8(getPageObject()->PageResult.getValue()));
-                view->setWindowIcon(Gui::BitmapFactory().pixmap("actions/drawing-landscape"));
-                view->setWindowTitle(QObject::tr("Drawing viewer"));
-                view->resize(400, 300);
-                view->oneToOneDrawing();
-                Gui::getMainWindow()->addWindow(view);
-            }
-            else {
-                try{
-                    view->load(QString::fromUtf8(getPageObject()->PageResult.getValue()));
-                }catch(...){}// dirty hack for the moment....
-            }
+        if (std::string(getPageObject()->PageResult.getValue()) != "") {
+            DrawingView* view = showDrawingView();
+            view->load(QString::fromUtf8(getPageObject()->PageResult.getValue()));
+            if (view->isHidden())
+                QTimer::singleShot(300, view, SLOT(viewAll()));
+            else
+                view->viewAll();
         }
     }
 }
 
 bool ViewProviderDrawingPage::doubleClicked(void)
 {
- /*   view = new DrawingView(Gui::getMainWindow());
-    view->load(QString::fromUtf8(getPageObject()->PageResult.getValue()));
-    view->setWindowIcon(Gui::BitmapFactory().pixmap("actions/drawing-landscape"));
-    view->setWindowTitle(QObject::tr("Drawing viewer"));
-    view->resize( 400, 300 );
-    Gui::getMainWindow()->addWindow(view);
-
-    return true;*/
-    return false;
+    Gui::getMainWindow()->setActiveWindow(showDrawingView());
+    return true;
 }
 
-Drawing::FeaturePage* ViewProviderDrawingPage::getPageObject(void)
+DrawingView* ViewProviderDrawingPage::showDrawingView()
+{
+    if (!view){
+        view = new DrawingView(Gui::getMainWindow());
+        view->setWindowIcon(Gui::BitmapFactory().pixmap("actions/drawing-landscape"));
+        view->setWindowTitle(QObject::tr("Drawing viewer"));
+        Gui::getMainWindow()->addWindow(view);
+    }
+
+    return view;
+}
+
+Drawing::FeaturePage* ViewProviderDrawingPage::getPageObject() const
 {
     return dynamic_cast<Drawing::FeaturePage*>(pcObject);
 }
