@@ -24,6 +24,7 @@
 #ifndef DRAWINGGUI_VIEWPROVIDERPAGE_H
 #define DRAWINGGUI_VIEWPROVIDERPAGE_H
 
+#include <QPointer>
 #include <Gui/ViewProviderFeature.h>
 #include <Gui/ViewProviderDocumentObjectGroup.h>
 
@@ -36,39 +37,36 @@ namespace Drawing{
 namespace DrawingGui {
 
 
-class DrawingGuiExport ViewProviderDrawingPage:public Gui::ViewProviderDocumentObjectGroup
+class DrawingGuiExport ViewProviderDrawingPage : public Gui::ViewProviderDocumentObjectGroup
 {
-  PROPERTY_HEADER(DrawingGui::ViewProviderDrawingPage);
+    PROPERTY_HEADER(DrawingGui::ViewProviderDrawingPage);
 
 public:
-  /// constructor
-  ViewProviderDrawingPage();
-  /// destructor
-  virtual ~ViewProviderDrawingPage();
+    /// constructor
+    ViewProviderDrawingPage();
+    /// destructor
+    virtual ~ViewProviderDrawingPage();
 
-  App::PropertyString         HintScale;
-  App::PropertyString         HintOffset;
+    App::PropertyFloat         HintScale;
+    App::PropertyFloat         HintOffsetX;
+    App::PropertyFloat         HintOffsetY;
 
-  virtual void attach(App::DocumentObject *);
-  virtual void setDisplayMode(const char* ModeName);
-  /// returns a list of all possible modes
-  virtual std::vector<std::string> getDisplayModes(void) const;
-  /// Update the Drawing representation
-  //virtual void update(const ChangeType&);
+    virtual void attach(App::DocumentObject *);
+    virtual void setDisplayMode(const char* ModeName);
+    /// returns a list of all possible modes
+    virtual std::vector<std::string> getDisplayModes(void) const;
 
-  /// Is called by the tree if the user double click on the object
-  virtual bool doubleClicked(void);
-
-
-  virtual void updateData(const App::Property*);
-
-  Drawing::FeaturePage* getPageObject(void);
+    /// Is called by the tree if the user double click on the object
+    virtual bool doubleClicked(void);
+    virtual void updateData(const App::Property*);
 
+    Drawing::FeaturePage* getPageObject() const;
 
 protected:
+    DrawingView* showDrawingView();
 
-    DrawingView* view;
-
+private:
+    QPointer<DrawingView> view;
 };
 
 } // namespace DrawingGui
