@@ -265,6 +265,12 @@ public:
   inline unsigned short CountOpenEdges() const;
   /** Returns true if there is an edge without neighbour, otherwise false. */
   inline bool HasOpenEdge() const;
+  /** Returns true if the two facets have the same orientation, false otherwise
+   * Therefore the two facets must be adjacent.
+   */
+  inline bool HasSameOrientation(const MeshFacet&) const;
+  /** Checks wether the facet is degenerated to a line of point. */
+  inline bool IsDegenerated() const;
   /** Flips the orientation of the facet. */
   void FlipNormal (void)
   {
@@ -299,7 +305,7 @@ public:
 public:
   /**
    * Checks if the point is part of the facet. A point is regarded as part
-   * of a facet if the distance is lower \s fDistance and the projected point
+   * of a facet if the distance is lower \a fDistance and the projected point
    * in the facet normal direction is inside the triangle.
    */
   bool IsPointOf (const Base::Vector3f &rclPoint, float fDistance) const;
@@ -508,12 +514,12 @@ public:
   MeshPointArray& operator = (const MeshPointArray &rclPAry);
   /**
    * Searches for the first point index  Two points are equal if the distance is less
-   * than FLOAT_EPS. If no such points is found ULONG_MAX is returned. 
+   * than EPSILON. If no such points is found ULONG_MAX is returned. 
    */
   unsigned long Get (const MeshPoint &rclPoint);
   /**
    * Searches for the first point index  Two points are equal if the distance is less
-   * than FLOAT_EPS. If no such points is found the point is added to the array at end
+   * than EPSILON. If no such points is found the point is added to the array at end
    * and its index is returned. 
    */
   unsigned long GetOrAddIndex (const MeshPoint &rclPoint);
@@ -842,6 +848,33 @@ inline bool MeshFacet::HasOpenEdge() const
     return (CountOpenEdges() != 0);
 }
 
+inline bool MeshFacet::HasSameOrientation(const MeshFacet& f) const
+{
+    for (int i = 0; i < 3; i++) {
+        for (int j = 0; j < 3; j++) {
+            if (_aulPoints[i] == f._aulPoints[j]) {
+                if ((_aulPoints[(i+1)%3] == f._aulPoints[(j+1)%3]) ||
+                    (_aulPoints[(i+2)%3] == f._aulPoints[(j+2)%3])) {
+                    return false;
+                }
+            }
+        }
+    }
+
+    return true;
+}
+
+inline bool MeshFacet::IsDegenerated() const
+{
+    if (_aulPoints[0] == _aulPoints[1])
+        return true;
+    if (_aulPoints[1] == _aulPoints[2])
+        return true;
+    if (_aulPoints[2] == _aulPoints[0])
+        return true;
+    return false;
+}
+
 inline unsigned short MeshFacet::Side (unsigned long ulNIndex) const
 {
     if (_aulNeighbours[0] == ulNIndex)
@@ -907,7 +940,7 @@ inline bool MeshFacet::IsEqual (const MeshFacet& rcFace) const
 }
 
 /**
- * Binary function to query the flags for use with generic STL functions..
+ * Binary function to query the flags for use with generic STL functions.
  */
 template <class TCLASS>
 class MeshIsFlag : public std::binary_function<TCLASS, typename TCLASS::TFlagType, bool>
@@ -918,7 +951,7 @@ public:
 };
 
 /**
- * Binary function to query the flags for use with generic STL functions..
+ * Binary function to query the flags for use with generic STL functions.
  */
 template <class TCLASS>
 class MeshIsNotFlag : public std::binary_function<TCLASS, typename TCLASS::TFlagType, bool>
@@ -929,7 +962,7 @@ public:
 };
 
 /**
- * Binary function to set the flags for use with generic STL functions..
+ * Binary function to set the flags for use with generic STL functions.
  */
 template <class TCLASS>
 class MeshSetFlag : public std::binary_function<TCLASS, typename TCLASS::TFlagType, bool>
@@ -940,7 +973,7 @@ public:
 };
 
 /**
- * Binary function to reset the flags for use with generic STL functions..
+ * Binary function to reset the flags for use with generic STL functions.
  */
 template <class TCLASS>
 class MeshResetFlag : public std::binary_function<TCLASS, typename TCLASS::TFlagType, bool>
