@@ -97,7 +97,6 @@ public:
     /** Returns false after the first inconsistence is found, true otherwise. */
     bool Visit (const MeshFacet &, const MeshFacet &, unsigned long , unsigned long );
     bool HasNonUnifomOrientedFacets() const;
-    bool IsSameOrientation(const MeshFacet &, const MeshFacet &) const;
 
 private:
     bool _nonuniformOrientation;
@@ -335,7 +334,8 @@ public:
 
   bool Evaluate();
   /** 
-   * Calculates the local coordinate system defined by \a u, \av, \a w and \a c. 
+   * Calculates the local coordinate system defined by \a u, \a v, \a w
+   * and \a c. 
    */
 protected:
   void CalculateLocalSystem();
