@@ -40,6 +40,7 @@
 
 #include <math.h>
 #include <sstream>
+#include <iomanip>
 #include <boost/regex.hpp>
 
 
@@ -92,6 +93,18 @@ std::string& ltrim(std::string& str)
     return str;
 }
 
+int numDigits(int number)
+{
+    number = std::abs(number);
+    int digits = 1;
+    int step = 10;
+    while (step <= number) {
+        digits++;
+        step *= 10;
+    }
+    return digits;
+}
+
 /* Define a vector.  */
 typedef struct _Vector
 	{
@@ -349,9 +362,9 @@ bool MeshInput::LoadSTL (std::istream &rstrIn)
     uint32_t ulCt;
     rstrIn.read((char*)&ulCt, sizeof(ulCt));
     // Either it's really an invalid STL file or it's just empty. In this case the number of facets must be 0.
-    if (rstrIn.read(szBuf, 50) == false)
+    if (rstrIn.read(szBuf, 100) == false)
         return (ulCt==0);
-    szBuf[50] = 0;
+    szBuf[100] = 0;
     upper(szBuf);
 
     try {
@@ -523,7 +536,7 @@ bool MeshInput::LoadPLY (std::istream &inp)
         return false; // wrong header
 
     std::string line, element;
-    bool xyz_float=false,xyz_double=false;
+    bool xyz_float=false,xyz_double=false,rgb_value=false;
     while (std::getline(inp, line)) {
         std::istringstream str(line);
         str.unsetf(std::ios_base::skipws);
@@ -599,6 +612,9 @@ bool MeshInput::LoadPLY (std::istream &inp)
                     else if (type == "double" || type == "float64")
                         xyz_double = true;
                 }
+                else if (name == "red") {
+                    rgb_value = true;
+                }
             }
             else if (element == "face") {
             }
@@ -642,17 +658,21 @@ bool MeshInput::LoadPLY (std::istream &inp)
             is.setByteOrder(Base::Stream::LittleEndian);
         else
             is.setByteOrder(Base::Stream::BigEndian);
+        int r,g,b;
         if (xyz_float) {
             Base::Vector3f pt;
             for (std::size_t i = 0; i < v_count; i++) {
                 is >> pt.x >> pt.y >> pt.z;
                 meshPoints.push_back(pt);
+                if (rgb_value)
+                    is >> r >> g >> b;
             }
         }
         else if (xyz_double) {
             Base::Vector3d pt;
             for (std::size_t i = 0; i < v_count; i++) {
                 is >> pt.x >> pt.y >> pt.z;
+                is >> r >> g >> b;
                 meshPoints.push_back(Base::Vector3f((float)pt.x,(float)pt.y,(float)pt.z));
             }
         }
@@ -1480,14 +1500,21 @@ bool MeshOutput::SavePLY (std::ostream &out) const
     std::size_t f_count = rFacets.size();
     if (!out || out.bad() == true)
         return false;
+    bool saveVertexColor = (_material && _material->binding == MeshIO::PER_VERTEX
+        && _material->diffuseColor.size() == rPoints.size());
     out << "ply" << std::endl
         << "format binary_little_endian 1.0" << std::endl
         << "comment Created by FreeCAD <http://free-cad.sourceforge.net>" << std::endl
         << "element vertex " << v_count << std::endl
         << "property float32 x" << std::endl
         << "property float32 y" << std::endl
-        << "property float32 z" << std::endl
-        << "element face " << f_count << std::endl
+        << "property float32 z" << std::endl;
+    if (saveVertexColor) {
+        out << "property uchar red" << std::endl
+            << "property uchar green" << std::endl
+            << "property uchar blue" << std::endl;
+    }
+    out << "element face " << f_count << std::endl
         << "property list uchar int vertex_index" << std::endl
         << "end_header" << std::endl;
 
@@ -1497,6 +1524,13 @@ bool MeshOutput::SavePLY (std::ostream &out) const
     for (std::size_t i = 0; i < v_count; i++) {
         const MeshPoint& p = rPoints[i];
         os << p.x << p.y << p.z;
+        if (saveVertexColor) {
+            const App::Color& c = _material->diffuseColor[i];
+            int r = (int)(255.0f * c.r);
+            int g = (int)(255.0f * c.g);
+            int b = (int)(255.0f * c.b);
+            os << r << g << b;
+        }
     }
     unsigned char n = 3;
     int f1, f2, f3;
@@ -1696,73 +1730,24 @@ bool MeshOutput::SaveNastran (std::ostream &rstrOut) const
 
     MeshPointIterator clPIter(_rclMesh);
     MeshFacetIterator clTIter(_rclMesh);
-    int iIndx = 1, iDec, iSign, iCount = 0;
+    int iIndx = 1;
 
     Base::SequencerLauncher seq("Saving...", _rclMesh.CountFacets() + 1);
 
     rstrOut.precision(3);
     rstrOut.setf(std::ios::fixed | std::ios::showpoint);
     for (clPIter.Init(); clPIter.More(); clPIter.Next()) {
+        float x = clPIter->x;
+        float y = clPIter->y;
+        float z = clPIter->z;
+
         rstrOut << "GRID";
 
-#ifdef __GNUC__
-        fcvt ((float)iIndx, iCount, &iDec, &iSign);
-#else
-        _fcvt ((float)iIndx, iCount, &iDec, &iSign);
-#endif
-        for (int i = iDec; i <= 11; i++) {
-            rstrOut << " ";
-        }
-
-        rstrOut << iIndx;
-
-#ifdef __GNUC__
-        fcvt ((*clPIter).x, iCount, &iDec, &iSign);
-#else
-        _fcvt ((*clPIter).x, iCount, &iDec, &iSign);
-#endif
-        if (iSign == 0) {
-            rstrOut << " ";
-        }
-        if (fabs((*clPIter).x) < pow (10.0, (iDec-1))) iDec--;
-        if (iDec <= 0) iDec = 1;
-        for (int i = iDec; i <= 10; i++) {
-            rstrOut << " ";
-        }
-
-        rstrOut << clPIter->x;
-
-#ifdef __GNUC__
-        fcvt ((*clPIter).y, iCount, &iDec, &iSign);
-#else
-        _fcvt ((*clPIter).y, iCount, &iDec, &iSign);
-#endif
-        if (iSign == 0) {
-            rstrOut << " ";
-        }
-        if (fabs((*clPIter).y) < pow (10.0, (iDec-1))) iDec--;
-        if (iDec <= 0) iDec = 1;
-        for (int i = iDec; i <= 2; i++) {
-            rstrOut << " ";
-        }
-
-        rstrOut << clPIter->y;
-
-#ifdef __GNUC__
-        fcvt ((*clPIter).z, iCount, &iDec, &iSign);
-#else
-        _fcvt ((*clPIter).z, iCount, &iDec, &iSign);
-#endif
-        if (iSign == 0) {
-            rstrOut << " ";
-        }
-        if (fabs((*clPIter).z) < pow (10.0, (iDec-1))) iDec--;
-        if (iDec <= 0) iDec = 1;
-        for (int i = iDec; i <= 2; i++) {
-            rstrOut << " ";
-        }
-
-        rstrOut << clPIter->z << std::endl;
+        rstrOut << std::setfill(' ') << std::setw(12) << iIndx;
+        rstrOut << std::setfill(' ') << std::setw(16) << x;
+        rstrOut << std::setfill(' ') << std::setw(8)  << y;
+        rstrOut << std::setfill(' ') << std::setw(8)  << z;
+        rstrOut << std::endl;
 
         iIndx++;
         seq.next();
@@ -1772,52 +1757,13 @@ bool MeshOutput::SaveNastran (std::ostream &rstrOut) const
     for (clTIter.Init(); clTIter.More(); clTIter.Next()) {
         rstrOut << "CTRIA3";
 
-#ifdef __GNUC__
-        fcvt ((float)iIndx, iCount, &iDec, &iSign);
-#else
-        _fcvt ((float)iIndx, iCount, &iDec, &iSign);
-#endif
-        for (int i = iDec; i <= 9; i++) {
-            rstrOut << " ";
-        }
+        rstrOut << std::setfill(' ') << std::setw(10) << iIndx;
+        rstrOut << std::setfill(' ') << std::setw(8) << (int)0;
+        rstrOut << std::setfill(' ') << std::setw(8) << clTIter.GetIndices()._aulPoints[1]+1;
+        rstrOut << std::setfill(' ') << std::setw(8) << clTIter.GetIndices()._aulPoints[0]+1;
+        rstrOut << std::setfill(' ') << std::setw(8) << clTIter.GetIndices()._aulPoints[2]+1;
+        rstrOut <<std::endl;
 
-        rstrOut << iIndx << "       0";
-
-#ifdef __GNUC__
-        fcvt ((float)clTIter.GetIndices()._aulPoints[1]+1, iCount, &iDec, &iSign);
-#else
-        _fcvt ((float)clTIter.GetIndices()._aulPoints[1]+1, iCount, &iDec, &iSign);
-#endif
-        for (int i = iDec; i <= 7; i++) {
-            rstrOut << " ";
-        }
-
-        rstrOut << clTIter.GetIndices()._aulPoints[1]+1;
-
-#ifdef __GNUC__
-        fcvt ((float)clTIter.GetIndices()._aulPoints[0]+1, iCount, &iDec, &iSign);
-#else
-        _fcvt ((float)clTIter.GetIndices()._aulPoints[0]+1, iCount, &iDec, &iSign);
-#endif
-        for (int i = iDec; i <= 7; i++) {
-            rstrOut << " ";
-        }
-
-        rstrOut << clTIter.GetIndices()._aulPoints[0]+1;
-
-#ifdef __GNUC__
-        fcvt ((float)clTIter.GetIndices()._aulPoints[2]+1, iCount, &iDec, &iSign);
-#else
-        _fcvt ((float)clTIter.GetIndices()._aulPoints[2]+1, iCount, &iDec, &iSign);
-#endif
-        for (int i = iDec; i <= 7; i++) {
-            rstrOut << " ";
-        }
-
-        rstrOut << clTIter.GetIndices()._aulPoints[2]+1 << std::endl;
-
-//    sprintf(szBuf, "CTRIA3 %d 0 %d %d %d\n", iIndx, clTIter.GetIndices()._aulPoints[0]+1, clTIter.GetIndices()._aulPoints[1]+1, clTIter.GetIndices()._aulPoints[2]+1);
-//    rstrOut.write(szBuf, strlen(szBuf));
         iIndx++;
         seq.next();
     }
