@@ -44,6 +44,7 @@
 #include "Core/Evaluation.h"
 #include "Core/Degeneration.h"
 #include "Core/SetOperations.h"
+#include "Core/Visitor.h"
 
 #include "Mesh.h"
 #include "MeshPy.h"
@@ -102,6 +103,14 @@ Data::Segment* MeshObject::getSubElement(const char* Type, unsigned long n) cons
     return 0;
 }
 
+void MeshObject::transformGeometry(const Base::Matrix4D &rclMat)
+{
+    MeshCore::MeshKernel kernel;
+    swap(kernel);
+    kernel.Transform(rclMat);
+    swap(kernel);
+}
+
 void MeshObject::setTransform(const Base::Matrix4D& rclTrf)
 {
     _Mtrx = rclTrf;
@@ -270,10 +279,19 @@ void MeshObject::RestoreDocFile(Base::Reader &reader)
     load(reader);
 }
 
-void MeshObject::save(const char* file, MeshCore::MeshIO::Format f) const
+void MeshObject::save(const char* file, MeshCore::MeshIO::Format f,
+                      const MeshCore::Material* mat) const
 {
-    MeshCore::MeshOutput aWriter(_kernel);
-    aWriter.SaveAny(file, f);
+    if (this->_Mtrx != Base::Matrix4D()) {
+        MeshCore::MeshKernel kernel(this->_kernel);
+        kernel.Transform(this->_Mtrx);
+        MeshCore::MeshOutput aWriter(kernel, mat);
+        aWriter.SaveAny(file, f);
+    }
+    else {
+        MeshCore::MeshOutput aWriter(this->_kernel, mat);
+        aWriter.SaveAny(file, f);
+    }
 }
 
 void MeshObject::save(std::ostream& out) const
@@ -303,7 +321,7 @@ bool MeshObject::load(const char* file)
         if (prop < it->_ulProp) {
             prop = it->_ulProp;
             if (!segment.empty()) {
-                this->_segments.push_back(Segment(this,segment));
+                this->_segments.push_back(Segment(this,segment,true));
                 segment.clear();
             }
         }
@@ -313,7 +331,7 @@ bool MeshObject::load(const char* file)
 
     // if the whole mesh is a single object then don't mark as segment
     if (!segment.empty() && (segment.size() < faces.size())) {
-        this->_segments.push_back(Segment(this,segment));
+        this->_segments.push_back(Segment(this,segment,true));
     }
 
 #ifndef FC_DEBUG
@@ -375,6 +393,11 @@ void MeshObject::addFacets(const std::vector<MeshCore::MeshGeomFacet>& facets)
     _kernel.AddFacets(facets);
 }
 
+void MeshObject::addFacets(const std::vector<MeshCore::MeshFacet> &facets)
+{
+    _kernel.AddFacets(facets);
+}
+
 void MeshObject::addFacets(const std::vector<MeshCore::MeshFacet> &facets,
                            const std::vector<Base::Vector3f>& points)
 {
@@ -508,6 +531,16 @@ void MeshObject::deleteSelectedPoints()
     deletePoints(points);
 }
 
+void MeshObject::clearFacetSelection() const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).ResetFacetFlag(MeshCore::MeshFacet::SELECTED);
+}
+
+void MeshObject::clearPointSelection() const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).ResetPointFlag(MeshCore::MeshPoint::SELECTED);
+}
+
 void MeshObject::addFacetsToSelection(const std::vector<unsigned long>& inds) const
 {
     MeshCore::MeshAlgorithm(this->_kernel).SetFacetsFlag(inds, MeshCore::MeshFacet::SELECTED);
@@ -728,13 +761,13 @@ Base::Vector3d MeshObject::getPointNormal(unsigned long index) const
     return normal;
 }
 
-void MeshObject::crossSections(const std::vector<MeshObject::Plane>& planes, std::vector<MeshObject::Polylines> &sections,
+void MeshObject::crossSections(const std::vector<MeshObject::TPlane>& planes, std::vector<MeshObject::TPolylines> &sections,
                                float fMinEps, bool bConnectPolygons) const
 {
     MeshCore::MeshFacetGrid grid(_kernel);
     MeshCore::MeshAlgorithm algo(_kernel);
-    for (std::vector<MeshObject::Plane>::const_iterator it = planes.begin(); it != planes.end(); ++it) {
-        MeshObject::Polylines polylines;
+    for (std::vector<MeshObject::TPlane>::const_iterator it = planes.begin(); it != planes.end(); ++it) {
+        MeshObject::TPolylines polylines;
         algo.CutWithPlane(it->first, it->second, grid, polylines, fMinEps, bConnectPolygons);
         sections.push_back(polylines);
     }
@@ -824,7 +857,10 @@ void MeshObject::refine()
 void MeshObject::optimizeTopology(float fMaxAngle)
 {
     MeshCore::MeshTopoAlgorithm topalg(_kernel);
-    topalg.OptimizeTopology(fMaxAngle);
+    if (fMaxAngle > 0.0f)
+        topalg.OptimizeTopology(fMaxAngle);
+    else
+        topalg.OptimizeTopology();
 
     // clear the segments because we don't know how the new
     // topology looks like
@@ -984,11 +1020,41 @@ void MeshObject::removeSelfIntersections()
 
 void MeshObject::removeFoldsOnSurface()
 {
-    unsigned long count = _kernel.CountFacets();
-    MeshCore::MeshFixFoldsOnSurface(_kernel).Fixup();
-    MeshCore::MeshFixFoldsOnBoundary(_kernel).Fixup();
-    if (_kernel.CountFacets() < count)
-        this->_segments.clear();
+    std::vector<unsigned long> indices;
+    MeshCore::MeshEvalFoldsOnSurface s_eval(_kernel);
+    MeshCore::MeshEvalFoldOversOnSurface f_eval(_kernel);
+
+    f_eval.Evaluate();
+    std::vector<unsigned long> inds  = f_eval.GetIndices();
+
+    s_eval.Evaluate();
+    std::vector<unsigned long> inds1 = s_eval.GetIndices();
+
+    // remove duplicates
+    inds.insert(inds.end(), inds1.begin(), inds1.end());
+    std::sort(inds.begin(), inds.end());
+    inds.erase(std::unique(inds.begin(), inds.end()), inds.end());
+
+    if (!inds.empty())
+        deleteFacets(inds);
+
+    // do this as additional check after removing folds on closed area
+    for (int i=0; i<5; i++) {
+        MeshCore::MeshEvalFoldsOnBoundary b_eval(_kernel);
+        if (b_eval.Evaluate())
+            break;
+        inds = b_eval.GetIndices();
+        if (!inds.empty())
+            deleteFacets(inds);
+    }
+}
+
+void MeshObject::removeFullBoundaryFacets()
+{
+    std::vector<unsigned long> facets;
+    if (!MeshCore::MeshEvalBorderFacet(_kernel, facets).Evaluate()) {
+        deleteFacets(facets);
+    }
 }
 
 void MeshObject::validateIndices()
@@ -1231,7 +1297,7 @@ void MeshObject::addSegment(const std::vector<unsigned long>& inds)
             throw Base::Exception("Index out of range");
     }
 
-    this->_segments.push_back(Segment(this,inds));
+    this->_segments.push_back(Segment(this,inds,true));
 }
 
 const Segment& MeshObject::getSegment(unsigned long index) const
@@ -1260,6 +1326,53 @@ MeshObject* MeshObject::meshFromSegment(const std::vector<unsigned long>& indice
     return new MeshObject(kernel, _Mtrx);
 }
 
+std::vector<Segment> MeshObject::getSegmentsFromType(MeshObject::Type type, const Segment& aSegment, float dev) const
+{
+    std::vector<Segment> segm;
+    unsigned long startFacet, visited;
+    if (this->_kernel.CountFacets() == 0)
+        return segm;
+
+    // reset VISIT flags
+    MeshCore::MeshAlgorithm cAlgo(this->_kernel);
+    if (aSegment.isEmpty()) {
+        cAlgo.ResetFacetFlag(MeshCore::MeshFacet::VISIT);
+    }
+    else {
+        cAlgo.SetFacetFlag(MeshCore::MeshFacet::VISIT);
+        cAlgo.ResetFacetsFlag(aSegment.getIndices(), MeshCore::MeshFacet::VISIT);
+    }
+
+    const MeshCore::MeshFacetArray& rFAry = this->_kernel.GetFacets();
+    MeshCore::MeshFacetArray::_TConstIterator iTri = rFAry.begin();
+    MeshCore::MeshFacetArray::_TConstIterator iBeg = rFAry.begin();
+    MeshCore::MeshFacetArray::_TConstIterator iEnd = rFAry.end();
+
+    // start from the first not visited facet
+    visited = cAlgo.CountFacetFlag(MeshCore::MeshFacet::VISIT);
+    iTri = std::find_if(iTri, iEnd, std::bind2nd(MeshCore::MeshIsNotFlag<MeshCore::MeshFacet>(),
+        MeshCore::MeshFacet::VISIT));
+    startFacet = iTri - iBeg;
+
+    while (startFacet != ULONG_MAX) {
+        // collect all facets of the same geometry
+        std::vector<unsigned long> indices;
+        indices.push_back(startFacet);
+        MeshCore::MeshPlaneVisitor pv(this->_kernel, startFacet, dev, indices);
+        visited += this->_kernel.VisitNeighbourFacets(pv, startFacet);
+
+        iTri = std::find_if(iTri, iEnd, std::bind2nd(MeshCore::MeshIsNotFlag<MeshCore::MeshFacet>(),
+            MeshCore::MeshFacet::VISIT));
+        if (iTri < iEnd)
+            startFacet = iTri - iBeg;
+        else
+            startFacet = ULONG_MAX;
+        segm.push_back(Segment(const_cast<MeshObject*>(this), indices, false));
+    }
+
+    return segm;
+}
+
 // ----------------------------------------------------------------------------
 
 MeshObject::const_point_iterator::const_point_iterator(const MeshObject* mesh, unsigned long index)
