@@ -66,9 +66,11 @@ class MeshExport MeshObject : public Data::ComplexGeoData
     TYPESYSTEM_HEADER();
 
 public:
+    enum Type {PLANE, CYLINDER, SPHERE};
+
     // typedef needed for cross-section
-    typedef std::pair<Base::Vector3f, Base::Vector3f> Plane;
-    typedef std::list<std::vector<Base::Vector3f> > Polylines;
+    typedef std::pair<Base::Vector3f, Base::Vector3f> TPlane;
+    typedef std::list<std::vector<Base::Vector3f> > TPolylines;
 
     MeshObject();
     explicit MeshObject(const MeshCore::MeshKernel& Kernel);
@@ -92,6 +94,7 @@ public:
 
     void setTransform(const Base::Matrix4D& rclTrf);
     Base::Matrix4D getTransform(void) const;
+    void transformGeometry(const Base::Matrix4D &rclMat);
 
     /**
      * Swaps the content of \a Kernel and the internal mesh kernel.
@@ -132,7 +135,8 @@ public:
     void SaveDocFile (Base::Writer &writer) const;
     void Restore(Base::XMLReader &reader);
     void RestoreDocFile(Base::Reader &reader);
-    void save(const char* file,MeshCore::MeshIO::Format f=MeshCore::MeshIO::Undefined) const;
+    void save(const char* file,MeshCore::MeshIO::Format f=MeshCore::MeshIO::Undefined,
+        const MeshCore::Material* mat = 0) const;
     void save(std::ostream&) const;
     bool load(const char* file);
     void load(std::istream&);
@@ -142,6 +146,7 @@ public:
     //@{
     void addFacet(const MeshCore::MeshGeomFacet& facet);
     void addFacets(const std::vector<MeshCore::MeshGeomFacet>& facets);
+    void addFacets(const std::vector<MeshCore::MeshFacet> &facets);
     void addFacets(const std::vector<MeshCore::MeshFacet> &facets,
                    const std::vector<Base::Vector3f>& points);
     void addFacets(const std::vector<Data::ComplexGeoData::FacetTopo> &facets,
@@ -150,27 +155,19 @@ public:
     void setFacets(const std::vector<Data::ComplexGeoData::FacetTopo> &facets,
                    const std::vector<Base::Vector3d>& points);
     /**
-     * Combines two independant mesh objects.
+     * Combines two independent mesh objects.
      * @note The mesh object we want to add must not overlap or intersect with
      * this mesh object.
      */
     void addMesh(const MeshObject&);
     /**
-     * Combines two independant mesh objects.
+     * Combines two independent mesh objects.
      * @note The mesh object we want to add must not overlap or intersect with
      * this mesh object.
      */
     void addMesh(const MeshCore::MeshKernel&);
     void deleteFacets(const std::vector<unsigned long>& removeIndices);
     void deletePoints(const std::vector<unsigned long>& removeIndices);
-    void deleteSelectedFacets();
-    void deleteSelectedPoints();
-    void addFacetsToSelection(const std::vector<unsigned long>&) const;
-    void addPointsToSelection(const std::vector<unsigned long>&) const;
-    void removeFacetsFromSelection(const std::vector<unsigned long>&) const;
-    void removePointsFromSelection(const std::vector<unsigned long>&) const;
-    void getFacetsFromSelection(std::vector<unsigned long>&) const;
-    void getPointsFromSelection(std::vector<unsigned long>&) const;
     std::vector<std::vector<unsigned long> > getComponents() const;
     unsigned long countComponents() const;
     void removeComponents(unsigned long);
@@ -185,10 +182,24 @@ public:
     void setPoint(unsigned long, const Base::Vector3d& v);
     void smooth(int iterations, float d_max);
     Base::Vector3d getPointNormal(unsigned long) const;
-    void crossSections(const std::vector<Plane>&, std::vector<Polylines> &sections,
+    void crossSections(const std::vector<TPlane>&, std::vector<TPolylines> &sections,
                        float fMinEps = 1.0e-2f, bool bConnectPolygons = false) const;
     //@}
 
+    /** @name Selection */
+    //@{
+    void deleteSelectedFacets();
+    void deleteSelectedPoints();
+    void addFacetsToSelection(const std::vector<unsigned long>&) const;
+    void addPointsToSelection(const std::vector<unsigned long>&) const;
+    void removeFacetsFromSelection(const std::vector<unsigned long>&) const;
+    void removePointsFromSelection(const std::vector<unsigned long>&) const;
+    void getFacetsFromSelection(std::vector<unsigned long>&) const;
+    void getPointsFromSelection(std::vector<unsigned long>&) const;
+    void clearFacetSelection() const;
+    void clearPointSelection() const;
+    //@}
+
     /** @name Boolean operations */
     //@{
     MeshObject* unite(const MeshObject&) const;
@@ -229,6 +240,7 @@ public:
     bool hasSelfIntersections() const;
     void removeSelfIntersections();
     void removeFoldsOnSurface();
+    void removeFullBoundaryFacets();
     //@}
 
     /** @name Mesh segments */
@@ -238,6 +250,7 @@ public:
     const Segment& getSegment(unsigned long) const;
     Segment& getSegment(unsigned long);
     MeshObject* meshFromSegment(const std::vector<unsigned long>&) const;
+    std::vector<Segment> getSegmentsFromType(Type, const Segment& aSegment, float dev) const;
     //@}
 
     /** @name Primitives */
