@@ -281,7 +281,7 @@ void PropertyCurvatureList::Paste(const App::Property &from)
 // ----------------------------------------------------------------------------
 
 PropertyMeshKernel::PropertyMeshKernel()
-  : _meshObject(new MeshObject())
+  : _meshObject(new MeshObject()), meshPyObject(0)
 {
     // Note: Normally this property is a member of a document object, i.e. the setValue()
     // method gets called in the constructor of a sublcass of DocumentObject, e.g. Mesh::Feature.
@@ -291,6 +291,12 @@ PropertyMeshKernel::PropertyMeshKernel()
 
 PropertyMeshKernel::~PropertyMeshKernel()
 {
+    if (meshPyObject) {
+        // Note: Do not call setInvalid() of the Python binding 
+        // because the mesh should still be accessible afterwards.
+        meshPyObject->parentProperty = 0;
+        Py_DECREF(meshPyObject);
+    }
 }
 
 void PropertyMeshKernel::setValuePtr(MeshObject* mesh)
@@ -341,6 +347,11 @@ const MeshObject* PropertyMeshKernel::getValuePtr(void)const
     return (MeshObject*)_meshObject;
 }
 
+const Data::ComplexGeoData* PropertyMeshKernel::getComplexData() const
+{
+    return (MeshObject*)_meshObject;
+}
+
 Base::BoundBox3d PropertyMeshKernel::getBoundingBox() const
 {
     return _meshObject->getBoundBox();
@@ -375,10 +386,7 @@ void PropertyMeshKernel::finishEditing()
 void PropertyMeshKernel::transformGeometry(const Base::Matrix4D &rclMat)
 {
     aboutToSetValue();
-    MeshCore::MeshKernel kernel;
-    _meshObject->swap(kernel);
-    kernel.Transform(rclMat);
-    _meshObject->swap(kernel);
+    _meshObject->transformGeometry(rclMat);
     hasSetValue();
 }
 
@@ -500,9 +508,14 @@ void PropertyMeshKernel::removeFoldsOnSurface()
 
 PyObject *PropertyMeshKernel::getPyObject(void)
 {
-    MeshPy* mesh = new MeshPy(&*_meshObject);
-    mesh->setConst(); // set immutable
-    return mesh;
+    if (!meshPyObject) {
+        meshPyObject = new MeshPy(&*_meshObject);
+        meshPyObject->setConst(); // set immutable
+        meshPyObject->parentProperty = this;
+    }
+
+    Py_INCREF(meshPyObject);
+    return meshPyObject;
 }
 
 void PropertyMeshKernel::setPyObject(PyObject *value)
