@@ -44,6 +44,8 @@
 namespace Mesh
 {
 
+class MeshPy;
+
 /** The normals property class.
  * Note: We need an own class for that to distinguish from the base vector list.
  * @author Werner Mayer
@@ -162,6 +164,7 @@ public:
 
     /** @name Getting basic geometric entities */
     //@{
+    const Data::ComplexGeoData* getComplexData() const;
     /** Returns the bounding box around the underlying mesh kernel */
     Base::BoundBox3d getBoundingBox() const;
     /** Get faces from object with given accuracy */
@@ -230,6 +233,7 @@ public:
 
 private:
     Base::Reference<MeshObject> _meshObject;
+    MeshPy* meshPyObject;
 };
 
 } // namespace Mesh
