@@ -33,6 +33,7 @@
 #include "MeshPointPy.h"
 #include "FacetPy.h"
 #include "MeshPy.cpp"
+#include "MeshProperties.h"
 #include "Core/Algorithm.h"
 #include "Core/Iterator.h"
 #include "Core/Elements.h"
@@ -43,6 +44,15 @@
 using namespace Mesh;
 
 
+struct MeshPropertyLock {
+    MeshPropertyLock(PropertyMeshKernel* p) : prop(p)
+    { if (prop) prop->startEditing(); }
+    ~MeshPropertyLock()
+    { if (prop) prop->finishEditing(); }
+private:
+    PropertyMeshKernel* prop;
+};
+
 int MeshPy::PyInit(PyObject* args, PyObject*)
 {
     PyObject *pcObj=0;
@@ -50,6 +60,7 @@ int MeshPy::PyInit(PyObject* args, PyObject*)
         return -1;                             // NULL triggers exception
 
     try {
+        this->parentProperty = 0;
         // if no mesh is given
         if (!pcObj) return 0;
         if (PyObject_TypeCheck(pcObj, &(MeshPy::Type))) {
@@ -109,6 +120,15 @@ PyObject *MeshPy::PyMake(struct _typeobject *, PyObject *, PyObject *)  // Pytho
     return new MeshPy(new MeshObject);
 }
 
+PyObject* MeshPy::copy(PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+
+    const MeshCore::MeshKernel& kernel = getMeshObjectPtr()->getKernel();
+    return new MeshPy(new MeshObject(kernel));
+}
+
 PyObject*  MeshPy::read(PyObject *args)
 {
     const char* Name;
@@ -223,13 +243,13 @@ PyObject*  MeshPy::crossSections(PyObject *args)
     union PyType_Object pyType = {&(Base::VectorPy::Type)};
     Py::Type vType(pyType.o);
 
-    std::vector<MeshObject::Plane> csPlanes;
+    std::vector<MeshObject::TPlane> csPlanes;
     for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
         Py::Tuple pair(*it);
         Py::Object p1 = pair.getItem(0);
         Py::Object p2 = pair.getItem(1);
         if (p1.isType(vType) && p2.isType(vType)) {
-            MeshObject::Plane plane;
+            MeshObject::TPlane plane;
             Base::Vector3d b = static_cast<Base::VectorPy*>(p1.ptr())->value();
             Base::Vector3d n = static_cast<Base::VectorPy*>(p2.ptr())->value();
             plane.first.Set((float)b.x,(float)b.y,(float)b.z);
@@ -246,21 +266,21 @@ PyObject*  MeshPy::crossSections(PyObject *args)
             float ny = (float)Py::Float(n.getItem(1));
             float nz = (float)Py::Float(n.getItem(2));
 
-            MeshObject::Plane plane;
+            MeshObject::TPlane plane;
             plane.first .Set(bx,by,bz);
             plane.second.Set(nx,ny,nz);
             csPlanes.push_back(plane);
         }
     }
 
-    std::vector<MeshObject::Polylines> sections;
+    std::vector<MeshObject::TPolylines> sections;
     getMeshObjectPtr()->crossSections(csPlanes, sections, min_eps, (poly == Py_True));
 
     // convert to Python objects
     Py::List crossSections;
-    for (std::vector<MeshObject::Polylines>::iterator it = sections.begin(); it != sections.end(); ++it) {
+    for (std::vector<MeshObject::TPolylines>::iterator it = sections.begin(); it != sections.end(); ++it) {
         Py::List section;
-        for (MeshObject::Polylines::const_iterator jt = it->begin(); jt != it->end(); ++jt) {
+        for (MeshObject::TPolylines::const_iterator jt = it->begin(); jt != it->end(); ++jt) {
             Py::List polyline;
             for (std::vector<Base::Vector3f>::const_iterator kt = jt->begin(); kt != jt->end(); ++kt) {
                 polyline.append(Py::Object(new Base::VectorPy(*kt)));
@@ -725,6 +745,7 @@ PyObject*  MeshPy::flipNormals(PyObject *args)
         return NULL;
 
     PY_TRY {
+        MeshPropertyLock lock(this->parentProperty);
         getMeshObjectPtr()->flipNormals();
     } PY_CATCH;
 
@@ -753,6 +774,7 @@ PyObject*  MeshPy::harmonizeNormals(PyObject *args)
         return NULL;
 
     PY_TRY {
+        MeshPropertyLock lock(this->parentProperty);
         getMeshObjectPtr()->harmonizeNormals();
     } PY_CATCH;
 
@@ -786,17 +808,30 @@ PyObject*  MeshPy::removeComponents(PyObject *args)
 PyObject*  MeshPy::fillupHoles(PyObject *args)
 {
     unsigned long len;
-    float area;
-    int level=0;
-    if (!PyArg_ParseTuple(args, "kf|i", &len, &area, &level))
-        return NULL;                         
+    int level = 0;
+    float max_area = 0.0f;
+    if (!PyArg_ParseTuple(args, "k|if", &len,&level,&max_area))
+        return NULL;
+    try {
+        std::auto_ptr<MeshCore::AbstractPolygonTriangulator> tria;
+        if (max_area > 0.0f) {
+            tria = std::auto_ptr<MeshCore::AbstractPolygonTriangulator>
+                (new MeshCore::ConstraintDelaunayTriangulator(max_area));
+        }
+        else {
+            tria = std::auto_ptr<MeshCore::AbstractPolygonTriangulator>
+                (new MeshCore::FlatTriangulator());
+        }
 
-    PY_TRY {
-        MeshCore::ConstraintDelaunayTriangulator cTria(area);
-        getMeshObjectPtr()->fillupHoles(len, level, cTria);
-    } PY_CATCH;
+        MeshPropertyLock lock(this->parentProperty);
+        getMeshObjectPtr()->fillupHoles(len, level, *tria);
+    }
+    catch (const Base::Exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
 
-    Py_Return; 
+    Py_Return;
 }
 
 PyObject*  MeshPy::fixIndices(PyObject *args)
@@ -874,11 +909,12 @@ PyObject*  MeshPy::refine(PyObject *args)
 
 PyObject*  MeshPy::optimizeTopology(PyObject *args)
 {
-    float fMaxAngle;
-    if (!PyArg_ParseTuple(args, "f; specify the maximum allowed angle between the normals of two adjacent facets", &fMaxAngle))
+    float fMaxAngle=-1.0f;
+    if (!PyArg_ParseTuple(args, "|f; specify the maximum allowed angle between the normals of two adjacent facets", &fMaxAngle))
         return NULL;
 
     PY_TRY {
+        MeshPropertyLock lock(this->parentProperty);
         getMeshObjectPtr()->optimizeTopology(fMaxAngle);
     } PY_CATCH;
 
@@ -891,6 +927,7 @@ PyObject*  MeshPy::optimizeEdges(PyObject *args)
         return NULL;
 
     PY_TRY {
+        MeshPropertyLock lock(this->parentProperty);
         getMeshObjectPtr()->optimizeEdges();
     } PY_CATCH;
 
@@ -1181,6 +1218,7 @@ PyObject*  MeshPy::smooth(PyObject *args)
         return NULL;
 
     PY_TRY {
+        MeshPropertyLock lock(this->parentProperty);
         getMeshObjectPtr()->smooth(iter, d_max);
     } PY_CATCH;
 
@@ -1250,6 +1288,29 @@ PyObject* MeshPy::nearestFacetOnRay(PyObject *args)
     }
 }
 
+PyObject*  MeshPy::getPlanes(PyObject *args)
+{
+    float dev;
+    if (!PyArg_ParseTuple(args, "f",&dev))
+        return NULL;
+
+    Mesh::MeshObject* mesh = getMeshObjectPtr();
+    std::vector<Mesh::Segment> segments = mesh->getSegmentsFromType
+        (Mesh::MeshObject::PLANE, Mesh::Segment(mesh,false), dev);
+
+    Py::List s;
+    for (std::vector<Mesh::Segment>::iterator it = segments.begin(); it != segments.end(); ++it) {
+        const std::vector<unsigned long>& segm = it->getIndices();
+        Py::List ary;
+        for (std::vector<unsigned long>::const_iterator jt = segm.begin(); jt != segm.end(); ++jt) {
+            ary.append(Py::Int((int)*jt));
+        }
+        s.append(ary);
+    }
+
+    return Py::new_reference_to(s);
+}
+
 Py::Int MeshPy::getCountPoints(void) const
 {
     return Py::Int((long)getMeshObjectPtr()->countPoints());
