@@ -40,8 +40,9 @@
 # include <Gui/InventorAll.h>
 #endif
 
-#include "../App/MeshFeature.h"
-#include "../App/FeatureMeshCurvature.h"
+#include <Mod/Mesh/App/Core/Smoothing.h>
+#include <Mod/Mesh/App/MeshFeature.h>
+#include <Mod/Mesh/App/FeatureMeshCurvature.h>
 
 #include <Base/Console.h>
 #include <Base/Exception.h>
@@ -52,18 +53,22 @@
 #include <Gui/BitmapFactory.h>
 #include <Gui/MainWindow.h>
 #include <Gui/Command.h>
+#include <Gui/Control.h>
 #include <Gui/Document.h>
 #include <Gui/FileDialog.h>
 #include <Gui/Selection.h>
 #include <Gui/ViewProvider.h>
 #include <Gui/View3DInventor.h>
 #include <Gui/View3DInventorViewer.h>
+#include <Gui/WaitCursor.h>
 
 #include "DlgEvaluateMeshImp.h"
 #include "DlgRegularSolidImp.h"
 #include "RemoveComponents.h"
+#include "DlgSmoothing.h"
 #include "ViewProviderMeshFaceSet.h"
 #include "ViewProviderCurvature.h"
+#include "MeshEditor.h"
 
 using namespace Mesh;
 
@@ -434,7 +439,7 @@ void CmdMeshImport::activated(int iMsg)
 {
     // use current path as default
     QStringList filter;
-    filter << QObject::tr("All Mesh Files (*.stl *.ast *.bms *.obj)");
+    filter << QObject::tr("All Mesh Files (*.stl *.ast *.bms *.obj *.ply)");
     filter << QObject::tr("Binary STL (*.stl)");
     filter << QObject::tr("ASCII STL (*.ast)");
     filter << QObject::tr("Binary Mesh (*.bms)");
@@ -658,8 +663,10 @@ void CmdMeshVertexCurvatureInfo::activated(int iMsg)
     if (view) {
         Gui::View3DInventorViewer* viewer = view->getViewer();
         viewer->setEditing(true);
+        viewer->setRedirectToSceneGraph(true);
         viewer->setEditingCursor(QCursor(Gui::BitmapFactory().pixmap("mesh_pipette"),4,29));
-        viewer->addEventCallback(SoMouseButtonEvent::getClassTypeId(), MeshGui::ViewProviderMeshCurvature::curvatureInfoCallback);
+        viewer->addEventCallback(SoEvent::getClassTypeId(),
+            MeshGui::ViewProviderMeshCurvature::curvatureInfoCallback);
      }
 }
 
@@ -704,7 +711,7 @@ void CmdMeshPolySegm::activated(int iMsg)
             if (view->getTypeId().isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
                 Gui::View3DInventorViewer* viewer = ((Gui::View3DInventor*)view)->getViewer();
                 viewer->setEditing(true);
-                viewer->startPicking(Gui::View3DInventorViewer::Clip);
+                viewer->startSelection(Gui::View3DInventorViewer::Clip);
                 viewer->addEventCallback(SoMouseButtonEvent::getClassTypeId(), MeshGui::ViewProviderMeshFaceSet::segmMeshCallback);
             }
             else {
@@ -713,7 +720,7 @@ void CmdMeshPolySegm::activated(int iMsg)
         }
 
         Gui::ViewProvider* pVP = getActiveGuiDocument()->getViewProvider(*it);
-        pVP->setEdit();
+        pVP->startEditing();
     }
 }
 
@@ -757,7 +764,7 @@ void CmdMeshPolySelect::activated(int iMsg)
             if (view->getTypeId().isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
                 Gui::View3DInventorViewer* viewer = ((Gui::View3DInventor*)view)->getViewer();
                 viewer->setEditing(true);
-                viewer->startPicking(Gui::View3DInventorViewer::Rectangle);
+                viewer->startSelection(Gui::View3DInventorViewer::Rectangle);
                 viewer->addEventCallback(SoMouseButtonEvent::getClassTypeId(), MeshGui::ViewProviderMeshFaceSet::selectGLCallback);
             }
             else {
@@ -766,7 +773,7 @@ void CmdMeshPolySelect::activated(int iMsg)
         }
 
         Gui::ViewProvider* pVP = getActiveGuiDocument()->getViewProvider(*it);
-        pVP->setEdit();
+        pVP->startEditing();
     }
 }
 
@@ -787,6 +794,52 @@ bool CmdMeshPolySelect::isActive(void)
 
 //--------------------------------------------------------------------------------------
 
+DEF_STD_CMD_A(CmdMeshAddFacet);
+
+CmdMeshAddFacet::CmdMeshAddFacet()
+  : Command("Mesh_AddFacet")
+{
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Add triangle");
+    sToolTipText  = QT_TR_NOOP("Add triangle manually to a mesh");
+    sWhatsThis    = "Mesh_AddFacet";
+    sStatusTip    = QT_TR_NOOP("Add triangle manually to a mesh");
+}
+
+void CmdMeshAddFacet::activated(int iMsg)
+{
+    std::vector<App::DocumentObject*> docObj = Gui::Selection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+    for (std::vector<App::DocumentObject*>::iterator it = docObj.begin(); it != docObj.end(); ++it) {
+        Gui::Document* doc = Gui::Application::Instance->getDocument((*it)->getDocument());
+        Gui::MDIView* view = doc->getActiveView();
+        if (view->getTypeId().isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
+            MeshGui::MeshFaceAddition* edit = new MeshGui::MeshFaceAddition
+                (static_cast<Gui::View3DInventor*>(view));
+            edit->startEditing(static_cast<MeshGui::ViewProviderMesh*>
+                (Gui::Application::Instance->getViewProvider(*it)));
+            break;
+        }
+    }
+}
+
+bool CmdMeshAddFacet::isActive(void)
+{
+    // Check for the selected mesh feature (all Mesh types)
+    if (getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) != 1)
+        return false;
+
+    Gui::MDIView* view = Gui::getMainWindow()->activeWindow();
+    if (view && view->isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
+        Gui::View3DInventorViewer* viewer = static_cast<Gui::View3DInventor*>(view)->getViewer();
+        return !viewer->isEditing();
+    }
+
+    return false;
+}
+
+//--------------------------------------------------------------------------------------
+
 DEF_STD_CMD_A(CmdMeshPolyCut);
 
 CmdMeshPolyCut::CmdMeshPolyCut()
@@ -811,7 +864,7 @@ void CmdMeshPolyCut::activated(int iMsg)
             if (view->getTypeId().isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
                 Gui::View3DInventorViewer* viewer = ((Gui::View3DInventor*)view)->getViewer();
                 viewer->setEditing(true);
-                viewer->startPicking(Gui::View3DInventorViewer::Clip);
+                viewer->startSelection(Gui::View3DInventorViewer::Clip);
                 viewer->addEventCallback(SoMouseButtonEvent::getClassTypeId(), MeshGui::ViewProviderMeshFaceSet::clipMeshCallback);
             }
             else {
@@ -820,7 +873,8 @@ void CmdMeshPolyCut::activated(int iMsg)
         }
 
         Gui::ViewProvider* pVP = getActiveGuiDocument()->getViewProvider(*it);
-        pVP->setEdit();
+        if (pVP->isVisible())
+            pVP->startEditing();
     }
 }
 
@@ -864,7 +918,7 @@ void CmdMeshPolySplit::activated(int iMsg)
             if (view->getTypeId().isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
                 Gui::View3DInventorViewer* viewer = ((Gui::View3DInventor*)view)->getViewer();
                 viewer->setEditing(true);
-                viewer->startPicking(Gui::View3DInventorViewer::Clip);
+                viewer->startSelection(Gui::View3DInventorViewer::Clip);
                 viewer->addEventCallback(SoMouseButtonEvent::getClassTypeId(), MeshGui::ViewProviderMeshFaceSet::partMeshCallback);
             }
             else {
@@ -873,7 +927,7 @@ void CmdMeshPolySplit::activated(int iMsg)
         }
 
         Gui::ViewProvider* pVP = getActiveGuiDocument()->getViewProvider(*it);
-        pVP->setEdit();
+        pVP->startEditing();
     }
 }
 
@@ -1042,24 +1096,28 @@ CmdMeshRemoveComponents::CmdMeshRemoveComponents()
     sAppModule    = "Mesh";
     sGroup        = QT_TR_NOOP("Mesh");
     sMenuText     = QT_TR_NOOP("Remove components...");
-    sToolTipText  = QT_TR_NOOP("Remove topologic independant components from the mesh");
+    sToolTipText  = QT_TR_NOOP("Remove topologic independent components from the mesh");
     sWhatsThis    = "Mesh_RemoveComponents";
-    sStatusTip    = QT_TR_NOOP("Remove topologic independant components from the mesh");
+    sStatusTip    = QT_TR_NOOP("Remove topologic independent components from the mesh");
 }
 
 void CmdMeshRemoveComponents::activated(int iMsg)
 {
-    static QPointer<MeshGui::RemoveComponents> dlg=0;
-    if (!dlg)
-        dlg = new MeshGui::RemoveComponents(Gui::getMainWindow());
-    dlg->setAttribute(Qt::WA_DeleteOnClose);
-    dlg->show();
+    Gui::TaskView::TaskDialog* dlg = Gui::Control().activeDialog();
+    if (!dlg) {
+        dlg = new MeshGui::TaskRemoveComponents();
+        dlg->setButtonPosition(Gui::TaskView::TaskDialog::South);
+    }
+    Gui::Control().showDialog(dlg);
 }
 
 bool CmdMeshRemoveComponents::isActive(void)
 {
     // Check for the selected mesh feature (all Mesh types)
-    return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
+    App::Document* doc = getDocument();
+    return (doc && doc->countObjectsOfType
+            (Mesh::Feature::getClassTypeId()) > 0
+            && !Gui::Control().activeDialog());
 }
 
 //--------------------------------------------------------------------------------------
@@ -1111,35 +1169,34 @@ DEF_STD_CMD_A(CmdMeshEvaluateSolid);
 CmdMeshEvaluateSolid::CmdMeshEvaluateSolid()
   :Command("Mesh_EvaluateSolid")
 {
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Check solid mesh");
-  sToolTipText  = QT_TR_NOOP("Checks whether the mesh is a solid");
-  sWhatsThis    = "Mesh_EvaluateSolid";
-  sStatusTip    = QT_TR_NOOP("Checks whether the mesh is a solid");
-//  sPixmap       = "curv_info";
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Check solid mesh");
+    sToolTipText  = QT_TR_NOOP("Checks whether the mesh is a solid");
+    sWhatsThis    = "Mesh_EvaluateSolid";
+    sStatusTip    = QT_TR_NOOP("Checks whether the mesh is a solid");
 }
 
 void CmdMeshEvaluateSolid::activated(int iMsg)
 {
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    Mesh::Feature* mesh = (Mesh::Feature*)(*it);
-    QString msg = QObject::tr("The mesh '%1' is ")
-        .arg(QString::fromAscii(mesh->getNameInDocument()));
-    if ( mesh->Mesh.getValue().getKernel().HasOpenEdges() )
-      msg += QObject::tr("not a solid.");
-    else
-      msg += QObject::tr("a solid.");
-    QMessageBox::information(Gui::getMainWindow(), QObject::tr("Solid Mesh"), msg);
-  }
+    std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+    for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
+        Mesh::Feature* mesh = (Mesh::Feature*)(*it);
+        QString msg;
+        if (mesh->Mesh.getValue().getKernel().HasOpenEdges())
+            msg = QObject::tr("The mesh '%1' is not a solid.")
+                .arg(QString::fromAscii(mesh->Label.getValue()));
+        else
+            msg = QObject::tr("The mesh '%1' is a solid.")
+                .arg(QString::fromAscii(mesh->Label.getValue()));
+        QMessageBox::information(Gui::getMainWindow(), QObject::tr("Solid Mesh"), msg);
+    }
 }
 
 bool CmdMeshEvaluateSolid::isActive(void)
 {
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) == 1;
+    // Check for the selected mesh feature (all Mesh types)
+    return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) == 1;
 }
 
 //--------------------------------------------------------------------------------------
@@ -1151,7 +1208,7 @@ CmdMeshSmoothing::CmdMeshSmoothing()
 {
     sAppModule    = "Mesh";
     sGroup        = QT_TR_NOOP("Mesh");
-    sMenuText     = QT_TR_NOOP("Smooth");
+    sMenuText     = QT_TR_NOOP("Smooth...");
     sToolTipText  = QT_TR_NOOP("Smooth the selected meshes");
     sWhatsThis    = "Mesh_Smoothing";
     sStatusTip    = QT_TR_NOOP("Smooth the selected meshes");
@@ -1159,14 +1216,35 @@ CmdMeshSmoothing::CmdMeshSmoothing()
 
 void CmdMeshSmoothing::activated(int iMsg)
 {
-    std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-    openCommand("Smooth");
-    for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
-        App::Document* doc = (*it)->getDocument();
-        doCommand(Doc,"App.getDocument(\"%s\").getObject(\"%s\").smooth()"
-                     ,doc->getName(), (*it)->getNameInDocument());
+    MeshGui::DlgSmoothing dlg(Gui::getMainWindow());
+    if (dlg.exec() == QDialog::Accepted) {
+        Gui::WaitCursor wc;
+        openCommand("Mesh Smoothing");
+        std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+        for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
+            Mesh::Feature* mesh = (Mesh::Feature*)*it;
+            Mesh::MeshObject* mm = mesh->Mesh.startEditing();
+            switch (dlg.method()) {
+                case MeshGui::DlgSmoothing::Taubin:
+                    {
+                        MeshCore::TaubinSmoothing s(mm->getKernel());
+                        s.SetLambda(dlg.lambdaStep());
+                        s.SetMicro(dlg.microStep());
+                        s.Smooth(dlg.iterations());
+                    }   break;
+                case MeshGui::DlgSmoothing::Laplace:
+                    {
+                        MeshCore::LaplaceSmoothing s(mm->getKernel());
+                        s.SetLambda(dlg.lambdaStep());
+                        s.Smooth(dlg.iterations());
+                    }   break;
+                default:
+                    break;
+            }
+            mesh->Mesh.finishEditing();
+        }
+        commitCommand();
     }
-    commitCommand();
 }
 
 bool CmdMeshSmoothing::isActive(void)
@@ -1182,36 +1260,30 @@ DEF_STD_CMD_A(CmdMeshHarmonizeNormals);
 CmdMeshHarmonizeNormals::CmdMeshHarmonizeNormals()
   :Command("Mesh_HarmonizeNormals")
 {
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Harmonize normals");
-  sToolTipText  = QT_TR_NOOP("Harmonizes the normals of the mesh");
-  sWhatsThis    = "Mesh_HarmonizeNormals";
-  sStatusTip    = QT_TR_NOOP("Harmonizes the normals of the mesh");
-//  sPixmap       = "curv_info";
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Harmonize normals");
+    sToolTipText  = QT_TR_NOOP("Harmonizes the normals of the mesh");
+    sWhatsThis    = "Mesh_HarmonizeNormals";
+    sStatusTip    = QT_TR_NOOP("Harmonizes the normals of the mesh");
 }
 
 void CmdMeshHarmonizeNormals::activated(int iMsg)
 {
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_Harmonize";
-    fName = getUniqueObjectName(fName.c_str());
-    openCommand("Mesh Harmonize Normals");
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::HarmonizeNormals\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
+    std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+    openCommand("Harmonize mesh normals");
+    for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
+        doCommand(Doc,"App.activeDocument().getObject(\"%s\").Mesh.harmonizeNormals()"
+                     ,(*it)->getNameInDocument());
+    }
     commitCommand();
     updateActive();
-    doCommand(Gui,"Gui.hide(\"%s\")",(*it)->getNameInDocument());
-  }
 }
 
 bool CmdMeshHarmonizeNormals::isActive(void)
 {
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
+    // Check for the selected mesh feature (all Mesh types)
+    return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
 }
 
 //--------------------------------------------------------------------------------------
@@ -1221,192 +1293,30 @@ DEF_STD_CMD_A(CmdMeshFlipNormals);
 CmdMeshFlipNormals::CmdMeshFlipNormals()
   :Command("Mesh_FlipNormals")
 {
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Flip normals");
-  sToolTipText  = QT_TR_NOOP("Flips the normals of the mesh");
-  sWhatsThis    = "Mesh_FlipNormals";
-  sStatusTip    = QT_TR_NOOP("Flips the normals of the mesh");
-//  sPixmap       = "curv_info";
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Flip normals");
+    sToolTipText  = QT_TR_NOOP("Flips the normals of the mesh");
+    sWhatsThis    = "Mesh_FlipNormals";
+    sStatusTip    = QT_TR_NOOP("Flips the normals of the mesh");
 }
 
 void CmdMeshFlipNormals::activated(int iMsg)
 {
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_Flip";
-    fName = getUniqueObjectName(fName.c_str());
-    openCommand("Mesh Flip Normals");
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::FlipNormals\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
+    std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+    openCommand("Flip mesh normals");
+    for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
+        doCommand(Doc,"App.activeDocument().getObject(\"%s\").Mesh.flipNormals()"
+                     ,(*it)->getNameInDocument());
+    }
     commitCommand();
     updateActive();
-    doCommand(Gui,"Gui.hide(\"%s\")",(*it)->getNameInDocument());
-  }
 }
 
 bool CmdMeshFlipNormals::isActive(void)
 {
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
-}
-
-//--------------------------------------------------------------------------------------
-
-DEF_STD_CMD_A(CmdMeshFixDegenerations);
-
-CmdMeshFixDegenerations::CmdMeshFixDegenerations()
-  :Command("Mesh_FixDegenerations")
-{
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Remove degenerated faces");
-  sToolTipText  = QT_TR_NOOP("Remove degenerated faces from the mesh");
-  sWhatsThis    = "Mesh_FixDegenerations";
-  sStatusTip    = QT_TR_NOOP("Remove degenerated faces from the mesh");
-//  sPixmap       = "curv_info";
-}
-
-void CmdMeshFixDegenerations::activated(int iMsg)
-{
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_Fixed";
-    fName = getUniqueObjectName(fName.c_str());
-    openCommand("Mesh Harmonize Normals");
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::FixDegenerations\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
-    commitCommand();
-    updateActive();
-    doCommand(Gui,"Gui.hide(\"%s\")",(*it)->getNameInDocument());
-  }
-}
-
-bool CmdMeshFixDegenerations::isActive(void)
-{
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
-}
-
-//--------------------------------------------------------------------------------------
-
-DEF_STD_CMD_A(CmdMeshFixDuplicateFaces);
-
-CmdMeshFixDuplicateFaces::CmdMeshFixDuplicateFaces()
-  :Command("Mesh_FixDuplicateFaces")
-{
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Remove duplicated faces");
-  sToolTipText  = QT_TR_NOOP("Remove duplicated faces from the mesh");
-  sWhatsThis    = "Mesh_FixDuplicateFaces";
-  sStatusTip    = QT_TR_NOOP("Remove duplicated faces from the mesh");
-//  sPixmap       = "curv_info";
-}
-
-void CmdMeshFixDuplicateFaces::activated(int iMsg)
-{
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_Fixed";
-    fName = getUniqueObjectName(fName.c_str());
-    openCommand("Mesh Harmonize Normals");
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::FixDuplicatedFaces\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
-    commitCommand();
-    updateActive();
-    doCommand(Gui,"Gui.hide(\"%s\")",(*it)->getNameInDocument());
-  }
-}
-
-bool CmdMeshFixDuplicateFaces::isActive(void)
-{
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
-}
-
-//--------------------------------------------------------------------------------------
-
-DEF_STD_CMD_A(CmdMeshFixDuplicatePoints);
-
-CmdMeshFixDuplicatePoints::CmdMeshFixDuplicatePoints()
-  :Command("Mesh_FixDuplicatePoints")
-{
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Remove duplicated points");
-  sToolTipText  = QT_TR_NOOP("Remove duplicated points from the mesh");
-  sWhatsThis    = "Mesh_FixDuplicatePoints";
-  sStatusTip    = QT_TR_NOOP("Remove duplicated points from the mesh");
-//  sPixmap       = "curv_info";
-}
-
-void CmdMeshFixDuplicatePoints::activated(int iMsg)
-{
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_Fixed";
-    fName = getUniqueObjectName(fName.c_str());
-    openCommand("Remove duplicated points");
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::FixDuplicatedPoints\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
-    commitCommand();
-    updateActive();
-    doCommand(Gui,"Gui.hide(\"%s\")",(*it)->getNameInDocument());
-  }
-}
-
-bool CmdMeshFixDuplicatePoints::isActive(void)
-{
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
-}
-
-//--------------------------------------------------------------------------------------
-
-DEF_STD_CMD_A(CmdMeshFixIndices);
-
-CmdMeshFixIndices::CmdMeshFixIndices()
-  :Command("Mesh_FixIndices")
-{
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Fix indices");
-  sToolTipText  = QT_TR_NOOP("Fixes invalid indices in the mesh structure");
-  sWhatsThis    = "Mesh_FixIndices";
-  sStatusTip    = QT_TR_NOOP("Fixes invalid indices in the mesh structure");
-//  sPixmap       = "curv_info";
-}
-
-void CmdMeshFixIndices::activated(int iMsg)
-{
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_Fixed";
-    fName = getUniqueObjectName(fName.c_str());
-    openCommand("Mesh Harmonize Normals");
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::FixIndices\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
-    commitCommand();
-    updateActive();
-    doCommand(Gui,"Gui.hide(\"%s\")",(*it)->getNameInDocument());
-  }
-}
-
-bool CmdMeshFixIndices::isActive(void)
-{
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
+    // Check for the selected mesh feature (all Mesh types)
+    return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
 }
 
 //--------------------------------------------------------------------------------------
@@ -1416,38 +1326,36 @@ DEF_STD_CMD_A(CmdMeshBoundingBox);
 CmdMeshBoundingBox::CmdMeshBoundingBox()
   :Command("Mesh_BoundingBox")
 {
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Boundings info...");
-  sToolTipText  = QT_TR_NOOP("Shows the boundings of the selected mesh");
-  sWhatsThis    = "Mesh_BoundingBox";
-  sStatusTip    = QT_TR_NOOP("Shows the boundings of the selected mesh");
-//  sPixmap       = "curv_info";
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Boundings info...");
+    sToolTipText  = QT_TR_NOOP("Shows the boundings of the selected mesh");
+    sWhatsThis    = "Mesh_BoundingBox";
+    sStatusTip    = QT_TR_NOOP("Shows the boundings of the selected mesh");
 }
 
 void CmdMeshBoundingBox::activated(int iMsg)
 {
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    const MeshCore::MeshKernel& rMesh = ((Mesh::Feature*)(*it))->Mesh.getValue().getKernel();
-    const Base::BoundBox3f& box = rMesh.GetBoundBox();
+    std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+    for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
+        const MeshCore::MeshKernel& rMesh = ((Mesh::Feature*)(*it))->Mesh.getValue().getKernel();
+        const Base::BoundBox3f& box = rMesh.GetBoundBox();
 
-    Base::Console().Message("Boundings: Min=<%f,%f,%f>, Max=<%f,%f,%f>\n",
-                            box.MinX,box.MinY,box.MinZ,box.MaxX,box.MaxY,box.MaxZ);
+        Base::Console().Message("Boundings: Min=<%f,%f,%f>, Max=<%f,%f,%f>\n",
+                                box.MinX,box.MinY,box.MinZ,box.MaxX,box.MaxY,box.MaxZ);
 
-    QString bound = QObject::tr("Min=<%1,%2,%3>\n\nMax=<%4,%5,%6>")
-        .arg(box.MinX).arg(box.MinY).arg(box.MinZ)
-        .arg(box.MaxX).arg(box.MaxY).arg(box.MaxZ);
-    QMessageBox::information(Gui::getMainWindow(), QObject::tr("Boundings"), bound);
-    break;
-  }
+        QString bound = QObject::tr("Min=<%1,%2,%3>\n\nMax=<%4,%5,%6>")
+            .arg(box.MinX).arg(box.MinY).arg(box.MinZ)
+            .arg(box.MaxX).arg(box.MaxY).arg(box.MaxZ);
+        QMessageBox::information(Gui::getMainWindow(), QObject::tr("Boundings"), bound);
+        break;
+    }
 }
 
 bool CmdMeshBoundingBox::isActive(void)
 {
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) == 1;
+    // Check for the selected mesh feature (all Mesh types)
+    return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) == 1;
 }
 
 //--------------------------------------------------------------------------------------
@@ -1457,24 +1365,24 @@ DEF_STD_CMD_A(CmdMeshBuildRegularSolid);
 CmdMeshBuildRegularSolid::CmdMeshBuildRegularSolid()
   :Command("Mesh_BuildRegularSolid")
 {
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Regular solid...");
-  sToolTipText  = QT_TR_NOOP("Builds a regular solid");
-  sWhatsThis    = "Mesh_BuildRegularSolid";
-  sStatusTip    = QT_TR_NOOP("Builds a regular solid");
-  sPixmap       = "solid_mesh";
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Regular solid...");
+    sToolTipText  = QT_TR_NOOP("Builds a regular solid");
+    sWhatsThis    = "Mesh_BuildRegularSolid";
+    sStatusTip    = QT_TR_NOOP("Builds a regular solid");
+    sPixmap       = "solid_mesh";
 }
 
 void CmdMeshBuildRegularSolid::activated(int iMsg)
 {
-  MeshGui::SingleDlgRegularSolidImp::instance()->show();
+    MeshGui::SingleDlgRegularSolidImp::instance()->show();
 }
 
 bool CmdMeshBuildRegularSolid::isActive(void)
 {
-  // Check for the selected mesh feature (all Mesh types)
-  return (!MeshGui::SingleDlgRegularSolidImp::hasInstance())&&hasActiveDocument();
+    // Check for the selected mesh feature (all Mesh types)
+    return (!MeshGui::SingleDlgRegularSolidImp::hasInstance())&&hasActiveDocument();
 }
 
 //--------------------------------------------------------------------------------------
@@ -1484,41 +1392,34 @@ DEF_STD_CMD_A(CmdMeshFillupHoles);
 CmdMeshFillupHoles::CmdMeshFillupHoles()
   :Command("Mesh_FillupHoles")
 {
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Fill holes...");
-  sToolTipText  = QT_TR_NOOP("Fill holes of the mesh");
-  sWhatsThis    = "Mesh_FillupHoles";
-  sStatusTip    = QT_TR_NOOP("Fill holes of the mesh");
-//  sPixmap       = "curv_info";
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Fill holes...");
+    sToolTipText  = QT_TR_NOOP("Fill holes of the mesh");
+    sWhatsThis    = "Mesh_FillupHoles";
+    sStatusTip    = QT_TR_NOOP("Fill holes of the mesh");
 }
 
 void CmdMeshFillupHoles::activated(int iMsg)
 {
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  bool ok;
-  int FillupHolesOfLength = QInputDialog::getInteger( Gui::getMainWindow(), QObject::tr("Fill holes"), QObject::tr("Fill holes with maximum number of edges:"), 
-                                                      3, 3, 10000, 1, &ok  );
-  if (!ok) return;
-
-  openCommand("Fill up holes");
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_fill";
-    fName = getUniqueObjectName(fName.c_str());
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::FillHoles\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
-    doCommand(Doc,"App.activeDocument().%s.FillupHolesOfLength = %d",fName.c_str(), FillupHolesOfLength);
-  }
-  commitCommand();
-  updateActive();
+    std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+    bool ok;
+    int FillupHolesOfLength = QInputDialog::getInteger(Gui::getMainWindow(), QObject::tr("Fill holes"),
+                                QObject::tr("Fill holes with maximum number of edges:"), 3, 3, 10000, 1, &ok);
+    if (!ok) return;
+    openCommand("Fill up holes");
+    for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
+        doCommand(Doc,"App.activeDocument().getObject(\"%s\").Mesh.fillupHoles(%d)"
+                     ,(*it)->getNameInDocument(), FillupHolesOfLength);
+    }
+    commitCommand();
+    updateActive();
 }
 
 bool CmdMeshFillupHoles::isActive(void)
 {
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
+    // Check for the selected mesh feature (all Mesh types)
+    return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
 }
 
 //--------------------------------------------------------------------------------------
@@ -1580,6 +1481,7 @@ void CreateMeshCommands(void)
   rcCmdMgr.addCommand(new CmdMeshDemolding());
   rcCmdMgr.addCommand(new CmdMeshPolySegm());
   rcCmdMgr.addCommand(new CmdMeshPolySelect());
+  rcCmdMgr.addCommand(new CmdMeshAddFacet());
   rcCmdMgr.addCommand(new CmdMeshPolyCut());
   rcCmdMgr.addCommand(new CmdMeshPolySplit());
   rcCmdMgr.addCommand(new CmdMeshToolMesh());
@@ -1590,10 +1492,6 @@ void CreateMeshCommands(void)
   rcCmdMgr.addCommand(new CmdMeshHarmonizeNormals());
   rcCmdMgr.addCommand(new CmdMeshFlipNormals());
   rcCmdMgr.addCommand(new CmdMeshSmoothing());
-  rcCmdMgr.addCommand(new CmdMeshFixDegenerations());
-  rcCmdMgr.addCommand(new CmdMeshFixDuplicateFaces());
-  rcCmdMgr.addCommand(new CmdMeshFixDuplicatePoints());
-  rcCmdMgr.addCommand(new CmdMeshFixIndices());
   rcCmdMgr.addCommand(new CmdMeshBoundingBox());
   rcCmdMgr.addCommand(new CmdMeshBuildRegularSolid());
   rcCmdMgr.addCommand(new CmdMeshFillupHoles());
