@@ -71,6 +71,15 @@ void CleanupHandler::cleanup()
 
 /* TRANSLATOR MeshGui::DlgEvaluateMeshImp */
 
+void DlgEvaluateMeshImp::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::LanguageChange) {
+        this->retranslateUi(this);
+        meshNameButton->setItemText(0, tr("No selection"));
+    }
+    QDialog::changeEvent(e);
+}
+
 void DlgEvaluateMeshImp::slotCreatedObject(const App::DocumentObject& Obj)
 {
     // add new mesh object to the list
@@ -211,7 +220,7 @@ void DlgEvaluateMeshImp::setMesh(Mesh::Feature* m)
     }
 }
 
-void DlgEvaluateMeshImp::addViewProvider(const char* name)
+void DlgEvaluateMeshImp::addViewProvider(const char* name, const std::vector<unsigned long>& indices)
 {
     removeViewProvider(name);
 
@@ -220,7 +229,7 @@ void DlgEvaluateMeshImp::addViewProvider(const char* name)
         assert(vp->getTypeId().isDerivedFrom(Gui::ViewProvider::getClassTypeId()));
         vp->attach(_meshFeature);
         _view->getViewer()->addViewProvider( vp );
-        vp->showDefects();
+        vp->showDefects(indices);
         _vp[name] = vp;
     }
 }
@@ -295,6 +304,7 @@ void DlgEvaluateMeshImp::showInformation()
     analyzeDegeneratedButton->setEnabled(true);
     analyzeIndicesButton->setEnabled(true);
     analyzeSelfIntersectionButton->setEnabled(true);
+    analyzeFoldsButton->setEnabled(true);
     analyzeAllTogether->setEnabled(true);
 
     const MeshKernel& rMesh = _meshFeature->Mesh.getValue().getKernel();
@@ -315,6 +325,7 @@ void DlgEvaluateMeshImp::cleanInformation()
     checkDegenerationButton->setText( tr("No information") );
     checkIndicesButton->setText( tr("No information") );
     checkSelfIntersectionButton->setText( tr("No information") );
+    checkFoldsButton->setText( tr("No information") );
     analyzeOrientationButton->setDisabled(true);
     repairOrientationButton->setDisabled(true);
     analyzeDuplicatedFacesButton->setDisabled(true);
@@ -329,6 +340,8 @@ void DlgEvaluateMeshImp::cleanInformation()
     repairIndicesButton->setDisabled(true);
     analyzeSelfIntersectionButton->setDisabled(true);
     repairSelfIntersectionButton->setDisabled(true);
+    analyzeFoldsButton->setDisabled(true);
+    repairFoldsButton->setDisabled(true);
     analyzeAllTogether->setDisabled(true);
     repairAllTogether->setDisabled(true);
 }
@@ -372,8 +385,18 @@ void DlgEvaluateMeshImp::on_analyzeOrientationButton_clicked()
         const MeshKernel& rMesh = _meshFeature->Mesh.getValue().getKernel();
         MeshEvalOrientation eval(rMesh);
         std::vector<unsigned long> inds = eval.GetIndices();
-    
-        if (inds.empty()) {
+        if (inds.empty() && !eval.Evaluate()) {
+            checkOrientationButton->setText(tr("Flipped normals found"));
+            MeshEvalFoldOversOnSurface f_eval(rMesh);
+            if (!f_eval.Evaluate()) {
+                qApp->restoreOverrideCursor();
+                QMessageBox::warning(this, tr("Orientation"),
+                    tr("Check failed due to folds on the surface.\n"
+                    "Please run the command to repair folds first"));
+                qApp->setOverrideCursor(Qt::WaitCursor);
+            }
+        }
+        else if (inds.empty()) {
             checkOrientationButton->setText( tr("No flipped normals") );
             checkOrientationButton->setChecked(false);
             repairOrientationButton->setEnabled(false);
@@ -384,7 +407,7 @@ void DlgEvaluateMeshImp::on_analyzeOrientationButton_clicked()
             checkOrientationButton->setChecked(true);
             repairOrientationButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-            addViewProvider( "MeshGui::ViewProviderMeshOrientation" );
+            addViewProvider( "MeshGui::ViewProviderMeshOrientation", eval.GetIndices());
         }
 
         qApp->restoreOverrideCursor();
@@ -401,7 +424,7 @@ void DlgEvaluateMeshImp::on_repairOrientationButton_clicked()
         doc->openCommand("Harmonize normals");
         try {
             Gui::Application::Instance->runCommand(
-                true, "App.getDocument(\"%s\").getObject(\"%s\").harmonizeNormals()\n"
+                true, "App.getDocument(\"%s\").getObject(\"%s\").harmonizeNormals()"
                     , docName, objName);
         }
         catch (const Base::Exception& e) {
@@ -439,17 +462,26 @@ void DlgEvaluateMeshImp::on_analyzeNonmanifoldsButton_clicked()
         MeshEvalTopology eval(rMesh);
     
         if (eval.Evaluate()) {
-          checkNonmanifoldsButton->setText(tr("No non-manifolds"));
-          checkNonmanifoldsButton->setChecked(false);
-          repairNonmanifoldsButton->setEnabled(false);
-          removeViewProvider("MeshGui::ViewProviderMeshNonManifolds");
+            checkNonmanifoldsButton->setText(tr("No non-manifolds"));
+            checkNonmanifoldsButton->setChecked(false);
+            repairNonmanifoldsButton->setEnabled(false);
+            removeViewProvider("MeshGui::ViewProviderMeshNonManifolds");
         }
         else {
-          checkNonmanifoldsButton->setText(tr("%1 non-manifolds").arg(eval.CountManifolds()));
-          checkNonmanifoldsButton->setChecked(true);
-          repairNonmanifoldsButton->setEnabled(true);
-          repairAllTogether->setEnabled(true);
-          addViewProvider("MeshGui::ViewProviderMeshNonManifolds");
+            checkNonmanifoldsButton->setText(tr("%1 non-manifolds").arg(eval.CountManifolds()));
+            checkNonmanifoldsButton->setChecked(true);
+            repairNonmanifoldsButton->setEnabled(true);
+            repairAllTogether->setEnabled(true);
+            const std::vector<std::pair<unsigned long, unsigned long> >& inds = eval.GetIndices();
+            std::vector<unsigned long> indices;
+            indices.reserve(2*inds.size());
+            std::vector<std::pair<unsigned long, unsigned long> >::const_iterator it;
+            for (it = inds.begin(); it != inds.end(); ++it) {
+                indices.push_back(it->first);
+                indices.push_back(it->second);
+            }
+
+            addViewProvider("MeshGui::ViewProviderMeshNonManifolds", indices);
         }
 
         qApp->restoreOverrideCursor();
@@ -466,7 +498,7 @@ void DlgEvaluateMeshImp::on_repairNonmanifoldsButton_clicked()
         doc->openCommand("Remove non-manifolds");
         try {
             Gui::Application::Instance->runCommand(
-                true, "App.getDocument(\"%s\").getObject(\"%s\").removeNonManifolds()\n"
+                true, "App.getDocument(\"%s\").getObject(\"%s\").removeNonManifolds()"
                     , docName, objName);
         } 
         catch (const Base::Exception& e) {
@@ -514,28 +546,28 @@ void DlgEvaluateMeshImp::on_analyzeIndicesButton_clicked()
             checkIndicesButton->setChecked(true);
             repairIndicesButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-           addViewProvider("MeshGui::ViewProviderMeshIndices");
+            addViewProvider("MeshGui::ViewProviderMeshIndices", rf.GetIndices());
         }
         else if (!rp.Evaluate()) {
             checkIndicesButton->setText(tr("Invalid point indices"));
             checkIndicesButton->setChecked(true);
             repairIndicesButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-            addViewProvider("MeshGui::ViewProviderMeshIndices");
+            //addViewProvider("MeshGui::ViewProviderMeshIndices", rp.GetIndices());
         }
         else if (!cf.Evaluate()) {
             checkIndicesButton->setText(tr("Multiple point indices"));
             checkIndicesButton->setChecked(true);
             repairIndicesButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-            addViewProvider("MeshGui::ViewProviderMeshIndices");
+            addViewProvider("MeshGui::ViewProviderMeshIndices", cf.GetIndices());
         }
         else if (!nb.Evaluate()) {
             checkIndicesButton->setText(tr("Invalid neighbour indices"));
             checkIndicesButton->setChecked(true);
             repairIndicesButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-            addViewProvider("MeshGui::ViewProviderMeshIndices");
+            addViewProvider("MeshGui::ViewProviderMeshIndices", nb.GetIndices());
         }
         else {
             checkIndicesButton->setText(tr("No invalid indices"));
@@ -558,7 +590,7 @@ void DlgEvaluateMeshImp::on_repairIndicesButton_clicked()
         doc->openCommand("Fix indices");
         try {
             Gui::Application::Instance->runCommand(
-                true, "App.getDocument(\"%s\").getObject(\"%s\").fixIndices()\n"
+                true, "App.getDocument(\"%s\").getObject(\"%s\").fixIndices()"
                     , docName, objName);
         }
         catch (const Base::Exception& e) {
@@ -607,7 +639,7 @@ void DlgEvaluateMeshImp::on_analyzeDegeneratedButton_clicked()
             checkDegenerationButton->setChecked(true);
             repairDegeneratedButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-            addViewProvider("MeshGui::ViewProviderMeshDegenerations");
+            addViewProvider("MeshGui::ViewProviderMeshDegenerations", degen);
         }
 
         qApp->restoreOverrideCursor();
@@ -624,7 +656,7 @@ void DlgEvaluateMeshImp::on_repairDegeneratedButton_clicked()
         doc->openCommand("Remove degenerated faces");
         try {
             Gui::Application::Instance->runCommand(
-                true, "App.getDocument(\"%s\").getObject(\"%s\").fixDegenerations()\n"
+                true, "App.getDocument(\"%s\").getObject(\"%s\").fixDegenerations()"
                     , docName, objName);
         }
         catch (const Base::Exception& e) {
@@ -674,7 +706,7 @@ void DlgEvaluateMeshImp::on_analyzeDuplicatedFacesButton_clicked()
             repairDuplicatedFacesButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
 
-            addViewProvider("MeshGui::ViewProviderMeshDuplicatedFaces");
+            addViewProvider("MeshGui::ViewProviderMeshDuplicatedFaces", dupl);
         }
 
         qApp->restoreOverrideCursor();
@@ -691,7 +723,7 @@ void DlgEvaluateMeshImp::on_repairDuplicatedFacesButton_clicked()
         doc->openCommand("Remove duplicated faces");
         try {
             Gui::Application::Instance->runCommand(
-                true, "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedFacets()\n"
+                true, "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedFacets()"
                     , docName, objName);
         }
         catch (const Base::Exception& e) {
@@ -739,7 +771,7 @@ void DlgEvaluateMeshImp::on_analyzeDuplicatedPointsButton_clicked()
             checkDuplicatedPointsButton->setChecked(true);
             repairDuplicatedPointsButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-            addViewProvider("MeshGui::ViewProviderMeshDuplicatedPoints");
+            addViewProvider("MeshGui::ViewProviderMeshDuplicatedPoints", eval.GetIndices());
         }
 
         qApp->restoreOverrideCursor();
@@ -756,7 +788,7 @@ void DlgEvaluateMeshImp::on_repairDuplicatedPointsButton_clicked()
         doc->openCommand("Remove duplicated points");
         try {
             Gui::Application::Instance->runCommand(
-                true, "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedPoints()\n"
+                true, "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedPoints()"
                     , docName, objName);
         }
         catch (const Base::Exception& e) {
@@ -804,7 +836,7 @@ void DlgEvaluateMeshImp::on_analyzeSelfIntersectionButton_clicked()
             checkSelfIntersectionButton->setChecked(true);
             repairSelfIntersectionButton->setEnabled(true);
             repairAllTogether->setEnabled(true);
-            addViewProvider("MeshGui::ViewProviderMeshSelfIntersections");
+            addViewProvider("MeshGui::ViewProviderMeshSelfIntersections", std::vector<unsigned long>());
         }
 
         qApp->restoreOverrideCursor();
@@ -821,7 +853,7 @@ void DlgEvaluateMeshImp::on_repairSelfIntersectionButton_clicked()
         doc->openCommand("Fix self-intersections");
         try {
             Gui::Application::Instance->runCommand(
-                true, "App.getDocument(\"%s\").getObject(\"%s\").fixSelfIntersections()\n"
+                true, "App.getDocument(\"%s\").getObject(\"%s\").fixSelfIntersections()"
                     , docName, objName);
         }
         catch (const Base::Exception& e) {
@@ -837,6 +869,88 @@ void DlgEvaluateMeshImp::on_repairSelfIntersectionButton_clicked()
     }
 }
 
+void DlgEvaluateMeshImp::on_checkFoldsButton_clicked()
+{
+    std::map<std::string, ViewProviderMeshDefects*>::iterator it = _vp.find("MeshGui::ViewProviderMeshFolds");
+    if (it != _vp.end()) {
+        if (checkFoldsButton->isChecked())
+            it->second->show();
+        else
+            it->second->hide();
+    }
+}
+
+void DlgEvaluateMeshImp::on_analyzeFoldsButton_clicked()
+{
+    if (_meshFeature) {
+        analyzeFoldsButton->setEnabled(false);
+        qApp->processEvents();
+        qApp->setOverrideCursor(Qt::WaitCursor);
+
+        const MeshKernel& rMesh = _meshFeature->Mesh.getValue().getKernel();
+        MeshEvalFoldsOnSurface s_eval(rMesh);
+        MeshEvalFoldsOnBoundary b_eval(rMesh);
+        MeshEvalFoldOversOnSurface f_eval(rMesh);
+        bool ok1 = s_eval.Evaluate();
+        bool ok2 = b_eval.Evaluate();
+        bool ok3 = f_eval.Evaluate();
+    
+        if (ok1 && ok2 && ok3) {
+            checkFoldsButton->setText(tr("No folds on surface"));
+            checkFoldsButton->setChecked(false);
+            repairFoldsButton->setEnabled(false);
+            removeViewProvider("MeshGui::ViewProviderMeshFolds");
+        }
+        else {
+            std::vector<unsigned long> inds  = f_eval.GetIndices();
+            std::vector<unsigned long> inds1 = s_eval.GetIndices();
+            std::vector<unsigned long> inds2 = b_eval.GetIndices();
+            inds.insert(inds.end(), inds1.begin(), inds1.end());
+            inds.insert(inds.end(), inds2.begin(), inds2.end());
+
+            // remove duplicates
+            std::sort(inds.begin(), inds.end());
+            inds.erase(std::unique(inds.begin(), inds.end()), inds.end());
+            
+            checkFoldsButton->setText(tr("%1 folds on surface").arg(inds.size()));
+            checkFoldsButton->setChecked(true);
+            repairFoldsButton->setEnabled(true);
+            repairAllTogether->setEnabled(true);
+            addViewProvider("MeshGui::ViewProviderMeshFolds", inds);
+        }
+
+        qApp->restoreOverrideCursor();
+        analyzeFoldsButton->setEnabled(true);
+    }
+}
+
+void DlgEvaluateMeshImp::on_repairFoldsButton_clicked()
+{
+    if (_meshFeature) {
+        const char* docName = App::GetApplication().getDocumentName(_meshFeature->getDocument());
+        const char* objName = _meshFeature->getNameInDocument();
+        Gui::Document* doc = Gui::Application::Instance->getDocument(docName);
+        qApp->setOverrideCursor(Qt::WaitCursor);
+        doc->openCommand("Remove folds");
+        try {
+            Gui::Application::Instance->runCommand(
+                true, "App.getDocument(\"%s\").getObject(\"%s\").removeFoldsOnSurface()"
+                    , docName, objName);
+        }
+        catch (const Base::Exception& e) {
+            QMessageBox::warning(this, tr("Folds"), QString::fromLatin1(e.what()));
+        }
+
+        doc->commitCommand();
+        doc->getDocument()->recompute();
+    
+        qApp->restoreOverrideCursor();
+        repairFoldsButton->setEnabled(false);
+        checkFoldsButton->setChecked(false);
+        removeViewProvider("MeshGui::ViewProviderMeshFolds");
+    }
+}
+
 void DlgEvaluateMeshImp::on_analyzeAllTogether_clicked()
 {
     on_analyzeOrientationButton_clicked();
@@ -846,6 +960,7 @@ void DlgEvaluateMeshImp::on_analyzeAllTogether_clicked()
     on_analyzeDegeneratedButton_clicked();
     on_analyzeIndicesButton_clicked();
     on_analyzeSelfIntersectionButton_clicked();
+    on_analyzeFoldsButton_clicked();
 }
 
 void DlgEvaluateMeshImp::on_repairAllTogether_clicked()
@@ -856,45 +971,57 @@ void DlgEvaluateMeshImp::on_repairAllTogether_clicked()
         const char* objName = _meshFeature->getNameInDocument();
         Gui::Document* doc = Gui::Application::Instance->getDocument(docName);
         doc->openCommand("Repair mesh");
-        //bool state = Base::Sequencer().setLocked(true);
-
-        // containers with indices of invalid elements
-        std::vector<unsigned long> orientation;
-        std::vector<unsigned long> manifolds;
-        std::vector<unsigned long> degenerated;
-        std::vector<unsigned long> duplicatedf;
-        std::vector<unsigned long> duplicatedp;
-        std::vector<std::pair<unsigned long, unsigned long> > intersection;
 
         bool run = false;
+        bool self = true;
         int max_iter=10;
         const MeshKernel& rMesh = _meshFeature->Mesh.getValue().getKernel();
         try {
             do {
                 run = false;
                 {
-                    MeshEvalOrientation eval(rMesh);
-                    std::vector<unsigned long> inds = eval.GetIndices();
-                    if (!inds.empty() && inds != orientation) {
+                    MeshEvalSelfIntersection eval(rMesh);
+                    if (self && !eval.Evaluate()) {
                         Gui::Application::Instance->runCommand(true,
-                            "App.getDocument(\"%s\").getObject(\"%s\").harmonizeNormals()\n",
+                            "App.getDocument(\"%s\").getObject(\"%s\").fixSelfIntersections()",
+                            docName, objName);
+                        run = true;
+                    }
+                    else {
+                        self = false; // once no self-intersections found do not repeat it later on
+                    }
+                    qApp->processEvents();
+                }
+                {
+                    MeshEvalFoldsOnSurface s_eval(rMesh);
+                    MeshEvalFoldsOnBoundary b_eval(rMesh);
+                    MeshEvalFoldOversOnSurface f_eval(rMesh);
+                    if (!s_eval.Evaluate() || !b_eval.Evaluate() || !f_eval.Evaluate()) {
+                        Gui::Application::Instance->runCommand(true,
+                            "App.getDocument(\"%s\").getObject(\"%s\").removeFoldsOnSurface()",
+                            docName, objName);
+                        run = true;
+                    }
+                    qApp->processEvents();
+                }
+                {
+                    MeshEvalOrientation eval(rMesh);
+                    if (!eval.Evaluate()) {
+                        Gui::Application::Instance->runCommand(true,
+                            "App.getDocument(\"%s\").getObject(\"%s\").harmonizeNormals()",
                             docName, objName);
                         run = true;
                     }
-                    orientation = inds;
                     qApp->processEvents();
                 }
                 {
                     MeshEvalTopology eval(rMesh);
-                    std::vector<unsigned long> inds;
-                    eval.GetFacetManifolds(inds);
-                    if (!inds.empty() && inds != manifolds) {
+                    if (!eval.Evaluate()) {
                         Gui::Application::Instance->runCommand(true,
-                            "App.getDocument(\"%s\").getObject(\"%s\").removeNonManifolds()\n",
+                            "App.getDocument(\"%s\").getObject(\"%s\").removeNonManifolds()",
                             docName, objName);
                         run = true;
                     }
-                    manifolds = inds;
                     qApp->processEvents();
                 }
                 {
@@ -904,58 +1031,39 @@ void DlgEvaluateMeshImp::on_repairAllTogether_clicked()
                     MeshEvalNeighbourhood nb(rMesh);
                     if (!rf.Evaluate() || !rp.Evaluate() || !cf.Evaluate() || !nb.Evaluate()) {
                         Gui::Application::Instance->runCommand(true,
-                            "App.getDocument(\"%s\").getObject(\"%s\").fixIndices()\n",
+                            "App.getDocument(\"%s\").getObject(\"%s\").fixIndices()",
                             docName, objName);
                         run = true;
                     }
                 }
                 {
                     MeshEvalDegeneratedFacets eval(rMesh);
-                    std::vector<unsigned long> inds = eval.GetIndices();
-                    if (!inds.empty() && inds != degenerated) {
+                    if (!eval.Evaluate()) {
                         Gui::Application::Instance->runCommand(true,
-                            "App.getDocument(\"%s\").getObject(\"%s\").fixDegenerations()\n",
+                            "App.getDocument(\"%s\").getObject(\"%s\").fixDegenerations()",
                             docName, objName);
                         run = true;
                     }
-                    degenerated = inds;
                     qApp->processEvents();
                 }
                 {
                     MeshEvalDuplicateFacets eval(rMesh);
-                    std::vector<unsigned long> inds = eval.GetIndices();
-                    if (!inds.empty() && duplicatedf != inds) {
+                    if (!eval.Evaluate()) {
                         Gui::Application::Instance->runCommand(true,
-                            "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedFacets()\n",
+                            "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedFacets()",
                             docName, objName);
                         run = true;
                     }
-                    duplicatedf = inds;
                     qApp->processEvents();
                 }
                 {
                     MeshEvalDuplicatePoints eval(rMesh);
-                    std::vector<unsigned long> inds = eval.GetIndices();
-                    if (!inds.empty() && inds != duplicatedp) {
+                    if (!eval.Evaluate()) {
                         Gui::Application::Instance->runCommand(true,
-                            "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedPoints()\n",
+                            "App.getDocument(\"%s\").getObject(\"%s\").removeDuplicatedPoints()",
                             docName, objName);
                         run = true;
                     }
-                    duplicatedp = inds;
-                    qApp->processEvents();
-                }
-                {
-                    MeshEvalSelfIntersection eval(rMesh);
-                    std::vector<std::pair<unsigned long, unsigned long> > inds;
-                    eval.GetIntersections(inds);
-                    if (!inds.empty() && inds != intersection) {
-                        Gui::Application::Instance->runCommand(true,
-                            "App.getDocument(\"%s\").getObject(\"%s\").fixSelfIntersections()\n",
-                            docName, objName);
-                        run = true;
-                    }
-                    intersection = inds;
                     qApp->processEvents();
                 }
             } while(checkRepeatButton->isChecked() && run && (--max_iter > 0));
@@ -967,7 +1075,6 @@ void DlgEvaluateMeshImp::on_repairAllTogether_clicked()
             QMessageBox::warning(this, tr("Mesh repair"), QString::fromLatin1("Unknown error occurred."));
         }
 
-        //Base::Sequencer().setLocked(state);
         doc->commitCommand();
         doc->getDocument()->recompute();
     }
@@ -975,6 +1082,12 @@ void DlgEvaluateMeshImp::on_repairAllTogether_clicked()
 
 // -------------------------------------------------------------
 
+/* TRANSLATOR MeshGui::DockEvaluateMeshImp */
+
+#if 0 // needed for Qt's lupdate utility
+    qApp->translate("QDockWidget", "Evaluate & Repair Mesh");
+#endif
+
 DockEvaluateMeshImp* DockEvaluateMeshImp::_instance=0;
 
 DockEvaluateMeshImp* DockEvaluateMeshImp::instance()
@@ -1012,7 +1125,7 @@ DockEvaluateMeshImp::DockEvaluateMeshImp( QWidget* parent, Qt::WFlags fl )
     // embed this dialog into a dockable widget container
     Gui::DockWindowManager* pDockMgr = Gui::DockWindowManager::instance();
     // use Qt macro for preparing for translation stuff (but not translating yet)
-    QDockWidget* dw = pDockMgr->addDockWindow(QT_TR_NOOP("Evaluate & Repair Mesh"),
+    QDockWidget* dw = pDockMgr->addDockWindow("Evaluate & Repair Mesh",
         this, Qt::RightDockWidgetArea);
     //dw->setAttribute(Qt::WA_DeleteOnClose);
     dw->setFeatures(QDockWidget::DockWidgetMovable|QDockWidget::DockWidgetFloatable);
