@@ -25,11 +25,14 @@
 
 #ifndef _PreComp_
 # include <QMenu>
+# include <Inventor/SbBox2s.h>
 # include <Inventor/SoPickedPoint.h>
 # include <Inventor/details/SoFaceDetail.h>
 # include <Inventor/events/SoMouseButtonEvent.h>
 # include <Inventor/nodes/SoBaseColor.h>
+# include <Inventor/nodes/SoCallback.h>
 # include <Inventor/nodes/SoCoordinate3.h>
+# include <Inventor/nodes/SoLightModel.h>
 # include <Inventor/nodes/SoIndexedFaceSet.h>
 # include <Inventor/nodes/SoIndexedLineSet.h>
 # include <Inventor/nodes/SoDrawStyle.h>
@@ -37,6 +40,7 @@
 # include <Inventor/nodes/SoMaterialBinding.h>
 # include <Inventor/nodes/SoNormalBinding.h>
 # include <Inventor/nodes/SoOrthographicCamera.h>
+# include <Inventor/nodes/SoPerspectiveCamera.h>
 # include <Inventor/nodes/SoPolygonOffset.h>
 # include <Inventor/nodes/SoShapeHints.h>
 # include <Inventor/nodes/SoSeparator.h>
@@ -56,11 +60,12 @@
 #include <Gui/BitmapFactory.h>
 #include <Gui/Command.h>
 #include <Gui/Document.h>
+#include <Gui/SoFCOffscreenRenderer.h>
 #include <Gui/SoFCSelection.h>
 #include <Gui/SoFCSelectionAction.h>
 #include <Gui/MainWindow.h>
-#include <Gui/MouseModel.h>
 #include <Gui/Selection.h>
+#include <Gui/Utilities.h>
 #include <Gui/Window.h>
 #include <Gui/WaitCursor.h>
 #include <Gui/View3DInventor.h>
@@ -208,7 +213,7 @@ const char* ViewProviderMesh::LightingEnums[]= {"One side","Two side",NULL};
 
 PROPERTY_SOURCE(MeshGui::ViewProviderMesh, Gui::ViewProviderGeometryObject)
 
-ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
+ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0)
 {
     ADD_PROPERTY(LineTransparency,(0));
     LineTransparency.setConstraints(&intPercent);
@@ -222,7 +227,6 @@ ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
     ADD_PROPERTY(Lighting,(1));
     Lighting.setEnums(LightingEnums);
     ADD_PROPERTY(LineColor,(0,0,0));
-    //ADD_PROPERTY(BacksideColor,(0.8f,0.8f,0.8f));
 
     pOpenColor = new SoBaseColor();
     setOpenEdgeColorFrom(ShapeColor.getValue());
@@ -242,10 +246,6 @@ ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
     pShapeHints->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
     pShapeHints->ref();
 
-    //pBackHints = new SoShapeHints;
-    //pBackHints->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
-    //pBackHints->ref();
-
     pcMatBinding = new SoMaterialBinding;
     pcMatBinding->value = SoMaterialBinding::OVERALL;
     pcMatBinding->ref();
@@ -254,10 +254,6 @@ ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
     pLineColor->ref();
     LineColor.touch();
 
-    //pcBacksideMaterial = new SoMaterial;
-    //pcBacksideMaterial->ref();
-    //BacksideColor.touch();
-
     // read the correct shape color from the preferences
     Base::Reference<ParameterGrp> hGrp = Gui::WindowParameter::getDefaultParameter()->GetGroup("Mod/Mesh");
 
@@ -285,19 +281,14 @@ ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
     if (twoside) Lighting.setValue(1);
     else Lighting.setValue((long)0);
 
-    //color = BacksideColor.getValue();
-    //current = color.getPackedValue();
-    //setting = hGrp->GetUnsigned("BackfaceColor", current);
-    //if (current != setting) {
-    //    color.setPackedValue((uint32_t)setting);
-    //    BacksideColor.setValue(color);
-    //}
-
     bool normal_per_vertex = hGrp->GetBool("VertexPerNormals", false);
     if (normal_per_vertex) {
         double angle = hGrp->GetFloat("CreaseAngle", 0.0);
         CreaseAngle.setValue(angle);
     }
+
+    if (hGrp->GetBool("ShowBoundingBox", false))
+        pcHighlight->style = Gui::SoFCSelection::BOX;
 }
 
 ViewProviderMesh::~ViewProviderMesh()
@@ -306,14 +297,16 @@ ViewProviderMesh::~ViewProviderMesh()
     pcLineStyle->unref();
     pcPointStyle->unref();
     pShapeHints->unref();
-    //pBackHints->unref();
     pcMatBinding->unref();
     pLineColor->unref();
-    //pcBacksideMaterial->unref();
 }
 
 void ViewProviderMesh::onChanged(const App::Property* prop)
 {
+    // we gonna change the number of colors to one
+    if (prop == &ShapeColor || prop == &ShapeMaterial) {
+        pcMatBinding->value = SoMaterialBinding::OVERALL;
+    }
     if (prop == &LineTransparency) {
         float trans = LineTransparency.getValue()/100.0f;
         pLineColor->transparency = trans;
@@ -326,7 +319,6 @@ void ViewProviderMesh::onChanged(const App::Property* prop)
     }
     else if (prop == &CreaseAngle) {
         pShapeHints->creaseAngle = (F_PI*CreaseAngle.getValue())/180.0;
-        //pBackHints->creaseAngle = (F_PI*CreaseAngle.getValue())/180.0;
     }
     else if (prop == &OpenEdges) {
         showOpenEdges(OpenEdges.getValue());
@@ -335,22 +327,15 @@ void ViewProviderMesh::onChanged(const App::Property* prop)
         if (Lighting.getValue() == 0) {
             pShapeHints->vertexOrdering = SoShapeHints::UNKNOWN_ORDERING;
             //pShapeHints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
-            //pcBacksideMaterial->diffuseColor = pcShapeMaterial->diffuseColor;
         }
         else {
             pShapeHints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
-        //    const App::Color& c = BacksideColor.getValue();
-        //    pcBacksideMaterial->diffuseColor.setValue(c.r,c.g,c.b);
         }
     }
     else if (prop == &LineColor) {
         const App::Color& c = LineColor.getValue();
         pLineColor->diffuseColor.setValue(c.r,c.g,c.b);
     }
-    /*else if (prop == &BacksideColor && Lighting.getValue() != 0) {
-        const App::Color& c = BacksideColor.getValue();
-        pcBacksideMaterial->diffuseColor.setValue(c.r,c.g,c.b);
-    }*/
     else {
         // Set the inverse color for open edges
         if (prop == &ShapeColor) {
@@ -377,6 +362,11 @@ SoShape* ViewProviderMesh::getShapeNode() const
     return 0;
 }
 
+SoNode* ViewProviderMesh::getCoordNode() const
+{
+    return 0;
+}
+
 /** 
  * Extracts the mesh data from the feature \a pcFeature and creates
  * an Inventor node \a SoNode with these data. 
@@ -385,31 +375,14 @@ void ViewProviderMesh::attach(App::DocumentObject *pcFeat)
 {
     ViewProviderGeometryObject::attach(pcFeat);
 
+    // Note: Since for mesh data the SoFCSelection node has no SoSeparator but
+    // an SoGroup as parent the EMISSIVE style if set has fundamentally no effect.
+    // This behaviour is given due to the fact that SoFCSelection inherits from
+    // SoGroup (formerly SoSeparator). If we wanted to enable emissive overlay for
+    // highlighting or selection we would need an SoSeparator as parent node below.
+
     // faces
     SoGroup* pcFlatRoot = new SoGroup();
-    /*
-    {
-        // front face
-        SoSeparator* sep = new SoSeparator();
-        pShapeHints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
-        pShapeHints->shapeType = SoShapeHints::SOLID;
-        sep->addChild(pShapeHints);
-        sep->addChild(pcShapeMaterial);
-        sep->addChild(pcMatBinding);
-        sep->addChild(pcHighlight);
-        pcFlatRoot->addChild(sep);
-    }
-    {
-        // back face
-        SoSeparator* sep = new SoSeparator();
-        pBackHints->vertexOrdering = SoShapeHints::CLOCKWISE;
-        pBackHints->shapeType = SoShapeHints::SOLID;
-        sep->addChild(pBackHints);
-        sep->addChild(pcBacksideMaterial);
-        sep->addChild(pcHighlight);
-        pcFlatRoot->addChild(sep);
-    }
-    */
     pcFlatRoot->addChild(pShapeHints);
     pcFlatRoot->addChild(pcShapeMaterial);
     pcFlatRoot->addChild(pcMatBinding);
@@ -428,7 +401,9 @@ void ViewProviderMesh::attach(App::DocumentObject *pcFeat)
     SoGroup* pcWireRoot = new SoGroup();
     pcWireRoot->addChild(pcLineStyle);
     pcWireRoot->addChild(pcLightModel);
-    //pcWireRoot->addChild(pcShapeMaterial);
+    SoMaterialBinding* binding = new SoMaterialBinding;
+    binding->value = SoMaterialBinding::OVERALL; // doesn't set several colors
+    pcWireRoot->addChild(binding);
     pcWireRoot->addChild(pLineColor);
     pcWireRoot->addChild(pcHighlight);
     addDisplayMaskMode(pcWireRoot, "Wireframe");
@@ -445,16 +420,6 @@ void ViewProviderMesh::attach(App::DocumentObject *pcFeat)
     pcFlatWireRoot->addChild(offset);
     pcFlatWireRoot->addChild(pcWireRoot);
     addDisplayMaskMode(pcFlatWireRoot, "FlatWireframe");
-
-    // Turns on backface culling
-    //SoShapeHints * wirehints = new SoShapeHints;
-    //wirehints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE ;
-    //wirehints->shapeType = SoShapeHints::SOLID;
-    //wirehints->faceType = SoShapeHints::UNKNOWN_FACE_TYPE;
-    //SoGroup* pcHiddenLineRoot = new SoGroup();
-    //pcHiddenLineRoot->addChild(wirehints);
-    //pcHiddenLineRoot->addChild(pcWireRoot);
-    //addDisplayMaskMode(pcHiddenLineRoot, "HiddenLine");
 }
 
 QIcon ViewProviderMesh::getIcon() const
@@ -519,19 +484,11 @@ std::vector<std::string> ViewProviderMesh::getDisplayModes(void) const
 
 bool ViewProviderMesh::setEdit(int ModNum)
 {
-    if (m_bEdit) return true;
-    m_bEdit = true;
     return true;
 }
 
-void ViewProviderMesh::unsetEdit(void)
+void ViewProviderMesh::unsetEdit(int ModNum)
 {
-    m_bEdit = false;
-}
-
-const char* ViewProviderMesh::getEditModeName(void)
-{
-    return "Polygon picking";
 }
 
 bool ViewProviderMesh::createToolMesh(const std::vector<SbVec2f>& rclPoly, const SbViewVolume& vol,
@@ -632,7 +589,7 @@ void ViewProviderMesh::clipMeshCallback(void * ud, SoEventCallback * n)
     n->setHandled();
 
     SbBool clip_inner;
-    std::vector<SbVec2f> clPoly = view->getPickedPolygon(&clip_inner);
+    std::vector<SbVec2f> clPoly = view->getGLPolygon(&clip_inner);
     if (clPoly.size() < 3)
         return;
     if (clPoly.front() != clPoly.back())
@@ -643,8 +600,8 @@ void ViewProviderMesh::clipMeshCallback(void * ud, SoEventCallback * n)
         Gui::Application::Instance->activeDocument()->openCommand("Cut");
         for (std::vector<Gui::ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
             ViewProviderMesh* that = static_cast<ViewProviderMesh*>(*it);
-            if (that->m_bEdit) {
-                that->unsetEdit();
+            if (that->getEditingMode() > -1) {
+                that->finishEditing();
                 that->cutMesh(clPoly, *view, clip_inner);
             }
         }
@@ -667,7 +624,7 @@ void ViewProviderMesh::partMeshCallback(void * ud, SoEventCallback * cb)
     cb->setHandled();
 
     SbBool clip_inner;
-    std::vector<SbVec2f> clPoly = view->getPickedPolygon(&clip_inner);
+    std::vector<SbVec2f> clPoly = view->getGLPolygon(&clip_inner);
     if (clPoly.size() < 3)
         return;
     if (clPoly.front() != clPoly.back())
@@ -697,8 +654,8 @@ void ViewProviderMesh::partMeshCallback(void * ud, SoEventCallback * cb)
         std::vector<Gui::ViewProvider*> views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
         for (std::vector<Gui::ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
             ViewProviderMesh* that = static_cast<ViewProviderMesh*>(*it);
-            if (that->m_bEdit) {
-                that->unsetEdit();
+            if (that->getEditingMode() > -1) {
+                that->finishEditing();
                 that->splitMesh(toolMesh, cNormal, clip_inner);
             }
         }
@@ -724,7 +681,7 @@ void ViewProviderMesh::segmMeshCallback(void * ud, SoEventCallback * cb)
     cb->setHandled();
 
     SbBool clip_inner;
-    std::vector<SbVec2f> clPoly = view->getPickedPolygon(&clip_inner);
+    std::vector<SbVec2f> clPoly = view->getGLPolygon(&clip_inner);
     if (clPoly.size() < 3)
         return;
     if (clPoly.front() != clPoly.back())
@@ -754,8 +711,8 @@ void ViewProviderMesh::segmMeshCallback(void * ud, SoEventCallback * cb)
         std::vector<Gui::ViewProvider*> views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
         for (std::vector<Gui::ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
             ViewProviderMesh* that = static_cast<ViewProviderMesh*>(*it);
-            if (that->m_bEdit) {
-                that->unsetEdit();
+            if (that->getEditingMode() > -1) {
+                that->finishEditing();
                 that->segmentMesh(toolMesh, cNormal, clip_inner);
             }
         }
@@ -777,7 +734,7 @@ void ViewProviderMesh::selectGLCallback(void * ud, SoEventCallback * n)
     view->removeEventCallback(SoMouseButtonEvent::getClassTypeId(), selectGLCallback,ud);
     n->setHandled();
 
-    std::vector<SbVec2f> clPoly = view->getPickedPolygon();
+    std::vector<SbVec2f> clPoly = view->getGLPolygon();
     if (clPoly.size() != 1)
         return;
     const SoEvent* ev = n->getEvent();
@@ -812,9 +769,9 @@ void ViewProviderMesh::selectGLCallback(void * ud, SoEventCallback * n)
     views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
     for (std::vector<Gui::ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
         ViewProviderMesh* that = static_cast<ViewProviderMesh*>(*it);
-        if (that->m_bEdit) {
-            that->unsetEdit();
-            that->selectArea(x, y, w, h);
+        if (that->getEditingMode() > -1) {
+            that->finishEditing();
+            that->selectArea(x, y, w, h, view->getViewportRegion(), view->getCamera());
         }
     }
 
@@ -826,6 +783,20 @@ void ViewProviderMesh::getFacetsFromPolygon(const std::vector<SbVec2f>& picked,
                                             SbBool inner,
                                             std::vector<unsigned long>& indices) const
 {
+#if 1
+    bool ok = true;
+    SoCamera* cam = Viewer.getCamera();
+    SbViewVolume vv = cam->getViewVolume();
+    Gui::ViewVolumeProjection proj(vv);
+    Base::Polygon2D polygon;
+    for (std::vector<SbVec2f>::const_iterator it = picked.begin(); it != picked.end(); ++it)
+        polygon.Add(Base::Vector2D((*it)[0],(*it)[1]));
+
+    // Get the attached mesh property
+    Mesh::PropertyMeshKernel& meshProp = static_cast<Mesh::Feature*>(pcObject)->Mesh;
+    MeshCore::MeshAlgorithm cAlg(meshProp.getValue().getKernel());
+    cAlg.CheckFacets(&proj, polygon, true, indices);
+#else
     // get the normal of the front clipping plane
     SbVec3f b,n;
     Viewer.getNearPlane(b, n);
@@ -848,6 +819,7 @@ void ViewProviderMesh::getFacetsFromPolygon(const std::vector<SbVec2f>& picked,
     MeshCore::MeshFacetGrid cGrid(meshProp.getValue().getKernel());
     MeshCore::MeshAlgorithm cAlg(meshProp.getValue().getKernel());
     cAlg.GetFacetsFromToolMesh(cToolMesh, cNormal, cGrid, indices);
+#endif
     if (!inner) {
         // get the indices that are completely outside
         std::vector<unsigned long> complete(meshProp.getValue().countFacets());
@@ -863,6 +835,169 @@ void ViewProviderMesh::getFacetsFromPolygon(const std::vector<SbVec2f>& picked,
         Base::Console().Message("The picked polygon seems to have self-overlappings. This could lead to strange results.");
 }
 
+std::vector<unsigned long> ViewProviderMesh::getFacetsOfRegion(const SbViewportRegion& select,
+                                                               const SbViewportRegion& region,
+                                                               SoCamera* camera) const
+{
+    SoSeparator* root = new SoSeparator();
+    root->ref();
+    root->addChild(camera);
+    root->addChild(const_cast<ViewProviderMesh*>(this)->getCoordNode());
+    root->addChild(const_cast<ViewProviderMesh*>(this)->getShapeNode());
+    Gui::SoGLSelectAction gl(region, select);
+    gl.apply(root);
+    root->unref();
+
+    std::vector<unsigned long> faces;
+    faces.insert(faces.end(), gl.indices.begin(), gl.indices.end());
+    return faces;
+}
+
+void ViewProviderMesh::panCamera(SoCamera * cam, float aspectratio, const SbPlane & panplane,
+                                 const SbVec2f & currpos, const SbVec2f & prevpos)
+{
+    if (cam == NULL) return; // can happen for empty scenegraph
+    if (currpos == prevpos) return; // useless invocation
+
+
+    // Find projection points for the last and current mouse coordinates.
+    SbViewVolume vv = cam->getViewVolume(aspectratio);
+    SbLine line;
+    vv.projectPointToLine(currpos, line);
+    SbVec3f current_planept;
+    panplane.intersect(line, current_planept);
+    vv.projectPointToLine(prevpos, line);
+    SbVec3f old_planept;
+    panplane.intersect(line, old_planept);
+
+    // Reposition camera according to the vector difference between the
+    // projected points.
+    cam->position = cam->position.getValue() - (current_planept - old_planept);
+}
+
+void ViewProviderMesh::boxZoom(const SbBox2s& box, const SbViewportRegion & vp, SoCamera* cam)
+{
+    SbViewVolume vv = cam->getViewVolume(vp.getViewportAspectRatio());
+
+    short sizeX,sizeY;
+    box.getSize(sizeX, sizeY);
+    SbVec2s size = vp.getViewportSizePixels();
+
+    // The bbox must not be empty i.e. width and length is zero, but it is possible that
+    // either width or length is zero
+    if (sizeX == 0 && sizeY == 0) 
+        return;
+
+    // Get the new center in normalized pixel coordinates
+    short xmin,xmax,ymin,ymax;
+    box.getBounds(xmin,ymin,xmax,ymax);
+    const SbVec2f center((float) ((xmin+xmax)/2) / (float) SoQtMax((int)(size[0] - 1), 1),
+                         (float) (size[1]-(ymin+ymax)/2) / (float) SoQtMax((int)(size[1] - 1), 1));
+
+    SbPlane plane = vv.getPlane(cam->focalDistance.getValue());
+    panCamera(cam,vp.getViewportAspectRatio(),plane, SbVec2f(0.5,0.5), center);
+
+    // Set height or height angle of the camera
+    float scaleX = (float)sizeX/(float)size[0];
+    float scaleY = (float)sizeY/(float)size[1];
+    float scale = std::max<float>(scaleX, scaleY);
+    if (cam && cam->getTypeId() == SoOrthographicCamera::getClassTypeId()) {
+        float height = static_cast<SoOrthographicCamera*>(cam)->height.getValue() * scale;
+        static_cast<SoOrthographicCamera*>(cam)->height = height;
+    }
+    else if (cam && cam->getTypeId() == SoPerspectiveCamera::getClassTypeId()) {
+        float height = static_cast<SoPerspectiveCamera*>(cam)->heightAngle.getValue() / 2.0f;
+        height = 2.0f * atan(tan(height) * scale);
+        static_cast<SoPerspectiveCamera*>(cam)->heightAngle = height;
+    }
+}
+
+std::vector<unsigned long> ViewProviderMesh::getVisibleFacetsAfterZoom(const SbBox2s& rect,
+                                                                       const SbViewportRegion& vp,
+                                                                       SoCamera* camera) const
+{
+    // camera copy will be deleted inside getVisibleFacets()
+    // because the ref counter reaches 0
+    camera = static_cast<SoCamera*>(camera->copy());
+    boxZoom(rect,vp,camera);
+    return getVisibleFacets(vp, camera);
+}
+
+void ViewProviderMesh::renderGLCallback(void * ud, SoAction * action)
+{
+    if (action->isOfType(SoGLRenderAction::getClassTypeId())) {
+        ViewProviderMesh* mesh = reinterpret_cast<ViewProviderMesh*>(ud);
+        Gui::SoVisibleFaceAction fa;
+        fa.apply(mesh->getRoot());
+    }
+}
+
+std::vector<unsigned long> ViewProviderMesh::getVisibleFacets(const SbViewportRegion& vp,
+                                                              SoCamera* camera) const
+{
+    const Mesh::PropertyMeshKernel& meshProp = static_cast<Mesh::Feature*>(pcObject)->Mesh;
+    const Mesh::MeshObject& mesh = meshProp.getValue();
+    uint32_t count = (uint32_t)mesh.countFacets();
+
+    SoSeparator* root = new SoSeparator;
+    root->ref();
+    root->addChild(camera);
+
+#if 0
+    SoCallback* cb = new SoCallback;
+    cb->setCallback(renderGLCallback, const_cast<ViewProviderMesh*>(this));
+    root->addChild(cb);
+#else
+    SoLightModel* lm = new SoLightModel();
+    lm->model = SoLightModel::BASE_COLOR;
+    root->addChild(lm);
+    SoMaterial* mat = new SoMaterial();
+    mat->diffuseColor.setNum(count);
+    SbColor* diffcol = mat->diffuseColor.startEditing();
+    for (uint32_t i=0; i<count; i++) {
+        float t;
+        diffcol[i].setPackedValue(i<<8,t);
+    }
+
+    mat->diffuseColor.finishEditing();
+    SoMaterialBinding* bind = new SoMaterialBinding();
+    bind->value = SoMaterialBinding::PER_FACE;
+    root->addChild(mat);
+    root->addChild(bind);
+#endif
+    root->addChild(this->getCoordNode());
+    root->addChild(this->getShapeNode());
+
+    Gui::SoFCOffscreenRenderer& renderer = Gui::SoFCOffscreenRenderer::instance();
+    renderer.setViewportRegion(vp);
+    renderer.setBackgroundColor(SbColor(0.0f, 0.0f, 0.0f));
+
+    QImage img;
+    renderer.render(root);
+    renderer.writeToImage(img);
+    root->unref();
+
+    int width = img.width();
+    int height = img.height();
+    QRgb color=0;
+    std::vector<unsigned long> faces;
+    for (int y = 0; y < height; y++) {
+        for (int x = 0; x < width; x++) {
+            QRgb rgb = img.pixel(x,y);
+            rgb = rgb-(0xff << 24);
+            if (rgb != 0 && rgb != color) {
+                color = rgb;
+                faces.push_back((unsigned long)rgb);
+            }
+        }
+    }
+
+    std::sort(faces.begin(), faces.end());
+    faces.erase(std::unique(faces.begin(), faces.end()), faces.end());
+
+    return faces;
+}
+
 void ViewProviderMesh::cutMesh(const std::vector<SbVec2f>& picked, 
                                Gui::View3DInventorViewer &Viewer, SbBool inner)
 {
@@ -1248,8 +1383,7 @@ void ViewProviderMesh::deselectComponent(unsigned long uFacet)
 void ViewProviderMesh::setSelection(const std::vector<unsigned long>& indices)
 {
     const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
-    const MeshCore::MeshKernel& rKernel = rMesh.getKernel();
-    MeshCore::MeshAlgorithm(rKernel).ResetFacetFlag(MeshCore::MeshFacet::SELECTED);
+    rMesh.clearFacetSelection();
     rMesh.addFacetsToSelection(indices);
 
     // Colorize the selection
@@ -1277,8 +1411,7 @@ void ViewProviderMesh::removeSelection(const std::vector<unsigned long>& indices
 void ViewProviderMesh::clearSelection()
 {
     const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
-    const MeshCore::MeshKernel& rKernel = rMesh.getKernel();
-    MeshCore::MeshAlgorithm(rKernel).ResetFacetFlag(MeshCore::MeshFacet::SELECTED);
+    rMesh.clearFacetSelection();
     unhighlightSelection();
 }
 
@@ -1298,15 +1431,16 @@ void ViewProviderMesh::deleteSelection()
     }
 }
 
-void ViewProviderMesh::selectArea(short x, short y, short w, short h)
+void ViewProviderMesh::selectArea(short x, short y, short w, short h,
+                                  const SbViewportRegion& region,
+                                  SoCamera* camera)
 {
-    Gui::SoGLSelectAction gl;
-    gl.x = x; gl.y = y;
-    gl.w = w; gl.h = h;
-    gl.apply(this->pcHighlight);
+    SbViewportRegion vp;
+    vp.setViewportPixels (x, y, w, h);
+    std::vector<unsigned long> faces = getFacetsOfRegion(vp, region, camera);
 
     const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
-    rMesh.addFacetsToSelection(gl.indices);
+    rMesh.addFacetsToSelection(faces);
 
     // Colorize the selected part
     highlightSelection();
@@ -1317,6 +1451,11 @@ void ViewProviderMesh::highlightSelection()
     std::vector<unsigned long> selection;
     const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
     rMesh.getFacetsFromSelection(selection);
+    if (selection.empty()) {
+        // If no faces are selected then simply return even without
+        // without calling unhighlightSelection()
+        return;
+    }
 
     // Colorize the selection
     pcMatBinding->value = SoMaterialBinding::PER_FACE;
@@ -1379,6 +1518,7 @@ void ViewProviderIndexedFaceSet::updateData(const App::Property* prop)
         ViewProviderMeshBuilder builder;
         builder.createMesh(prop, pcMeshCoord, pcMeshFaces);
         showOpenEdges(OpenEdges.getValue());
+        highlightSelection();
     }
 }
 
@@ -1423,6 +1563,11 @@ SoShape* ViewProviderIndexedFaceSet::getShapeNode() const
     return this->pcMeshFaces;
 }
 
+SoNode* ViewProviderIndexedFaceSet::getCoordNode() const
+{
+    return this->pcMeshCoord;
+}
+
 // ------------------------------------------------------
 
 PROPERTY_SOURCE(MeshGui::ViewProviderMeshObject, MeshGui::ViewProviderMesh)
@@ -1487,3 +1632,8 @@ SoShape* ViewProviderMeshObject::getShapeNode() const
 {
     return this->pcMeshShape;
 }
+
+SoNode* ViewProviderMeshObject::getCoordNode() const
+{
+    return this->pcMeshNode;
+}
