@@ -41,6 +41,12 @@ class SoCoordinate3;
 class SoIndexedFaceSet;
 class SoShapeHints;
 class SoMaterialBinding;
+class SoCamera;
+class SoAction;
+class SbViewportRegion;
+class SbVec2f;
+class SbBox2s;
+class SbPlane;
 
 namespace App {
   class Color;
@@ -49,7 +55,6 @@ namespace App {
 namespace Gui {
   class View3DInventorViewer;
   class SoFCSelection;
-  class AbstractMouseModel;
 }
 
 
@@ -62,7 +67,7 @@ namespace MeshGui {
 class SoFCMeshObjectNode;
 class SoFCMeshObjectShape;
 
-class ViewProviderMeshBuilder : public Gui::ViewProviderBuilder
+class MeshGuiExport ViewProviderMeshBuilder : public Gui::ViewProviderBuilder
 {
 public:
     ViewProviderMeshBuilder(){}
@@ -110,7 +115,6 @@ public:
     App::PropertyBool OpenEdges;
     App::PropertyEnumeration Lighting;
     App::PropertyColor LineColor;
-    //App::PropertyColor BacksideColor;
 
     virtual void attach(App::DocumentObject *);
     virtual QIcon getIcon() const;
@@ -122,12 +126,7 @@ public:
     /** @name Editing */
     //@{
     bool doubleClicked(void){return false;}
-    /// Sets the edit mode
-    bool setEdit(int ModNum=0);
-    /// Unsets the edit mode
-    void unsetEdit(void);
-    /// Returns the edit mode
-    const char* getEditModeName(void);
+    void setupContextMenu(QMenu*, QObject*, const char*) {}
     void selectComponent(unsigned long facet);
     void deselectComponent(unsigned long facet);
     void selectFacet(unsigned long facet);
@@ -140,9 +139,16 @@ public:
     void getFacetsFromPolygon(const std::vector<SbVec2f>& picked,
                               Gui::View3DInventorViewer &Viewer, SbBool inner,
                               std::vector<unsigned long>& indices) const;
+    std::vector<unsigned long> getFacetsOfRegion(const SbViewportRegion&, const SbViewportRegion&, SoCamera*) const;
+    std::vector<unsigned long> getVisibleFacetsAfterZoom(const SbBox2s&, const SbViewportRegion&, SoCamera*) const;
+    std::vector<unsigned long> getVisibleFacets(const SbViewportRegion&, SoCamera*) const;
     //@}
 
 protected:
+    /// Sets the edit mode
+    bool setEdit(int ModNum);
+    /// Unsets the edit mode
+    void unsetEdit(int ModNum);
     /// get called by the container whenever a property has been changed
     void onChanged(const App::Property* prop);
     virtual void showOpenEdges(bool);
@@ -152,11 +158,12 @@ protected:
     virtual void segmentMesh(const MeshCore::MeshKernel& toolMesh, const Base::Vector3f& normal, SbBool inner);
     virtual void faceInfo(unsigned long facet);
     virtual void fillHole(unsigned long facet);
-    virtual void selectArea(short, short, short, short);
+    virtual void selectArea(short, short, short, short, const SbViewportRegion&, SoCamera*);
     void highlightSelection();
     void unhighlightSelection();
 
     virtual SoShape* getShapeNode() const;
+    virtual SoNode* getCoordNode() const;
 
 public:
     static void faceInfoCallback(void * ud, SoEventCallback * n);
@@ -170,6 +177,11 @@ public:
     static bool createToolMesh(const std::vector<SbVec2f>& rclPoly, const SbViewVolume& vol,
             const Base::Vector3f& rcNormal, std::vector<MeshCore::MeshGeomFacet>&);
 
+private:
+    static void renderGLCallback(void * ud, SoAction * a);
+    static void boxZoom(const SbBox2s& box, const SbViewportRegion & vp, SoCamera* cam);
+    static void panCamera(SoCamera*, float, const SbPlane&, const SbVec2f&, const SbVec2f&);
+
 protected:
     SoDrawStyle         * pcLineStyle;
     SoDrawStyle         * pcPointStyle;
@@ -177,13 +189,9 @@ protected:
     SoBaseColor         * pOpenColor;
     SoMaterial          * pLineColor;
     SoShapeHints        * pShapeHints;
-    //SoShapeHints        * pBackHints;
     SoMaterialBinding   * pcMatBinding;
-    //SoMaterial          * pcBacksideMaterial;
 
 private:
-    bool m_bEdit;
-
     static App::PropertyFloatConstraint::Constraints floatRange;
     static App::PropertyFloatConstraint::Constraints angleRange;
     static App::PropertyIntegerConstraint::Constraints intPercent;
@@ -210,6 +218,7 @@ public:
 protected:
     void showOpenEdges(bool);
     SoShape* getShapeNode() const;
+    SoNode* getCoordNode() const;
 
 private:
     SoCoordinate3       * pcMeshCoord;
@@ -234,6 +243,7 @@ public:
 
 protected:
     SoShape* getShapeNode() const;
+    SoNode* getCoordNode() const;
     void showOpenEdges(bool);
 
 private:
