@@ -28,23 +28,35 @@
 # include <Inventor/details/SoFaceDetail.h>
 # include <Inventor/details/SoPointDetail.h>
 # include <Inventor/events/SoMouseButtonEvent.h>
+# include <Inventor/events/SoKeyboardEvent.h>
+# include <Inventor/events/SoLocation2Event.h>
 # include <Inventor/nodes/SoDrawStyle.h>
 # include <Inventor/nodes/SoIndexedFaceSet.h>
 # include <Inventor/nodes/SoMaterial.h>
 # include <Inventor/nodes/SoMaterialBinding.h>
 # include <Inventor/nodes/SoShapeHints.h>
 # include <algorithm>
+# include <sstream>
+# include <QEvent>
+# include <QMenu>
+# include <QMessageBox>
 # include <QCursor>
 # include <QToolTip>
 # include <QWhatsThis>
 #endif
 
+# include <iomanip>
+# include <ios>
+
 // Here the FreeCAD includes sorted by Base,App,Gui......
 #include <Base/Console.h>
 #include <Base/Parameter.h>
 #include <Base/Exception.h>
 #include <Base/Sequencer.h>
+#include <App/Annotation.h>
 #include <App/Application.h>
+#include <App/Document.h>
+#include <App/DocumentObjectGroup.h>
 #include <Gui/Application.h>
 #include <Gui/Document.h>
 #include <Gui/MainWindow.h>
@@ -67,13 +79,18 @@ using namespace Mesh;
 using namespace MeshGui;
 using namespace std;
 
+bool ViewProviderMeshCurvature::addflag = false;
 
 PROPERTY_SOURCE(MeshGui::ViewProviderMeshCurvature, Gui::ViewProviderDocumentObject)
 
 ViewProviderMeshCurvature::ViewProviderMeshCurvature()
 {
+    pcColorRoot = new SoSeparator();
+    pcColorRoot->ref();
     pcColorMat = new SoMaterial;
     pcColorMat->ref();
+    pcColorStyle = new SoDrawStyle(); 
+    pcColorRoot->addChild(pcColorStyle);
     // simple color bar
     pcColorBar = new Gui::SoFCColorBar;
     pcColorBar->Attach(this);
@@ -85,12 +102,25 @@ ViewProviderMeshCurvature::ViewProviderMeshCurvature()
 
 ViewProviderMeshCurvature::~ViewProviderMeshCurvature()
 {
+    pcColorRoot->unref();
     pcColorMat->unref();
     pcColorBar->Detach(this);
     pcColorBar->unref();
     pcLinkRoot->unref();
 }
 
+void ViewProviderMeshCurvature::hide(void)
+{
+    inherited::hide();
+    pcColorStyle->style = SoDrawStyle::INVISIBLE;
+}
+
+void ViewProviderMeshCurvature::show(void)
+{
+    inherited::show();
+    pcColorStyle->style = SoDrawStyle::FILLED;
+}
+
 void ViewProviderMeshCurvature::init(const Mesh::PropertyCurvatureList* pCurvInfo)
 {
     std::vector<float> aMinValues, aMaxValues;
@@ -222,6 +252,8 @@ void ViewProviderMeshCurvature::attach(App::DocumentObject *pcFeat)
         pcColorBar->unref();
         pcColorBar = pcBar;
     }
+
+    pcColorRoot->addChild(pcColorBar);
 }
 
 void ViewProviderMeshCurvature::updateData(const App::Property* prop)
@@ -254,7 +286,7 @@ void ViewProviderMeshCurvature::updateData(const App::Property* prop)
 
 SoSeparator* ViewProviderMeshCurvature::getFrontRoot(void) const
 {
-    return pcColorBar;
+    return pcColorRoot;
 }
 
 void ViewProviderMeshCurvature::setVertexCurvatureMode(int mode)
@@ -368,19 +400,94 @@ void ViewProviderMeshCurvature::OnChange(Base::Subject<int> &rCaller,int rcReaso
     setActiveMode();
 }
 
+namespace MeshGui {
+class AnnotationEvent : public QEvent
+{
+public:
+    AnnotationEvent(Gui::ViewProviderDocumentObject* vp, const QString& s,const SbVec3f& p, const SbVec3f& n)
+        : QEvent(QEvent::User), vp(vp), s(s), p(p), n(n)
+    {
+    }
+
+    Gui::ViewProviderDocumentObject* vp;
+    QString s;
+    SbVec3f p;
+    SbVec3f n;
+};
+
+// Proxy class that receives an asynchronous custom event
+class ViewProviderProxyObject : public QObject
+{
+public:
+    ViewProviderProxyObject(QWidget* w) : QObject(0), widget(w) {}
+    ~ViewProviderProxyObject() {}
+    void customEvent(QEvent * e)
+    {
+        AnnotationEvent* ae = static_cast<AnnotationEvent*>(e);
+        App::Document* doc = ae->vp->getObject()->getDocument();
+
+        std::vector<App::DocumentObject*> groups = doc->getObjectsOfType
+            (App::DocumentObjectGroup::getClassTypeId());
+        App::DocumentObjectGroup* group = 0;
+        std::string internalname = "CurvatureGroup";
+        for (std::vector<App::DocumentObject*>::iterator it = groups.begin(); it != groups.end(); ++it) {
+            if (internalname == (*it)->getNameInDocument()) {
+                group = static_cast<App::DocumentObjectGroup*>(*it);
+                break;
+            }
+        }
+        if (!group) {
+            group = static_cast<App::DocumentObjectGroup*>(doc->addObject
+                ("App::DocumentObjectGroup",internalname.c_str()));
+        }
+
+        App::AnnotationLabel* anno = static_cast<App::AnnotationLabel*>
+            (group->addObject("App::AnnotationLabel", internalname.c_str()));
+        QStringList lines = ae->s.split(QLatin1String("\n"));
+        std::vector<std::string> text;
+        for (QStringList::Iterator it = lines.begin(); it != lines.end(); ++it)
+            text.push_back((const char*)it->toAscii());
+        anno->LabelText.setValues(text);
+        std::stringstream str;
+        str << "Curvature info (" << group->Group.getSize() << ")";
+        anno->Label.setValue(str.str());
+        anno->BasePosition.setValue(ae->p[0],ae->p[1],ae->p[2]);
+        anno->TextPosition.setValue(ae->n[0],ae->n[1],ae->n[2]);
+        this->deleteLater();
+    }
+
+private:
+    QPointer<QWidget> widget;
+};
+}
+
 void ViewProviderMeshCurvature::curvatureInfoCallback(void * ud, SoEventCallback * n)
 {
-    if (n->getEvent()->getTypeId() == SoMouseButtonEvent::getClassTypeId()) {
-        const SoMouseButtonEvent * mbe = static_cast<const SoMouseButtonEvent *>(n->getEvent());
-        Gui::View3DInventorViewer* view  = reinterpret_cast<Gui::View3DInventorViewer*>(n->getUserData());
+    Gui::View3DInventorViewer* view  = reinterpret_cast<Gui::View3DInventorViewer*>(n->getUserData());
+    const SoEvent* ev = n->getEvent();
+    if (ev->getTypeId() == SoMouseButtonEvent::getClassTypeId()) {
+        const SoMouseButtonEvent * mbe = static_cast<const SoMouseButtonEvent *>(ev);
 
         // Mark all incoming mouse button events as handled, especially, to deactivate the selection node
         n->getAction()->setHandled();
         if (mbe->getButton() == SoMouseButtonEvent::BUTTON2 && mbe->getState() == SoButtonEvent::UP) {
             n->setHandled();
-            view->setEditing(false);
-            view->getWidget()->setCursor(QCursor(Qt::ArrowCursor));
-            view->removeEventCallback(SoMouseButtonEvent::getClassTypeId(), curvatureInfoCallback);
+            // context-menu
+            QMenu menu;
+            QAction* fl = menu.addAction(QObject::tr("Annotation"));
+            fl->setCheckable(true);
+            fl->setChecked(addflag);
+            QAction* cl = menu.addAction(QObject::tr("Leave info mode"));
+            QAction* id = menu.exec(QCursor::pos());
+            if (fl == id) {
+                addflag = fl->isChecked();
+            }
+            else if (cl == id) {
+                view->setEditing(false);
+                view->getWidget()->setCursor(QCursor(Qt::ArrowCursor));
+                view->setRedirectToSceneGraph(false);
+                view->removeEventCallback(SoEvent::getClassTypeId(), curvatureInfoCallback);
+            }
         }
         else if (mbe->getButton() == SoMouseButtonEvent::BUTTON1 && mbe->getState() == SoButtonEvent::UP) {
             const SoPickedPoint * point = n->getPickedPoint();
@@ -405,16 +512,52 @@ void ViewProviderMeshCurvature::curvatureInfoCallback(void * ud, SoEventCallback
                 int index1 = facedetail->getPoint(0)->getCoordinateIndex();
                 int index2 = facedetail->getPoint(1)->getCoordinateIndex();
                 int index3 = facedetail->getPoint(2)->getCoordinateIndex();
-                that->curvatureInfo(index1, index2, index3);
+                std::string info = that->curvatureInfo(true, index1, index2, index3);
+                QString text = QString::fromAscii(info.c_str());
+                if (addflag) {
+                    SbVec3f pt = point->getPoint();
+                    SbVec3f nl = point->getNormal();
+                    QApplication::postEvent(
+                        new ViewProviderProxyObject(view->getGLWidget()),
+                        new AnnotationEvent(that, text, pt, nl));
+                }
+                else {
+                    Gui::ToolTip::showText(QCursor::pos(), text);
+                }
             }
         }
     }
+    else if (ev->getTypeId().isDerivedFrom(SoLocation2Event::getClassTypeId())) {
+        const SoPickedPoint * point = n->getPickedPoint();
+        if (point == NULL)
+            return;
+        n->setHandled();
+
+        // By specifying the indexed mesh node 'pcFaceSet' we make sure that the picked point is
+        // really from the mesh we render and not from any other geometry
+        Gui::ViewProvider* vp = static_cast<Gui::ViewProvider*>(view->getViewProviderByPath(point->getPath()));
+        if (!vp || !vp->getTypeId().isDerivedFrom(ViewProviderMeshCurvature::getClassTypeId()))
+            return;
+        ViewProviderMeshCurvature* that = static_cast<ViewProviderMeshCurvature*>(vp);
+        const SoDetail* detail = point->getDetail(point->getPath()->getTail());
+        if (detail && detail->getTypeId() == SoFaceDetail::getClassTypeId()) {
+            // safe downward cast, know the type
+            SoFaceDetail * facedetail = (SoFaceDetail *)detail;
+            // get the curvature info of the three points of the picked facet
+            int index1 = facedetail->getPoint(0)->getCoordinateIndex();
+            int index2 = facedetail->getPoint(1)->getCoordinateIndex();
+            int index3 = facedetail->getPoint(2)->getCoordinateIndex();
+            std::string info = that->curvatureInfo(false, index1, index2, index3);
+            Gui::getMainWindow()->setPaneText(1,QString::fromAscii(info.c_str()));
+        }
+    }
 }
 
-void ViewProviderMeshCurvature::curvatureInfo(int index1, int index2, int index3) const
+std::string ViewProviderMeshCurvature::curvatureInfo(bool detail, int index1, int index2, int index3) const
 {
     // get the curvature info of the three points of the picked facet
     App::Property* prop = pcObject->getPropertyByName("CurvInfo");
+    std::stringstream str;
     if (prop && prop->getTypeId() == Mesh::PropertyCurvatureList::getClassTypeId()) {
         Mesh::PropertyCurvatureList* curv = (Mesh::PropertyCurvatureList*)prop;
         const Mesh::CurvatureInfo& cVal1 = (*curv)[index1];
@@ -453,18 +596,23 @@ void ViewProviderMeshCurvature::curvatureInfo(int index1, int index2, int index3
             print = false;
         }
 
-        QString info, what;
         if (print) {
-            info = QString(QLatin1String("%1: <%2, %3, %4>"))
-                .arg(QString::fromLatin1(mode.c_str())).arg(fVal1).arg(fVal2).arg(fVal3);
-            what = QString::fromAscii("%1\nv1: %2\nv2: %3\nv3: %4")
-                .arg(QString::fromLatin1(mode.c_str())).arg(fVal1).arg(fVal2).arg(fVal3);
+            if (!detail) {
+                str << mode << ": <" << fVal1 << ", " << fVal2 << ", " << fVal3 << ">";
+            }
+            else {
+                str.setf(std::ios::fixed | std::ios::showpoint);
+                str.precision(5);
+                str << mode << std::endl
+                    << "v1: " << std::setw(5) << fVal1 << std::endl
+                    << "v2: " << std::setw(5) << fVal2 << std::endl
+                    << "v3: " << std::setw(5) << fVal3;
+            }
         }
-        else {
-            info = QString::fromLatin1("No curvature mode set");
+        else if (!detail) {
+            str << "No curvature mode set";
         }
-
-        Gui::getMainWindow()->setPaneText(1,info);
-        Gui::ToolTip::showText(QCursor::pos(), what);
     }
+
+    return str.str();
 }
