@@ -35,9 +35,12 @@
 #include "FeaturePartCircle.h"
 #include "FeaturePartPolygon.h"
 #include "FeatureGeometrySet.h"
+#include "FeatureChamfer.h"
 #include "FeatureExtrusion.h"
 #include "FeatureFillet.h"
+#include "FeatureMirroring.h"
 #include "FeatureRevolution.h"
+#include "Features.h"
 #include "PrimitiveFeature.h"
 #include "Part2DObject.h"
 #include "CustomFeature.h"
@@ -124,9 +127,6 @@ void AppPartExport initPart()
     Part::PropertyPartShape     ::init();
     Part::PropertyGeometryList  ::init();
     Part::PropertyFilletEdges   ::init();
-#if 1
-    Part::PropertyCircle        ::init();
-#endif
 
     Part::Feature               ::init();
     Part::FeaturePython         ::init();
@@ -137,12 +137,16 @@ void AppPartExport initPart()
     Part::Box                   ::init();
     Part::Boolean               ::init();
     Part::Common                ::init();
+    Part::MultiCommon           ::init();
     Part::Cut                   ::init();
     Part::Fuse                  ::init();
+    Part::MultiFuse             ::init();
     Part::Section               ::init();
+    Part::Chamfer               ::init();
     Part::Extrusion             ::init();
     Part::Fillet                ::init();
     Part::Revolution            ::init();
+    Part::Mirroring             ::init();
     Part::ImportStep            ::init();
     Part::ImportIges            ::init();
     Part::ImportBrep            ::init();
@@ -157,6 +161,7 @@ void AppPartExport initPart()
     Part::Torus                 ::init();
     Part::Helix                 ::init();
     Part::Part2DObject          ::init();
+    Part::RuledSurface          ::init();
 
     // Geometry types
     Part::Geometry                ::init();
