@@ -24,10 +24,14 @@
 #include "PreCompiled.h"
 #ifndef _PreComp_
 # include <Geom_BSplineCurve.hxx>
+# include <GeomAPI_PointsToBSpline.hxx>
+# include <GeomAPI_Interpolate.hxx>
 # include <gp_Pnt.hxx>
 # include <TColStd_Array1OfReal.hxx>
 # include <TColgp_Array1OfPnt.hxx>
+# include <TColgp_HArray1OfPnt.hxx>
 # include <TColStd_Array1OfInteger.hxx>
+# include <Handle_TColgp_HArray1OfPnt.hxx>
 #endif
 
 #include <Base/VectorPy.h>
@@ -52,9 +56,15 @@ PyObject *BSplineCurvePy::PyMake(struct _typeobject *, PyObject *, PyObject *)
 }
 
 // constructor method
-int BSplineCurvePy::PyInit(PyObject* /*args*/, PyObject* /*kwd*/)
+int BSplineCurvePy::PyInit(PyObject* args, PyObject* /*kwd*/)
 {
-    return 0;
+    if (PyArg_ParseTuple(args, "")) {
+        return 0;
+    }
+
+    PyErr_SetString(PyExc_TypeError, "B-Spline constructor accepts:\n"
+        "-- empty parameter list\n");
+    return -1;
 }
 
 PyObject* BSplineCurvePy::isRational(PyObject *args)
@@ -683,7 +693,133 @@ Py::List BSplineCurvePy::getKnotSequence(void) const
     return list;
 }
 
-PyObject *BSplineCurvePy::getCustomAttributes(const char* /*attr*/) const
+PyObject* BSplineCurvePy::approximate(PyObject *args)
+{
+    PyObject* obj;
+    if (!PyArg_ParseTuple(args, "O!",&(PyList_Type), &obj))
+        return 0;
+    try {
+        Py::List list(obj);
+        TColgp_Array1OfPnt pnts(1,list.size());
+        Standard_Integer index = 1;
+        for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+            Base::Vector3d vec = Py::Vector(*it).toVector();
+            pnts(index++) = gp_Pnt(vec.x,vec.y,vec.z);
+        }
+
+        GeomAPI_PointsToBSpline fit(pnts);
+        Handle_Geom_BSplineCurve spline = fit.Curve();
+        if (!spline.IsNull()) {
+            this->getGeomBSplineCurvePtr()->setHandle(spline);
+            Py_Return;
+        }
+        else {
+            Standard_Failure::Raise("failed to approximate points");
+            return 0; // goes to the catch block
+        }
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return 0;
+    }
+}
+
+PyObject* BSplineCurvePy::interpolate(PyObject *args)
+{
+    PyObject* obj;
+    double tol3d = 0.0001;
+    PyObject* closed = Py_False;
+    PyObject* t1=0; PyObject* t2=0;
+    if (!PyArg_ParseTuple(args, "O!|O!dO!O!",&(PyList_Type), &obj, &PyBool_Type, &closed, &tol3d,
+                                             &Base::VectorPy::Type, &t1, &Base::VectorPy::Type, &t2))
+        return 0;
+    try {
+        Py::List list(obj);
+        Handle_TColgp_HArray1OfPnt interpolationPoints = new TColgp_HArray1OfPnt(1, list.size());
+        Standard_Integer index = 1;
+        for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+            Py::Vector v(*it);
+            Base::Vector3d pnt = v.toVector();
+            interpolationPoints->SetValue(index++, gp_Pnt(pnt.x,pnt.y,pnt.z));
+        }
+
+        if (interpolationPoints->Length() < 2) {
+            Standard_Failure::Raise("not enough points given");
+        }
+
+        GeomAPI_Interpolate aBSplineInterpolation(interpolationPoints, (closed == Py_True), tol3d);
+        if (t1 && t2) {
+            Base::Vector3d v1 = Py::Vector(t1,false).toVector();
+            Base::Vector3d v2 = Py::Vector(t1,false).toVector();
+            gp_Vec initTangent(v1.x,v1.y,v1.z), finalTangent(v2.x,v2.y,v2.z);
+            aBSplineInterpolation.Load(initTangent, finalTangent);
+        }
+        aBSplineInterpolation.Perform();
+        if (aBSplineInterpolation.IsDone()) {
+            Handle_Geom_BSplineCurve aBSplineCurve(aBSplineInterpolation.Curve());
+            this->getGeomBSplineCurvePtr()->setHandle(aBSplineCurve);
+            Py_Return;
+        }
+        else {
+            Standard_Failure::Raise("failed to interpolate points");
+            return 0; // goes to the catch block
+        }
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return 0;
+    }
+}
+
+PyObject* BSplineCurvePy::buildFromPoles(PyObject *args)
+{
+    PyObject* obj;
+    int degree = 3;
+    PyObject* closed = Py_False;
+    if (!PyArg_ParseTuple(args, "O!|O!i",&(PyList_Type), &obj, &PyBool_Type, &closed, &degree))
+        return 0;
+    try {
+        Py::List list(obj);
+        TColgp_Array1OfPnt poles(1, list.size());
+        Standard_Integer index = 1;
+        for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+            Py::Vector v(*it);
+            Base::Vector3d pnt = v.toVector();
+            poles(index++) = gp_Pnt(pnt.x,pnt.y,pnt.z);
+        }
+
+        if (poles.Length() <= degree)
+             degree = poles.Length()-1;
+
+        TColStd_Array1OfReal knots(1, poles.Length()+degree+1-2*(degree));
+        TColStd_Array1OfInteger mults(1, poles.Length()+degree+1-2*(degree));
+        for (int i=1; i<=knots.Length(); i++){
+            knots.SetValue(i,(double)(i-1)/(knots.Length()-1));
+            mults.SetValue(i,1);
+        }
+        mults.SetValue(1, degree+1);
+        mults.SetValue(knots.Length(), degree+1);
+
+        Handle_Geom_BSplineCurve spline = new Geom_BSplineCurve(poles, knots, mults, degree, (closed == Py_True));
+        if (!spline.IsNull()) {
+            this->getGeomBSplineCurvePtr()->setHandle(spline);
+            Py_Return;
+        }
+        else {
+            Standard_Failure::Raise("failed to create spline");
+            return 0; // goes to the catch block
+        }
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return 0;
+    }
+}
+
+PyObject* BSplineCurvePy::getCustomAttributes(const char* /*attr*/) const
 {
     return 0;
 }
