@@ -46,6 +46,28 @@ protected:
     //@}
 };
 
+class MultiFuse : public Part::Feature
+{
+    PROPERTY_HEADER(Part::MultiFuse);
+
+public:
+    MultiFuse();
+
+    App::PropertyLinkList Shapes;
+
+    /** @name methods override feature */
+    //@{
+    /// recalculate the Feature
+    App::DocumentObjectExecReturn *execute(void);
+    short mustExecute() const;
+    //@}
+    /// returns the type name of the ViewProvider
+    const char* getViewProviderName(void) const {
+        return "PartGui::ViewProviderMultiFuse";
+    }
+
+};
+
 }
 
 #endif // PART_FEATUREPARTFUSE_H
