@@ -100,6 +100,11 @@ const TopoShape& PropertyPartShape::getShape() const
     return this->_Shape;
 }
 
+const Data::ComplexGeoData* PropertyPartShape::getComplexData() const
+{
+    return &(this->_Shape);
+}
+
 Base::BoundBox3d PropertyPartShape::getBoundingBox() const
 {
     Base::BoundBox3d box;
@@ -135,7 +140,9 @@ void PropertyPartShape::getFaces(std::vector<Base::Vector3d> &aPoints,
 
 void PropertyPartShape::transformGeometry(const Base::Matrix4D &rclTrf)
 {
-    setValue(_Shape.transformGeometry(rclTrf));
+    aboutToSetValue();
+    _Shape.transformGeometry(rclTrf);
+    hasSetValue();
 }
 
 PyObject *PropertyPartShape::getPyObject(void)
@@ -234,8 +241,9 @@ void PropertyPartShape::SaveDocFile (Base::Writer &writer) const
     // can be checked when reading in the data.
     if (_Shape._Shape.IsNull())
         return;
+    // NOTE: Cleaning the triangulation may cause problems on some algorithms like BOP
     // Before writing to the project we clean all triangulation data to save memory
-    BRepTools::Clean(_Shape._Shape);
+    // BRepTools::Clean(_Shape._Shape);
 
     // create a temporary file and copy the content to the zip stream
     // once the tmp. filename is known use always the same because otherwise
