@@ -26,6 +26,7 @@
 #ifndef _PreComp_
 # include <cstdlib>
 # include <sstream>
+# include <Bnd_Box.hxx>
 # include <BRep_Builder.hxx>
 # include <BRep_Tool.hxx>
 # include <BRepAdaptor_Curve.hxx>
@@ -34,29 +35,30 @@
 # include <BRepAlgoAPI_Cut.hxx>
 # include <BRepAlgoAPI_Fuse.hxx>
 # include <BRepAlgoAPI_Section.hxx>
+# include <BRepBndLib.hxx>
 # include <BRepBuilderAPI_GTransform.hxx>
 # include <BRepBuilderAPI_MakeFace.hxx>
 # include <BRepBuilderAPI_MakePolygon.hxx>
 # include <BRepBuilderAPI_MakeVertex.hxx>
 # include <BRepBuilderAPI_NurbsConvert.hxx>
-# include <BRepFilletAPI_MakeFillet.hxx>
-# include <BRepOffsetAPI_MakeThickSolid.hxx>
-# include <BRepOffsetAPI_MakePipe.hxx>
-# include <BRepOffsetAPI_MakePipeShell.hxx>
-# include <BRepPrimAPI_MakePrism.hxx>
-# include <BRepPrimAPI_MakeRevol.hxx>
+# include <BRepBuilderAPI_FaceError.hxx>
+# include <BRepBuilderAPI_Copy.hxx>
+# include <BRepBuilderAPI_Transform.hxx>
 # include <BRepCheck_Analyzer.hxx>
-# include <BRepBndLib.hxx>
+# include <BRepFilletAPI_MakeFillet.hxx>
 # include <BRepMesh.hxx>
 # include <BRepMesh_IncrementalMesh.hxx>
 # include <BRepMesh_Triangle.hxx>
 # include <BRepMesh_Edge.hxx>
-# include <Bnd_Box.hxx>
+# include <BRepOffsetAPI_MakeThickSolid.hxx>
+# include <BRepOffsetAPI_MakePipe.hxx>
+# include <BRepOffsetAPI_MakePipeShell.hxx>
+# include <BRepOffsetAPI_Sewing.hxx>
+# include <BRepPrimAPI_MakePrism.hxx>
+# include <BRepPrimAPI_MakeRevol.hxx>
 # include <BRepTools.hxx>
+# include <BRepTools_ReShape.hxx>
 # include <BRepTools_ShapeSet.hxx>
-# include <BRepBuilderAPI_FaceError.hxx>
-# include <BRepBuilderAPI_Copy.hxx>
-# include <BRepBuilderAPI_Transform.hxx>
 # include <Handle_TopTools_HSequenceOfShape.hxx>
 # include <TopTools_HSequenceOfShape.hxx>
 # include <Interface_Static.hxx>
@@ -95,6 +97,12 @@
 # include <StlAPI_Writer.hxx>
 # include <Standard_Failure.hxx>
 # include <gp_GTrsf.hxx>
+# include <ShapeAnalysis_Shell.hxx>
+# include <ShapeBuild_ReShape.hxx>
+# include <ShapeFix_Face.hxx>
+# include <ShapeFix_Shell.hxx>
+# include <ShapeFix_Solid.hxx>
+# include <ShapeUpgrade_ShellSewing.hxx>
 #endif
 # include <Poly_Polygon3D.hxx>
 # include <Poly_PolygonOnTriangulation.hxx>
@@ -110,6 +118,7 @@
 #include <Base/Exception.h>
 
 #include "TopoShape.h"
+#include "CrossSection.h"
 #include "TopoShapeFacePy.h"
 #include "TopoShapeEdgePy.h"
 #include "TopoShapeVertexPy.h"
@@ -131,8 +140,10 @@ const char* BRepBuilderAPI_FaceErrorText(BRepBuilderAPI_FaceError et)
         return "Curve projection failed";
     case BRepBuilderAPI_ParametersOutOfRange:
         return "Parameters out of range";
+#if OCC_HEX_VERSION < 0x060500
     case BRepBuilderAPI_SurfaceNotC2:
         return "Surface not C2-continous";
+#endif
     default:
         return "Unknown creation error";
     }
@@ -469,7 +480,7 @@ void TopoShape::exportStep(const char *filename) const
             throw Base::Exception("Error in transferring STEP");
 
         APIHeaderSection_MakeHeader makeHeader(aWriter.Model());
-        makeHeader.SetName(new TCollection_HAsciiString(filename));
+        makeHeader.SetName(new TCollection_HAsciiString((const Standard_CString)filename));
         makeHeader.SetAuthorValue (1, new TCollection_HAsciiString("FreeCAD"));
         makeHeader.SetOrganizationValue (1, new TCollection_HAsciiString("FreeCAD"));
         makeHeader.SetOriginatingSystem(new TCollection_HAsciiString("FreeCAD"));
@@ -872,6 +883,36 @@ TopoDS_Shape TopoShape::section(TopoDS_Shape shape) const
     return mkSection.Shape();
 }
 
+std::list<TopoDS_Wire> TopoShape::slice(const Base::Vector3d& dir, double d) const
+{
+    CrossSection cs(dir.x, dir.y, dir.z, this->_Shape);
+    return cs.section(d);
+}
+
+TopoDS_Compound TopoShape::slices(const Base::Vector3d& dir, const std::vector<double>& d) const
+{
+    std::vector< std::list<TopoDS_Wire> > wire_list;
+    CrossSection cs(dir.x, dir.y, dir.z, this->_Shape);
+    for (std::vector<double>::const_iterator jt = d.begin(); jt != d.end(); ++jt) {
+        wire_list.push_back(cs.section(*jt));
+    }
+
+    std::vector< std::list<TopoDS_Wire> >::const_iterator ft;
+    TopoDS_Compound comp;
+    BRep_Builder builder;
+    builder.MakeCompound(comp);
+
+    for (ft = wire_list.begin(); ft != wire_list.end(); ++ft) {
+        const std::list<TopoDS_Wire>& w = *ft;
+        for (std::list<TopoDS_Wire>::const_iterator wt = w.begin(); wt != w.end(); ++wt) {
+            if (!wt->IsNull())
+                builder.Add(comp, *wt);
+        }
+    }
+
+    return comp;
+}
+
 TopoDS_Shape TopoShape::makePipe(const TopoDS_Shape& profile) const
 {
     if (this->_Shape.IsNull())
@@ -926,7 +967,12 @@ TopoDS_Shape TopoShape::makeThickSolid(const TopTools_ListOfShape& remFace,
     return mkThick.Shape();
 }
 
-TopoDS_Shape TopoShape::transformGeometry(const Base::Matrix4D& rclTrf) const
+void TopoShape::transformGeometry(const Base::Matrix4D &rclMat)
+{
+    this->_Shape = transformGShape(rclMat);
+}
+
+TopoDS_Shape TopoShape::transformGShape(const Base::Matrix4D& rclTrf) const
 {
     // There is a strange behaviour of the gp_Trsf class if rclTrf has
     // a negative determinant.
@@ -996,24 +1042,85 @@ void TopoShape::transformShape(const Base::Matrix4D& rclTrf)
     this->_Shape = mkTrf.Shape();
 }
 
+TopoDS_Shape TopoShape::mirror(const gp_Ax2& ax2) const
+{
+    gp_Trsf mat;
+    mat.SetMirror(ax2);
+    TopLoc_Location loc = this->_Shape.Location();
+    gp_Trsf placement = loc.Transformation();
+    mat = placement * mat;
+    BRepBuilderAPI_Transform mkTrf(this->_Shape, mat);
+    return mkTrf.Shape();
+}
+
 TopoDS_Shape TopoShape::toNurbs() const
 {
     BRepBuilderAPI_NurbsConvert mkNurbs(this->_Shape);
     return mkNurbs.Shape();
 }
 
+TopoDS_Shape TopoShape::replaceShape(const std::vector< std::pair<TopoDS_Shape,TopoDS_Shape> >& s) const
+{
+    BRepTools_ReShape reshape;
+    std::vector< std::pair<TopoDS_Shape,TopoDS_Shape> >::const_iterator it;
+    for (it = s.begin(); it != s.end(); ++it)
+        reshape.Replace(it->first, it->second);
+    return reshape.Apply(this->_Shape, TopAbs_SHAPE);
+}
+
+TopoDS_Shape TopoShape::removeShape(const std::vector<TopoDS_Shape>& s) const
+{
+    BRepTools_ReShape reshape;
+    for (std::vector<TopoDS_Shape>::const_iterator it = s.begin(); it != s.end(); ++it)
+        reshape.Remove(*it);
+    return reshape.Apply(this->_Shape, TopAbs_SHAPE);
+}
+
 void TopoShape::sewShape()
 {
-    ShapeFix_Shape fixer(this->_Shape);
-    fixer.Perform();
+    //ShapeFix_Shape fixer(this->_Shape);
+    //fixer.Perform();
 
     BRepBuilderAPI_Sewing sew;
-    sew.Add(fixer.Shape());
+    sew.Add(this->_Shape/*fixer.Shape()*/);
     sew.Perform();
 
+    //shape = ShapeUpgrade_ShellSewing().ApplySewing(shape);
     this->_Shape = sew.SewedShape();
 }
 
+bool TopoShape::fix()
+{
+    bool ret = false;
+    if (this->_Shape.IsNull())
+        return false;
+    TopAbs_ShapeEnum type = this->_Shape.ShapeType();
+    if (type == TopAbs_SOLID) {
+        ShapeFix_Solid fix(TopoDS::Solid(this->_Shape));
+        ret = fix.Perform() ? true : false;
+        this->_Shape = fix.Shape();
+    }
+    else if (type == TopAbs_SHELL) {
+        //ShapeAnalysis_Shell eval;
+        //eval.LoadShells(this->_Shape);
+        ShapeFix_Shell fix(TopoDS::Shell(this->_Shape));
+        ret = fix.Perform() ? true : false;
+        this->_Shape = fix.Shape();
+    }
+    else if (type == TopAbs_FACE) {
+        ShapeFix_Face fix(TopoDS::Face(this->_Shape));
+        ret = fix.Perform() ? true : false;
+        this->_Shape = fix.Result();
+    }
+    else {
+        ShapeFix_Shape fix(this->_Shape);
+        ret = fix.Perform() ? true : false;
+        this->_Shape = fix.Shape();
+    }
+
+    return ret;
+}
+
 namespace Part {
 struct Vertex
 {
