@@ -26,10 +26,13 @@
 
 #include <iostream>
 #include <gp_Vec.hxx>
+#include <TopoDS_Compound.hxx>
 #include <TopoDS_Wire.hxx>
 #include <TopTools_ListOfShape.hxx>
 #include <App/ComplexGeoData.h>
 
+class gp_Ax2;
+
 namespace Part
 {
 
@@ -124,6 +127,8 @@ public:
     TopoDS_Shape common(TopoDS_Shape) const;
     TopoDS_Shape fuse(TopoDS_Shape) const;
     TopoDS_Shape section(TopoDS_Shape) const;
+    std::list<TopoDS_Wire> slice(const Base::Vector3d&, double) const;
+    TopoDS_Compound slices(const Base::Vector3d&, const std::vector<double>&) const;
     //@}
 
     /** Sweeping */
@@ -138,10 +143,15 @@ public:
 
     /** @name Manipulation*/
     //@{
-    TopoDS_Shape transformGeometry(const Base::Matrix4D&) const;
+    void transformGeometry(const Base::Matrix4D &rclMat);
+    TopoDS_Shape transformGShape(const Base::Matrix4D&) const;
     void transformShape(const Base::Matrix4D&);
+    TopoDS_Shape mirror(const gp_Ax2&) const;
     TopoDS_Shape toNurbs() const;
+    TopoDS_Shape replaceShape(const std::vector< std::pair<TopoDS_Shape,TopoDS_Shape> >& s) const;
+    TopoDS_Shape removeShape(const std::vector<TopoDS_Shape>& s) const;
     void sewShape();
+    bool fix();
     //@}
 
     /** @name Getting basic geometric entities */
