@@ -29,11 +29,15 @@
 # include <BRepCheck_Analyzer.hxx>
 # include <BRepCheck_ListIteratorOfListOfStatus.hxx>
 # include <BRepCheck_Result.hxx>
+# include <BRepClass3d_SolidClassifier.hxx>
 # include <BRepFilletAPI_MakeFillet.hxx>
 # include <BRepOffsetAPI_MakePipe.hxx>
 # include <BRepOffsetAPI_MakePipeShell.hxx>
 # include <BRepTools.hxx>
 # include <gp_Ax1.hxx>
+# include <gp_Ax2.hxx>
+# include <gp_Dir.hxx>
+# include <gp_Pnt.hxx>
 # include <gp_Trsf.hxx>
 # include <TopExp_Explorer.hxx>
 # include <TopoDS.hxx>
@@ -54,6 +58,7 @@
 #include <Base/MatrixPy.h>
 #include <Base/Vector3D.h>
 #include <Base/VectorPy.h>
+#include <CXX/Extensions.hxx>
 
 #include "TopoShape.h"
 #include "TopoShapePy.h"
@@ -79,6 +84,14 @@ using namespace Part;
     #define M_PI_2  1.57079632679489661923 /* pi/2 */
 #endif
 
+namespace Py {
+    typedef ExtensionObject<TopoShapePy> TopoShape;
+    template<>
+    bool TopoShape::accepts (PyObject *pyob) const
+    {
+        return (pyob && PyObject_TypeCheck(pyob, &(Part::TopoShapePy::Type)));
+    }
+}
 
 // returns a string which represents the object e.g. when printed in python
 std::string TopoShapePy::representation(void) const
@@ -132,6 +145,87 @@ int TopoShapePy::PyInit(PyObject* args, PyObject*)
     return 0;
 }
 
+PyObject* TopoShapePy::copy(PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+
+    const TopoDS_Shape& shape = this->getTopoShapePtr()->_Shape;
+    PyTypeObject* type = this->GetType();
+    PyObject* cpy = 0;
+    // let the type object decide
+    if (type->tp_new)
+        cpy = type->tp_new(type, this, 0);
+    if (!cpy) {
+        PyErr_SetString(PyExc_TypeError, "failed to create copy of shape");
+        return 0;
+    }
+
+    static_cast<TopoShapePy*>(cpy)->getTopoShapePtr()->_Shape = shape;
+    return cpy;
+}
+
+PyObject* TopoShapePy::replaceShape(PyObject *args)
+{
+    PyObject *l;
+    if (!PyArg_ParseTuple(args, "O!",&PyList_Type,&l))
+        return NULL;
+
+    try {
+        Py::List list(l);
+        std::vector< std::pair<TopoDS_Shape, TopoDS_Shape> > shapes;
+        for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+            Py::Tuple tuple(*it);
+            Py::TopoShape sh1(tuple[0]);
+            Py::TopoShape sh2(tuple[1]);
+            shapes.push_back(std::make_pair(
+                sh1.extensionObject()->getTopoShapePtr()->_Shape,
+                sh2.extensionObject()->getTopoShapePtr()->_Shape)
+            );
+        }
+        PyTypeObject* type = this->GetType();
+        PyObject* inst = type->tp_new(type, this, 0);
+        static_cast<TopoShapePy*>(inst)->getTopoShapePtr()->_Shape = 
+            this->getTopoShapePtr()->replaceShape(shapes);
+        return inst;
+    }
+    catch (const Py::Exception&) {
+        return 0;
+    }
+    catch (...) {
+        PyErr_SetString(PyExc_Exception, "failed to replace shape");
+        return 0;
+    }
+}
+
+PyObject* TopoShapePy::removeShape(PyObject *args)
+{
+    PyObject *l;
+    if (!PyArg_ParseTuple(args, "O!",&PyList_Type,&l))
+        return NULL;
+
+    try {
+        Py::List list(l);
+        std::vector<TopoDS_Shape> shapes;
+        for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+            Py::Tuple tuple(*it);
+            Py::TopoShape sh(tuple[0]);
+            shapes.push_back(
+                sh.extensionObject()->getTopoShapePtr()->_Shape
+            );
+        }
+        PyTypeObject* type = this->GetType();
+        PyObject* inst = type->tp_new(type, this, 0);
+        static_cast<TopoShapePy*>(inst)->getTopoShapePtr()->_Shape = 
+            this->getTopoShapePtr()->removeShape(shapes);
+        return inst;
+    }
+    catch (...) {
+        PyErr_SetString(PyExc_Exception, "failed to remove shape");
+        return 0;
+    }
+}
+
 PyObject*  TopoShapePy::read(PyObject *args)
 {
     char* filename;
@@ -159,7 +253,8 @@ PyObject* TopoShapePy::writeInventor(PyObject * args)
         getTopoShapePtr()->exportFaceSet(dev, angle, result);
         getTopoShapePtr()->exportLineSet(result);
     }
-    BRepTools::Clean(getTopoShapePtr()->_Shape); // remove triangulation
+    // NOTE: Cleaning the triangulation may cause problems on some algorithms like BOP
+    //BRepTools::Clean(getTopoShapePtr()->_Shape); // remove triangulation
     return Py::new_reference_to(Py::String(result.str()));
 }
 
@@ -246,7 +341,7 @@ PyObject* TopoShapePy::extrude(PyObject *args)
             switch (type)
             {
             case TopAbs_COMPOUND:
-                break;
+                return new TopoShapeCompoundPy(new TopoShape(shape));
             case TopAbs_COMPSOLID:
                 return new TopoShapeCompSolidPy(new TopoShape(shape));
             case TopAbs_SOLID:
@@ -286,6 +381,24 @@ PyObject* TopoShapePy::revolve(PyObject *args)
     double d=360;
     if (PyArg_ParseTuple(args, "O!O!|d", &(Base::VectorPy::Type), &pPos, &(Base::VectorPy::Type), &pDir,&d)) {
         try {
+            const TopoDS_Shape& input = this->getTopoShapePtr()->_Shape;
+            if (input.IsNull()) {
+                PyErr_SetString(PyExc_Exception, "empty shape cannot be revolved");
+                return 0;
+            }
+
+            TopExp_Explorer xp;
+            xp.Init(input,TopAbs_SOLID);
+            if (xp.More()) {
+                PyErr_SetString(PyExc_Exception, "shape must not contain solids");
+                return 0;
+            }
+            xp.Init(input,TopAbs_COMPSOLID);
+            if (xp.More()) {
+                PyErr_SetString(PyExc_Exception, "shape must not contain compound solids");
+                return 0;
+            }
+
             Base::Vector3d pos = static_cast<Base::VectorPy*>(pPos)->value();
             Base::Vector3d dir = static_cast<Base::VectorPy*>(pDir)->value();
             TopoDS_Shape shape = this->getTopoShapePtr()->revolve(
@@ -294,7 +407,7 @@ PyObject* TopoShapePy::revolve(PyObject *args)
             switch (type)
             {
             case TopAbs_COMPOUND:
-                break;
+                return new TopoShapeCompoundPy(new TopoShape(shape));
             case TopAbs_COMPSOLID:
                 return new TopoShapeCompSolidPy(new TopoShape(shape));
             case TopAbs_SOLID:
@@ -479,6 +592,10 @@ PyObject*  TopoShapePy::fuse(PyObject *args)
         PyErr_SetString(PyExc_Exception, e->GetMessageString());
         return NULL;
     }
+    catch (const std::exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
 }
 
 PyObject*  TopoShapePy::common(PyObject *args)
@@ -498,6 +615,10 @@ PyObject*  TopoShapePy::common(PyObject *args)
         PyErr_SetString(PyExc_Exception, e->GetMessageString());
         return NULL;
     }
+    catch (const std::exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
 }
 
 PyObject*  TopoShapePy::section(PyObject *args)
@@ -517,6 +638,66 @@ PyObject*  TopoShapePy::section(PyObject *args)
         PyErr_SetString(PyExc_Exception, e->GetMessageString());
         return NULL;
     }
+    catch (const std::exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
+}
+
+PyObject*  TopoShapePy::slice(PyObject *args)
+{
+    PyObject *dir;
+    double d;
+    if (!PyArg_ParseTuple(args, "O!d", &(Base::VectorPy::Type), &dir, &d))
+        return NULL;
+
+    try {
+        Base::Vector3d vec = Py::Vector(dir, false).toVector();
+        std::list<TopoDS_Wire> slice = this->getTopoShapePtr()->slice(vec, d);
+        Py::List wire;
+        for (std::list<TopoDS_Wire>::iterator it = slice.begin(); it != slice.end(); ++it) {
+            wire.append(Py::asObject(new TopoShapeWirePy(new TopoShape(*it))));
+        }
+
+        return Py::new_reference_to(wire);
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return NULL;
+    }
+    catch (const std::exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
+}
+
+PyObject*  TopoShapePy::slices(PyObject *args)
+{
+    PyObject *dir, *dist;
+    if (!PyArg_ParseTuple(args, "O!O!", &(Base::VectorPy::Type), &dir,
+                                        &PyList_Type, &dist))
+        return NULL;
+
+    try {
+        Base::Vector3d vec = Py::Vector(dir, false).toVector();
+        Py::List list(dist);
+        std::vector<double> d;
+        d.reserve(list.size());
+        for (Py::List::iterator it = list.begin(); it != list.end(); ++it)
+            d.push_back((double)Py::Float(*it));
+        TopoDS_Compound slice = this->getTopoShapePtr()->slices(vec, d);
+        return new TopoShapeCompoundPy(new TopoShape(slice));
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return NULL;
+    }
+    catch (const std::exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
 }
 
 PyObject*  TopoShapePy::cut(PyObject *args)
@@ -536,6 +717,10 @@ PyObject*  TopoShapePy::cut(PyObject *args)
         PyErr_SetString(PyExc_Exception, e->GetMessageString());
         return NULL;
     }
+    catch (const std::exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
 }
 
 PyObject*  TopoShapePy::sewShape(PyObject *args)
@@ -554,6 +739,28 @@ PyObject*  TopoShapePy::sewShape(PyObject *args)
     }
 }
 
+PyObject*  TopoShapePy::mirror(PyObject *args)
+{
+    PyObject *v1, *v2;
+    if (!PyArg_ParseTuple(args, "O!O!", &(Base::VectorPy::Type),&v1,
+                                        &(Base::VectorPy::Type),&v2))
+        return NULL;
+
+    Base::Vector3d base = Py::Vector(v1,false).toVector();
+    Base::Vector3d norm = Py::Vector(v2,false).toVector();
+
+    try {
+        gp_Ax2 ax2(gp_Pnt(base.x,base.y,base.z), gp_Dir(norm.x,norm.y,norm.z));
+        TopoDS_Shape shape = this->getTopoShapePtr()->mirror(ax2);
+        return new TopoShapePy(new TopoShape(shape));
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return NULL;
+    }
+}
+
 PyObject*  TopoShapePy::transformGeometry(PyObject *args)
 {
     PyObject *obj;
@@ -562,7 +769,7 @@ PyObject*  TopoShapePy::transformGeometry(PyObject *args)
 
     Base::Matrix4D mat = static_cast<Base::MatrixPy*>(obj)->value();
     try {
-        TopoDS_Shape shape = this->getTopoShapePtr()->transformGeometry(mat);
+        TopoDS_Shape shape = this->getTopoShapePtr()->transformGShape(mat);
         return new TopoShapePy(new TopoShape(shape));
     }
     catch (Standard_Failure) {
@@ -623,7 +830,8 @@ PyObject*  TopoShapePy::rotate(PyObject *args)
         return NULL;
 
     try {
-        Py::Tuple p1(obj1), p2(obj2);
+        // Vector also supports sequence
+        Py::Sequence p1(obj1), p2(obj2);
         // Convert into OCC representation
         gp_Pnt pos = gp_Pnt((double)Py::Float(p1[0]),
                             (double)Py::Float(p1[1]),
@@ -737,53 +945,6 @@ PyObject* TopoShapePy::makeFillet(PyObject *args)
     return NULL;
 }
 
-PyObject* TopoShapePy::makePipe(PyObject *args)
-{
-    PyObject *pShape;
-    if (PyArg_ParseTuple(args, "O!", &(Part::TopoShapePy::Type), &pShape)) {
-        try {
-            TopoDS_Shape profile = static_cast<TopoShapePy*>(pShape)->getTopoShapePtr()->_Shape;
-            TopoDS_Shape shape = this->getTopoShapePtr()->makePipe(profile);
-            return new TopoShapePy(new TopoShape(shape));
-        }
-        catch (Standard_Failure) {
-            Handle_Standard_Failure e = Standard_Failure::Caught();
-            PyErr_SetString(PyExc_Exception, e->GetMessageString());
-            return 0;
-        }
-    }
-
-    return 0;
-}
-
-PyObject* TopoShapePy::makePipeShell(PyObject *args)
-{
-    PyObject *obj;
-    int make_solid = 0;
-
-    if (PyArg_ParseTuple(args, "O!|i", &(PyList_Type), &obj, &make_solid)) {
-        try {
-            TopTools_ListOfShape sections;
-            Py::List list(obj);
-            for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
-                if (PyObject_TypeCheck((*it).ptr(), &(Part::TopoShapePy::Type))) {
-                    const TopoDS_Shape& shape = static_cast<TopoShapePy*>((*it).ptr())->getTopoShapePtr()->_Shape;
-                    sections.Append(shape);
-                }
-            }
-            TopoDS_Shape shape = this->getTopoShapePtr()->makePipeShell(sections, make_solid);
-            return new TopoShapePy(new TopoShape(shape));
-        }
-        catch (Standard_Failure) {
-            Handle_Standard_Failure e = Standard_Failure::Caught();
-            PyErr_SetString(PyExc_Exception, e->GetMessageString());
-            return NULL;
-        }
-    }
-
-    return 0;
-}
-
 PyObject* TopoShapePy::makeThickness(PyObject *args)
 {
     PyObject *obj;
@@ -905,29 +1066,36 @@ PyObject* TopoShapePy::hashCode(PyObject *args)
 
 PyObject* TopoShapePy::tessellate(PyObject *args)
 {
-    float tolerance;
-    if (!PyArg_ParseTuple(args, "f",&tolerance))
-        return 0;
-    std::vector<Base::Vector3d> Points;
-    std::vector<Data::ComplexGeoData::FacetTopo> Facets;
-    getTopoShapePtr()->getFaces(Points, Facets,tolerance);
-    Py::Tuple tuple(2);
-    Py::List vertex;
-    for (std::vector<Base::Vector3d>::const_iterator it = Points.begin();
-        it != Points.end(); ++it)
-        vertex.append(Py::Object(new Base::VectorPy(*it)));
-    tuple.setItem(0, vertex);
-    Py::List facet;
-    for (std::vector<Data::ComplexGeoData::FacetTopo>::const_iterator
-        it = Facets.begin(); it != Facets.end(); ++it) {
-        Py::Tuple f(3);
-        f.setItem(0,Py::Int((int)it->I1));
-        f.setItem(1,Py::Int((int)it->I2));
-        f.setItem(2,Py::Int((int)it->I3));
-        facet.append(f);
+    try {
+        float tolerance;
+        if (!PyArg_ParseTuple(args, "f",&tolerance))
+            return 0;
+        std::vector<Base::Vector3d> Points;
+        std::vector<Data::ComplexGeoData::FacetTopo> Facets;
+        getTopoShapePtr()->getFaces(Points, Facets,tolerance);
+        Py::Tuple tuple(2);
+        Py::List vertex;
+        for (std::vector<Base::Vector3d>::const_iterator it = Points.begin();
+            it != Points.end(); ++it)
+            vertex.append(Py::Object(new Base::VectorPy(*it)));
+        tuple.setItem(0, vertex);
+        Py::List facet;
+        for (std::vector<Data::ComplexGeoData::FacetTopo>::const_iterator
+            it = Facets.begin(); it != Facets.end(); ++it) {
+            Py::Tuple f(3);
+            f.setItem(0,Py::Int((int)it->I1));
+            f.setItem(1,Py::Int((int)it->I2));
+            f.setItem(2,Py::Int((int)it->I3));
+            facet.append(f);
+        }
+        tuple.setItem(1, facet);
+        return Py::new_reference_to(tuple);
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return NULL;
     }
-    tuple.setItem(1, facet);
-    return Py::new_reference_to(tuple);
 }
 
 PyObject* TopoShapePy::project(PyObject *args)
@@ -1014,6 +1182,36 @@ PyObject* TopoShapePy::toNurbs(PyObject *args)
     }
 }
 
+PyObject*  TopoShapePy::isInside(PyObject *args)
+{
+    PyObject *point;
+    double tolerance;
+    PyObject* checkFace = Py_False;
+    TopAbs_State stateIn = TopAbs_IN;
+    if (!PyArg_ParseTuple(args, "O!dO!", &(Base::VectorPy::Type), &point, &tolerance,  &PyBool_Type, &checkFace))
+        return NULL;
+    try {
+        TopoDS_Shape shape = getTopoShapePtr()->_Shape;
+        BRepClass3d_SolidClassifier solidClassifier(shape);
+        Base::Vector3d pnt = static_cast<Base::VectorPy*>(point)->value();
+        gp_Pnt vertex = gp_Pnt(pnt.x,pnt.y,pnt.z);
+        solidClassifier.Perform(vertex, tolerance);
+        Standard_Boolean test = (solidClassifier.State() == stateIn);
+        if ( (checkFace == Py_True) && (solidClassifier.IsOnAFace()) )
+            test = Standard_True;
+        return Py_BuildValue("O", (test ? Py_True : Py_False));
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return NULL;
+    }
+    catch (const std::exception& e) {
+        PyErr_SetString(PyExc_Exception, e.what());
+        return NULL;
+    }
+}
+
 #if 0 // see ComplexGeoDataPy::Matrix which does the same
 Py::Object TopoShapePy::getLocation(void) const
 {
