@@ -23,7 +23,9 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <Approx_Curve3d.hxx>
 # include <ShapeAlgo_AlgoContainer.hxx>
+# include <BRepAdaptor_CompCurve.hxx>
 # include <BRepBuilderAPI_MakeWire.hxx>
 # include <BRepOffsetAPI_MakeOffset.hxx>
 # include <TopoDS.hxx>
@@ -138,6 +140,53 @@ PyObject* TopoShapeWirePy::makeOffset(PyObject *args)
     return new TopoShapePy(new TopoShape(mkOffset.Shape()));
 }
 
+PyObject* TopoShapeWirePy::makePipe(PyObject *args)
+{
+    PyObject *pShape;
+    if (PyArg_ParseTuple(args, "O!", &(Part::TopoShapePy::Type), &pShape)) {
+        try {
+            TopoDS_Shape profile = static_cast<TopoShapePy*>(pShape)->getTopoShapePtr()->_Shape;
+            TopoDS_Shape shape = this->getTopoShapePtr()->makePipe(profile);
+            return new TopoShapePy(new TopoShape(shape));
+        }
+        catch (Standard_Failure) {
+            Handle_Standard_Failure e = Standard_Failure::Caught();
+            PyErr_SetString(PyExc_Exception, e->GetMessageString());
+            return 0;
+        }
+    }
+
+    return 0;
+}
+
+PyObject* TopoShapeWirePy::makePipeShell(PyObject *args)
+{
+    PyObject *obj;
+    int make_solid = 0;
+
+    if (PyArg_ParseTuple(args, "O!|i", &(PyList_Type), &obj, &make_solid)) {
+        try {
+            TopTools_ListOfShape sections;
+            Py::List list(obj);
+            for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+                if (PyObject_TypeCheck((*it).ptr(), &(Part::TopoShapePy::Type))) {
+                    const TopoDS_Shape& shape = static_cast<TopoShapePy*>((*it).ptr())->getTopoShapePtr()->_Shape;
+                    sections.Append(shape);
+                }
+            }
+            TopoDS_Shape shape = this->getTopoShapePtr()->makePipeShell(sections, make_solid);
+            return new TopoShapePy(new TopoShape(shape));
+        }
+        catch (Standard_Failure) {
+            Handle_Standard_Failure e = Standard_Failure::Caught();
+            PyErr_SetString(PyExc_Exception, e->GetMessageString());
+            return NULL;
+        }
+    }
+
+    return 0;
+}
+
 PyObject* TopoShapeWirePy::makeHomogenousWires(PyObject *args)
 {
     PyObject* wire;
@@ -164,6 +213,33 @@ PyObject* TopoShapeWirePy::makeHomogenousWires(PyObject *args)
     }
 }
 
+PyObject* TopoShapeWirePy::approximate(PyObject *args)
+{
+    double tol2d = gp::Resolution();
+    double tol3d = 0.0001;
+    int maxseg=10, maxdeg=3;
+    if (!PyArg_ParseTuple(args, "ddii",&tol2d,&tol3d,&maxseg,&maxdeg))
+        return 0;
+    try {
+        BRepAdaptor_CompCurve adapt(TopoDS::Wire(getTopoShapePtr()->_Shape));
+        Handle_Adaptor3d_HCurve hcurve = adapt.Trim(adapt.FirstParameter(),
+                                                    adapt.LastParameter(),
+                                                    tol2d);
+        Approx_Curve3d approx(hcurve, tol3d, GeomAbs_C0, maxseg, maxdeg);
+        if (approx.IsDone()) {
+            return new BSplineCurvePy(new GeomBSplineCurve(approx.Curve()));
+        }
+        else {
+            PyErr_SetString(PyExc_Exception, "failed to approximate wire");
+            return 0;
+        }
+    }
+    catch (Standard_Failure) {
+        PyErr_SetString(PyExc_Exception, "failed to approximate wire");
+        return 0;
+    }
+}
+
 Py::Object TopoShapeWirePy::getCenterOfMass(void) const
 {
     GProp_GProps props;
