@@ -32,6 +32,8 @@
 #include "ViewProviderImport.h"
 #include "ViewProviderExtrusion.h"
 #include "ViewProvider2DObject.h"
+#include "ViewProviderMirror.h"
+#include "ViewProviderBoolean.h"
 
 #include "DlgSettingsGeneral.h"
 #include "DlgSettings3DViewPartImp.h"
@@ -70,7 +72,7 @@ void AppPartGuiExport initPartGui()
 
     // load needed modules
     try {
-        Base::Interpreter().loadModule("Part");
+        Base::Interpreter().runString("import Part");
     }
     catch(const Base::Exception& e) {
         PyErr_SetString(PyExc_ImportError, e.what());
@@ -89,19 +91,25 @@ void AppPartGuiExport initPartGui()
     PartGui::ViewProviderCurveNet       ::init();
     PartGui::ViewProviderExtrusion      ::init();
     PartGui::ViewProvider2DObject       ::init();
+    PartGui::ViewProviderMirror         ::init();
+    PartGui::ViewProviderFillet         ::init();
+    PartGui::ViewProviderChamfer        ::init();
     PartGui::ViewProviderCustom         ::init();
     PartGui::ViewProviderCustomPython   ::init();
+    PartGui::ViewProviderBoolean        ::init();
+    PartGui::ViewProviderMultiFuse      ::init();
+    PartGui::ViewProviderMultiCommon    ::init();
 
     PartGui::Workbench                  ::init();
 
-    // instanciating the commands
+    // instantiating the commands
     CreatePartCommands();
     CreateSimplePartCommands();
     CreateParamPartCommands();
 
     // register preferences pages
-    (void)new Gui::PrefPageProducer<PartGui::DlgSettingsGeneral>      ("Part design");
-    (void)new Gui::PrefPageProducer<PartGui::DlgSettings3DViewPart>   ("Part design");
+    (void)new Gui::PrefPageProducer<PartGui::DlgSettingsGeneral>      ( QT_TRANSLATE_NOOP("QObject","Part design") );
+    (void)new Gui::PrefPageProducer<PartGui::DlgSettings3DViewPart>   ( QT_TRANSLATE_NOOP("QObject","Part design") );
     Gui::ViewProviderBuilder::add(
         Part::PropertyPartShape::getClassTypeId(),
         PartGui::ViewProviderPart::getClassTypeId());
