@@ -54,6 +54,8 @@
 #include "DlgRevolution.h"
 #include "DlgFilletEdges.h"
 #include "DlgPrimitives.h"
+#include "CrossSections.h"
+#include "Mirroring.h"
 #include "ViewProvider.h"
 
 
@@ -117,7 +119,7 @@ CmdPartBox2::CmdPartBox2()
     sAppModule    = "Part";
     sGroup        = QT_TR_NOOP("Part");
     sMenuText     = QT_TR_NOOP("Box fix 1");
-    sToolTipText  = QT_TR_NOOP("Create a box feature without dialog");
+    sToolTipText  = QT_TR_NOOP("Create a box solid without dialog");
     sWhatsThis    = "Part_Box2";
     sStatusTip    = sToolTipText;
     sPixmap       = "Part_Box";
@@ -159,7 +161,7 @@ CmdPartBox3::CmdPartBox3()
     sAppModule    = "Part";
     sGroup        = QT_TR_NOOP("Part");
     sMenuText     = QT_TR_NOOP("Box fix 2");
-    sToolTipText  = QT_TR_NOOP("Create a box feature without dialog");
+    sToolTipText  = QT_TR_NOOP("Create a box solid without dialog");
     sWhatsThis    = "Part_Box3";
     sStatusTip    = sToolTipText;
     sPixmap       = "Part_Box";
@@ -232,14 +234,13 @@ CmdPartCut::CmdPartCut()
     sAppModule    = "Part";
     sGroup        = QT_TR_NOOP("Part");
     sMenuText     = QT_TR_NOOP("Cut");
-    sToolTipText  = QT_TR_NOOP("Create a Cut feature");
+    sToolTipText  = QT_TR_NOOP("Make a cut of two shapes");
     sWhatsThis    = "Part_Cut";
     sStatusTip    = sToolTipText;
     sPixmap       = "Part_Cut";
     iAccel        = 0;
 }
 
-
 void CmdPartCut::activated(int iMsg)
 {
     unsigned int n = getSelection().countObjectsOfType(Part::Feature::getClassTypeId());
@@ -249,16 +250,18 @@ void CmdPartCut::activated(int iMsg)
         return;
     }
 
-    std::string FeatName = getUniqueObjectName("Cut");
-
     std::vector<Gui::SelectionSingleton::SelObj> Sel = getSelection().getSelection();
 
+    std::string FeatName = getUniqueObjectName("Cut");
+    std::string BaseName  = Sel[0].FeatName;
+    std::string ToolName  = Sel[1].FeatName;
+
     openCommand("Part Cut");
     doCommand(Doc,"App.activeDocument().addObject(\"Part::Cut\",\"%s\")",FeatName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Base = App.activeDocument().%s",FeatName.c_str(),Sel[0].FeatName);
-    doCommand(Doc,"App.activeDocument().%s.Tool = App.activeDocument().%s",FeatName.c_str(),Sel[1].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[0].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[1].FeatName);
+    doCommand(Doc,"App.activeDocument().%s.Base = App.activeDocument().%s",FeatName.c_str(),BaseName.c_str());
+    doCommand(Doc,"App.activeDocument().%s.Tool = App.activeDocument().%s",FeatName.c_str(),ToolName.c_str());
+    doCommand(Gui,"Gui.activeDocument().hide('%s')",BaseName.c_str());
+    doCommand(Gui,"Gui.activeDocument().hide('%s')",ToolName.c_str());
     updateActive();
     commitCommand();
 }
@@ -279,7 +282,7 @@ CmdPartCommon::CmdPartCommon()
     sAppModule    = "Part";
     sGroup        = QT_TR_NOOP("Part");
     sMenuText     = QT_TR_NOOP("Intersection");
-    sToolTipText  = QT_TR_NOOP("Intersection of two shapes");
+    sToolTipText  = QT_TR_NOOP("Make an intersection of two shapes");
     sWhatsThis    = "Part_Common";
     sStatusTip    = sToolTipText;
     sPixmap       = "Part_Common";
@@ -289,29 +292,35 @@ CmdPartCommon::CmdPartCommon()
 void CmdPartCommon::activated(int iMsg)
 {
     unsigned int n = getSelection().countObjectsOfType(Part::Feature::getClassTypeId());
-    if (n != 2) {
+    if (n < 2) {
         QMessageBox::warning(Gui::getMainWindow(), QObject::tr("Wrong selection"),
-            QObject::tr("Select two shapes please."));
+            QObject::tr("Select two shapes or more, please."));
         return;
     }
 
     std::string FeatName = getUniqueObjectName("Common");
 
     std::vector<Gui::SelectionSingleton::SelObj> Sel = getSelection().getSelection();
+    std::string ObjectBuf;
+    std::vector<std::string> tempSelNames;
+    for (std::vector<Gui::SelectionSingleton::SelObj>::iterator it = Sel.begin(); it != Sel.end(); ++it){
+        ObjectBuf += std::string("App.activeDocument().") + it->FeatName + ",";
+        tempSelNames.push_back(it->FeatName);
+    }
+    ObjectBuf.erase(--ObjectBuf.end());
 
     openCommand("Common");
-    doCommand(Doc,"App.activeDocument().addObject(\"Part::Common\",\"%s\")",FeatName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Base = App.activeDocument().%s",FeatName.c_str(),Sel[0].FeatName);
-    doCommand(Doc,"App.activeDocument().%s.Tool = App.activeDocument().%s",FeatName.c_str(),Sel[1].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[0].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[1].FeatName);
+    doCommand(Doc,"App.activeDocument().addObject(\"Part::MultiCommon\",\"%s\")",FeatName.c_str());
+    doCommand(Doc,"App.activeDocument().%s.Shapes = [%s]",FeatName.c_str(),ObjectBuf.c_str());
+    for (std::vector<std::string>::iterator it = tempSelNames.begin(); it != tempSelNames.end(); ++it)
+        doCommand(Gui,"Gui.activeDocument().%s.Visibility=False",it->c_str());
     updateActive();
     commitCommand();
 }
 
 bool CmdPartCommon::isActive(void)
 {
-    return getSelection().countObjectsOfType(Part::Feature::getClassTypeId())==2;
+    return getSelection().countObjectsOfType(Part::Feature::getClassTypeId())>=2;
 }
 
 //===========================================================================
@@ -325,40 +334,45 @@ CmdPartFuse::CmdPartFuse()
     sAppModule    = "Part";
     sGroup        = QT_TR_NOOP("Part");
     sMenuText     = QT_TR_NOOP("Union");
-    sToolTipText  = QT_TR_NOOP("Make union of two shapes");
+    sToolTipText  = QT_TR_NOOP("Make a union of several shapes");
     sWhatsThis    = "Part_Fuse";
     sStatusTip    = sToolTipText;
     sPixmap       = "Part_Fuse";
     iAccel        = 0;
 }
 
-
 void CmdPartFuse::activated(int iMsg)
 {
     unsigned int n = getSelection().countObjectsOfType(Part::Feature::getClassTypeId());
-    if (n != 2) {
+    if (n < 2) {
         QMessageBox::warning(Gui::getMainWindow(), QObject::tr("Wrong selection"),
-            QObject::tr("Select two shapes please."));
+            QObject::tr("Select two shapes or more, please."));
         return;
     }
 
     std::string FeatName = getUniqueObjectName("Fusion");
 
     std::vector<Gui::SelectionSingleton::SelObj> Sel = getSelection().getSelection();
+    std::string ObjectBuf;
+    std::vector<std::string> tempSelNames;
+    for (std::vector<Gui::SelectionSingleton::SelObj>::iterator it = Sel.begin(); it != Sel.end(); ++it){
+        ObjectBuf += std::string("App.activeDocument().") + it->FeatName + ",";
+        tempSelNames.push_back(it->FeatName);
+    }
+    ObjectBuf.erase(--ObjectBuf.end());
 
     openCommand("Fusion");
-    doCommand(Doc,"App.activeDocument().addObject(\"Part::Fuse\",\"%s\")",FeatName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Base = App.activeDocument().%s",FeatName.c_str(),Sel[0].FeatName);
-    doCommand(Doc,"App.activeDocument().%s.Tool = App.activeDocument().%s",FeatName.c_str(),Sel[1].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[0].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[1].FeatName);
+    doCommand(Doc,"App.activeDocument().addObject(\"Part::MultiFuse\",\"%s\")",FeatName.c_str());
+    doCommand(Doc,"App.activeDocument().%s.Shapes = [%s]",FeatName.c_str(),ObjectBuf.c_str());
+    for (std::vector<std::string>::iterator it = tempSelNames.begin(); it != tempSelNames.end(); ++it)
+        doCommand(Gui,"Gui.activeDocument().%s.Visibility=False",it->c_str());
     updateActive();
     commitCommand();
 }
 
 bool CmdPartFuse::isActive(void)
 {
-    return getSelection().countObjectsOfType(Part::Feature::getClassTypeId())==2;
+    return getSelection().countObjectsOfType(Part::Feature::getClassTypeId())>=2;
 }
 
 //===========================================================================
@@ -372,7 +386,7 @@ CmdPartSection::CmdPartSection()
     sAppModule    = "Part";
     sGroup        = QT_TR_NOOP("Part");
     sMenuText     = QT_TR_NOOP("Section");
-    sToolTipText  = QT_TR_NOOP("Make section of two shapes");
+    sToolTipText  = QT_TR_NOOP("Make a section of two shapes");
     sWhatsThis    = "Part_Section";
     sStatusTip    = sToolTipText;
     sPixmap       = "Part_Section";
@@ -397,8 +411,8 @@ void CmdPartSection::activated(int iMsg)
     doCommand(Doc,"App.activeDocument().addObject(\"Part::Section\",\"%s\")",FeatName.c_str());
     doCommand(Doc,"App.activeDocument().%s.Base = App.activeDocument().%s",FeatName.c_str(),Sel[0].FeatName);
     doCommand(Doc,"App.activeDocument().%s.Tool = App.activeDocument().%s",FeatName.c_str(),Sel[1].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[0].FeatName);
-    doCommand(Gui,"Gui.activeDocument().hide(\"%s\")",Sel[1].FeatName);
+    doCommand(Gui,"Gui.activeDocument().hide('%s')",Sel[0].FeatName);
+    doCommand(Gui,"Gui.activeDocument().hide('%s')",Sel[1].FeatName);
     updateActive();
     commitCommand();
 }
@@ -740,7 +754,7 @@ CmdPartFillet::CmdPartFillet()
 
 void CmdPartFillet::activated(int iMsg)
 {
-    Gui::Control().showDialog(new PartGui::TaskFilletEdges());
+    Gui::Control().showDialog(new PartGui::TaskFilletEdges(0));
 }
 
 bool CmdPartFillet::isActive(void)
@@ -748,6 +762,73 @@ bool CmdPartFillet::isActive(void)
     return (hasActiveDocument() && !Gui::Control().activeDialog());
 }
 
+//===========================================================================
+// Part_Mirror
+//===========================================================================
+DEF_STD_CMD_A(CmdPartMirror);
+
+CmdPartMirror::CmdPartMirror()
+  :Command("Part_Mirror")
+{
+    sAppModule    = "Part";
+    sGroup        = QT_TR_NOOP("Part");
+    sMenuText     = QT_TR_NOOP("Mirroring...");
+    sToolTipText  = QT_TR_NOOP("Mirroring a selected shape");
+    sWhatsThis    = sToolTipText;
+    sStatusTip    = sToolTipText;
+    sPixmap       = "Part_MirrorPNG";
+    iAccel        = 0;
+}
+
+void CmdPartMirror::activated(int iMsg)
+{
+    Gui::Control().showDialog(new PartGui::TaskMirroring());
+}
+
+bool CmdPartMirror::isActive(void)
+{
+    return (hasActiveDocument() && !Gui::Control().activeDialog());
+}
+
+//===========================================================================
+// Part_CrossSections
+//===========================================================================
+DEF_STD_CMD_A(CmdPartCrossSections);
+
+CmdPartCrossSections::CmdPartCrossSections()
+  :Command("Part_CrossSections")
+{
+    sAppModule    = "Part";
+    sGroup        = QT_TR_NOOP("Part");
+    sMenuText     = QT_TR_NOOP("Cross-sections...");
+    sToolTipText  = QT_TR_NOOP("Cross-sections");
+    sWhatsThis    = "Part_CrossSections";
+    sStatusTip    = sToolTipText;
+//  sPixmap       = "Part_Booleans";
+    iAccel        = 0;
+}
+
+void CmdPartCrossSections::activated(int iMsg)
+{
+    Gui::TaskView::TaskDialog* dlg = Gui::Control().activeDialog();
+    if (!dlg) {
+        std::vector<App::DocumentObject*> obj = Gui::Selection().getObjectsOfType
+            (Part::Feature::getClassTypeId());
+        Base::BoundBox3d bbox;
+        for (std::vector<App::DocumentObject*>::iterator it = obj.begin(); it != obj.end(); ++it) {
+            bbox.Add(static_cast<Part::Feature*>(*it)->Shape.getBoundingBox());
+        }
+        dlg = new PartGui::TaskCrossSections(bbox);
+    }
+    Gui::Control().showDialog(dlg);
+}
+
+bool CmdPartCrossSections::isActive(void)
+{
+    return (Gui::Selection().countObjectsOfType(Part::Feature::getClassTypeId()) > 0 &&
+            !Gui::Control().activeDialog());
+}
+
 //--------------------------------------------------------------------------------------
 
 DEF_STD_CMD_A(CmdShapeInfo);
@@ -842,6 +923,116 @@ bool CmdShapeInfo::isActive(void)
     return false;
 }
 
+DEF_STD_CMD_A(CmdPartRuledSurface);
+
+CmdPartRuledSurface::CmdPartRuledSurface()
+  : Command("Part_RuledSurface")
+{
+    sAppModule      = "Part";
+    sGroup          = QT_TR_NOOP("Part");
+    sMenuText       = QT_TR_NOOP("Create ruled surface");
+    sToolTipText    = QT_TR_NOOP("Create a ruled surface from two curves");
+    sWhatsThis      = sToolTipText;
+    sStatusTip      = sToolTipText;
+}
+
+void CmdPartRuledSurface::activated(int iMsg)
+{
+    bool ok = false;
+    TopoDS_Shape curve1, curve2;
+    std::string link1, link2, obj1, obj2;
+    Gui::SelectionFilter edgeFilter  ("SELECT Part::Feature SUBELEMENT Edge COUNT 1..2");
+    Gui::SelectionFilter wireFilter  ("SELECT Part::Feature SUBELEMENT Wire COUNT 1..2");
+    Gui::SelectionFilter partFilter  ("SELECT Part::Feature COUNT 2");
+    bool matchEdge = edgeFilter.match();
+    bool matchWire = wireFilter.match();
+    if (matchEdge || matchWire) {
+        // get the selected object
+        const std::vector<Gui::SelectionObject>& result = matchEdge
+            ? edgeFilter.Result[0] : wireFilter.Result[0];
+        // two edges from one object
+        if (result.size() == 1) {
+            const Part::Feature* part = static_cast<const Part::Feature*>(result[0].getObject());
+            const std::vector<std::string>& edges = result[0].getSubNames();
+            if (edges.size() != 2) {
+                ok = false;
+            }
+            else {
+                ok = true;
+                // get the selected sub-shapes
+                const Part::TopoShape& shape = part->Shape.getValue();
+                curve1 = shape.getSubShape(edges[0].c_str());
+                curve2 = shape.getSubShape(edges[1].c_str());
+                obj1 = result[0].getObject()->getNameInDocument();
+                link1 = edges[0];
+                obj2 = result[0].getObject()->getNameInDocument();
+                link2 = edges[1];
+            }
+        }
+        // two objects and one edge per object
+        else if (result.size() == 2) {
+            const Part::Feature* part1 = static_cast<const Part::Feature*>(result[0].getObject());
+            const std::vector<std::string>& edges1 = result[0].getSubNames();
+            const Part::Feature* part2 = static_cast<const Part::Feature*>(result[1].getObject());
+            const std::vector<std::string>& edges2 = result[1].getSubNames();
+            if (edges1.size() != 1 || edges2.size() != 1) {
+                ok = false;
+            }
+            else {
+                ok = true;
+                const Part::TopoShape& shape1 = part1->Shape.getValue();
+                curve1 = shape1.getSubShape(edges1[0].c_str());
+                const Part::TopoShape& shape2 = part2->Shape.getValue();
+                curve2 = shape2.getSubShape(edges2[0].c_str());
+                obj1 = result[0].getObject()->getNameInDocument();
+                link1 = edges1[0];
+                obj2 = result[1].getObject()->getNameInDocument();
+                link2 = edges2[0];
+            }
+        }
+    }
+    else if (partFilter.match()) {
+        const std::vector<Gui::SelectionObject>& result = partFilter.Result[0];
+        const Part::Feature* part1 = static_cast<const Part::Feature*>(result[0].getObject());
+        const Part::Feature* part2 = static_cast<const Part::Feature*>(result[1].getObject());
+        const Part::TopoShape& shape1 = part1->Shape.getValue();
+        curve1 = shape1._Shape;
+        const Part::TopoShape& shape2 = part2->Shape.getValue();
+        curve2 = shape2._Shape;
+        obj1 = part1->getNameInDocument();
+        obj2 = part2->getNameInDocument();
+
+        if (!curve1.IsNull() && !curve2.IsNull()) {
+            if (curve1.ShapeType() == TopAbs_EDGE &&
+                curve2.ShapeType() == TopAbs_EDGE)
+                ok = true;
+            if (curve1.ShapeType() == TopAbs_WIRE &&
+                curve2.ShapeType() == TopAbs_WIRE)
+                ok = true;
+        }
+    }
+
+    if (!ok) {
+        QMessageBox::warning(Gui::getMainWindow(), QObject::tr("Wrong selection"),
+            QObject::tr("You have to select either two edges or two wires."));
+        return;
+    }
+
+    openCommand("Create ruled surface");
+    doCommand(Doc, "FreeCAD.ActiveDocument.addObject('Part::RuledSurface','Filled shape')");
+    doCommand(Doc, "FreeCAD.ActiveDocument.ActiveObject.Curve1=(FreeCAD.ActiveDocument.%s,['%s'])"
+                 ,obj1.c_str(), link1.c_str());
+    doCommand(Doc, "FreeCAD.ActiveDocument.ActiveObject.Curve2=(FreeCAD.ActiveDocument.%s,['%s'])"
+                 ,obj2.c_str(), link2.c_str());
+    commitCommand();
+    updateActive();
+}
+
+bool CmdPartRuledSurface::isActive(void)
+{
+    return getActiveGuiDocument();
+}
+
 
 void CreatePartCommands(void)
 {
@@ -851,7 +1042,9 @@ void CreatePartCommands(void)
     rcCmdMgr.addCommand(new CmdPartReverseShape());
     rcCmdMgr.addCommand(new CmdPartBoolean());
     rcCmdMgr.addCommand(new CmdPartExtrude());
+    rcCmdMgr.addCommand(new CmdPartMirror());
     rcCmdMgr.addCommand(new CmdPartRevolve());
+    rcCmdMgr.addCommand(new CmdPartCrossSections());
     rcCmdMgr.addCommand(new CmdPartFillet());
     rcCmdMgr.addCommand(new CmdPartCommon());
     rcCmdMgr.addCommand(new CmdPartCut());
@@ -865,5 +1058,6 @@ void CreatePartCommands(void)
     rcCmdMgr.addCommand(new CmdPartImportCurveNet());
     rcCmdMgr.addCommand(new CmdPartPickCurveNet());
     rcCmdMgr.addCommand(new CmdShapeInfo());
+    rcCmdMgr.addCommand(new CmdPartRuledSurface());
 } 
 
