@@ -23,6 +23,7 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <algorithm>
 # include <BRep_Tool.hxx>
 # include <TopoDS.hxx>
 # include <TopoDS_Edge.hxx>
@@ -46,6 +47,7 @@
 #include "ui_DlgFilletEdges.h"
 
 #include "../App/PartFeature.h"
+#include "../App/FeatureFillet.h"
 #include <App/Application.h>
 #include <App/Document.h>
 #include <App/DocumentObject.h>
@@ -54,6 +56,7 @@
 #include <Gui/Command.h>
 #include <Gui/WaitCursor.h>
 #include <Gui/Selection.h>
+#include <Gui/ViewProvider.h>
 
 using namespace PartGui;
 
@@ -131,6 +134,7 @@ namespace PartGui {
     {
     public:
         App::DocumentObject* object;
+        Part::Fillet* fillet;
         typedef boost::signals::connection Connection;
         Connection connectApplicationDeletedObject;
         Connection connectApplicationDeletedDocument;
@@ -139,12 +143,13 @@ namespace PartGui {
 
 /* TRANSLATOR PartGui::DlgFilletEdges */
 
-DlgFilletEdges::DlgFilletEdges(QWidget* parent, Qt::WFlags fl)
+DlgFilletEdges::DlgFilletEdges(Part::Fillet* fillet, QWidget* parent, Qt::WFlags fl)
   : QWidget(parent, fl), ui(new Ui_DlgFilletEdges()), d(new DlgFilletEdgesP())
 {
     ui->setupUi(this);
 
     d->object = 0;
+    d->fillet = fillet;
     d->connectApplicationDeletedObject = App::GetApplication().signalDeletedObject
         .connect(boost::bind(&DlgFilletEdges::onDeleteObject, this, _1));
     d->connectApplicationDeletedDocument = App::GetApplication().signalDeleteDocument
@@ -194,7 +199,8 @@ void DlgFilletEdges::onSelectionChanged(const Gui::SelectionChanges& msg)
             QString subelement = QString::fromAscii(msg.pSubName);
             QAbstractItemModel* model = ui->treeView->model();
             for (int i=0; i<model->rowCount(); ++i) {
-                QString name = model->data(model->index(i,0), Qt::DisplayRole).toString();
+                int id = model->data(model->index(i,0), Qt::UserRole).toInt();
+                QString name = QString::fromAscii("Edge%1").arg(id);
                 if (name == subelement) {
                     // ok, check the selected sub-element
                     Qt::CheckState checkState = Qt::Checked;
@@ -214,7 +220,16 @@ void DlgFilletEdges::onSelectionChanged(const Gui::SelectionChanges& msg)
 
 void DlgFilletEdges::onDeleteObject(const App::DocumentObject& obj)
 {
-    if (d->object == &obj) {
+    if (d->fillet == &obj) {
+        d->fillet = 0;
+    }
+    else if (d->fillet && d->fillet->Base.getValue() == &obj) {
+        d->fillet = 0;
+        d->object = 0;
+        ui->shapeObject->setCurrentIndex(0);
+        on_shapeObject_activated(0);
+    }
+    else if (d->object == &obj) {
         d->object = 0;
         ui->shapeObject->removeItem(ui->shapeObject->currentIndex());
         ui->shapeObject->setCurrentIndex(0);
@@ -253,7 +268,8 @@ void DlgFilletEdges::toogleCheckState(const QModelIndex& index)
     if (!d->object)
         return;
     QVariant check = index.data(Qt::CheckStateRole);
-    QString name = index.data(Qt::DisplayRole).toString();
+    int id = index.data(Qt::UserRole).toInt();
+    QString name = QString::fromAscii("Edge%1").arg(id);
     Qt::CheckState checkState = static_cast<Qt::CheckState>(check.toInt());
 
     bool block = this->blockConnection(false);
@@ -302,6 +318,73 @@ void DlgFilletEdges::findShapes()
         ui->shapeObject->setCurrentIndex(current_index);
         on_shapeObject_activated(current_index);
     }
+
+    // if an existing fillet object is given start the edit mode
+    if (d->fillet) {
+        setupFillet(objs);
+    }
+}
+
+void DlgFilletEdges::setupFillet(const std::vector<App::DocumentObject*>& objs)
+{
+    App::DocumentObject* base = d->fillet->Base.getValue();
+    const std::vector<Part::FilletElement>& e = d->fillet->Edges.getValues();
+    std::vector<App::DocumentObject*>::const_iterator it = std::find(objs.begin(), objs.end(), base);
+    if (it != objs.end()) {
+        // toggle visibility
+        Gui::ViewProvider* vp;
+        vp = Gui::Application::Instance->getViewProvider(d->fillet);
+        if (vp) vp->hide();
+        vp = Gui::Application::Instance->getViewProvider(base);
+        if (vp) vp->show();
+
+        int current_index = (it - objs.begin()) + 1;
+        ui->shapeObject->setCurrentIndex(current_index);
+        on_shapeObject_activated(current_index);
+        ui->shapeObject->setEnabled(false);
+        QStandardItemModel *model = qobject_cast<QStandardItemModel*>(ui->treeView->model());
+        for (std::vector<Part::FilletElement>::const_iterator et = e.begin(); et != e.end(); ++et) {
+            int index = et->edgeid-1;
+            model->setData(model->index(index, 0), Qt::Checked, Qt::CheckStateRole);
+            model->setData(model->index(index, 1), QVariant(QLocale::system().toString(et->radius1,'f',2)));
+            model->setData(model->index(index, 2), QVariant(QLocale::system().toString(et->radius2,'f',2)));
+        }
+    }
+}
+
+/**
+ * Sets the strings of the subwidgets using the current language.
+ */
+void DlgFilletEdges::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::LanguageChange) {
+        int index = ui->shapeObject->currentIndex();
+        // only get the items from index 1 on since the first one will be added automatically
+        int count = ui->shapeObject->count() - 1;
+        QStringList text;
+        QList<QVariant> data;
+        for (int i=0; i<count; i++) {
+            text << ui->shapeObject->itemText(i+1);
+            data << ui->shapeObject->itemData(i+1);
+        }
+
+        ui->retranslateUi(this);
+        for (int i=0; i<count; i++) {
+            ui->shapeObject->addItem(text.at(i));
+            ui->shapeObject->setItemData(i+1, data.at(i));
+        }
+
+        ui->shapeObject->setCurrentIndex(index);
+        QStandardItemModel *model = qobject_cast<QStandardItemModel*>(ui->treeView->model());
+        count = model->rowCount();
+        for (int i=0; i<count; i++) {
+            int id = model->data(model->index(i, 0), Qt::UserRole).toInt();
+            model->setData(model->index(i, 0), QVariant(tr("Edge%1").arg(id)));
+        }
+    }
+    else {
+        QWidget::changeEvent(e);
+    }
 }
 
 void DlgFilletEdges::on_shapeObject_activated(int index)
@@ -448,14 +531,21 @@ bool DlgFilletEdges::accept()
     int index = ui->shapeObject->currentIndex();
     shape = ui->shapeObject->itemData(index).toString();
     type = QString::fromAscii("Part::Fillet");
-    name = QString::fromAscii(activeDoc->getUniqueObjectName("Fillet").c_str());
+
+    if (d->fillet)
+        name = QString::fromAscii(d->fillet->getNameInDocument());
+    else
+        name = QString::fromAscii(activeDoc->getUniqueObjectName("Fillet").c_str());
 
     activeDoc->openTransaction("Fillet");
-    QString code = QString::fromAscii(
+    QString code;
+    if (!d->fillet) {
+        code = QString::fromAscii(
         "FreeCAD.ActiveDocument.addObject(\"%1\",\"%2\")\n"
-        "FreeCAD.ActiveDocument.%2.Base = FreeCAD.ActiveDocument.%3\n"
-        "__fillets__ = []\n")
+        "FreeCAD.ActiveDocument.%2.Base = FreeCAD.ActiveDocument.%3\n")
         .arg(type).arg(name).arg(shape);
+    }
+    code += QString::fromAscii("__fillets__ = []\n");
     for (int i=0; i<model->rowCount(); ++i) {
         QVariant value = model->index(i,0).data(Qt::CheckStateRole);
         Qt::CheckState checkState = static_cast<Qt::CheckState>(value.toInt());
@@ -491,15 +581,20 @@ bool DlgFilletEdges::accept()
     Gui::Application::Instance->runPythonCode((const char*)code.toAscii());
     activeDoc->commitTransaction();
     activeDoc->recompute();
+    if (d->fillet) {
+        Gui::ViewProvider* vp;
+        vp = Gui::Application::Instance->getViewProvider(d->fillet);
+        if (vp) vp->show();
+    }
     return true;
 }
 
 // ---------------------------------------
 
-FilletEdgesDialog::FilletEdgesDialog(QWidget* parent, Qt::WFlags fl)
+FilletEdgesDialog::FilletEdgesDialog(Part::Fillet* fillet, QWidget* parent, Qt::WFlags fl)
   : QDialog(parent, fl)
 {
-    widget = new DlgFilletEdges(this);
+    widget = new DlgFilletEdges(fillet, this);
     this->setWindowTitle(widget->windowTitle());
 
     QVBoxLayout* hboxLayout = new QVBoxLayout(this);
@@ -525,27 +620,13 @@ void FilletEdgesDialog::accept()
 
 // ---------------------------------------
 
-TaskBoxFilletEdges::TaskBoxFilletEdges(QWidget* parent)
-    : TaskBox(Gui::BitmapFactory().pixmap("Part_Fillet"),
-        tr("Fillet"),true, parent)
-{
-    widget = new DlgFilletEdges(this);
-    this->groupLayout()->addWidget(widget);
-}
-
-TaskBoxFilletEdges::~TaskBoxFilletEdges()
-{
-}
-
-bool TaskBoxFilletEdges::accept()
-{
-    return widget->accept();
-}
-
-// ---------------------------------------
-
-TaskFilletEdges::TaskFilletEdges() : taskbox(new TaskBoxFilletEdges)
+TaskFilletEdges::TaskFilletEdges(Part::Fillet* fillet)
 {
+    widget = new DlgFilletEdges(fillet);
+    taskbox = new Gui::TaskView::TaskBox(
+        Gui::BitmapFactory().pixmap("Part_Fillet"),
+        widget->windowTitle(), true, 0);
+    taskbox->groupLayout()->addWidget(widget);
     Content.push_back(taskbox);
 }
 
@@ -564,7 +645,7 @@ void TaskFilletEdges::clicked(int)
 
 bool TaskFilletEdges::accept()
 {
-    return taskbox->accept();
+    return widget->accept();
 }
 
 bool TaskFilletEdges::reject()
