@@ -29,6 +29,8 @@
 #include <QStandardItemModel>
 #include <QItemDelegate>
 
+namespace Part { class Fillet; }
+
 namespace PartGui {
 
 class Ui_DlgFilletEdges;
@@ -70,12 +72,14 @@ class DlgFilletEdges : public QWidget, public Gui::SelectionObserver
     Q_OBJECT
 
 public:
-    DlgFilletEdges(QWidget* parent = 0, Qt::WFlags fl = 0);
+    DlgFilletEdges(Part::Fillet*, QWidget* parent = 0, Qt::WFlags fl = 0);
     ~DlgFilletEdges();
     bool accept();
 
 protected:
     void findShapes();
+    void setupFillet(const std::vector<App::DocumentObject*>&);
+    void changeEvent(QEvent *e);
 
 private:
     void onSelectionChanged(const Gui::SelectionChanges& msg);
@@ -101,7 +105,7 @@ class FilletEdgesDialog : public QDialog
     Q_OBJECT
 
 public:
-    FilletEdgesDialog(QWidget* parent = 0, Qt::WFlags fl = 0);
+    FilletEdgesDialog(Part::Fillet* fillet, QWidget* parent = 0, Qt::WFlags fl = 0);
     ~FilletEdgesDialog();
     void accept();
 
@@ -109,25 +113,12 @@ private:
     DlgFilletEdges* widget;
 };
 
-class TaskBoxFilletEdges : public Gui::TaskView::TaskBox
-{
-    Q_OBJECT
-
-public:
-    TaskBoxFilletEdges(QWidget *parent = 0);
-    ~TaskBoxFilletEdges();
-    bool accept();
-
-private:
-    DlgFilletEdges* widget;
-};
-
 class TaskFilletEdges : public Gui::TaskView::TaskDialog
 {
     Q_OBJECT
 
 public:
-    TaskFilletEdges();
+    TaskFilletEdges(Part::Fillet*);
     ~TaskFilletEdges();
 
 public:
@@ -140,7 +131,8 @@ public:
     { return QDialogButtonBox::Ok|QDialogButtonBox::Cancel; }
 
 private:
-    TaskBoxFilletEdges* taskbox;
+    DlgFilletEdges* widget;
+    Gui::TaskView::TaskBox* taskbox;
 };
 
 } // namespace PartGui
