@@ -67,6 +67,7 @@
 # include <Inventor/nodes/SoNormal.h>
 # include <Inventor/nodes/SoNormalBinding.h>
 # include <Inventor/nodes/SoPointSet.h>
+# include <Inventor/nodes/SoPolygonOffset.h>
 # include <Inventor/nodes/SoShapeHints.h>
 # include <Inventor/nodes/SoSwitch.h>
 # include <Inventor/nodes/SoGroup.h>
@@ -119,7 +120,7 @@ ViewProviderPart::ViewProviderPart() : pcControlPoints(0)
     mat.diffuseColor.set(0.1f,0.1f,0.1f);
     mat.specularColor.set(0.0f,0.0f,0.0f);
     mat.emissiveColor.set(0.0f,0.0f,0.0f);
-    mat.shininess = 0.0f;
+    mat.shininess = 1.0f;
     mat.transparency = 0.0f;
     ADD_PROPERTY(LineMaterial,(mat));
     ADD_PROPERTY(PointMaterial,(mat));
@@ -250,10 +251,15 @@ void ViewProviderPart::attach(App::DocumentObject *pcFeat)
     pShapeHints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
     pShapeHints->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
 
+    // Avoid any Z-buffer artefacts, so that the lines always appear on top of the faces
+    // The correct order is Edges, Polygon offset, Faces.
+    SoPolygonOffset* offset = new SoPolygonOffset();
+
     // normal viewing with edges and points
     pcNormalRoot->addChild(pShapeHints);
-    pcNormalRoot->addChild(FaceRoot);
     pcNormalRoot->addChild(EdgeRoot);
+    pcNormalRoot->addChild(offset);
+    pcNormalRoot->addChild(FaceRoot);
     pcNormalRoot->addChild(VertexRoot);
 
     // just faces with no edges or points
@@ -417,7 +423,8 @@ void ViewProviderPart::updateData(const App::Property* prop)
             computeFaces   (FaceRoot,cShape,deflection);
             computeEdges   (EdgeRoot,cShape);
             computeVertices(VertexRoot,cShape);
-            BRepTools::Clean(cShape); // remove triangulation
+            // NOTE: Cleaning the triangulation may cause problems on some algorithms like BOP
+            //BRepTools::Clean(cShape); // remove triangulation
 
             // update control points if there
             if (pcControlPoints) {
