@@ -53,7 +53,7 @@ using namespace std;
 
 PROPERTY_SOURCE(PartGui::ViewProvider2DObject, PartGui::ViewProviderPart)
 
-       
+
 ViewProvider2DObject::ViewProvider2DObject()
 {
     ADD_PROPERTY_TYPE(ShowGrid,(false),"Grid",(App::PropertyType)(App::Prop_None),"Switch the grid on/off");
@@ -61,8 +61,8 @@ ViewProvider2DObject::ViewProvider2DObject()
 
     GridRoot = new SoSeparator();
     GridRoot->ref();
-	MinX = MinY = -100;
-	MaxX = MaxY = 100;
+    MinX = MinY = -100;
+    MaxX = MaxY = 100;
 
     pcRoot->addChild(GridRoot);
  
@@ -71,91 +71,89 @@ ViewProvider2DObject::ViewProvider2DObject()
 
 ViewProvider2DObject::~ViewProvider2DObject()
 {
-  GridRoot->unref();
+     GridRoot->unref();
 }
 
 
-
 // **********************************************************************************
 
-SoSeparator* ViewProvider2DObject::createGrid(void) {
-  //double dx = 10 * GridSize.getValue();                       // carpet size
-  //double dy = 10 * GridSize.getValue();
-	float Size = (MaxX-MinX > MaxY-MinY)?MaxX-MinX: MaxY-MinY;
-	float Step = pow(10,floor(log10(Size/5.0)));
-	float MiX = MinX - (MaxX-MinX)*0.5;
-	float MaX = MaxX + (MaxX-MinX)*0.5;
-	float MiY = MinY - (MaxY-MinY)*0.5;
-	float MaY = MaxY + (MaxY-MinY)*0.5;
-	//float Step = 10.0;
-  double dz = 0.0;                     // carpet-grid separation
-  //int gridsize = 20;                    // grid size
+SoSeparator* ViewProvider2DObject::createGrid(void)
+{
+    //double dx = 10 * GridSize.getValue();                       // carpet size
+    //double dy = 10 * GridSize.getValue();
+    float Size = (MaxX-MinX > MaxY-MinY)?MaxX-MinX: MaxY-MinY;
+    float Step = pow(10,floor(log10(Size/5.0)));
+    float MiX = MinX - (MaxX-MinX)*0.5;
+    float MaX = MaxX + (MaxX-MinX)*0.5;
+    float MiY = MinY - (MaxY-MinY)*0.5;
+    float MaY = MaxY + (MaxY-MinY)*0.5;
+    //float Step = 10.0;
+    double dz = 0.0;                     // carpet-grid separation
+    //int gridsize = 20;                    // grid size
 
-  SoSeparator *parent = GridRoot;
-  GridRoot->removeAllChildren();
-  SoBaseColor *mycolor;
-  SoVertexProperty *vts;
+    SoSeparator *parent = GridRoot;
+    GridRoot->removeAllChildren();
+    SoBaseColor *mycolor;
+    SoVertexProperty *vts;
 
-  // carpet
+   // carpet
  /* mycolor = new SoBaseColor;
-  mycolor->rgb.setValue(0.2f, 0.7f, 0.7f);
-  parent->addChild(mycolor);
+    mycolor->rgb.setValue(0.2f, 0.7f, 0.7f);
+    parent->addChild(mycolor);
 
-  vts = new SoVertexProperty;
-  vts->vertex.set1Value(0, -0.5*dx, -1.5*dy,  0.5*dz);
-  vts->vertex.set1Value(1, -0.5*dx, -1.5*dy, -0.5*dz);
-  vts->vertex.set1Value(2,  0.5*dx, -1.5*dy,  0.5*dz);
-  vts->vertex.set1Value(3,  0.5*dx, -1.5*dy, -0.5*dz);
+    vts = new SoVertexProperty;
+    vts->vertex.set1Value(0, -0.5*dx, -1.5*dy,  0.5*dz);
+    vts->vertex.set1Value(1, -0.5*dx, -1.5*dy, -0.5*dz);
+    vts->vertex.set1Value(2,  0.5*dx, -1.5*dy,  0.5*dz);
+    vts->vertex.set1Value(3,  0.5*dx, -1.5*dy, -0.5*dz);
 
-  SoQuadMesh *carpet = new SoQuadMesh;
-  carpet->verticesPerColumn = 2;
-  carpet->verticesPerRow = 2;
-  carpet->vertexProperty = vts;
-  parent->addChild(carpet);*/
+    SoQuadMesh *carpet = new SoQuadMesh;
+    carpet->verticesPerColumn = 2;
+    carpet->verticesPerRow = 2;
+    carpet->vertexProperty = vts;
+    parent->addChild(carpet);*/
 
-  // gridlines
-  mycolor = new SoBaseColor;
+    // gridlines
+    mycolor = new SoBaseColor;
 
-  mycolor->rgb.setValue(0.7f, 0.7f ,0.7f);
-  parent->addChild(mycolor);
+    mycolor->rgb.setValue(0.7f, 0.7f ,0.7f);
+    parent->addChild(mycolor);
 
-  SoDrawStyle* DrawStyle = new SoDrawStyle;
-  DrawStyle->lineWidth = 1;
-  DrawStyle->linePattern = 0x0f0f;
-  parent->addChild(DrawStyle);
+    SoDrawStyle* DrawStyle = new SoDrawStyle;
+    DrawStyle->lineWidth = 1;
+    DrawStyle->linePattern = 0x0f0f;
+    parent->addChild(DrawStyle);
 
-  SoPickStyle* PickStyle = new SoPickStyle;
-  PickStyle->style = SoPickStyle::UNPICKABLE;
-  parent->addChild(PickStyle);
+    SoPickStyle* PickStyle = new SoPickStyle;
+    PickStyle->style = SoPickStyle::UNPICKABLE;
+    parent->addChild(PickStyle);
 
-  SoLineSet *grid = new SoLineSet;
-  vts = new SoVertexProperty;
-  grid->vertexProperty = vts;
+    SoLineSet *grid = new SoLineSet;
+    vts = new SoVertexProperty;
+    grid->vertexProperty = vts;
 
-  int vi=0, l=0;
+    int vi=0, l=0;
 
-  // vertical lines
-  float i;
-  for (i=MiX; i<MaX; i+=Step) {
-    /*float h=-0.5*dx + float(i) / gridsize * dx;*/
-    vts->vertex.set1Value(vi++, i, MiY, dz);
-    vts->vertex.set1Value(vi++, i,  MaY, dz);
-    grid->numVertices.set1Value(l++, 2);
-  }
+    // vertical lines
+    float i;
+    for (i=MiX; i<MaX; i+=Step) {
+        /*float h=-0.5*dx + float(i) / gridsize * dx;*/
+        vts->vertex.set1Value(vi++, i, MiY, dz);
+        vts->vertex.set1Value(vi++, i,  MaY, dz);
+        grid->numVertices.set1Value(l++, 2);
+    }
 
-  // horizontal lines
-  for (i=MiY; i<MaY; i+=Step) {
-    //float v=-0.5*dy + float(i) / gridsize * dy;
-    vts->vertex.set1Value(vi++, MiX, i, dz );
-    vts->vertex.set1Value(vi++,  MaX, i, dz );
-    grid->numVertices.set1Value(l++, 2);
-  }
-  parent->addChild(vts);
-
-  parent->addChild(grid);
-
-  return parent;
+    // horizontal lines
+    for (i=MiY; i<MaY; i+=Step) {
+        //float v=-0.5*dy + float(i) / gridsize * dy;
+        vts->vertex.set1Value(vi++, MiX, i, dz );
+        vts->vertex.set1Value(vi++,  MaX, i, dz );
+        grid->numVertices.set1Value(l++, 2);
+    }
+    parent->addChild(vts);
+    parent->addChild(grid);
 
+    return parent;
 }
 
 void ViewProvider2DObject::updateData(const App::Property* prop)
@@ -164,17 +162,16 @@ void ViewProvider2DObject::updateData(const App::Property* prop)
 
     if (prop->getTypeId() == Part::PropertyPartShape::getClassTypeId()) {
         Base::BoundBox3d Bnd = static_cast<const Part::PropertyPartShape*>(prop)->getBoundingBox();
-		GridRoot->removeAllChildren();
-		MinX = Bnd.MinX;
-		MaxX = Bnd.MaxX;
-		MinY = Bnd.MinY;
-		MaxY = Bnd.MaxY;
-		if(ShowGrid.getValue()){
-			GridRoot->removeAllChildren();
-			createGrid();
-		}
-	}
- 
+        GridRoot->removeAllChildren();
+        MinX = Bnd.MinX;
+        MaxX = Bnd.MaxX;
+        MinY = Bnd.MinY;
+        MaxY = Bnd.MaxY;
+        if (ShowGrid.getValue()){
+            GridRoot->removeAllChildren();
+            createGrid();
+        }
+    }
 }
 
 void ViewProvider2DObject::onChanged(const App::Property* prop)
@@ -200,19 +197,33 @@ void ViewProvider2DObject::attach(App::DocumentObject *pcFeat)
 {
     ViewProviderPart::attach(pcFeat);
 
-    if(ShowGrid.getValue())
+    if (ShowGrid.getValue())
         createGrid();
 
 
 }
 
-void ViewProvider2DObject::setEdit(void)
+bool ViewProvider2DObject::setEdit(int)
+{
+    return false;
+}
+
+void ViewProvider2DObject::unsetEdit(int)
 {
 
 }
 
-void ViewProvider2DObject::unsetEdit(void)
+std::vector<std::string> ViewProvider2DObject::getDisplayModes(void) const
 {
+    // get the modes of the father
+    std::vector<std::string> StrList = ViewProviderGeometryObject::getDisplayModes();
 
+    // add your own modes
+    //StrList.push_back("Flat Lines");
+    //StrList.push_back("Shaded");
+    StrList.push_back("Wireframe");
+    StrList.push_back("Points");
+
+    return StrList;
 }
 
