@@ -57,16 +57,14 @@ public:
 
   virtual void attach(App::DocumentObject *);
   virtual void updateData(const App::Property*);
-
-
-  virtual void setEdit(void);
-  virtual void unsetEdit(void);
+  virtual std::vector<std::string> getDisplayModes(void) const;
 
   /// creats the grid
   SoSeparator* createGrid(void); 
 
-
 protected:
+  virtual bool setEdit(int ModNum);
+  virtual void unsetEdit(int ModNum);
   /// get called by the container whenever a property has been changed
   virtual void onChanged(const App::Property* prop);
 
