@@ -30,7 +30,13 @@
 #include <Base/Interpreter.h>
  
 #include "FeaturePad.h"
+#include "FeaturePocket.h"
 #include "FeatureFillet.h"
+#include "FeatureSketchBased.h"
+#include "FeatureRevolution.h"
+#include "FeatureMainPart.h"
+#include "FeatureDressUp.h"
+#include "FeatureChamfer.h"
 
 extern struct PyMethodDef PartDesign_methods[];
 
@@ -42,10 +48,10 @@ PyDoc_STRVAR(module_PartDesign_doc,
 extern "C" {
 void AppPartDesignExport initPartDesign()
 {
-    // load dependend module
+    // load dependent module
     try {
-        Base::Interpreter().loadModule("Part");
-        Base::Interpreter().loadModule("Sketcher");
+        Base::Interpreter().runString("import Part");
+        Base::Interpreter().runString("import Sketcher");
     }
     catch(const Base::Exception& e) {
         PyErr_SetString(PyExc_ImportError, e.what());
@@ -59,8 +65,14 @@ void AppPartDesignExport initPartDesign()
     // call PyType_Ready, otherwise we run into a segmentation fault, later on.
     // This function is responsible for adding inherited slots from a type's base class.
  
+    PartDesign::DressUp    ::init();
+    PartDesign::SketchBased::init();
+    PartDesign::MainPart   ::init();
     PartDesign::Pad        ::init();
+    PartDesign::Pocket     ::init();
     PartDesign::Fillet     ::init();
+    PartDesign::Revolution ::init();
+    PartDesign::Chamfer    ::init();
 }
 
 } // extern "C"
