@@ -30,7 +30,12 @@
 #include <Base/Interpreter.h>
 #include <Gui/Application.h>
 #include <Gui/Language/Translator.h>
+
 #include "Workbench.h"
+#include "ViewProviderPocket.h"
+#include "ViewProviderPad.h"
+#include "TaskChamfer.h"
+
 //#include "resources/qrc_PartDesign.cpp"
 
 // use a different name to CreateCommand()
@@ -57,8 +62,8 @@ void PartDesignGuiExport initPartDesignGui()
     }
 
     try {
-        Base::Interpreter().loadModule("PartGui");
-        Base::Interpreter().loadModule("SketcherGui");
+        Base::Interpreter().runString("import PartGui");
+        Base::Interpreter().runString("import SketcherGui");
     }
     catch(const Base::Exception& e) {
         PyErr_SetString(PyExc_ImportError, e.what());
@@ -68,9 +73,13 @@ void PartDesignGuiExport initPartDesignGui()
     (void) Py_InitModule("PartDesignGui", PartDesignGui_Import_methods);   /* mod name, table ptr */
     Base::Console().Log("Loading GUI of PartDesign module... done\n");
 
-    // instanciating the commands
+    // instantiating the commands
     CreatePartDesignCommands();
-    PartDesignGui::Workbench::init();
+
+    PartDesignGui::Workbench           ::init();
+    PartDesignGui::ViewProviderPocket  ::init();
+    PartDesignGui::ViewProviderPad     ::init();
+    PartDesignGui::ViewProviderChamfer ::init();
 
      // add resources and reloads the translators
     loadPartDesignResource();
