@@ -67,7 +67,7 @@ PROPERTY_SOURCE(PointsGui::ViewProviderPoints, Gui::ViewProviderGeometryObject)
 
 App::PropertyFloatConstraint::Constraints ViewProviderPoints::floatRange = {1.0f,64.0f,1.0f};
 
-ViewProviderPoints::ViewProviderPoints() : _bEdit(false)
+ViewProviderPoints::ViewProviderPoints()
 {
     ADD_PROPERTY(PointSize,(2.0f));
     PointSize.setConstraints(&floatRange);
@@ -347,15 +347,13 @@ QIcon ViewProviderPoints::getIcon() const
   return px;
 }
 
-void ViewProviderPoints::setEdit(void)
+bool ViewProviderPoints::setEdit(int)
 {
-    if ( _bEdit ) return;
-    _bEdit = true;
+    return true;
 }
 
-void ViewProviderPoints::unsetEdit(void)
+void ViewProviderPoints::unsetEdit(int)
 {
-    _bEdit = false;
 }
 
 void ViewProviderPoints::clipPointsCallback(void * ud, SoEventCallback * n)
@@ -366,7 +364,7 @@ void ViewProviderPoints::clipPointsCallback(void * ud, SoEventCallback * n)
     view->removeEventCallback(SoMouseButtonEvent::getClassTypeId(), clipPointsCallback);
     n->setHandled();
 
-    std::vector<SbVec2f> clPoly = view->getPickedPolygon();
+    std::vector<SbVec2f> clPoly = view->getGLPolygon();
     if (clPoly.size() < 3)
         return;
     if (clPoly.front() != clPoly.back())
@@ -375,8 +373,8 @@ void ViewProviderPoints::clipPointsCallback(void * ud, SoEventCallback * n)
     std::vector<Gui::ViewProvider*> views = view->getViewProvidersOfType(ViewProviderPoints::getClassTypeId());
     for (std::vector<Gui::ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
         ViewProviderPoints* that = static_cast<ViewProviderPoints*>(*it);
-        if (that->_bEdit) {
-            that->unsetEdit();
+        if (that->getEditingMode() > -1) {
+            that->finishEditing();
             that->cut(clPoly, *view);
         }
     }
@@ -384,7 +382,7 @@ void ViewProviderPoints::clipPointsCallback(void * ud, SoEventCallback * n)
     view->render();
 }
 
-void ViewProviderPoints::cut( const std::vector<SbVec2f>& picked, Gui::View3DInventorViewer &Viewer)
+void ViewProviderPoints::cut(const std::vector<SbVec2f>& picked, Gui::View3DInventorViewer &Viewer)
 {
     // create the polygon from the picked points
     Base::Polygon2D cPoly;
@@ -405,7 +403,7 @@ void ViewProviderPoints::cut( const std::vector<SbVec2f>& picked, Gui::View3DInv
         SbVec3f pt(jt->x,jt->y,jt->z);
 
         // project from 3d to 2d
-        vol.projectToScreen( pt, pt );
+        vol.projectToScreen(pt, pt);
         if (!cPoly.Contains(Base::Vector2D(pt[0],pt[1])))
             newKernel.push_back(*jt);
     }
@@ -427,7 +425,10 @@ void ViewProviderPoints::cut( const std::vector<SbVec2f>& picked, Gui::View3DInv
 // -------------------------------------------------
 
 namespace Gui {
+/// @cond DOXERR
 PROPERTY_SOURCE_TEMPLATE(PointsGui::ViewProviderPython, PointsGui::ViewProviderPoints)
+/// @endcond
+
 // explicit template instantiation
 template class PointsGuiExport ViewProviderPythonFeatureT<PointsGui::ViewProviderPoints>;
 }
