@@ -4,23 +4,27 @@
 <context>
     <name>CmdApproxSurface</name>
     <message>
+        <location filename="../../Command.cpp" line="+50"/>
         <source>Reverse Engineering</source>
-        <translation>Reverse Engineering</translation>
+        <translation>Rückführung</translation>
     </message>
     <message>
+        <location line="+1"/>
         <source>Approximate surface...</source>
-        <translation>Flächenapproximation...</translation>
+        <translation>Ungefähre Oberfläche ...</translation>
     </message>
     <message>
+        <location line="+1"/>
         <source>Approximate a B-Spline surface</source>
-        <translation>Approximiere B-Spline-Fläche</translation>
+        <translation>Ungefähre B-Spline-Oberfläche</translation>
     </message>
 </context>
 <context>
     <name>Workbench</name>
     <message>
+        <location filename="../../Workbench.cpp" line="+36"/>
         <source>Reverse Engineering</source>
-        <translation>Reverse Engineering</translation>
+        <translation>Rückführung</translation>
     </message>
 </context>
 </TS>
