@@ -1,24 +1,35 @@
 <?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE TS>
-<TS version="2.0" language="fr_FR">
+<TS version="2.0">
 <context>
     <name>CmdApproxSurface</name>
     <message>
+        <location filename="../../Command.cpp" line="+50"/>
         <source>Reverse Engineering</source>
         <translation>Rétro-ingénierie</translation>
     </message>
     <message>
+        <location line="+1"/>
         <source>Approximate surface...</source>
         <translation>Surface approchée...</translation>
     </message>
     <message>
+        <location line="+1"/>
         <source>Approximate a B-Spline surface</source>
         <translation>Approche une surface B-Spline</translation>
     </message>
 </context>
+<context>
+    <name>ReverseEngineeringGui::Workbench</name>
+    <message>
+        <source>Reverse Engineering</source>
+        <translation type="obsolete">Rétro-ingénierie</translation>
+    </message>
+</context>
 <context>
     <name>Workbench</name>
     <message>
+        <location filename="../../Workbench.cpp" line="+36"/>
         <source>Reverse Engineering</source>
         <translation>Rétro-ingénierie</translation>
     </message>
