@@ -1,27 +1,36 @@
 <?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE TS>
-<TS version="2.0" language="sv_SE">
+<TS version="2.0">
 <context>
     <name>CmdApproxSurface</name>
     <message>
+        <location filename="../../Command.cpp" line="+50"/>
         <source>Reverse Engineering</source>
-        <translatorcomment>The term &quot;Reverse engineering&quot; is also used in Sweden</translatorcomment>
         <translation>Reverse Engineering</translation>
     </message>
     <message>
+        <location line="+1"/>
         <source>Approximate surface...</source>
         <translation>Approximera yta...</translation>
     </message>
     <message>
+        <location line="+1"/>
         <source>Approximate a B-Spline surface</source>
-        <translation>Approximiera en B-Spline-Yta</translation>
+        <translation>Approximera en B-Spline yta</translation>
+    </message>
+</context>
+<context>
+    <name>ReverseEngineeringGui::Workbench</name>
+    <message>
+        <source>Reverse Engineering</source>
+        <translation type="obsolete">Reverse Engineering</translation>
     </message>
 </context>
 <context>
     <name>Workbench</name>
     <message>
+        <location filename="../../Workbench.cpp" line="+36"/>
         <source>Reverse Engineering</source>
-        <translatorcomment>The term &quot;Reverse engineering&quot; is also used in Sweden</translatorcomment>
         <translation>Reverse Engineering</translation>
     </message>
 </context>
