@@ -48,10 +48,9 @@ PyDoc_STRVAR(module_Sketcher_doc,
 extern "C" {
 void AppSketcherExport initSketcher()
 {
-    // load dependend module
+    // load dependent module
     try {
         Base::Interpreter().runString("import Part");
-        //Base::Interpreter().loadModule("Mesh");
     }
     catch(const Base::Exception& e) {
         PyErr_SetString(PyExc_ImportError, e.what());
@@ -70,6 +69,7 @@ void AppSketcherExport initSketcher()
  
     Sketcher::SketchObjectSF        ::init();
     Sketcher::SketchObject          ::init();
+    Sketcher::SketchObjectPython    ::init();
     Sketcher::Sketch                ::init();
     Sketcher::Constraint            ::init();
     Sketcher::PropertyConstraintList::init();
