@@ -23,6 +23,8 @@
  
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <BRep_Builder.hxx>
+# include <TopoDS_Compound.hxx>
 #endif
 
 #include <Base/Writer.h>
@@ -37,6 +39,10 @@
 #include <Mod/Part/App/GeometryCurvePy.h>
 #include <Mod/Part/App/LinePy.h>
 
+#include <TopoDS.hxx>
+#include <TopoDS_Edge.hxx>
+#include <BRepBuilderAPI_MakeWire.hxx>
+
 
 #include "Sketch.h"
 #include "Constraint.h"
@@ -49,6 +55,7 @@ using namespace Sketcher;
 using namespace Base;
 using namespace Part;
 
+#define POINT_CONSTRAIN_OPTIMIZE 1
 
 TYPESYSTEM_SOURCE(Sketcher::Sketch, Base::Persistence)
 
@@ -70,8 +77,7 @@ Sketch::Sketch()
 }
 Sketch::~Sketch()
 {
-    for(std::vector<double*>::iterator it=Parameters.begin();it!=Parameters.end();++it)
-        delete *it;
+    clear();
 }
 
 void Sketch::clear(void)
@@ -86,6 +92,9 @@ void Sketch::clear(void)
     for(std::vector<double*>::iterator it = Parameters.begin();it!=Parameters.end();++it)
         if(*it) delete *it;
     Parameters.clear();
+    for(std::vector<double*>::iterator it = FixParameters.begin();it!=FixParameters.end();++it)
+        if(*it) delete *it;
+    FixParameters.clear();
 
     // deleting the geometry copied into this sketch
     for(std::vector<GeoDef>::iterator it = Geoms.begin();it!=Geoms.end();++it)
@@ -95,6 +104,70 @@ void Sketch::clear(void)
 
 }
 
+void Sketch::setUpSketch(const std::vector<Part::Geometry *> &geo,const std::vector<Constraint *> &ConstraintList)
+{
+    clear();
+    int rtn = -1;
+
+# if (POINT_CONSTRAIN_OPTIMIZE)
+
+    // pass 1 first check the PointCoincidentConstraint =================================================
+
+    for(std::vector<Constraint *>::const_iterator it = ConstraintList.begin();it!=ConstraintList.end();++it){
+        switch ((*it)->Type){
+           case Coincident: addPointCoincidentConstraint((*it)->First,(*it)->FirstPos,(*it)->Second,(*it)->SecondPos,(*it)->Name.c_str()); break;
+           default: break;
+        }
+    }
+# endif
+
+    // pass 2: building up internel geo list ===========================================================
+    for (std::vector<Part::Geometry *>::const_iterator it = geo.begin();it!=geo.end();++it){
+        if ((*it)->getTypeId()== GeomLineSegment::getClassTypeId()) { // add a line
+            const GeomLineSegment *lineSeg = dynamic_cast<const GeomLineSegment*>((*it));
+            // create the definition struct for that geom
+            addLineSegment(*lineSeg);
+        }
+        else if ((*it)->getTypeId()== GeomCircle::getClassTypeId()) {
+            const GeomCircle *circle = dynamic_cast<const GeomCircle*>((*it));
+            addCircle(*circle);
+        }
+        else {
+            Base::Exception("Sketch::addGeometry(): Unknown or unsoported type added to a sketch");
+        }
+    }
+
+    // pass 3 building up constraints =================================================================
+    // constraints on nothing makes no sense 
+    assert((int)Geoms.size()>0 || ConstraintList.size() == 0 );
+
+    rtn = -1;
+    for(std::vector<Constraint *>::const_iterator it = ConstraintList.begin();it!=ConstraintList.end();++it){
+        // constraints on nothing makes no sense 
+        assert((int)Geoms.size()>0  );
+        switch ((*it)->Type){
+           case Horizontal: addHorizontalConstraint((*it)->First,(*it)->Name.c_str()); break;
+           case Vertical  : addVerticalConstraint((*it)->First,(*it)->Name.c_str());   break;
+#          if !(POINT_CONSTRAIN_OPTIMIZE)
+              case Coincident: addPointCoincidentConstraint((*it)->First,(*it)->FirstPos,(*it)->Second,(*it)->SecondPos,(*it)->Name.c_str()); break;
+#          endif 
+           case Parallel  : addParallelConstraint((*it)->First,(*it)->Second,(*it)->Name.c_str());  break;
+           case Distance  :
+               if((*it)->Second == -1)
+                    rtn = addDistanceConstraint((*it)->First,(*it)->Value, (*it)->Name.c_str());
+               else
+                   rtn = addDistanceConstraint((*it)->First,(*it)->Second,(*it)->Value, (*it)->Name.c_str());
+               break;
+           case Angle: break;
+           case None :  break;
+           case Coincident: break;
+        }
+    }
+
+    // clear the optimize data structure
+    PoPMap.clear();
+}
+
 // Geometry adding ==========================================================
 
 int Sketch::addGeometry(const Part::Geometry *geo)
@@ -110,7 +183,7 @@ int Sketch::addGeometry(const Part::Geometry *geo)
     }
 }
 
-void Sketch::addGeometry(const std::vector<Part::Geometry *> geo)
+void Sketch::addGeometry(const std::vector<Part::Geometry *> &geo)
 {
     for(std::vector<Part::Geometry *>::const_iterator it = geo.begin();it!=geo.end();++it)
         addGeometry(*it);
@@ -153,7 +226,78 @@ int Sketch::addLine(const Part::GeomLineSegment &line)
 }
 
 int Sketch::addLineSegment(const Part::GeomLineSegment &lineSegment)
+
 {
+# if (POINT_CONSTRAIN_OPTIMIZE)
+
+    // create our own copy
+    GeomLineSegment *lineSeg = new GeomLineSegment(lineSegment);
+    // create the definition struct for that geom
+    GeoDef def;
+    def.geo  = lineSeg;
+    def.type = Line;
+    def.construction = false;
+
+    // get the points from the line
+    Base::Vector3d start = lineSeg->getStartPoint();
+    Base::Vector3d end   = lineSeg->getEndPoint();
+
+    // the points for later constraints
+    point p1,p2;
+    def.parameterStartIndex = Parameters.size();
+
+    // check if for the start point is already a constraint point present
+    if(PoPMap[Geoms.size()].StartPointIndex != -1){
+        // if yes, use the coincident point
+        p1.x = Parameters[PoPMap[Geoms.size()].StartPointIndex+0];
+        p1.y = Parameters[PoPMap[Geoms.size()].StartPointIndex+1];
+        // set the values
+        *(Parameters[PoPMap[Geoms.size()].StartPointIndex+0]) = start.x;
+        *(Parameters[PoPMap[Geoms.size()].StartPointIndex+1]) = start.y;
+    }else{
+        // otherwise set the parameter for the solver
+        Parameters.push_back(new double(start.x));
+        Parameters.push_back(new double(start.y));
+        // set the points for later constraints
+        p1.x = Parameters[Parameters.size()-2];
+        p1.y = Parameters[Parameters.size()-1];
+    }
+
+
+    // check if for the end point is already a constraint point present
+    if(PoPMap[Geoms.size()].EndPointIndex != -1){
+        // if yes, use the coincident point
+        p2.x = Parameters[PoPMap[Geoms.size()].EndPointIndex+0];
+        p2.y = Parameters[PoPMap[Geoms.size()].EndPointIndex+1];
+        // set the values
+        *(Parameters[PoPMap[Geoms.size()].EndPointIndex+0]) = end.x;
+        *(Parameters[PoPMap[Geoms.size()].EndPointIndex+1]) = end.y;
+    }else{
+        Parameters.push_back(new double(end.x));
+        Parameters.push_back(new double(end.y));
+        // set the points for later constraints
+        p2.x = Parameters[Parameters.size()-2];
+        p2.y = Parameters[Parameters.size()-1];
+    }
+
+    // add the points
+    def.pointStartIndex = Points.size();
+    Points.push_back(p1);
+    Points.push_back(p2);
+
+    // set the line for later constraints
+    line l;
+    l.p1 = p1;
+    l.p2 = p2;
+    def.lineStartIndex = Lines.size();
+    Lines.push_back(l);
+
+    // store complete set
+    Geoms.push_back(def);
+    
+    // return the position of the newly added geometry
+    return Geoms.size()-1;
+# else
     // create our own copy
     GeomLineSegment *lineSeg = new GeomLineSegment(lineSegment);
     // create the definition struct for that geom
@@ -194,6 +338,7 @@ int Sketch::addLineSegment(const Part::GeomLineSegment &lineSegment)
     
     // return the position of the newly added geometry
     return Geoms.size()-1;
+# endif
 }
 
 int Sketch::addArc(const Part::GeomTrimmedCurve &circleSegment)
@@ -327,6 +472,17 @@ int Sketch::addConstraint(const Constraint * constraint)
        case Coincident:
            rtn = addPointCoincidentConstraint(constraint->First,constraint->FirstPos,constraint->Second,constraint->SecondPos,constraint->Name.c_str());
            break;
+       case Parallel:
+           rtn = addParallelConstraint(constraint->First,constraint->Second,constraint->Name.c_str());
+           break;
+       case Distance:
+           if(constraint->Second == -1)
+                rtn = addDistanceConstraint(constraint->First,constraint->Value, constraint->Name.c_str());
+           else
+               rtn = addDistanceConstraint(constraint->First,constraint->Second,constraint->Value, constraint->Name.c_str());
+           break;
+       case Angle:
+           break;
        case None:
            break;
     }
@@ -384,7 +540,33 @@ int Sketch::addVerticalConstraint(int geoIndex, const char* name)
 }
 
 int Sketch::addPointCoincidentConstraint(int geoIndex1,PointPos Pos1,int geoIndex2,PointPos Pos2, const char* name)
-{   
+{ 
+# if (POINT_CONSTRAIN_OPTIMIZE)
+    // this optimization alter point on point constraints for e.g Line segments
+    // to one point. That means the Lines segments get altered to a polyline. 
+
+    // create one points for the constraint
+    int parameterStartIndex = Parameters.size();
+    Parameters.push_back(new double(0));
+    Parameters.push_back(new double(0));
+
+    // save the index belonging to the geo id for later usage in build up geo
+    switch(Pos1) {
+        case start: PoPMap[geoIndex1].StartPointIndex = parameterStartIndex; break;
+        case end  : PoPMap[geoIndex1].EndPointIndex = parameterStartIndex; break;
+        case mid  : PoPMap[geoIndex1].MidPointIndex = parameterStartIndex; break;
+        case none : break;
+    }
+    switch(Pos2) {
+        case start: PoPMap[geoIndex2].StartPointIndex = parameterStartIndex; break;
+        case end  : PoPMap[geoIndex2].EndPointIndex = parameterStartIndex; break;
+        case mid  : PoPMap[geoIndex2].MidPointIndex = parameterStartIndex; break;
+        case none : break;
+    }
+
+    return Const.size()-1;
+
+# else
     // index out of bounds?
     assert(geoIndex1 < (int)Geoms.size());
     assert(geoIndex2 < (int)Geoms.size());
@@ -412,6 +594,81 @@ int Sketch::addPointCoincidentConstraint(int geoIndex1,PointPos Pos1,int geoInde
 
     Const.push_back(constrain);
 
+    return Const.size()-1;
+# endif
+}
+
+int Sketch::addParallelConstraint(int geoIndex1,int geoIndex2, const char* name)
+{   
+    // index out of bounds?
+    assert(geoIndex1 < (int)Geoms.size());
+    assert(geoIndex2 < (int)Geoms.size());
+    // constraint the right type?
+    assert(Geoms[geoIndex1].type == Line );
+    assert(Geoms[geoIndex2].type == Line );
+
+    // creat the constraint and fill it up
+    ConstrainDef constrain;
+    constrain.constrain.type = parallel;
+    constrain.constrain.line1 = Lines[Geoms[geoIndex1].lineStartIndex];
+    constrain.constrain.line2 = Lines[Geoms[geoIndex2].lineStartIndex];
+
+    if(name)
+        constrain.name = name;
+
+    Const.push_back(constrain);
+
+    return Const.size()-1;
+}
+
+int Sketch::addDistanceConstraint(int geoIndex1, double Value, const char* name)
+{   
+    // index out of bounds?
+    assert(geoIndex1 < (int)Geoms.size());
+    // constraint the right type?
+    assert(Geoms[geoIndex1].type == Line );
+
+    // creat the constraint and fill it up
+    ConstrainDef constrain;
+    constrain.constrain.type = P2PDistance;
+    constrain.constrain.point1 = Points[Geoms[geoIndex1].pointStartIndex];
+    constrain.constrain.point2 = Points[Geoms[geoIndex1].pointStartIndex+1];
+
+    // add the parameter for the length
+    FixParameters.push_back(new double(Value));
+    constrain.constrain.parameter = FixParameters[FixParameters.size()-1];
+
+    if(name)
+        constrain.name = name;
+
+    Const.push_back(constrain);
+
+    return Const.size()-1;
+}
+int Sketch::addDistanceConstraint(int geoIndex1,int geoIndex2, double Value, const char* name)
+{   
+    // index out of bounds?
+    assert(geoIndex1 < (int)Geoms.size());
+    assert(geoIndex2 < (int)Geoms.size());
+    // constraint the right type?
+    assert(Geoms[geoIndex1].type == Line );
+    assert(Geoms[geoIndex2].type == Line );
+
+    // creat the constraint and fill it up
+    ConstrainDef constrain;
+    constrain.constrain.type = P2LDistance;
+    constrain.constrain.line1 = Lines[Geoms[geoIndex1].lineStartIndex];
+    constrain.constrain.point1 = Points[Geoms[geoIndex2].pointStartIndex];
+
+    // add the parameter for the length
+    Parameters.push_back(new double(Value));
+    constrain.constrain.parameter = Parameters[Parameters.size()-1];
+
+    if(name)
+        constrain.name = name;
+
+    Const.push_back(constrain);
+
     return Const.size()-1;
 }
 
@@ -432,7 +689,7 @@ void _redirectPoint(point &c,double &fixedValue1,double &fixedValue2,double * fi
 int Sketch::solve(double * fixed[2]) {
 
     Base::TimeInfo start_time;
-    Base::Console().Log("solv: Start solving (C:%d;G%d) ",Const.size(),Geoms.size());
+    //Base::Console().Log("solv: Start solving (C:%d;G%d) ",Const.size(),Geoms.size());
     Solver s;
     double fixedValue1=0,fixedValue2=0;
 
@@ -475,12 +732,12 @@ int Sketch::solve(double * fixed[2]) {
                     GeomLineSegment *lineSeg = dynamic_cast<GeomLineSegment*>(it->geo);
                     lineSeg->setPoints(
                              Vector3d(
-                                 *Parameters[it->parameterStartIndex+0],
-                                 *Parameters[it->parameterStartIndex+1],
+                                 *Points[it->pointStartIndex+0].x,
+                                 *Points[it->pointStartIndex+0].y,
                                  0.0),
                              Vector3d(
-                                 *Parameters[it->parameterStartIndex+2],
-                                 *Parameters[it->parameterStartIndex+3],
+                                 *Points[it->pointStartIndex+1].x,
+                                 *Points[it->pointStartIndex+1].y,
                                  0.0)
                          );
                 }
@@ -490,10 +747,12 @@ int Sketch::solve(double * fixed[2]) {
             }
 
         }
+    }else{
+        //Base::Console().Log("NotSolved ");
     }
     Base::TimeInfo end_time;
-    Base::Console().Log("T:%s\n",Base::TimeInfo::diffTime(start_time,end_time).c_str());
-
+    //Base::Console().Log("T:%s\n",Base::TimeInfo::diffTime(start_time,end_time).c_str());
+    SolveTime = Base::TimeInfo::diffTimeF(start_time,end_time);
 	return ret;
 }
 
@@ -523,12 +782,14 @@ int Sketch::movePoint(int geoIndex1,PointPos Pos1,Base::Vector3d toPoint)
     return solve(fixed);
 }
 
-TopoShape Sketch::toShape(void)
+TopoShape Sketch::toShape(void) const
 {
     TopoShape result;
     std::vector<GeoDef>::const_iterator it=Geoms.begin();
-    // skeep the default elements
+
+    // skip the default elements
     it += IntGeoOffset;
+#if 0
 
     bool first = true;
     for(;it!=Geoms.end();++it){
@@ -544,6 +805,79 @@ TopoShape Sketch::toShape(void)
         }
     }
     return result;
+#else
+    std::list<TopoDS_Edge> edge_list;
+    std::list<TopoDS_Wire> wires;
+
+    // collecting all (non constructive) edges out of the sketch
+    for(;it!=Geoms.end();++it){
+        if(!it->construction){
+            edge_list.push_back(TopoDS::Edge(it->geo->toShape()));
+        }
+    }
+
+    // sort them together to wires
+    while (edge_list.size() > 0) {
+        BRepBuilderAPI_MakeWire mkWire;
+        // add and erase first edge
+        mkWire.Add(edge_list.front());
+        edge_list.erase(edge_list.begin());
+
+        TopoDS_Wire new_wire = mkWire.Wire(); // current new wire
+
+        // try to connect each edge to the wire, the wire is complete if no more egdes are connectible
+        bool found = false;
+        do {
+            found = false;
+            for (std::list<TopoDS_Edge>::iterator pE = edge_list.begin(); pE != edge_list.end();++pE) {
+                mkWire.Add(*pE);
+                if (mkWire.Error() != BRepBuilderAPI_DisconnectedWire) {
+                    // edge added ==> remove it from list
+                    found = true;
+                    edge_list.erase(pE);
+                    new_wire = mkWire.Wire();
+                    break;
+                }
+            }
+        }
+        while (found);
+        wires.push_back(new_wire);
+    }
+    if (wires.size() == 1)
+        result = *wires.begin();
+    else if (wires.size() > 1) {
+        // FIXME: The right way here would be to determine the outer and inner wires and 
+        // generate a face with holes (inner wires have to be taged REVERSE or INNER).
+        // thats the only way to transport a somwhat more complex sketch...
+        //result = *wires.begin();
+        
+        // I think a compound can be used as container because it is just a collection of
+        // shapes and doesn't need too much information about the topology.
+        // The actual knowledge how to create a prism from several wires should go to the Pad
+        // feature (Werner).
+        BRep_Builder builder;
+        TopoDS_Compound comp;
+        builder.MakeCompound(comp);
+        for (std::list<TopoDS_Wire>::iterator wt = wires.begin(); wt != wires.end(); ++wt)
+            builder.Add(comp, *wt);
+        result._Shape = comp;
+    }
+    // FIXME: if free edges are left over its probably better to 
+    // create a compound with the closed structures and let the 
+    // features decide what to do with it...
+    if (edge_list.size() > 0)
+        Base::Console().Warning("Left offer edges in Sketch. Only closed structures will be propagated at the moment!");
+
+#endif
+
+    return result;
+
+}
+
+void Sketch::getGeoVertexIndex(int VertexId,int &GeoId,int &PointPos)
+{
+    PointPos = VertexId%2 + 1;
+    GeoId = (int)VertexId/2;
 }
 
 // Persistance implementer -------------------------------------------------
@@ -555,187 +889,11 @@ unsigned int Sketch::getMemSize (void) const
 
 void Sketch::Save (Writer &writer) const
 {
- /*   for(unsigned int i=0;i<6;i++){
-        Base::Placement Tip = toPlacement(Kinematic.getSegment(i).getFrameToTip());
-	    writer.Stream() << writer.ind() << "<Axis "
-                        << "Px=\""          <<  Tip.getPosition().x  << "\" " 
-                        << "Py=\""          <<  Tip.getPosition().y  << "\" "
-                        << "Pz=\""          <<  Tip.getPosition().z  << "\" "
-					    << "Q0=\""          <<  Tip.getRotation()[0] << "\" "
-                        << "Q1=\""          <<  Tip.getRotation()[1] << "\" "
-                        << "Q2=\""          <<  Tip.getRotation()[2] << "\" "
-                        << "Q3=\""          <<  Tip.getRotation()[3] << "\" "
-                        << "maxAngle=\""    <<  MaxAngle[i]		     << "\" "
-                        << "minAngle=\""    <<  MinAngle[i]			 << "\" "
-                        << "AxisVelocity=\""<<  Velocity[i]          << "\" "
-                        << "Pos=\""         <<  Actuall(i)           << "\"/>"
-					                      
-                        << std::endl;
-    }*/
 
 }
 
 void Sketch::Restore(XMLReader &reader)
 {
-    //Chain Temp;
-    //Base::Placement Tip;
-
-    //for(unsigned int i=0;i<6;i++){
-    //    // read my Element
-    //    reader.readElement("Axis");
-    //    // get the value of the placement
-    //    Tip =    Base::Placement(Base::Vector3d(reader.getAttributeAsFloat("Px"),
-    //                                            reader.getAttributeAsFloat("Py"),
-    //                                            reader.getAttributeAsFloat("Pz")),
-    //                             Base::Rotation(reader.getAttributeAsFloat("Q0"),
-    //                                            reader.getAttributeAsFloat("Q1"),
-    //                                            reader.getAttributeAsFloat("Q2"),
-    //                                            reader.getAttributeAsFloat("Q3")));
-    //    Temp.addSegment(Segment(Joint(Joint::RotZ),toFrame(Tip)));
-
-
-    //    MaxAngle[i]   = reader.getAttributeAsFloat("maxAngle");
-    //    MinAngle[i]	  = reader.getAttributeAsFloat("minAngle");
-    //    if(reader.hasAttribute("AxisVelocity"))
-    //        Velocity[i] = reader.getAttributeAsFloat("AxisVelocity");
-    //    else
-    //        Velocity[i] = 156.0;
-    //    Actuall(i) = reader.getAttributeAsFloat("Pos");
-    //}
-    //Kinematic = Temp;
-
-    //calcTcp();
-
+ 
 }
 
-#if 0
-
-using namespace std;
-
-double parameters[100],constants[100];
-double *P,*C;
-
-point points[100];
-line lines[100];
-
-circle circles[100];
-arc arcs[100];
-constraint cons[100];
-
-
-int Sketch::solve(void) {
-	P= parameters;
-
-	point origin;
-	double zero = 0;
-	origin.x = &zero;
-	origin.y = &zero;
-
-	constants[0]=6;
-	constants[1]=6;
-	constants[2]=2;
-	constants[3]=M_PI/3;
-	constants[4]=3;
-
-	points[0].x = &parameters[0];
-	points[0].y = &parameters[1];
-	points[1].x = &parameters[2];
-	points[1].y = &parameters[3];
-	points[2].x = &parameters[4];
-	points[2].y = &parameters[5];
-	points[3].x = &parameters[6];
-	points[3].y = &parameters[7];
-	points[4].x = &constants[0];
-	points[4].y = &constants[1];
-
-
-
-
-	lines[0].p1 = points[0];
-	lines[0].p2 = points[1];
-	lines[1].p1 = points[1];
-	lines[1].p2 = points[2];
-
-
-	cons[0].type = horizontal;
-	cons[0].line1 = lines[0];
-
-	cons[1].type = vertical;
-	cons[1].line1 = lines[1];
-
-	cons[2].type = pointOnPoint;
-	cons[2].point1 = points[2];
-	cons[2].point2 = points[4];
-
-	cons[3].type = pointOnPoint;
-	cons[3].point1 = points[3];
-	cons[3].point2 = points[4];
-
-	for(int i=0;i<1;i++)
-	{
-	parameters[0]=0;//1x
-	parameters[1]=0;//y
-	parameters[2]=5;//x
-	parameters[3]=0;//y
-	parameters[4]=6;//xstart
-	parameters[5]=5;//y
-
-	parameters[6]=6;//xend
-	parameters[7]=5;//y
-
-
-
-	int sol;
-
-
-
-	double  *pparameters[100];
-
-	for(i=0;i<100;i++)
-	{
-		pparameters[i] = &parameters[i];
-	}
-
-    Solver s;
-    sol=s.solve(pparameters ,8,cons,4,rough);
-
-	if(sol==succsess)
-	{
-		cout<<"A good Solution was found"<<endl;
-	}
-
-	else if(sol==noSolution)
-	{
-		cout<<"No valid Solutions were found from this start point"<<endl;
-	}
-	for(int i=0;i<8;i++)
-	{
-		cout<<"Point"<<*pparameters[i]<<endl;
-	}
-	double hey = parameters[0]*parameters[2]+parameters[1]*parameters[3];
-	cout<<"dot product: "<<hey<<endl;
-
-	double gradF[20];
-		double *ggradF[20];
-
-
-		for(int i=0;i<20;i++)
-			{
-			gradF[i]=0;
-			ggradF[i]=&gradF[i];
-			}
-
-		//derivatives(pparameters, gradF, 4, cons,3);
-
-		for(int i=0;i<4;i++) cout<<"GradF["<<i<<"]: "<<*ggradF[i]<<endl;
-
-
-
-	}
-	//end
-
-
-	return 0;
-}
-
-#endif
