@@ -33,6 +33,7 @@
 #include "Workbench.h"
 #include "ViewProviderSketchSF.h"
 #include "ViewProviderSketch.h"
+#include "ViewProviderPython.h"
 
 
 // create the commands
@@ -60,7 +61,8 @@ void SketcherGuiExport initSketcherGui()
         return;
     }
     try {
-        Base::Interpreter().loadModule("PartGui");
+        Base::Interpreter().runString("import PartGui");
+        Base::Interpreter().runString("import Sketcher");
     }
     catch(const Base::Exception& e) {
         PyErr_SetString(PyExc_ImportError, e.what());
@@ -70,7 +72,7 @@ void SketcherGuiExport initSketcherGui()
     (void) Py_InitModule("SketcherGui", SketcherGui_Import_methods);   /* mod name, table ptr */
     Base::Console().Log("Loading GUI of Sketcher module... done\n");
 
-    // instanciating the commands
+    // instantiating the commands
     CreateSketcherCommands();
     CreateSketcherCommandsCreateGeo();
     CreateSketcherCommandsConstraints();
@@ -78,8 +80,11 @@ void SketcherGuiExport initSketcherGui()
     SketcherGui::Workbench::init();
 
 	// init objects
-	SketcherGui::ViewProviderSketchSF::init();
-	SketcherGui::ViewProviderSketch  ::init();
+	SketcherGui::ViewProviderSketchSF       ::init();
+	SketcherGui::ViewProviderSketch         ::init();
+    SketcherGui::ViewProviderPython         ::init();
+    SketcherGui::ViewProviderCustom         ::init();
+    SketcherGui::ViewProviderCustomPython   ::init();
 
      // add resources and reloads the translators
     loadSketcherResource();
