@@ -23,6 +23,11 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <TopoDS_Shape.hxx>
+# include <TopoDS_Face.hxx>
+# include <TopoDS.hxx>
+# include <BRepAdaptor_Surface.hxx>
+# include <QMessageBox>
 #endif
 
 #include <Gui/Application.h>
@@ -30,13 +35,17 @@
 #include <Gui/Command.h>
 #include <Gui/MainWindow.h>
 #include <Gui/DlgEditFileIncludeProptertyExternal.h>
+#include <Gui/SelectionFilter.h>
 
 #include <Mod/Sketcher/App/SketchObjectSF.h>
+#include <Mod/Sketcher/App/SketchObject.h>
+#include <Mod/Part/App/Part2DObject.h>
 
 #include "ViewProviderSketch.h"
 
 using namespace std;
 using namespace SketcherGui;
+using namespace Part;
 
 /* Sketch commands =======================================================*/
 DEF_STD_CMD_A(CmdSketcherNewSketch);
@@ -56,13 +65,75 @@ CmdSketcherNewSketch::CmdSketcherNewSketch()
 
 void CmdSketcherNewSketch::activated(int iMsg)
 {
+    Gui::SelectionFilter SketchFilter("SELECT Sketcher::SketchObject COUNT 1");
+    Gui::SelectionFilter FaceFilter  ("SELECT Part::Feature SUBELEMENT Face COUNT 1");
 
-    std::string FeatName = getUniqueObjectName("Sketch");
+    if (SketchFilter.match()) {
+        Sketcher::SketchObject *Sketch = static_cast<Sketcher::SketchObject*>(SketchFilter.Result[0][0].getObject());
+        openCommand("Edit Sketch");
+        doCommand(Gui,"Gui.activeDocument().setEdit('%s')",Sketch->getNameInDocument());
+    }
+    else if (FaceFilter.match()) {
+        // get the selected object
+        Part::Feature *part = static_cast<Part::Feature*>(FaceFilter.Result[0][0].getObject());
+        Base::Placement ObjectPos = part->Placement.getValue();
+        const std::vector<std::string> &sub = FaceFilter.Result[0][0].getSubNames();
+        if (sub.size() > 1){
+            // No assert for wrong user input!
+            QMessageBox::warning(Gui::getMainWindow(), QObject::tr("Several sub-elements selected"),
+                QObject::tr("You have to select a single face as support for a sketch!"));
+            return;
+        }
+        // get the selected sub shape (a Face)
+        const Part::TopoShape &shape = part->Shape.getValue();
+        TopoDS_Shape sh = shape.getSubShape(sub[0].c_str());
+        const TopoDS_Face& face = TopoDS::Face(sh);
+        if (face.IsNull()){
+            // No assert for wrong user input!
+            QMessageBox::warning(Gui::getMainWindow(), QObject::tr("No support face selected"),
+                QObject::tr("You have to select a face as support for a sketch!"));
+            return;
+        }
 
-    openCommand("Create a new Sketch");
-    doCommand(Doc,"App.activeDocument().addObject('Sketcher::SketchObject','%s')",FeatName.c_str());
-    commitCommand();
-    getDocument()->recompute();
+        BRepAdaptor_Surface adapt(face);
+        if (adapt.GetType() != GeomAbs_Plane){
+            QMessageBox::warning(Gui::getMainWindow(), QObject::tr("No planar support"),
+                QObject::tr("You need a planar face as support for a sketch!"));
+            return;
+        }
+        Base::Placement placement = Part2DObject::positionBySupport(face,ObjectPos);
+        double a,b,c;
+        placement.getRotation().getYawPitchRoll(a,b,c);
+
+        std::string supportString = FaceFilter.Result[0][0].getAsPropertyLinkSubString();
+ 
+        // create Sketch on Face
+        std::string FeatName = getUniqueObjectName("Sketch");
+
+        openCommand("Create a Sketch on Face");
+        doCommand(Doc,"App.activeDocument().addObject('Sketcher::SketchObject','%s')",FeatName.c_str());
+        doCommand(Gui,"App.activeDocument().%s.Placement = FreeCAD.Placement(FreeCAD.Vector(%f,%f,%f),FreeCAD.Rotation(%f,%f,%f))",FeatName.c_str(),placement.getPosition().x,placement.getPosition().y,placement.getPosition().z,a,b,c);
+        doCommand(Gui,"App.activeDocument().%s.Support = %s",FeatName.c_str(),supportString.c_str());
+        //doCommand(Gui,"Gui.activeDocument().activeView().setCamera('%s')",cam.c_str());
+        doCommand(Gui,"Gui.activeDocument().setEdit('%s')",FeatName.c_str());
+
+
+
+
+    }
+    else {
+        // do the standard Z+ new Sketch
+        const char camstring[] = "#Inventor V2.1 ascii \\n OrthographicCamera { \\n viewportMapping ADJUST_CAMERA \\n position 0 0 87 \\n orientation 0 0 1  0 \\n nearDistance 37 \\n farDistance 137 \\n aspectRatio 1 \\n focalDistance 87 \\n height 119 }";
+        std::string FeatName = getUniqueObjectName("Sketch");
+
+        std::string cam(camstring);
+
+        openCommand("Create a new Sketch");
+        doCommand(Doc,"App.activeDocument().addObject('Sketcher::SketchObject','%s')",FeatName.c_str());
+        doCommand(Gui,"Gui.activeDocument().activeView().setCamera('%s')",cam.c_str());
+        doCommand(Gui,"Gui.activeDocument().setEdit('%s')",FeatName.c_str());
+    }
+ 
 }
 
 bool CmdSketcherNewSketch::isActive(void)
@@ -91,8 +162,11 @@ CmdSketcherLeaveSketch::CmdSketcherLeaveSketch()
 
 void CmdSketcherLeaveSketch::activated(int iMsg)
 {
-    Gui::Document *doc = getActiveGuiDocument();
-    doc->resetEdit();      
+    openCommand("Sketch changed");
+    doCommand(Gui,"Gui.activeDocument().resetEdit()");
+    doCommand(Doc,"App.ActiveDocument.recompute()");
+    commitCommand();
+
 }
 
 bool CmdSketcherLeaveSketch::isActive(void)
