@@ -63,9 +63,6 @@ public:
 	virtual void attach(App::DocumentObject *);
 	virtual void updateData(const App::Property*);
 
-	virtual bool setEdit(int ModNum);
-	virtual void unsetEdit(void);
-
 	/// Is called by the tree if the user double click on the object
 	virtual bool doubleClicked(void);
 
@@ -114,6 +111,8 @@ public:
     Sketcher::SketchObjectSF* getSketchObjectSF(void);
 
 protected:
+    virtual bool setEdit(int ModNum);
+    virtual void unsetEdit(int ModNum);
     /// helper to detect whether the picked point lies on the sketch
     bool isPointOnSketch(const SoPickedPoint* pp) const;
     /// get called by the container whenever a property has been changed
