@@ -28,10 +28,18 @@
 #endif
 
 #include "Workbench.h"
+#include <Gui/MenuManager.h>
 #include <Gui/ToolBarManager.h>
 
 using namespace SketcherGui;
 
+#if 0 // needed for Qt's lupdate utility
+    qApp->translate("Workbench", "Sketcher");
+    qApp->translate("Workbench", "Sketcher geoms");
+    qApp->translate("Workbench", "Sketcher constraints");
+#endif
+
+/// @namespace SketcherGui @class Workbench
 TYPESYSTEM_SOURCE(SketcherGui::Workbench, Gui::StdWorkbench)
 
 Workbench::Workbench()
@@ -42,31 +50,71 @@ Workbench::~Workbench()
 {
 }
 
+Gui::MenuItem* Workbench::setupMenuBar() const
+{
+    Gui::MenuItem* root = StdWorkbench::setupMenuBar();
+    Gui::MenuItem* item = root->findItem("&Windows");
+
+    Gui::MenuItem* sketch = new Gui::MenuItem;
+    root->insertItem(item, sketch);
+    sketch->setCommand("Ske&tch");
+    Gui::MenuItem* geom = new Gui::MenuItem();
+    geom->setCommand("Sketcher geoms");
+    *geom /*<< "Sketcher_CreatePoint"*/
+          /*<< "Sketcher_CreateArc"*/
+          /*<< "Sketcher_CreateCircle"*/
+          << "Sketcher_CreateLine"
+          << "Sketcher_CreatePolyline"
+          << "Sketcher_CreateBox"
+          /*<< "Sketcher_CreateText"*/
+          /*<< "Sketcher_CreateDraftLine"*/;
+    Gui::MenuItem* cons = new Gui::MenuItem();
+    cons->setCommand("Sketcher constraints");
+    *cons /*<< "Sketcher_ConstrainLock"*/
+          << "Sketcher_ConstrainCoincident"
+          << "Sketcher_ConstrainVertical"
+          << "Sketcher_ConstrainHorizontal"
+          << "Sketcher_ConstrainDistance"
+          << "Sketcher_ConstrainParallel";
+    *sketch 
+        << "Sketcher_NewSketch"
+        << geom
+        << cons
+        << "Separator" 
+        << "PartDesign_Pad" 
+        << "PartDesign_Fillet"
+    ;
+
+    return root;
+}
+
 Gui::ToolBarItem* Workbench::setupToolBars() const
 {
     Gui::ToolBarItem* root = StdWorkbench::setupToolBars();
 
     Gui::ToolBarItem* part = new Gui::ToolBarItem(root);
-    part->setCommand(QT_TR_NOOP("Sketcher"));
+    part->setCommand("Sketcher");
     *part << "Sketcher_NewSketch"<< "Sketcher_LeaveSketch";
 
     part = new Gui::ToolBarItem(root);
-    part->setCommand(QT_TR_NOOP("Sketcher geoms"));
-    *part << "Sketcher_CreatePoint" 
-		  << "Sketcher_CreateArc"
-		  << "Sketcher_CreateCircle"
+    part->setCommand("Sketcher geoms");
+    *part /*<< "Sketcher_CreatePoint" */
+		  /*<< "Sketcher_CreateArc"*/
+		  /*<< "Sketcher_CreateCircle"*/
 		  << "Sketcher_CreateLine"
 		  << "Sketcher_CreatePolyline"
-		  << "Sketcher_CreateRectangle"
-		  << "Sketcher_CreateText"
-		  << "Sketcher_CreateDraftLine";
+		  << "Sketcher_CreateBox"
+		  /*<< "Sketcher_CreateText"*/
+		  /*<< "Sketcher_CreateDraftLine"*/;
 
     part = new Gui::ToolBarItem(root);
-    part->setCommand(QT_TR_NOOP("Sketcher constrains"));
-    *part << "Sketcher_ConstrainLock"
+    part->setCommand("Sketcher constraints");
+    *part /*<< "Sketcher_ConstrainLock"*/
 		  << "Sketcher_ConstrainCoincident"
 		  << "Sketcher_ConstrainVertical"
-		  << "Sketcher_ConstrainHorizontal";
+		  << "Sketcher_ConstrainHorizontal"
+		  << "Sketcher_ConstrainDistance"
+		  << "Sketcher_ConstrainParallel";
      return root;
 }
 
