@@ -37,6 +37,7 @@ def All():
     suite.addTest(unittest.defaultTestLoader.loadTestsFromName("UnitTests") )
     suite.addTest(unittest.defaultTestLoader.loadTestsFromName("Base") )
     suite.addTest(unittest.defaultTestLoader.loadTestsFromName("MeshTestsApp") )
+    suite.addTest(unittest.defaultTestLoader.loadTestsFromName("SketcherTestsApp") )
     if ( FreeCAD.GuiUp == 1):
         suite.addTest(unittest.defaultTestLoader.loadTestsFromName("Workbench") )
         suite.addTest(unittest.defaultTestLoader.loadTestsFromName("Menu") )
@@ -44,6 +45,7 @@ def All():
 
     
 def TestText(s):
+    s = unittest.defaultTestLoader.loadTestsFromName(s)
     r = unittest.TextTestRunner()
     r.run(s)
 
