@@ -41,6 +41,7 @@ from urllib2 import urlopen, HTTPError
 DEFAULTURL = "http://sourceforge.net/apps/mediawiki/free-cad" #default URL if no URL is passed
 INDEX = "Online_Help_Toc" # the start page from where to crawl the wiki
 NORETRIEVE = ['Manual','Developer_hub','Power_users_hub','Users_hub','Source_documentation', 'User_hub','Main_Page','About_this_site'] # pages that won't be fetched (kept online)
+GETTRANSLATIONS = True # Set true if you want to get the translations too.
 MAXFAIL = 3 # max number of retries if download fails
 VERBOSE = True # to display what's going on. Otherwise, runs totally silent.
 COMPILE = True # Wether qt assistant will be used to compile the final help file
@@ -457,6 +458,8 @@ def cleanhtml(html):
     html = re.compile('<div class="NavHead.*?</div>').sub('',html) # removing nav stuff
     html = re.compile('<div class="NavContent.*?</div>').sub('',html) # removing nav stuff
     html = re.compile('<div class="NavEnd.*?</div>').sub('',html) # removing nav stuff
+    if not GETTRANSLATIONS:
+        html = re.compile('<div class="languages.*?</div>').sub('',html) # removing translations links
     html = re.compile('Wlinebreak').sub('\n',html) # restoring original linebreaks
     return html
     
@@ -477,6 +480,9 @@ def getlinks(html):
                 NORETRIEVE.append(rg)
             if "&" in rg:
                 NORETRIEVE.append(rg)
+            if "/" in rg:
+                if not GETTRANSLATIONS:
+                    NORETRIEVE.append(rg)
             pages.append(rg)
     return pages
 
@@ -491,7 +497,7 @@ def cleanlinks(html, pages=None):
         if  page in NORETRIEVE:
             output = 'href="' + URL + wikiindex + page + '"'
         else:
-            output = 'href="' + page + '.html"'
+            output = 'href="' + page.replace("/","-") + '.html"'
         html = re.compile('href="[^"]+' + page + '"').sub(output,html)
     return html
 
@@ -561,13 +567,13 @@ def output(html,page):
     header += "</head><body>"
     footer = "</body></html>"
     html = header+html+footer
-    filename = local(page)
+    filename = local(page.replace("/","-"))
     file = open(filename,'wb')
     file.write(html)
     file.close()
 
 def main(arg):
-	global QHELPCOMPILER,QCOLLECTIOMGENERATOR,OUTPUTPATH,PDFOUTPUT,PDFCONVERTOR
+	global QHELPCOMPILER,QCOLLECTIOMGENERATOR,OUTPUTPATH,PDFOUTPUT,PDFCONVERTOR,TMPFOLDER
 	try:
 		opts, args = getopt.getopt(sys.argv[1:], "hp:t:c:g:o:", ["help", "pdf=", "noremove", "tempfolder=", "helpcompiler-exe=", "out-path=", "helpgenerator-exe="])
 	except getopt.GetoptError:
@@ -579,14 +585,14 @@ def main(arg):
 	for o, a in opts:
 		if o == "-v":
 			VERBOSE = True
-                if o in ("-p","--pdf"):
-                        PDFOUTPUT = True
-                        if a in ['pisa','htmldoc']:
-                            print "using pdf converter:",a
-                            PDFCONVERTOR = a
-                if o in ("-t","--tempfolder"):
-                        print "using tempfolder:",a
-                        TMPFOLDER = a
+		if o in ("-p","--pdf"):
+			PDFOUTPUT = True
+			if a in ['pisa','htmldoc']:
+				print "using pdf converter:",a
+				PDFCONVERTOR = a
+		if o in ("-t","--tempfolder"):
+			print "using tempfolder:",a
+			TMPFOLDER = a
 		if o in ("-h", "--help"):
 			sys.stderr.write(usage)
 			sys.exit()
